#include "testconfig.h"
#if defined(HAVE_LLRINT)
#define TEST_FUNC_F_LL __MATH_PRECNAME(llrint)
#include "testdriver.h"



static test_f_ll_data const llrint_data[] = {
	{ __LINE__, QNAN_P, LONG_LONG_MIN, 0 },
	{ __LINE__, QNAN_M, LONG_LONG_MIN, 0 },
	{ __LINE__, SNAN_P, LONG_LONG_MIN, 0 },
	{ __LINE__, SNAN_M, LONG_LONG_MIN, 0 },
	{ __LINE__, INF_P, LONG_LONG_MIN, 0 },
	{ __LINE__, INF_M, LONG_LONG_MIN, 0 },
	{ __LINE__, ZERO_P, 0, 0 },
	{ __LINE__, ZERO_M, 0, 0 },
	{ __LINE__, SUBNORM_P, 0, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, 0, FLAG_SUBNORM },
	{ __LINE__, MIN_P, 0, 0 },
	{ __LINE__, MIN_M, 0, 0 },
	{ __LINE__, MAX_P, LONG_LONG_MIN, 0 },
	{ __LINE__, MAX_M, LONG_LONG_MIN, 0 },

	{ __LINE__, HEXCONSTE(0.2L, 0x3ffc, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-0.2L, 0xbffc, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION },

	{ __LINE__, HEXCONSTE(1.0L, 0x3fff, 0x80000000L, 0x00000000L), 1, NO_INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.4L, 0x3fff, 0xb3333333L, 0x33333333L), 1, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.5L, 0x3fff, 0xc0000000L, 0x00000000L), 2, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.6L, 0x3fff, 0xccccccccL, 0xcccccccdL), 2, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(2.4L, 0x4000, 0x99999999L, 0x9999999aL), 2, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(2.5L, 0x4000, 0xa0000000L, 0x00000000L), 2, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(2.6L, 0x4000, 0xa6666666L, 0x66666666L), 3, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.4L, 0x4000, 0xd9999999L, 0x9999999aL), 3, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.5L, 0x4000, 0xe0000000L, 0x00000000L), 4, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.6L, 0x4000, 0xe6666666L, 0x66666666L), 4, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-0.4L, 0xbffd, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-0.5L, 0xbffe, 0x80000000L, 0x00000000L), 0, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-0.6L, 0xbffe, 0x99999999L, 0x9999999aL), -1, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-1.0L, 0xbfff, 0x80000000L, 0x00000000L), -1, NO_INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-1.4L, 0xbfff, 0xb3333333L, 0x33333333L), -1, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-1.5L, 0xbfff, 0xc0000000L, 0x00000000L), -2, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-1.6L, 0xbfff, 0xccccccccL, 0xcccccccdL), -2, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-2.4L, 0xc000, 0x99999999L, 0x9999999aL), -2, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-2.5L, 0xc000, 0xa0000000L, 0x00000000L), -2, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-2.6L, 0xc000, 0xa6666666L, 0x66666666L), -3, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-3.4L, 0xc000, 0xd9999999L, 0x9999999aL), -3, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-3.5L, 0xc000, 0xe0000000L, 0x00000000L), -4, INEXACT_EXCEPTION },
	{ __LINE__, HEXCONSTE(-3.6L, 0xc000, 0xe6666666L, 0x66666666L), -4, INEXACT_EXCEPTION },

	{ __LINE__, HEXCONSTE(8388600.3L, 0x4015, 0xfffff099L, 0x9999999aL), 8388600LL, 0 },
	{ __LINE__, HEXCONSTE(-8388600.3L, 0xc015, 0xfffff099L, 0x9999999aL), -8388600LL, 0 },

#ifndef TEST_FLOAT
	{ __LINE__, HEXCONSTE(1071930.0008, 0x4013, 0x82d9d001L, 0xa36e2eb2L), 1071930LL, 0 },
#endif

	/* Test boundary conditions.  */
	/* 0x1FFFFF */
	{ __LINE__, HEXCONSTE(2097151.0, 0x4013, 0xfffff800L, 0x00000000L), 2097151LL, 0 },
	/* 0x800000 */
	{ __LINE__, HEXCONSTE(8388608.0, 0x4016, 0x80000000L, 0x00000000L), 8388608LL, 0 },
	/* 0x1000000 */
	{ __LINE__, HEXCONSTE(16777216.0, 0x4017, 0x80000000L, 0x00000000L), 16777216LL, NO_INEXACT_EXCEPTION },
	/* 0x20000000000 */
	{ __LINE__, HEXCONSTE(2199023255552.0, 0x4028, 0x80000000L, 0x00000000L), 2199023255552LL, NO_INEXACT_EXCEPTION },
	/* 0x40000000000 */
	{ __LINE__, HEXCONSTE(4398046511104.0, 0x4029, 0x80000000L, 0x00000000L), 4398046511104LL, NO_INEXACT_EXCEPTION },
	/* 0x1000000000000 */
	{ __LINE__, HEXCONSTE(281474976710656.0, 0x402f, 0x80000000L, 0x00000000L), 281474976710656LL, NO_INEXACT_EXCEPTION },
	/* 0x10000000000000 */
	{ __LINE__, HEXCONSTE(4503599627370496.0, 0x4033, 0x80000000L, 0x00000000L), 4503599627370496LL, NO_INEXACT_EXCEPTION },
	/* 0x10000080000000 */
	{ __LINE__, HEXCONSTE(4503601774854144.0, 0x4033, 0x80000400L, 0x00000000L), 4503601774854144LL, 0 },
	/* 0x20000000000000 */
	{ __LINE__, HEXCONSTE(9007199254740992.0, 0x4034, 0x80000000L, 0x00000000L), 9007199254740992LL, NO_INEXACT_EXCEPTION },
	/* 0x80000000000000 */
	{ __LINE__, HEXCONSTE(36028797018963968.0, 0x4036, 0x80000000L, 0x00000000L), 36028797018963968LL, NO_INEXACT_EXCEPTION },
	/* 0x100000000000000 */
	{ __LINE__, HEXCONSTE(72057594037927936.0, 0x4037, 0x80000000L, 0x00000000L), 72057594037927936LL, NO_INEXACT_EXCEPTION },
#ifdef TEST_LDOUBLE
	/* The input can only be represented in long double.  */
	{ __LINE__, HEXCONSTE(4503599627370495.5L, 0x4032, 0xffffffffL, 0xfffff800L), 4503599627370496LL, 0 },
	{ __LINE__, HEXCONSTE(4503599627370496.25L, 0x4033, 0x80000000L, 0x00000200L), 4503599627370496LL, 0 },
	{ __LINE__, HEXCONSTE(4503599627370496.5L, 0x4033, 0x80000000L, 0x00000400L), 4503599627370496LL, 0 },
	{ __LINE__, HEXCONSTE(4503599627370496.75L, 0x4033, 0x80000000L, 0x00000600L), 4503599627370497LL, 0 },
	{ __LINE__, HEXCONSTE(4503599627370497.5L, 0x4033, 0x80000000L, 0x00000c00L), 4503599627370498LL, 0 },

	{ __LINE__, HEXCONSTE(-4503599627370495.5L, 0xc032, 0xffffffffL, 0xfffff800L), -4503599627370496LL, 0 },
	{ __LINE__, HEXCONSTE(-4503599627370496.25L, 0xc033, 0x80000000L, 0x00000200L), -4503599627370496LL, 0 },
	{ __LINE__, HEXCONSTE(-4503599627370496.5L, 0xc033, 0x80000000L, 0x00000400L), -4503599627370496LL, 0 },
	{ __LINE__, HEXCONSTE(-4503599627370496.75L, 0xc033, 0x80000000L, 0x00000600L), -4503599627370497LL, 0 },
	{ __LINE__, HEXCONSTE(-4503599627370497.5L, 0xc033, 0x80000000L, 0x00000c00L), -4503599627370498LL, 0 },

	{ __LINE__, HEXCONSTE(9007199254740991.5L, 0x4033, 0xffffffffL, 0xfffffc00L), 9007199254740992LL, 0 },
	{ __LINE__, HEXCONSTE(9007199254740992.25L, 0x4034, 0x80000000L, 0x00000100L), 9007199254740992LL, 0 },
	{ __LINE__, HEXCONSTE(9007199254740992.5L, 0x4034, 0x80000000L, 0x00000200L), 9007199254740992LL, 0 },
	{ __LINE__, HEXCONSTE(9007199254740992.75L, 0x4034, 0x80000000L, 0x00000300L), 9007199254740993LL, 0 },
	{ __LINE__, HEXCONSTE(9007199254740993.5L, 0x4034, 0x80000000L, 0x00000600L), 9007199254740994LL, 0 },

	{ __LINE__, HEXCONSTE(-9007199254740991.5L, 0xc033, 0xffffffffL, 0xfffffc00L), -9007199254740992LL, 0 },
	{ __LINE__, HEXCONSTE(-9007199254740992.25L, 0xc034, 0x80000000L, 0x00000100L), -9007199254740992LL, 0 },
	{ __LINE__, HEXCONSTE(-9007199254740992.5L, 0xc034, 0x80000000L, 0x00000200L), -9007199254740992LL, 0 },
	{ __LINE__, HEXCONSTE(-9007199254740992.75L, 0xc034, 0x80000000L, 0x00000300L), -9007199254740993LL, 0 },
	{ __LINE__, HEXCONSTE(-9007199254740993.5L, 0xc034, 0x80000000L, 0x00000600L), -9007199254740994LL, 0 },

	{ __LINE__, HEXCONSTE(72057594037927935.5L, 0x4036, 0xffffffffL, 0xffffff80L), 72057594037927936LL, 0 },
	{ __LINE__, HEXCONSTE(72057594037927936.25L, 0x4037, 0x80000000L, 0x00000020L), 72057594037927936LL, 0 },
	{ __LINE__, HEXCONSTE(72057594037927936.5L, 0x4037, 0x80000000L, 0x00000040L), 72057594037927936LL, 0 },
	{ __LINE__, HEXCONSTE(72057594037927936.75L, 0x4037, 0x80000000L, 0x00000060L), 72057594037927937LL, 0 },
	{ __LINE__, HEXCONSTE(72057594037927937.5L, 0x4037, 0x80000000L, 0x000000c0L), 72057594037927938LL, 0 },

	{ __LINE__, HEXCONSTE(-72057594037927935.5L, 0xc036, 0xffffffffL, 0xffffff80L), -72057594037927936LL, 0 },
	{ __LINE__, HEXCONSTE(-72057594037927936.25L, 0xc037, 0x80000000L, 0x00000020L), -72057594037927936LL, 0 },
	{ __LINE__, HEXCONSTE(-72057594037927936.5L, 0xc037, 0x80000000L, 0x00000040L), -72057594037927936LL, 0 },
	{ __LINE__, HEXCONSTE(-72057594037927936.75L, 0xc037, 0x80000000L, 0x00000060L), -72057594037927937LL, 0 },
	{ __LINE__, HEXCONSTE(-72057594037927937.5L, 0xc037, 0x80000000L, 0x000000c0L), -72057594037927938LL, 0 },
#endif

#if 0
	{ __LINE__, HEXCONSTE(0.2L, 0x3ffc, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION|ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-0.2L, 0xbffc, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION|ROUND_TONEAREST },

	{ __LINE__, HEXCONSTE(1.4L, 0x3fff, 0xb3333333L, 0x33333333L), 1, INEXACT_EXCEPTION|ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-1.4L, 0xbfff, 0xb3333333L, 0x33333333L), -1, INEXACT_EXCEPTION|ROUND_TONEAREST },

	{ __LINE__, HEXCONSTE(8388600.3L, 0x4015, 0xfffff099L, 0x9999999aL), 8388600LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-8388600.3L, 0xc015, 0xfffff099L, 0x9999999aL), -8388600LL, ROUND_TONEAREST },

#ifndef TEST_FLOAT
	{ __LINE__, HEXCONSTE(1071930.0008, 0x4013, 0x82d9d001L, 0xa36e2eb2L), 1071930LL, ROUND_TONEAREST },
#endif

	/* Test boundary conditions.  */
	/* 0x1FFFFF */
	{ __LINE__, HEXCONSTE(2097151.0, 0x4013, 0xfffff800L, 0x00000000L),2097151LL, ROUND_TONEAREST },
	/* 0x800000 */
	{ __LINE__, HEXCONSTE(8388608.0, 0x4016, 0x80000000L, 0x00000000L), 8388608LL, NO_INEXACT_EXCEPTION|ROUND_TONEAREST },
	/* 0x1000000 */
	{ __LINE__, HEXCONSTE(16777216.0, 0x4017, 0x80000000L, 0x00000000L), 16777216LL, NO_INEXACT_EXCEPTION|ROUND_TONEAREST },
	/* 0x80000000 */
	{ __LINE__, HEXCONSTE(2147483648.0, 0x401e, 0x80000000L, 0x00000000L), 2147483648LL, NO_INEXACT_EXCEPTION|ROUND_TONEAREST },
#ifndef TEST_FLOAT
	{ __LINE__, HEXCONSTE(2147483648.2, 0x401e, 0x80000000L, 0x33333333L), 2147483648LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(2147483648.5, 0x401e, 0x80000000L, 0x80000000L), 2147483648LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(2147483648.7, 0x401e, 0x80000000L, 0xb3333333L), 2147483649LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-2147483648.2, 0xc01e, 0x80000000L, 0x33333333L), -2147483648LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-2147483648.5, 0xc01e, 0x80000000L, 0x80000000L), -2147483648LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-2147483648.7, 0xc01e, 0x80000000L, 0xb3333333L), -2147483649LL, ROUND_TONEAREST },
#endif
	/* 0x20000000000 */
	{ __LINE__, HEXCONSTE(2199023255552.0, 0x4028, 0x80000000L, 0x00000000L), 2199023255552LL, NO_INEXACT_EXCEPTION|ROUND_TONEAREST },
	/* 0x40000000000 */
	{ __LINE__, HEXCONSTE(4398046511104.0, 0x4029, 0x80000000L, 0x00000000L), 4398046511104LL, NO_INEXACT_EXCEPTION|ROUND_TONEAREST },
	/* 0x1000000000000 */
	{ __LINE__, HEXCONSTE(281474976710656.0, 0x402f, 0x80000000L, 0x00000000L), 281474976710656LL, NO_INEXACT_EXCEPTION|ROUND_TONEAREST },
	/* 0x10000000000000 */
	{ __LINE__, HEXCONSTE(4503599627370496.0, 0x4033, 0x80000000L, 0x00000000L), 4503599627370496LL, NO_INEXACT_EXCEPTION|ROUND_TONEAREST },
	/* 0x10000080000000 */
	{ __LINE__, HEXCONSTE(4503601774854144.0, 0x4033, 0x80000400L, 0x00000000L), 4503601774854144LL, ROUND_TONEAREST },
	/* 0x20000000000000 */
	{ __LINE__, HEXCONSTE(9007199254740992.0, 0x4034, 0x80000000L, 0x00000000L), 9007199254740992LL, NO_INEXACT_EXCEPTION|ROUND_TONEAREST },
	/* 0x80000000000000 */
	{ __LINE__, HEXCONSTE(36028797018963968.0, 0x4036, 0x80000000L, 0x00000000L), 36028797018963968LL, NO_INEXACT_EXCEPTION|ROUND_TONEAREST },
	/* 0x100000000000000 */
	{ __LINE__, HEXCONSTE(72057594037927936.0, 0x4037, 0x80000000L, 0x00000000L), 72057594037927936LL, NO_INEXACT_EXCEPTION|ROUND_TONEAREST },
#ifdef TEST_LDOUBLE
	/* The input can only be represented in long double.  */
	{ __LINE__, HEXCONSTE(4503599627370495.5L, 0x4032, 0xffffffffL, 0xfffff800L), 4503599627370496LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(4503599627370496.25L, 0x4033, 0x80000000L, 0x00000200L), 4503599627370496LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(4503599627370496.5L, 0x4033, 0x80000000L, 0x00000400L), 4503599627370496LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(4503599627370496.75L, 0x4033, 0x80000000L, 0x00000600L), 4503599627370497LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(4503599627370497.5L, 0x4033, 0x80000000L, 0x00000c00L), 4503599627370498LL, ROUND_TONEAREST },

	{ __LINE__, HEXCONSTE(-4503599627370495.5L, 0xc032, 0xffffffffL, 0xfffff800L), -4503599627370496LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-4503599627370496.25L, 0xc033, 0x80000000L, 0x00000200L), -4503599627370496LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-4503599627370496.5L, 0xc033, 0x80000000L, 0x00000400L), -4503599627370496LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-4503599627370496.75L, 0xc033, 0x80000000L, 0x00000600L), -4503599627370497LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-4503599627370497.5L, 0xc033, 0x80000000L, 0x00000c00L), -4503599627370498LL, ROUND_TONEAREST },

	{ __LINE__, HEXCONSTE(9007199254740991.5L, 0x4033, 0xffffffffL, 0xfffffc00L), 9007199254740992LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(9007199254740992.25L, 0x4034, 0x80000000L, 0x00000100L), 9007199254740992LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(9007199254740992.5L, 0x4034, 0x80000000L, 0x00000200L), 9007199254740992LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(9007199254740992.75L, 0x4034, 0x80000000L, 0x00000300L), 9007199254740993LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(9007199254740993.5L, 0x4034, 0x80000000L, 0x00000600L), 9007199254740994LL, ROUND_TONEAREST },

	{ __LINE__, HEXCONSTE(-9007199254740991.5L, 0xc033, 0xffffffffL, 0xfffffc00L), -9007199254740992LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-9007199254740992.25L, 0xc034, 0x80000000L, 0x00000100L), -9007199254740992LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-9007199254740992.5L, 0xc034, 0x80000000L, 0x00000200L), -9007199254740992LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-9007199254740992.75L, 0xc034, 0x80000000L, 0x00000300L), -9007199254740993LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-9007199254740993.5L, 0xc034, 0x80000000L, 0x00000600L), -9007199254740994LL, ROUND_TONEAREST },

	{ __LINE__, HEXCONSTE(72057594037927935.5L, 0x4036, 0xffffffffL, 0xffffff80L), 72057594037927936LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(72057594037927936.25L, 0x4037, 0x80000000L, 0x00000020L), 72057594037927936LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(72057594037927936.5L, 0x4037, 0x80000000L, 0x00000040L), 72057594037927936LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(72057594037927936.75L, 0x4037, 0x80000000L, 0x00000060L), 72057594037927937LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(72057594037927937.5L, 0x4037, 0x80000000L, 0x000000c0L), 72057594037927938LL, ROUND_TONEAREST },

	{ __LINE__, HEXCONSTE(-72057594037927935.5L, 0xc036, 0xffffffffL, 0xffffff80L), -72057594037927936LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-72057594037927936.25L, 0xc037, 0x80000000L, 0x00000020L), -72057594037927936LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-72057594037927936.5L, 0xc037, 0x80000000L, 0x00000040L), -72057594037927936LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-72057594037927936.75L, 0xc037, 0x80000000L, 0x00000060L), -72057594037927937LL, ROUND_TONEAREST },
	{ __LINE__, HEXCONSTE(-72057594037927937.5L, 0xc037, 0x80000000L, 0x000000c0L), -72057594037927938LL, ROUND_TONEAREST },
#endif

	{ __LINE__, ZERO_P, 0, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, ZERO_M, 0, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(0.2L, 0x3ffc, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-0.2L, 0xbffc, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(0.5L, 0x3ffe, 0x80000000L, 0x00000000L), 0, INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-0.5L, 0xbffe, 0x80000000L, 0x00000000L), 0, INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(0.8L, 0x3ffe, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-0.8L, 0xbffe, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION|ROUND_TOWARDZERO },

	{ __LINE__, HEXCONSTE(1.4L, 0x3fff, 0xb3333333L, 0x33333333L), 1, INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-1.4L, 0xbfff, 0xb3333333L, 0x33333333L), -1, INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(1.5L, 0x3fff, 0xc0000000L, 0x00000000L), 1, INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-1.5L, 0xbfff, 0xc0000000L, 0x00000000L), -1, INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(1.8L, 0x3fff, 0xe6666666L, 0x66666666L), 1, INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-1.8L, 0xbfff, 0xe6666666L, 0x66666666L), -1, INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(3.0L, 0x4000, 0xc0000000L, 0x00000000L), 3, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-3.0L, 0xc000, 0xc0000000L, 0x00000000L), -3, ROUND_TOWARDZERO },

	{ __LINE__, HEXCONSTE(8388600.3L, 0x4015, 0xfffff099L, 0x9999999aL), 8388600LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-8388600.3L, 0xc015, 0xfffff099L, 0x9999999aL), -8388600LL, ROUND_TOWARDZERO },

#ifndef TEST_FLOAT
	{ __LINE__, HEXCONSTE(1071930.0008, 0x4013, 0x82d9d001L, 0xa36e2eb2L), 1071930LL, ROUND_TOWARDZERO },
#endif

	/* Test boundary conditions.  */
	/* 0x1FFFFF */
	{ __LINE__, HEXCONSTE(2097151.0, 0x4013, 0xfffff800L, 0x00000000L),2097151LL, ROUND_TOWARDZERO },
	/* 0x800000 */
	{ __LINE__, HEXCONSTE(8388608.0, 0x4016, 0x80000000L, 0x00000000L), 8388608LL, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	/* 0x1000000 */
	{ __LINE__, HEXCONSTE(16777216.0, 0x4017, 0x80000000L, 0x00000000L), 16777216LL, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	/* 0x80000000 */
	{ __LINE__, HEXCONSTE(2147483648.0, 0x401e, 0x80000000L, 0x00000000L), 2147483648LL, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
#ifndef TEST_FLOAT
	{ __LINE__, HEXCONSTE(2147483648.2, 0x401e, 0x80000000L, 0x33333333L), 2147483648LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(2147483648.5, 0x401e, 0x80000000L, 0x80000000L), 2147483648LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(2147483648.7, 0x401e, 0x80000000L, 0xb3333333L), 2147483648LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-2147483648.2, 0xc01e, 0x80000000L, 0x33333333L), -2147483648LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-2147483648.5, 0xc01e, 0x80000000L, 0x80000000L), -2147483648LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-2147483648.7, 0xc01e, 0x80000000L, 0xb3333333L), -2147483648LL, ROUND_TOWARDZERO },
#endif
	/* 0x20000000000 */
	{ __LINE__, HEXCONSTE(2199023255552.0, 0x4028, 0x80000000L, 0x00000000L), 2199023255552LL, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	/* 0x40000000000 */
	{ __LINE__, HEXCONSTE(4398046511104.0, 0x4029, 0x80000000L, 0x00000000L), 4398046511104LL, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	/* 0x1000000000000 */
	{ __LINE__, HEXCONSTE(281474976710656.0, 0x402f, 0x80000000L, 0x00000000L), 281474976710656LL, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	/* 0x10000000000000 */
	{ __LINE__, HEXCONSTE(4503599627370496.0, 0x4033, 0x80000000L, 0x00000000L), 4503599627370496LL, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	/* 0x10000080000000 */
	{ __LINE__, HEXCONSTE(4503601774854144.0, 0x4033, 0x80000400L, 0x00000000L), 4503601774854144LL, ROUND_TOWARDZERO },
	/* 0x20000000000000 */
	{ __LINE__, HEXCONSTE(9007199254740992.0, 0x4034, 0x80000000L, 0x00000000L), 9007199254740992LL, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	/* 0x80000000000000 */
	{ __LINE__, HEXCONSTE(36028797018963968.0, 0x4036, 0x80000000L, 0x00000000L), 36028797018963968LL, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
	/* 0x100000000000000 */
	{ __LINE__, HEXCONSTE(72057594037927936.0, 0x4037, 0x80000000L, 0x00000000L), 72057594037927936LL, NO_INEXACT_EXCEPTION|ROUND_TOWARDZERO },
#ifdef TEST_LDOUBLE
	/* The input can only be represented in long double.  */
	{ __LINE__, HEXCONSTE(4503599627370495.5L, 0x4032, 0xffffffffL, 0xfffff800L), 4503599627370495LL, ROUND_TOWARDZEROL },
	{ __LINE__, HEXCONSTE(4503599627370496.25L, 0x4033, 0x80000000L, 0x00000200L), 4503599627370496LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(4503599627370496.5L, 0x4033, 0x80000000L, 0x00000400L), 4503599627370496LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(4503599627370496.75L, 0x4033, 0x80000000L, 0x00000600L), 4503599627370496LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(4503599627370497.5L, 0x4033, 0x80000000L, 0x00000c00L), 4503599627370497LL, ROUND_TOWARDZERO },

	{ __LINE__, HEXCONSTE(-4503599627370495.5L, 0xc032, 0xffffffffL, 0xfffff800L), -4503599627370495LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-4503599627370496.25L, 0xc033, 0x80000000L, 0x00000200L), -4503599627370496LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-4503599627370496.5L, 0xc033, 0x80000000L, 0x00000400L), -4503599627370496LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-4503599627370496.75L, 0xc033, 0x80000000L, 0x00000600L), -4503599627370496LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-4503599627370497.5L, 0xc033, 0x80000000L, 0x00000c00L), -4503599627370497LL, ROUND_TOWARDZERO },

	{ __LINE__, HEXCONSTE(9007199254740991.5L, 0x4033, 0xffffffffL, 0xfffffc00L), 9007199254740991LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(9007199254740992.25L, 0x4034, 0x80000000L, 0x00000100L), 9007199254740992LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(9007199254740992.5L, 0x4034, 0x80000000L, 0x00000200L), 9007199254740992LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(9007199254740992.75L, 0x4034, 0x80000000L, 0x00000300L), 9007199254740992LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(9007199254740993.5L, 0x4034, 0x80000000L, 0x00000600L), 9007199254740993LL, ROUND_TOWARDZERO },

	{ __LINE__, HEXCONSTE(-9007199254740991.5L, 0xc033, 0xffffffffL, 0xfffffc00L), -9007199254740991LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-9007199254740992.25L, 0xc034, 0x80000000L, 0x00000100L), -9007199254740992LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-9007199254740992.5L, 0xc034, 0x80000000L, 0x00000200L), -9007199254740992LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-9007199254740992.75L, 0xc034, 0x80000000L, 0x00000300L), -9007199254740992LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-9007199254740993.5L, 0xc034, 0x80000000L, 0x00000600L), -9007199254740993LL, ROUND_TOWARDZERO },

	{ __LINE__, HEXCONSTE(72057594037927935.5L, 0x4036, 0xffffffffL, 0xffffff80L), 72057594037927935LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(72057594037927936.25L, 0x4037, 0x80000000L, 0x00000020L), 72057594037927936LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(72057594037927936.5L, 0x4037, 0x80000000L, 0x00000040L), 72057594037927936LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(72057594037927936.75L, 0x4037, 0x80000000L, 0x00000060L), 72057594037927936LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(72057594037927937.5L, 0x4037, 0x80000000L, 0x000000c0L), 72057594037927937LL, ROUND_TOWARDZERO },

	{ __LINE__, HEXCONSTE(-72057594037927935.5L, 0xc036, 0xffffffffL, 0xffffff80L), -72057594037927935LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-72057594037927936.25L, 0xc037, 0x80000000L, 0x00000020L), -72057594037927936LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-72057594037927936.5L, 0xc037, 0x80000000L, 0x00000040L), -72057594037927936LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-72057594037927936.75L, 0xc037, 0x80000000L, 0x00000060L), -72057594037927936LL, ROUND_TOWARDZERO },
	{ __LINE__, HEXCONSTE(-72057594037927937.5L, 0xc037, 0x80000000L, 0x000000c0L), -72057594037927937LL, ROUND_TOWARDZERO },
#endif

	{ __LINE__, ZERO_P, 0, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
	{ __LINE__, ZERO_M, 0, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(0.2L, 0x3ffc, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION|ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-0.2L, 0xbffc, 0xccccccccL, 0xcccccccdL), -1, INEXACT_EXCEPTION|ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-0.2L, 0xbffc, 0xccccccccL, 0xcccccccdL), -1, INEXACT_EXCEPTION|ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(0.5L, 0x3ffe, 0x80000000L, 0x00000000L), 0, INEXACT_EXCEPTION|ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-0.5L, 0xbffe, 0x80000000L, 0x00000000L), -1, INEXACT_EXCEPTION|ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(0.8L, 0x3ffe, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION|ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-0.8L, 0xbffe, 0xccccccccL, 0xcccccccdL), -1, INEXACT_EXCEPTION|ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(3.0L, 0x4000, 0xc0000000L, 0x00000000L), 3, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-3.0L, 0xc000, 0xc0000000L, 0x00000000L), -3, ROUND_DOWNWARD },

	{ __LINE__, HEXCONSTE(1.4L, 0x3fff, 0xb3333333L, 0x33333333L), 1, INEXACT_EXCEPTION|ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-1.4L, 0xbfff, 0xb3333333L, 0x33333333L), -2, INEXACT_EXCEPTION|ROUND_DOWNWARD },

	{ __LINE__, HEXCONSTE(8388600.3L, 0x4015, 0xfffff099L, 0x9999999aL), 8388600LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-8388600.3L, 0xc015, 0xfffff099L, 0x9999999aL), -8388601LL, ROUND_DOWNWARD },

#ifndef TEST_FLOAT
	{ __LINE__, HEXCONSTE(1071930.0008, 0x4013, 0x82d9d001L, 0xa36e2eb2L), 1071930LL, ROUND_DOWNWARD },
#endif

	/* Test boundary conditions.  */
	/* 0x1FFFFF */
	{ __LINE__, HEXCONSTE(2097151.0, 0x4013, 0xfffff800L, 0x00000000L),2097151LL, ROUND_DOWNWARD },
	/* 0x800000 */
	{ __LINE__, HEXCONSTE(8388608.0, 0x4016, 0x80000000L, 0x00000000L), 8388608LL, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
	/* 0x1000000 */
	{ __LINE__, HEXCONSTE(16777216.0, 0x4017, 0x80000000L, 0x00000000L), 16777216LL, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
	/* 0x80000000 */
	{ __LINE__, HEXCONSTE(2147483648.0, 0x401e, 0x80000000L, 0x00000000L), 2147483648LL, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
#ifndef TEST_FLOAT
	{ __LINE__, HEXCONSTE(2147483648.2, 0x401e, 0x80000000L, 0x33333333L), 2147483648LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(2147483648.5, 0x401e, 0x80000000L, 0x80000000L), 2147483648LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(2147483648.7, 0x401e, 0x80000000L, 0xb3333333L), 2147483648LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-2147483648.2, 0xc01e, 0x80000000L, 0x33333333L), -2147483649LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-2147483648.5, 0xc01e, 0x80000000L, 0x80000000L), -2147483649LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-2147483648.7, 0xc01e, 0x80000000L, 0xb3333333L), -2147483649LL, ROUND_DOWNWARD },
#endif
	/* 0x20000000000 */
	{ __LINE__, HEXCONSTE(2199023255552.0, 0x4028, 0x80000000L, 0x00000000L), 2199023255552LL, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
	/* 0x40000000000 */
	{ __LINE__, HEXCONSTE(4398046511104.0, 0x4029, 0x80000000L, 0x00000000L), 4398046511104LL, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
	/* 0x1000000000000 */
	{ __LINE__, HEXCONSTE(281474976710656.0, 0x402f, 0x80000000L, 0x00000000L), 281474976710656LL, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
	/* 0x10000000000000 */
	{ __LINE__, HEXCONSTE(4503599627370496.0, 0x4033, 0x80000000L, 0x00000000L), 4503599627370496LL, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
	/* 0x10000080000000 */
	{ __LINE__, HEXCONSTE(4503601774854144.0, 0x4033, 0x80000400L, 0x00000000L), 4503601774854144LL, ROUND_DOWNWARD },
	/* 0x20000000000000 */
	{ __LINE__, HEXCONSTE(9007199254740992.0, 0x4034, 0x80000000L, 0x00000000L), 9007199254740992LL, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
	/* 0x80000000000000 */
	{ __LINE__, HEXCONSTE(36028797018963968.0, 0x4036, 0x80000000L, 0x00000000L), 36028797018963968LL, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
	/* 0x100000000000000 */
	{ __LINE__, HEXCONSTE(72057594037927936.0, 0x4037, 0x80000000L, 0x00000000L), 72057594037927936LL, NO_INEXACT_EXCEPTION|ROUND_DOWNWARD },
#ifdef TEST_LDOUBLE
	/* The input can only be represented in long double.  */
	{ __LINE__, HEXCONSTE(4503599627370495.5L, 0x4032, 0xffffffffL, 0xfffff800L), 4503599627370495LL, ROUND_DOWNWARDL },
	{ __LINE__, HEXCONSTE(4503599627370496.25L, 0x4033, 0x80000000L, 0x00000200L), 4503599627370496LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(4503599627370496.5L, 0x4033, 0x80000000L, 0x00000400L), 4503599627370496LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(4503599627370496.75L, 0x4033, 0x80000000L, 0x00000600L), 4503599627370496LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(4503599627370497.5L, 0x4033, 0x80000000L, 0x00000c00L), 4503599627370497LL, ROUND_DOWNWARD },

	{ __LINE__, HEXCONSTE(4503599627370495.4999999999999L, 0x4032, 0xffffffffL, 0xfffff800L), 4503599627370495LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(4503599627370496.4999999999999L, 0x4033, 0x80000000L, 0x00000400L), 4503599627370496LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(4503599627370497.4999999999999L, 0x4033, 0x80000000L, 0x00000c00L), 4503599627370497LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(4503599627370494.5000000000001L, 0x4032, 0xffffffffL, 0xffffe800L), 4503599627370494LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(4503599627370495.5000000000001L, 0x4032, 0xffffffffL, 0xfffff800L), 4503599627370495LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(4503599627370496.5000000000001L, 0x4033, 0x80000000L, 0x00000400L), 4503599627370496LL, ROUND_DOWNWARD },

	{ __LINE__, HEXCONSTE(-4503599627370495.5L, 0xc032, 0xffffffffL, 0xfffff800L), -4503599627370496LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-4503599627370496.25L, 0xc033, 0x80000000L, 0x00000200L), -4503599627370497LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-4503599627370496.5L, 0xc033, 0x80000000L, 0x00000400L), -4503599627370497LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-4503599627370496.75L, 0xc033, 0x80000000L, 0x00000600L), -4503599627370497LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-4503599627370497.5L, 0xc033, 0x80000000L, 0x00000c00L), -4503599627370498LL, ROUND_DOWNWARD },

	{ __LINE__, HEXCONSTE(-4503599627370495.4999999999999L, 0xc032, 0xffffffffL, 0xfffff800L), -4503599627370496LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-4503599627370496.4999999999999L, 0xc033, 0x80000000L, 0x00000400L), -4503599627370497LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-4503599627370497.4999999999999L, 0xc033, 0x80000000L, 0x00000c00L), -4503599627370498LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-4503599627370494.5000000000001L, 0xc032, 0xffffffffL, 0xffffe800L), -4503599627370495LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-4503599627370495.5000000000001L, 0xc032, 0xffffffffL, 0xfffff800L), -4503599627370496LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-4503599627370496.5000000000001L, 0xc033, 0x80000000L, 0x00000400L), -4503599627370497LL, ROUND_DOWNWARD },

	{ __LINE__, HEXCONSTE(9007199254740991.5L, 0x4033, 0xffffffffL, 0xfffffc00L), 9007199254740991LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(9007199254740992.25L, 0x4034, 0x80000000L, 0x00000100L), 9007199254740992LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(9007199254740992.5L, 0x4034, 0x80000000L, 0x00000200L), 9007199254740992LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(9007199254740992.75L, 0x4034, 0x80000000L, 0x00000300L), 9007199254740992LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(9007199254740993.5L, 0x4034, 0x80000000L, 0x00000600L), 9007199254740993LL, ROUND_DOWNWARD },

	{ __LINE__, HEXCONSTE(9007199254740991.4999999999999L, 0x4033, 0xffffffffL, 0xfffffc00L), 9007199254740991LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(9007199254740992.4999999999999L, 0x4034, 0x80000000L, 0x00000200L), 9007199254740992LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(9007199254740993.4999999999999L, 0x4034, 0x80000000L, 0x00000600L), 9007199254740993LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(9007199254740991.5000000000001L, 0x4033, 0xffffffffL, 0xfffffc00L), 9007199254740991LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(9007199254740992.5000000000001L, 0x4034, 0x80000000L, 0x00000200L), 9007199254740992LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(9007199254740993.5000000000001L, 0x4034, 0x80000000L, 0x00000600L), 9007199254740993LL, ROUND_DOWNWARD },

	{ __LINE__, HEXCONSTE(-9007199254740991.5L, 0xc033, 0xffffffffL, 0xfffffc00L), -9007199254740992LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-9007199254740992.25L, 0xc034, 0x80000000L, 0x00000100L), -9007199254740993LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-9007199254740992.5L, 0xc034, 0x80000000L, 0x00000200L), -9007199254740993LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-9007199254740992.75L, 0xc034, 0x80000000L, 0x00000300L), -9007199254740993LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-9007199254740993.5L, 0xc034, 0x80000000L, 0x00000600L), -9007199254740994LL, ROUND_DOWNWARD },

	{ __LINE__, HEXCONSTE(-9007199254740991.4999999999999L, 0xc033, 0xffffffffL, 0xfffffc00L), -9007199254740992LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-9007199254740992.4999999999999L, 0xc034, 0x80000000L, 0x00000200L), -9007199254740993LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-9007199254740993.4999999999999L, 0xc034, 0x80000000L, 0x00000600L), -9007199254740994LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-9007199254740991.5000000000001L, 0xc033, 0xffffffffL, 0xfffffc00L), -9007199254740992LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-9007199254740992.5000000000001L, 0xc034, 0x80000000L, 0x00000200L), -9007199254740993LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-9007199254740993.5000000000001L, 0xc034, 0x80000000L, 0x00000600L), -9007199254740994LL, ROUND_DOWNWARD },

	{ __LINE__, HEXCONSTE(72057594037927935.5L, 0x4036, 0xffffffffL, 0xffffff80L), 72057594037927935LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(72057594037927936.25L, 0x4037, 0x80000000L, 0x00000020L), 72057594037927936LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(72057594037927936.5L, 0x4037, 0x80000000L, 0x00000040L), 72057594037927936LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(72057594037927936.75L, 0x4037, 0x80000000L, 0x00000060L), 72057594037927936LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(72057594037927937.5L, 0x4037, 0x80000000L, 0x000000c0L), 72057594037927937LL, ROUND_DOWNWARD },

	{ __LINE__, HEXCONSTE(-72057594037927935.5L, 0xc036, 0xffffffffL, 0xffffff80L), -72057594037927936LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-72057594037927936.25L, 0xc037, 0x80000000L, 0x00000020L), -72057594037927937LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-72057594037927936.5L, 0xc037, 0x80000000L, 0x00000040L), -72057594037927937LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-72057594037927936.75L, 0xc037, 0x80000000L, 0x00000060L), -72057594037927937LL, ROUND_DOWNWARD },
	{ __LINE__, HEXCONSTE(-72057594037927937.5L, 0xc037, 0x80000000L, 0x000000c0L), -72057594037927938LL, ROUND_DOWNWARD },
#endif

	{ __LINE__, ZERO_P, 0, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
	{ __LINE__, ZERO_M, 0, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(0.2L, 0x3ffc, 0xccccccccL, 0xcccccccdL), 1, INEXACT_EXCEPTION|ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-0.2L, 0xbffc, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION|ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(0.5L, 0x3ffe, 0x80000000L, 0x00000000L), 1, INEXACT_EXCEPTION|ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-0.5L, 0xbffe, 0x80000000L, 0x00000000L), 0, INEXACT_EXCEPTION|ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(0.8L, 0x3ffe, 0xccccccccL, 0xcccccccdL), 1, INEXACT_EXCEPTION|ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-0.8L, 0xbffe, 0xccccccccL, 0xcccccccdL), 0, INEXACT_EXCEPTION|ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(3.0L, 0x4000, 0xc0000000L, 0x00000000L), 3, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-3.0L, 0xc000, 0xc0000000L, 0x00000000L), -3, ROUND_UPWARD },

	{ __LINE__, HEXCONSTE(1.4L, 0x3fff, 0xb3333333L, 0x33333333L), 2, INEXACT_EXCEPTION|ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-1.4L, 0xbfff, 0xb3333333L, 0x33333333L), -1, INEXACT_EXCEPTION|ROUND_UPWARD },

	{ __LINE__, HEXCONSTE(8388600.3L, 0x4015, 0xfffff099L, 0x9999999aL), 8388601LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-8388600.3L, 0xc015, 0xfffff099L, 0x9999999aL), -8388600LL, ROUND_UPWARD },
#ifndef TEST_FLOAT
	{ __LINE__, HEXCONSTE(1071930.0008, 0x4013, 0x82d9d001L, 0xa36e2eb2L), 1071931LL, ROUND_UPWARD },
#endif
	/* Test boundary conditions.  */
	/* 0x1FFFFF */
	{ __LINE__, HEXCONSTE(2097151.0, 0x4013, 0xfffff800L, 0x00000000L),2097151LL, ROUND_UPWARD },
	/* 0x800000 */
	{ __LINE__, HEXCONSTE(8388608.0, 0x4016, 0x80000000L, 0x00000000L), 8388608LL, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
	/* 0x1000000 */
	{ __LINE__, HEXCONSTE(16777216.0, 0x4017, 0x80000000L, 0x00000000L), 16777216LL, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
	/* 0x80000000 */
	{ __LINE__, HEXCONSTE(2147483648.0, 0x401e, 0x80000000L, 0x00000000L), 2147483648LL, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
#ifndef TEST_FLOAT
	{ __LINE__, HEXCONSTE(2147483648.2, 0x401e, 0x80000000L, 0x33333333L), 2147483649LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(2147483648.5, 0x401e, 0x80000000L, 0x80000000L), 2147483649LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(2147483648.7, 0x401e, 0x80000000L, 0xb3333333L), 2147483649LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-2147483648.2, 0xc01e, 0x80000000L, 0x33333333L), -2147483648LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-2147483648.5, 0xc01e, 0x80000000L, 0x80000000L), -2147483648LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-2147483648.7, 0xc01e, 0x80000000L, 0xb3333333L), -2147483648LL, ROUND_UPWARD },
#endif
	/* 0x20000000000 */
	{ __LINE__, HEXCONSTE(2199023255552.0, 0x4028, 0x80000000L, 0x00000000L), 2199023255552LL, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
	/* 0x40000000000 */
	{ __LINE__, HEXCONSTE(4398046511104.0, 0x4029, 0x80000000L, 0x00000000L), 4398046511104LL, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
	/* 0x1000000000000 */
	{ __LINE__, HEXCONSTE(281474976710656.0, 0x402f, 0x80000000L, 0x00000000L), 281474976710656LL, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
	/* 0x10000000000000 */
	{ __LINE__, HEXCONSTE(4503599627370496.0, 0x4033, 0x80000000L, 0x00000000L), 4503599627370496LL, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
	/* 0x10000080000000 */
	{ __LINE__, HEXCONSTE(4503601774854144.0, 0x4033, 0x80000400L, 0x00000000L), 4503601774854144LL, ROUND_UPWARD },
	/* 0x20000000000000 */
	{ __LINE__, HEXCONSTE(9007199254740992.0, 0x4034, 0x80000000L, 0x00000000L), 9007199254740992LL, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
	/* 0x80000000000000 */
	{ __LINE__, HEXCONSTE(36028797018963968.0, 0x4036, 0x80000000L, 0x00000000L), 36028797018963968LL, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
	/* 0x100000000000000 */
	{ __LINE__, HEXCONSTE(72057594037927936.0, 0x4037, 0x80000000L, 0x00000000L), 72057594037927936LL, NO_INEXACT_EXCEPTION|ROUND_UPWARD },
#ifdef TEST_LDOUBLE
	/* The input can only be represented in long double.  */
	{ __LINE__, HEXCONSTE(4503599627370495.5L, 0x4032, 0xffffffffL, 0xfffff800L), 4503599627370496LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(4503599627370496.25L, 0x4033, 0x80000000L, 0x00000200L), 4503599627370497LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(4503599627370496.5L, 0x4033, 0x80000000L, 0x00000400L), 4503599627370497LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(4503599627370496.75L, 0x4033, 0x80000000L, 0x00000600L), 4503599627370497LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(4503599627370497.5L, 0x4033, 0x80000000L, 0x00000c00L), 4503599627370498LL, ROUND_UPWARD },

	{ __LINE__, HEXCONSTE(4503599627370495.4999999999999L, 0x4032, 0xffffffffL, 0xfffff800L), 4503599627370496LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(4503599627370496.4999999999999L, 0x4033, 0x80000000L, 0x00000400L), 4503599627370497LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(4503599627370497.4999999999999L, 0x4033, 0x80000000L, 0x00000c00L), 4503599627370498LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(4503599627370494.5000000000001L, 0x4032, 0xffffffffL, 0xffffe800L), 4503599627370495LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(4503599627370495.5000000000001L, 0x4032, 0xffffffffL, 0xfffff800L), 4503599627370496LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(4503599627370496.5000000000001L, 0x4033, 0x80000000L, 0x00000400L), 4503599627370497LL, ROUND_UPWARD },

	{ __LINE__, HEXCONSTE(-4503599627370495.5L, 0xc032, 0xffffffffL, 0xfffff800L), -4503599627370495LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-4503599627370496.25L, 0xc033, 0x80000000L, 0x00000200L), -4503599627370496LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-4503599627370496.5L, 0xc033, 0x80000000L, 0x00000400L), -4503599627370496LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-4503599627370496.75L, 0xc033, 0x80000000L, 0x00000600L), -4503599627370496LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-4503599627370497.5L, 0xc033, 0x80000000L, 0x00000c00L), -4503599627370497LL, ROUND_UPWARD },

	{ __LINE__, HEXCONSTE(-4503599627370495.4999999999999L, 0xc032, 0xffffffffL, 0xfffff800L), -4503599627370495LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-4503599627370496.4999999999999L, 0xc033, 0x80000000L, 0x00000400L), -4503599627370496LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-4503599627370497.4999999999999L, 0xc033, 0x80000000L, 0x00000c00L), -4503599627370497LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-4503599627370494.5000000000001L, 0xc032, 0xffffffffL, 0xffffe800L), -4503599627370494LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-4503599627370495.5000000000001L, 0xc032, 0xffffffffL, 0xfffff800L), -4503599627370495LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-4503599627370496.5000000000001L, 0xc033, 0x80000000L, 0x00000400L), -4503599627370496LL, ROUND_UPWARD },

	{ __LINE__, HEXCONSTE(9007199254740991.5L, 0x4033, 0xffffffffL, 0xfffffc00L), 9007199254740992LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(9007199254740992.25L, 0x4034, 0x80000000L, 0x00000100L), 9007199254740993LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(9007199254740992.5L, 0x4034, 0x80000000L, 0x00000200L), 9007199254740993LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(9007199254740992.75L, 0x4034, 0x80000000L, 0x00000300L), 9007199254740993LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(9007199254740993.5L, 0x4034, 0x80000000L, 0x00000600L), 9007199254740994LL, ROUND_UPWARD },

	{ __LINE__, HEXCONSTE(9007199254740991.4999999999999L, 0x4033, 0xffffffffL, 0xfffffc00L), 9007199254740992LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(9007199254740992.4999999999999L, 0x4034, 0x80000000L, 0x00000200L), 9007199254740993LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(9007199254740993.4999999999999L, 0x4034, 0x80000000L, 0x00000600L), 9007199254740994LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(9007199254740991.5000000000001L, 0x4033, 0xffffffffL, 0xfffffc00L), 9007199254740992LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(9007199254740992.5000000000001L, 0x4034, 0x80000000L, 0x00000200L), 9007199254740993LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(9007199254740993.5000000000001L, 0x4034, 0x80000000L, 0x00000600L), 9007199254740994LL, ROUND_UPWARD },

	{ __LINE__, HEXCONSTE(-9007199254740991.5L, 0xc033, 0xffffffffL, 0xfffffc00L), -9007199254740991LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-9007199254740992.25L, 0xc034, 0x80000000L, 0x00000100L), -9007199254740992LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-9007199254740992.5L, 0xc034, 0x80000000L, 0x00000200L), -9007199254740992LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-9007199254740992.75L, 0xc034, 0x80000000L, 0x00000300L), -9007199254740992LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-9007199254740993.5L, 0xc034, 0x80000000L, 0x00000600L), -9007199254740993LL, ROUND_UPWARD },

	{ __LINE__, HEXCONSTE(-9007199254740991.4999999999999L, 0xc033, 0xffffffffL, 0xfffffc00L), -9007199254740991LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-9007199254740992.4999999999999L, 0xc034, 0x80000000L, 0x00000200L), -9007199254740992LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-9007199254740993.4999999999999L, 0xc034, 0x80000000L, 0x00000600L), -9007199254740993LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-9007199254740991.5000000000001L, 0xc033, 0xffffffffL, 0xfffffc00L), -9007199254740991LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-9007199254740992.5000000000001L, 0xc034, 0x80000000L, 0x00000200L), -9007199254740992LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-9007199254740993.5000000000001L, 0xc034, 0x80000000L, 0x00000600L), -9007199254740993LL, ROUND_UPWARD },

	{ __LINE__, HEXCONSTE(72057594037927935.5L, 0x4036, 0xffffffffL, 0xffffff80L), 72057594037927936LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(72057594037927936.25L, 0x4037, 0x80000000L, 0x00000020L), 72057594037927937LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(72057594037927936.5L, 0x4037, 0x80000000L, 0x00000040L), 72057594037927937LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(72057594037927936.75L, 0x4037, 0x80000000L, 0x00000060L), 72057594037927937LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(72057594037927937.5L, 0x4037, 0x80000000L, 0x000000c0L), 72057594037927938LL, ROUND_UPWARD },

	{ __LINE__, HEXCONSTE(-72057594037927935.5L, 0xc036, 0xffffffffL, 0xffffff80L), -72057594037927935LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-72057594037927936.25L, 0xc037, 0x80000000L, 0x00000020L), -72057594037927936LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-72057594037927936.5L, 0xc037, 0x80000000L, 0x00000040L), -72057594037927936LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-72057594037927936.75L, 0xc037, 0x80000000L, 0x00000060L), -72057594037927936LL, ROUND_UPWARD },
	{ __LINE__, HEXCONSTE(-72057594037927937.5L, 0xc037, 0x80000000L, 0x000000c0L), -72057594037927937LL, ROUND_UPWARD },
#endif
#endif
};


int main(int argc, char **argv)
{
	int status;
	
	status = 0;
	
	test_init(argc, argv);
	
	status |= test_table_f_ll(llrint_data, ARRAY_SIZE(llrint_data), __FILE__);
	(void)check_fp;

	return status ? EXIT_FAILURE : EXIT_SUCCESS;
}

#else

int main(void)
{
	return 77;
}

#endif
