

static void test_cabs(void)
{
	struct test *testp = lookup("cabs" },

	TEST_c_f(cabs, 1.000000000000e00, 2.000000000000e00, 2.23606798052788e00, 0 },
	TEST_c_f(cabs, 0.000000000000e00, 2.000000000000e00, 2.00000000000000e00, 0 },
	TEST_c_f(cabs, 1.000000000000e00, 0.000000000000e00, 1.00000000000000e00, 0 },
	TEST_c_f(cabs, -2.000000000000e00, 2.000000000000e00, 2.82842713594437e00, 0 },
	TEST_c_f(cabs, -1.000000000000e00, -2.000000000000e00, 2.23606798052788e00, 0 },
	TEST_c_f(cabs, 1.000000000000e00, -2.000000000000e00, 2.23606798052788e00, 0 },

	/* cabs (x + iy) is specified as hypot (x,y) */

	/* cabs (+inf + i x) == +inf.  */
	TEST_c_f (cabs, INF_P, 1.0, INF_P, 0 },
	/* cabs (-inf + i x) == +inf.  */
	TEST_c_f (cabs, INF_M, 1.0, INF_P, 0 },

	TEST_c_f (cabs, INF_M, QNAN_P, INF_P, 0 },
	TEST_c_f (cabs, INF_M, QNAN_P, INF_P, 0 },

	TEST_c_f (cabs, QNAN_P, QNAN_P, QNAN_P, 0 },

	/* cabs (x,y) == cabs (y,x).  */
	TEST_c_f (cabs, 0.75L, 12.390625L, 12.4133028598606664302388810868156657L, 0 },
	/* cabs (x,y) == cabs (-x,y).  */
	TEST_c_f (cabs, -12.390625L, 0.75L, 12.4133028598606664302388810868156657L, 0 },
	/* cabs (x,y) == cabs (-y,x).  */
	TEST_c_f (cabs, -0.75L, 12.390625L, 12.4133028598606664302388810868156657L, 0 },
	/* cabs (x,y) == cabs (-x,-y).	*/
	TEST_c_f (cabs, -12.390625L, -0.75L, 12.4133028598606664302388810868156657L, 0 },
	/* cabs (x,y) == cabs (-y,-x).	*/
	TEST_c_f (cabs, -0.75L, -12.390625L, 12.4133028598606664302388810868156657L, 0 },
	/* cabs (x,0) == fabs (x).	*/
	TEST_c_f (cabs, -0.75L, 0, 0.75L, 0 },
	TEST_c_f (cabs, 0.75L, 0, 0.75L, 0 },
	TEST_c_f (cabs, -1.0L, 0, 1.0L, 0 },
	TEST_c_f (cabs, 1.0L, 0, 1.0L, 0 },
	TEST_c_f (cabs, -5.7e7L, 0, 5.7e7L, 0 },
	TEST_c_f (cabs, 5.7e7L, 0, 5.7e7L, 0 },

	TEST_c_f (cabs, 0.75L, 1.25L, 1.45773797371132511771853821938639577L, 0 },
}


static void test_carg(void)
{
	struct test *testp = lookup("carg" },

	/* carg (x + iy) is specified as atan2 (y, x) */

	/* carg (x + i 0) == 0 for x > 0.  */
	TEST_c_f (carg, 2.0, 0, 0, 0 },
	/* carg (x - i 0) == -0 for x > 0.	*/
	TEST_c_f (carg, 2.0, ZERO_M, ZERO_M, 0 },

	TEST_c_f (carg, 0, 0, 0, 0 },
	TEST_c_f (carg, 0, ZERO_M, ZERO_M, 0 },

	/* carg (x + i 0) == +pi for x < 0.  */
	TEST_c_f (carg, -2.0, 0, M_PIl, 0 },

	/* carg (x - i 0) == -pi for x < 0.  */
	TEST_c_f (carg, -2.0, ZERO_M, -M_PIl, 0 },

	TEST_c_f (carg, ZERO_M, 0, M_PIl, 0 },
	TEST_c_f (carg, ZERO_M, ZERO_M, -M_PIl, 0 },

	/* carg (+0 + i y) == pi/2 for y > 0.  */
	TEST_c_f (carg, 0, 2.0, M_PI_2l, 0 },

	/* carg (-0 + i y) == pi/2 for y > 0.  */
	TEST_c_f (carg, ZERO_M, 2.0, M_PI_2l, 0 },

	/* carg (+0 + i y) == -pi/2 for y < 0.	*/
	TEST_c_f (carg, 0, -2.0, -M_PI_2l, 0 },

	/* carg (-0 + i y) == -pi/2 for y < 0.	*/
	TEST_c_f (carg, ZERO_M, -2.0, -M_PI_2l, 0 },

	/* carg (inf + i y) == +0 for finite y > 0.  */
	TEST_c_f (carg, INF_P, 2.0, 0, 0 },

	/* carg (inf + i y) == -0 for finite y < 0.  */
	TEST_c_f (carg, INF_P, -2.0, ZERO_M, 0 },

	/* carg(x + i inf) == pi/2 for finite x.  */
	TEST_c_f (carg, 10.0, INF_P, M_PI_2l, 0 },

	/* carg(x - i inf) == -pi/2 for finite x.  */
	TEST_c_f (carg, 10.0, INF_M, -M_PI_2l, 0 },

	/* carg (-inf + i y) == +pi for finite y > 0.  */
	TEST_c_f (carg, INF_M, 10.0, M_PIl, 0 },

	/* carg (-inf + i y) == -pi for finite y < 0.  */
	TEST_c_f (carg, INF_M, -10.0, -M_PIl, 0 },

	TEST_c_f (carg, INF_P, INF_P, M_PI_4l, 0 },

	TEST_c_f (carg, INF_P, INF_M, -M_PI_4l, 0 },

	TEST_c_f (carg, INF_M, INF_P, M_PI_34l, 0 },

	TEST_c_f (carg, INF_M, INF_M, -M_PI_34l, 0 },

	TEST_c_f (carg, QNAN_P, QNAN_P, QNAN_P, 0 },
}


static void test_cimag(void)
{
	struct test *testp = lookup("cimag" },

	TEST_c_f (cimag, 1.0, 0.0, 0.0, NO_INEXACT_EXCEPTION },
	TEST_c_f (cimag, 1.0, ZERO_M, ZERO_M, NO_INEXACT_EXCEPTION },
	TEST_c_f (cimag, 1.0, QNAN_P, QNAN_P, NO_INEXACT_EXCEPTION },
	TEST_c_f (cimag, QNAN_P, QNAN_P, QNAN_P, NO_INEXACT_EXCEPTION },
	TEST_c_f (cimag, 1.0, INF_P, INF_P, NO_INEXACT_EXCEPTION },
	TEST_c_f (cimag, 1.0, INF_M, INF_M, NO_INEXACT_EXCEPTION },
	TEST_c_f (cimag, 2.0, 3.0, 3.0, NO_INEXACT_EXCEPTION },
}


static void test_creal(void)
{
	struct test *testp = lookup("creal" },

	TEST_c_f (creal, 0.0, 1.0, 0.0, NO_INEXACT_EXCEPTION },
	TEST_c_f (creal, ZERO_M, 1.0, ZERO_M, NO_INEXACT_EXCEPTION },
	TEST_c_f (creal, QNAN_P, 1.0, QNAN_P, NO_INEXACT_EXCEPTION },
	TEST_c_f (creal, QNAN_P, QNAN_P, QNAN_P, NO_INEXACT_EXCEPTION },
	TEST_c_f (creal, INF_P, 1.0, INF_P, NO_INEXACT_EXCEPTION },
	TEST_c_f (creal, INF_M, 1.0, INF_M, NO_INEXACT_EXCEPTION },
	TEST_c_f (creal, 2.0, 3.0, 2.0, NO_INEXACT_EXCEPTION },
}


static void test_cadd(void)
{
	struct test *testp = lookup("cadd" },

	TEST_cc_c(cadd, 1.122999995946884155e00, 2.340000003576278686e00, 3.560000002384185791e00, 4.100000023841857910e00, 4.683000028133392334e00, 6.440000057220458984e00, 0 },
	TEST_cc_c(cadd, -1.345669999718666076e00, 2.234566986560821533e00, 3.348675012588500976e00, -4.122219979763031005e00, 2.003005027770996093e00, -1.887652993202209472e00, 0 },
	TEST_cc_c(cadd, 3.857562988996505737e-01, 1.000030040740966796e-01, 3.333939403295516967e00, 4.349979996681213378e00, 3.719695717096328735e00, 4.449983000755310058e00, 0 },
	TEST_cc_c(cadd, 1.575757563114166259e00, 0.000000000000000000e00, 3.378574609756469726e00, 0.000000000000000000e00, 4.954332172870635986e00, 0.000000000000000000e00, 0 },
	TEST_cc_c(cadd, -1.233999997377395629e00, -2.334455549716949462e00, -3.000000000000000000e00, -4.000000000000000000e00, -4.234000027179718017e00, -6.334455549716949462e00, 0 },
	TEST_cc_c(cadd, 1.227000000000000000e04, 1.300000000000000000e03, 0.000000000000000000e00, 3.452000021934509277e-02, 1.227000000000000000e04, 1.300034515380859375e03, 0 },
}


static void test_csub(void)
{
	struct test *testp = lookup("csub" },

	TEST_cc_c(csub, 1.122999995946884155e00, 2.340000003576278686e00, 3.560000002384185791e00, 4.100000023841857910e00, -2.437000006437301635e00, -1.760000020265579223e00, 0 },
	TEST_cc_c(csub, -1.345669999718666076e00, 2.234566986560821533e00, 3.348675012588500976e00, -4.122219979763031005e00, -4.694344997406005859e00, 6.356786966323852539e00, 0 },
	TEST_cc_c(csub, 3.857562988996505737e-01, 1.000030040740966796e-01, 3.333939403295516967e00, 4.349979996681213378e00, -2.948183119297027587e00, -4.249976992607116699e00, 0 },
	TEST_cc_c(csub, 1.575757563114166259e00, 0.000000000000000000e00, 3.378574609756469726e00, 0.000000000000000000e00, -1.802817046642303466e00, 0.000000000000000000e00, 0 },
	TEST_cc_c(csub, -1.233999997377395629e00, -2.334455549716949462e00, -3.000000000000000000e00, -4.000000000000000000e00, 1.766000002622604370e00, 1.665544450283050537e00, 0 },
	TEST_cc_c(csub, 1.227000000000000000e04, 1.300000000000000000e03, 0.000000000000000000e00, 3.452000021934509277e-02, 1.227000000000000000e04, 1.299965484619140625e03, 0 },
}


static void test_cmul(void)
{
	struct test *testp = lookup("cmul" },

	TEST_cc_c(cmul, 1.122999995946884155e00, 2.340000003576278686e00, 3.560000002384185791e00, 4.100000023841857910e00, -5.596120119094848632e00, 1.293470001220703125e01, 0 },
	TEST_cc_c(cmul, -1.345669999718666076e00, 2.234566986560821533e00, 3.348675012588500976e00, -4.122219979763031005e00, 4.705165147781372070e00, 1.302998638153076171e01, 0 },
	TEST_cc_c(cmul, 3.857562988996505737e-01, 1.000030040740966796e-01, 3.333939403295516967e00, 4.349979996681213378e00, 8.510770574212074279e-01, 2.011436134576797485e00, 0 },
	TEST_cc_c(cmul, 1.575757563114166259e00, 0.000000000000000000e00, 3.378574609756469726e00, 0.000000000000000000e00, 5.323814511299133300e00, 0.000000000000000000e00, 0 },
	TEST_cc_c(cmul, -1.233999997377395629e00, -2.334455549716949462e00, -3.000000000000000000e00, -4.000000000000000000e00, -5.635822236537933349e00, 1.193936669826507568e01, 0 },
	TEST_cc_c(cmul, 1.227000000000000000e04, 1.300000000000000000e03, 0.000000000000000000e00, 3.452000021934509277e-02, -4.487600040435791015e01, 4.235604019165039062e02, 0 },
	TEST_cc_c(cmul, 1, 2, 3, 4, -5, 10, 0 },
}


static void test_cdiv(void)
{
	struct test *testp = lookup("cdiv" },

	TEST_cc_c(cdiv, 1.122999995946884155e00, 2.340000003576278686e00, 3.560000002384185791e00, 4.100000023841857910e00, 4.609979800879955291e-01, 1.263787318021059036e-01, 0 },
	TEST_cc_c(cdiv, -1.345669999718666076e00, 2.234566986560821533e00, 3.348675012588500976e00, -4.122219979763031005e00, -4.863302707672119140e-01, 6.862613558769226074e-02, 0 },
	TEST_cc_c(cdiv, 3.857562988996505737e-01, 1.000030040740966796e-01, 3.333939403295516967e00, 4.349979996681213378e00, 5.729839205741882324e-02, -4.476501746103167533e-02, 0 },
	TEST_cc_c(cdiv, 1.575757563114166259e00, 0.000000000000000000e00, 3.378574609756469726e00, 0.000000000000000000e00, 4.663971476256847381e-01, 0.000000000000000000e00, 0 },
	TEST_cc_c(cdiv, -1.233999997377395629e00, -2.334455549716949462e00, -3.000000000000000000e00, -4.000000000000000000e00, 5.215928852558135986e-01, 8.269466646015644073e-02, 0 },
	TEST_cc_c(cdiv, 1.227000000000000000e04, 1.300000000000000000e03, 0.000000000000000000e00, 3.452000021934509277e-02, 3.765932763671875000e04, -3.554461171875000000e05, 0 },
	TEST_cc_c(cdiv, -5, 10, 3, 4, 1, 2, 0 },
}


static void test_cacos(void)
{
	struct test *testp = lookup("cacos" },

	TEST_c_c(cacos, 1.0, 2.0, 1.143717750906944274e00, -1.528570920228958129e00, 0 },
	TEST_c_c(cacos, 0.0, 2.0, 1.570796325802803039e00, -1.443635478615760803e00, 0 },
	TEST_c_c(cacos, 1.0, 0.0, ZERO_P, ZERO_M, 0 },
	TEST_c_c(cacos, -2.0, 2.0, 2.325045466423034668e00, -1.734324529767036438e00, 0 },
	TEST_c_c(cacos, -1.0, -2.0, 1.997874900698661804e00, 1.528570890426635742e00, 0 },
	TEST_c_c(cacos, 1.0, -2.0, 1.143717750906944274e00, 1.528570890426635742e00, 0 },

	TEST_c_c(cacos, 0, 0, M_PI_2l, ZERO_M, 0 },
	TEST_c_c(cacos, ZERO_M, 0, M_PI_2l, ZERO_M, 0 },
	TEST_c_c(cacos, ZERO_M, ZERO_M, M_PI_2l, 0.0, 0 },
	TEST_c_c(cacos, 0, ZERO_M, M_PI_2l, 0.0, 0 },

	TEST_c_c(cacos, INF_M, INF_P, M_PI_34l, INF_M, 0 },
	TEST_c_c(cacos, INF_M, INF_M, M_PI_34l, INF_P, 0 },

	TEST_c_c(cacos, INF_P, INF_P, M_PI_4l, INF_M, 0 },
	TEST_c_c(cacos, INF_P, INF_M, M_PI_4l, INF_P, 0 },

	TEST_c_c(cacos, -10.0, INF_P, M_PI_2l, INF_M, 0 },
	TEST_c_c(cacos, -10.0, INF_M, M_PI_2l, INF_P, 0 },
	TEST_c_c(cacos, 0, INF_P, M_PI_2l, INF_M, 0 },
	TEST_c_c(cacos, 0, INF_M, M_PI_2l, INF_P, 0 },
	TEST_c_c(cacos, 0.1L, INF_P, M_PI_2l, INF_M, 0 },
	TEST_c_c(cacos, 0.1L, INF_M, M_PI_2l, INF_P, 0 },

	TEST_c_c(cacos, INF_M, 0, M_PIl, INF_M, 0 },
	TEST_c_c(cacos, INF_M, ZERO_M, M_PIl, INF_P, 0 },
	TEST_c_c(cacos, INF_M, 100, M_PIl, INF_M, 0 },
	TEST_c_c(cacos, INF_M, -100, M_PIl, INF_P, 0 },

	TEST_c_c(cacos, INF_P, 0, 0.0, INF_M, 0 },
	TEST_c_c(cacos, INF_P, ZERO_M, 0.0, INF_P, 0 },
	TEST_c_c(cacos, INF_P, 0.5, 0.0, INF_M, 0 },
	TEST_c_c(cacos, INF_P, -0.5, 0.0, INF_P, 0 },

	TEST_c_c(cacos, INF_P, QNAN_P, QNAN_P, INF_P, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(cacos, INF_M, QNAN_P, QNAN_P, INF_P, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(cacos, 0, QNAN_P, M_PI_2l, QNAN_P, 0 },
	TEST_c_c(cacos, ZERO_M, QNAN_P, M_PI_2l, QNAN_P, 0 },

	TEST_c_c(cacos, QNAN_P, INF_P, QNAN_P, INF_M, 0 },
	TEST_c_c(cacos, QNAN_P, INF_M, QNAN_P, INF_P, 0 },

	TEST_c_c(cacos, 10.5, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(cacos, -10.5, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(cacos, QNAN_P, 0.75, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(cacos, QNAN_P, -0.75, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(cacos, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(cacos, ZERO_P, -1.5L, M_PI_2l, 1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, ZERO_M, -1.5L, M_PI_2l, 1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, ZERO_P, -1.0L, M_PI_2l, 0.8813735870195430252326093249797923090282L, 0 },
	TEST_c_c(cacos, ZERO_M, -1.0L, M_PI_2l, 0.8813735870195430252326093249797923090282L, 0 },
	TEST_c_c(cacos, ZERO_P, -0.5L, M_PI_2l, 0.4812118250596034474977589134243684231352L, 0 },
	TEST_c_c(cacos, ZERO_M, -0.5L, M_PI_2l, 0.4812118250596034474977589134243684231352L, 0 },
	TEST_c_c(cacos, ZERO_P, 0.5L, M_PI_2l, -0.4812118250596034474977589134243684231352L, 0 },
	TEST_c_c(cacos, ZERO_M, 0.5L, M_PI_2l, -0.4812118250596034474977589134243684231352L, 0 },
	TEST_c_c(cacos, ZERO_P, 1.0L, M_PI_2l, -0.8813735870195430252326093249797923090282L, 0 },
	TEST_c_c(cacos, ZERO_M, 1.0L, M_PI_2l, -0.8813735870195430252326093249797923090282L, 0 },
	TEST_c_c(cacos, ZERO_P, 1.5L, M_PI_2l, -1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, ZERO_M, 1.5L, M_PI_2l, -1.194763217287109304111930828519090523536L, 0 },

	TEST_c_c(cacos, -1.5L, ZERO_P, M_PIl, -0.9624236501192068949955178268487368462704L, 0 },
	TEST_c_c(cacos, -1.5L, ZERO_M, M_PIl, 0.9624236501192068949955178268487368462704L, 0 },
	TEST_c_c(cacos, -1.0L, ZERO_P, M_PIl, ZERO_M, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.0L, ZERO_M, M_PIl, ZERO_P, 0 },
	TEST_c_c(cacos, -0.5L, ZERO_P, 2.094395102393195492308428922186335256131L, ZERO_M, 0|XFAIL_NATIVE_LINUXL|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0.5L, ZERO_M, 2.094395102393195492308428922186335256131L, ZERO_P, 0 },
	TEST_c_c(cacos, 0.5L, ZERO_P, 1.047197551196597746154214461093167628066L, ZERO_M, 0|XFAIL_NATIVE_LINUXL|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 0.5L, ZERO_M, 1.047197551196597746154214461093167628066L, ZERO_P, 0 },
	TEST_c_c(cacos, 1.0L, ZERO_P, ZERO_P, ZERO_M, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 1.0L, ZERO_M, ZERO_P, ZERO_P, 0 },
	TEST_c_c(cacos, 1.5L, ZERO_P, ZERO_P, -0.9624236501192068949955178268487368462704L, 0 },
	TEST_c_c(cacos, 1.5L, ZERO_M, ZERO_P, 0.9624236501192068949955178268487368462704L, 0 },

	TEST_c_c(cacos, 0x1p50L, 1.0L, 8.881784197001252323389053344727730248720e-16L, -3.535050620855721078027883819436720218708e1L, XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p50L, -1.0L, 8.881784197001252323389053344727730248720e-16L, 3.535050620855721078027883819436720218708e1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0x1p50L, 1.0L, 3.141592653589792350284223683154270545292L, -3.535050620855721078027883819436720218708e1L, XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p50L, -1.0L, 3.141592653589792350284223683154270545292L, 3.535050620855721078027883819436720218708e1L, 0 },
	TEST_c_c(cacos, 1.0L, 0x1p50L, 1.570796326794895731052901991514519103193L, -3.535050620855721078027883819436759661753e1L, XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.0L, 0x1p50L, 1.570796326794897507409741391764983781004L, -3.535050620855721078027883819436759661753e1L, XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 1.0L, -0x1p50L, 1.570796326794895731052901991514519103193L, 3.535050620855721078027883819436759661753e1L, 0 },
	TEST_c_c(cacos, -1.0L, -0x1p50L, 1.570796326794897507409741391764983781004L, 3.535050620855721078027883819436759661753e1L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(cacos, 0x1p500L, 1.0L, 3.054936363499604682051979393213617699789e-151L, -3.472667374605326000180332928505464606058e2L, XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p500L, -1.0L, 3.054936363499604682051979393213617699789e-151L, 3.472667374605326000180332928505464606058e2L, XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p500L, 1.0L, 3.141592653589793238462643383279502884197L, -3.472667374605326000180332928505464606058e2L, XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p500L, -1.0L, 3.141592653589793238462643383279502884197L, 3.472667374605326000180332928505464606058e2L, 0 },
	TEST_c_c(cacos, 1.0L, 0x1p500L, 1.570796326794896619231321691639751442099L, -3.472667374605326000180332928505464606058e2L, XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.0L, 0x1p500L, 1.570796326794896619231321691639751442099L, -3.472667374605326000180332928505464606058e2L, XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 1.0L, -0x1p500L, 1.570796326794896619231321691639751442099L, 3.472667374605326000180332928505464606058e2L, 0 },
	TEST_c_c(cacos, -1.0L, -0x1p500L, 1.570796326794896619231321691639751442099L, 3.472667374605326000180332928505464606058e2L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(cacos, 0x1p5000L, 1.0L, 7.079811261048172892385615158694057552948e-1506L, -3.466429049980286492395577839412341016946e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 0x1p5000L, -1.0L, 7.079811261048172892385615158694057552948e-1506L, 3.466429049980286492395577839412341016946e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1p5000L, 1.0L, 3.141592653589793238462643383279502884197L, -3.466429049980286492395577839412341016946e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1p5000L, -1.0L, 3.141592653589793238462643383279502884197L, 3.466429049980286492395577839412341016946e3L, 0 },
	TEST_c_c(cacos, 1.0L, 0x1p5000L, 1.570796326794896619231321691639751442099L, -3.466429049980286492395577839412341016946e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -1.0L, 0x1p5000L, 1.570796326794896619231321691639751442099L, -3.466429049980286492395577839412341016946e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 1.0L, -0x1p5000L, 1.570796326794896619231321691639751442099L, 3.466429049980286492395577839412341016946e3L, 0 },
	TEST_c_c(cacos, -1.0L, -0x1p5000L, 1.570796326794896619231321691639751442099L, 3.466429049980286492395577839412341016946e3L, 0 },
#endif

	TEST_c_c(cacos, 0x1.fp127L, 0x1.fp127L, 7.853981633974483096156608458198757210493e-1L, -8.973081118419833726837456344608533993585e1L, XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(cacos, 0x1.fp1023L, 0x1.fp1023L, 7.853981633974483096156608458198757210493e-1L, -7.107906849659093345062145442726115449315e2L, XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(cacos, 0x1.fp16383L, 0x1.fp16383L, 7.853981633974483096156608458198757210493e-1L, -1.135753137836666928715489992987020363057e4L, 0|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(cacos, 0x1.fp-129L, 1.5L, 1.570796326794896619231321691639751442097L, -1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, 0x1.fp-129L, -1.5L, 1.570796326794896619231321691639751442097L, 1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, 1.5L, 1.570796326794896619231321691639751442100L, -1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, -1.5L, 1.570796326794896619231321691639751442100L, 1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, 1.5L, 0x1.fp-129L, 2.546345110742945032959687790021055102355e-39L, -9.624236501192068949955178268487368462704e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.5L, 0x1.fp-129L, 3.141592653589793238462643383279502884195L, -9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(cacos, 1.5L, -0x1.fp-129L, 2.546345110742945032959687790021055102355e-39L, 9.624236501192068949955178268487368462704e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.5L, -0x1.fp-129L, 3.141592653589793238462643383279502884195L, 9.624236501192068949955178268487368462704e-1L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(cacos, 0x1.fp-1025L, 1.5L, 1.570796326794896619231321691639751442099L, -1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, 0x1.fp-1025L, -1.5L, 1.570796326794896619231321691639751442099L, 1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, 1.5L, 1.570796326794896619231321691639751442099L, -1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, -1.5L, 1.570796326794896619231321691639751442099L, 1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, 1.5L, 0x1.fp-1025L, 4.819934639999230680322935210539402497827e-309L, -9.624236501192068949955178268487368462704e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.5L, 0x1.fp-1025L, 3.141592653589793238462643383279502884197L, -9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(cacos, 1.5L, -0x1.fp-1025L, 4.819934639999230680322935210539402497827e-309L, 9.624236501192068949955178268487368462704e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.5L, -0x1.fp-1025L, 3.141592653589793238462643383279502884197L, 9.624236501192068949955178268487368462704e-1L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacos, 0x1.fp-16385L, 1.5L, 1.570796326794896619231321691639751442099L, -1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, 0x1.fp-16385L, -1.5L, 1.570796326794896619231321691639751442099L, 1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, 1.5L, 1.570796326794896619231321691639751442099L, -1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, -1.5L, 1.570796326794896619231321691639751442099L, 1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(cacos, 1.5L, 0x1.fp-16385L, 7.282957076134209141226696333885150260319e-4933L, -9.624236501192068949955178268487368462704e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -1.5L, 0x1.fp-16385L, 3.141592653589793238462643383279502884197L, -9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(cacos, 1.5L, -0x1.fp-16385L, 7.282957076134209141226696333885150260319e-4933L, 9.624236501192068949955178268487368462704e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -1.5L, -0x1.fp-16385L, 3.141592653589793238462643383279502884197L, 9.624236501192068949955178268487368462704e-1L, 0 },
#endif

	TEST_c_c(cacos, 0.5L, 1.0L, 1.221357263937683325603909865564381489366L, -9.261330313501824245501244453057873152694e-1L, 0 },
	TEST_c_c(cacos, 0.5L, -1.0L, 1.221357263937683325603909865564381489366L, 9.261330313501824245501244453057873152694e-1L, 0 },
	TEST_c_c(cacos, -0.5L, 1.0L, 1.920235389652109912858733517715121394831L, -9.261330313501824245501244453057873152694e-1L, 0 },
	TEST_c_c(cacos, -0.5L, -1.0L, 1.920235389652109912858733517715121394831L, 9.261330313501824245501244453057873152694e-1L, 0 },
	TEST_c_c(cacos, 1.0L, 0.5L, 6.748888455860063801646649673121744318756e-1L, -7.328576759736452608886724437653071523305e-1L, 0 },
	TEST_c_c(cacos, -1.0L, 0.5L, 2.466703808003786858297978415967328452322L, -7.328576759736452608886724437653071523305e-1L, 0 },
	TEST_c_c(cacos, 1.0L, -0.5L, 6.748888455860063801646649673121744318756e-1L, 7.328576759736452608886724437653071523305e-1L, 0 },
	TEST_c_c(cacos, -1.0L, -0.5L, 2.466703808003786858297978415967328452322L, 7.328576759736452608886724437653071523305e-1L, 0 },
	TEST_c_c(cacos, 0.25L, 1.0L, 1.394493894017929688812643125003661339452L, -8.924633639033482359562124741744951972772e-1L, 0 },
	TEST_c_c(cacos, 0.25L, -1.0L, 1.394493894017929688812643125003661339452L, 8.924633639033482359562124741744951972772e-1L, 0 },
	TEST_c_c(cacos, -0.25L, 1.0L, 1.747098759571863549650000258275841544745L, -8.924633639033482359562124741744951972772e-1L, 0 },
	TEST_c_c(cacos, -0.25L, -1.0L, 1.747098759571863549650000258275841544745L, 8.924633639033482359562124741744951972772e-1L, 0 },
	TEST_c_c(cacos, 1.0L, 0.25L, 4.890443302710802929202843732146540079124e-1L, -5.097911466811016354623559941115413499164e-1L, 0 },
	TEST_c_c(cacos, -1.0L, 0.25L, 2.652548323318712945542359010064848876285L, -5.097911466811016354623559941115413499164e-1L, 0 },
	TEST_c_c(cacos, 1.0L, -0.25L, 4.890443302710802929202843732146540079124e-1L, 5.097911466811016354623559941115413499164e-1L, 0 },
	TEST_c_c(cacos, -1.0L, -0.25L, 2.652548323318712945542359010064848876285L, 5.097911466811016354623559941115413499164e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-10L, 1.0L, 1.569458417435338878318763342108699202986L, -8.813742198809567991336704287826445879025e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-10L, -1.0L, 1.569458417435338878318763342108699202986L, 8.813742198809567991336704287826445879025e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-10L, 1.0L, 1.572134236154454360143880041170803681211L, -8.813742198809567991336704287826445879025e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-10L, -1.0L, 1.572134236154454360143880041170803681211L, 8.813742198809567991336704287826445879025e-1L, 0 },
	TEST_c_c(cacos, 1.0L, 0x1.fp-10L, 4.349129763101882771258049954181971959031e-2L, -4.350501469856803800217957402220976497152e-2L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -1.0L, 0x1.fp-10L, 3.098101355958774410750062883737683164607L, -4.350501469856803800217957402220976497152e-2L, 0 },
	TEST_c_c(cacos, 1.0L, -0x1.fp-10L, 4.349129763101882771258049954181971959031e-2L, 4.350501469856803800217957402220976497152e-2L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -1.0L, -0x1.fp-10L, 3.098101355958774410750062883737683164607L, 4.350501469856803800217957402220976497152e-2L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 0x1.fp-30L, 1.0L, 1.570796325518966635014803151387033957091L, -8.813735870195430258081932989769495326854e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-30L, -1.0L, 1.570796325518966635014803151387033957091L, 8.813735870195430258081932989769495326854e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-30L, 1.0L, 1.570796328070826603447840231892468927106L, -8.813735870195430258081932989769495326854e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-30L, -1.0L, 1.570796328070826603447840231892468927106L, 8.813735870195430258081932989769495326854e-1L, 0 },
	TEST_c_c(cacos, 1.0L, 0x1.fp-30L, 4.247867097467650115899790787875186617316e-5L, -4.247867098745151888768727039216644758847e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -1.0L, 0x1.fp-30L, 3.141550174918818561961484385371624132331L, -4.247867098745151888768727039216644758847e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 1.0L, -0x1.fp-30L, 4.247867097467650115899790787875186617316e-5L, 4.247867098745151888768727039216644758847e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -1.0L, -0x1.fp-30L, 3.141550174918818561961484385371624132331L, 4.247867098745151888768727039216644758847e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 0x1.fp-100L, 1.0L, 1.570796326794896619231321691638670687364L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-100L, -1.0L, 1.570796326794896619231321691638670687364L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-100L, 1.0L, 1.570796326794896619231321691640832196834L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-100L, -1.0L, 1.570796326794896619231321691640832196834L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 1.0L, 0x1.fp-100L, 1.236292038260260888664514866456887257525e-15L, -1.236292038260260888664514866457202186027e-15L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.0L, 0x1.fp-100L, 3.141592653589792002170605123018614219682L, -1.236292038260260888664514866457202186027e-15L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 1.0L, -0x1.fp-100L, 1.236292038260260888664514866456887257525e-15L, 1.236292038260260888664514866457202186027e-15L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.0L, -0x1.fp-100L, 3.141592653589792002170605123018614219682L, 1.236292038260260888664514866457202186027e-15L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1.fp-129L, 1.0L, 1.570796326794896619231321691639751442097L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-129L, -1.0L, 1.570796326794896619231321691639751442097L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, 1.0L, 1.570796326794896619231321691639751442101L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, -1.0L, 1.570796326794896619231321691639751442101L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 1.0L, 0x1.fp-129L, 5.335635276982233498398987585285818977930e-20L, -5.335635276982233498398987585285818977933e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.0L, 0x1.fp-129L, 3.141592653589793238409287030509680549213L, -5.335635276982233498398987585285818977933e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 1.0L, -0x1.fp-129L, 5.335635276982233498398987585285818977930e-20L, 5.335635276982233498398987585285818977933e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.0L, -0x1.fp-129L, 3.141592653589793238409287030509680549213L, 5.335635276982233498398987585285818977933e-20L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(cacos, 0x1.fp-1000L, 1.0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-1000L, -1.0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1000L, 1.0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1000L, -1.0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 1.0L, 0x1.fp-1000L, 4.252291453851660175550490409247739011867e-151L, -4.252291453851660175550490409247739011867e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.0L, 0x1.fp-1000L, 3.141592653589793238462643383279502884197L, -4.252291453851660175550490409247739011867e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 1.0L, -0x1.fp-1000L, 4.252291453851660175550490409247739011867e-151L, 4.252291453851660175550490409247739011867e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.0L, -0x1.fp-1000L, 3.141592653589793238462643383279502884197L, 4.252291453851660175550490409247739011867e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1.fp-1025L, 1.0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-1025L, -1.0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, 1.0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, -1.0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 1.0L, 0x1.fp-1025L, 7.340879205566679497036857179189356754017e-155L, -7.340879205566679497036857179189356754017e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.0L, 0x1.fp-1025L, 3.141592653589793238462643383279502884197L, -7.340879205566679497036857179189356754017e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 1.0L, -0x1.fp-1025L, 7.340879205566679497036857179189356754017e-155L, 7.340879205566679497036857179189356754017e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -1.0L, -0x1.fp-1025L, 3.141592653589793238462643383279502884197L, 7.340879205566679497036857179189356754017e-155L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacos, 0x1.fp-10000L, 1.0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-10000L, -1.0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-10000L, 1.0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-10000L, -1.0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 1.0L, 0x1.fp-10000L, 9.854680208706673586644342922051388714633e-1506L, -9.854680208706673586644342922051388714633e-1506L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -1.0L, 0x1.fp-10000L, 3.141592653589793238462643383279502884197L, -9.854680208706673586644342922051388714633e-1506L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 1.0L, -0x1.fp-10000L, 9.854680208706673586644342922051388714633e-1506L, 9.854680208706673586644342922051388714633e-1506L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -1.0L, -0x1.fp-10000L, 3.141592653589793238462643383279502884197L, 9.854680208706673586644342922051388714633e-1506L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 0x1.fp-16385L, 1.0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-16385L, -1.0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, 1.0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, -1.0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(cacos, 1.0L, 0x1.fp-16385L, 9.023632056840860275214893047597614177639e-2467L, -9.023632056840860275214893047597614177639e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -1.0L, 0x1.fp-16385L, 3.141592653589793238462643383279502884197L, -9.023632056840860275214893047597614177639e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 1.0L, -0x1.fp-16385L, 9.023632056840860275214893047597614177639e-2467L, 9.023632056840860275214893047597614177639e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -1.0L, -0x1.fp-16385L, 3.141592653589793238462643383279502884197L, 9.023632056840860275214893047597614177639e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(cacos, 0x1p-23L, 0x1.000002p0L, 1.570796242501204621739026081991856762292L, -8.813736713132400470205730751186547909968e-1L, 0 },
	TEST_c_c(cacos, 0x1p-23L, -0x1.000002p0L, 1.570796242501204621739026081991856762292L, 8.813736713132400470205730751186547909968e-1L, 0 },
	TEST_c_c(cacos, -0x1p-23L, 0x1.000002p0L, 1.570796411088588616723617301287646121905L, -8.813736713132400470205730751186547909968e-1L, 0 },
	TEST_c_c(cacos, -0x1p-23L, -0x1.000002p0L, 1.570796411088588616723617301287646121905L, 8.813736713132400470205730751186547909968e-1L, 0 },
	TEST_c_c(cacos, 0x1.000002p0L, 0x1p-23L, 2.222118384408546368406374049167636760903e-4L, -5.364668491573609633134147164031476452679e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0x1.000002p0L, 0x1p-23L, 3.141370441751352383825802745874586120521L, -5.364668491573609633134147164031476452679e-4L, 0 },
	TEST_c_c(cacos, 0x1.000002p0L, -0x1p-23L, 2.222118384408546368406374049167636760903e-4L, 5.364668491573609633134147164031476452679e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0x1.000002p0L, -0x1p-23L, 3.141370441751352383825802745874586120521L, 5.364668491573609633134147164031476452679e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 0x1.fp-129L, 0x1.000002p0L, 1.570796326794896619231321691639751442097L, -8.813736713132375348727889167749389235161e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-129L, -0x1.000002p0L, 1.570796326794896619231321691639751442097L, 8.813736713132375348727889167749389235161e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, 0x1.000002p0L, 1.570796326794896619231321691639751442101L, -8.813736713132375348727889167749389235161e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, -0x1.000002p0L, 1.570796326794896619231321691639751442101L, 8.813736713132375348727889167749389235161e-1L, 0 },
	TEST_c_c(cacos, 0x1.000002p0L, 0x1.fp-129L, 5.830451806317544230969669308596361881467e-36L, -4.882812451493617206486388134172712975070e-4L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1.000002p0L, 0x1.fp-129L, 3.141592653589793238462643383279502878367L, -4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(cacos, 0x1.000002p0L, -0x1.fp-129L, 5.830451806317544230969669308596361881467e-36L, 4.882812451493617206486388134172712975070e-4L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1.000002p0L, -0x1.fp-129L, 3.141592653589793238462643383279502878367L, 4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(cacos, 0.0L, 0x1.000002p0L, 1.570796326794896619231321691639751442099L, -8.813736713132375348727889167749389235161e-1L, 0 },
	TEST_c_c(cacos, 0.0L, -0x1.000002p0L, 1.570796326794896619231321691639751442099L, 8.813736713132375348727889167749389235161e-1L, 0 },
	TEST_c_c(cacos, -0.0L, 0x1.000002p0L, 1.570796326794896619231321691639751442099L, -8.813736713132375348727889167749389235161e-1L, 0 },
	TEST_c_c(cacos, -0.0L, -0x1.000002p0L, 1.570796326794896619231321691639751442099L, 8.813736713132375348727889167749389235161e-1L, 0 },
	TEST_c_c(cacos, 0x1.000002p0L, 0.0L, 0.0L, -4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(cacos, -0x1.000002p0L, 0.0L, 3.141592653589793238462643383279502884197L, -4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(cacos, 0x1.000002p0L, -0.0L, 0.0L, 4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(cacos, -0x1.000002p0L, -0.0L, 3.141592653589793238462643383279502884197L, 4.882812451493617206486388134172712975070e-4L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(cacos, 0x1p-52L, 0x1.0000000000001p0L, 1.570796326794896462222075823262262934288L, -8.813735870195431822418551933572982483664e-1L, 0 },
	TEST_c_c(cacos, 0x1p-52L, -0x1.0000000000001p0L, 1.570796326794896462222075823262262934288L, 8.813735870195431822418551933572982483664e-1L, 0 },
	TEST_c_c(cacos, -0x1p-52L, 0x1.0000000000001p0L, 1.570796326794896776240567560017239949909L, -8.813735870195431822418551933572982483664e-1L, 0 },
	TEST_c_c(cacos, -0x1p-52L, -0x1.0000000000001p0L, 1.570796326794896776240567560017239949909L, 8.813735870195431822418551933572982483664e-1L, 0 },
	TEST_c_c(cacos, 0x1.0000000000001p0L, 0x1p-52L, 9.590301705980041385828904092662391018164e-9L, -2.315303644582684770975188768022139415020e-8L, 0 },
	TEST_c_c(cacos, -0x1.0000000000001p0L, 0x1p-52L, 3.141592643999491532482601997450598791535L, -2.315303644582684770975188768022139415020e-8L, 0 },
	TEST_c_c(cacos, 0x1.0000000000001p0L, -0x1p-52L, 9.590301705980041385828904092662391018164e-9L, 2.315303644582684770975188768022139415020e-8L, 0 },
	TEST_c_c(cacos, -0x1.0000000000001p0L, -0x1p-52L, 3.141592643999491532482601997450598791535L, 2.315303644582684770975188768022139415020e-8L, 0 },
	TEST_c_c(cacos, 0x1.fp-1025L, 0x1.0000000000001p0L, 1.570796326794896619231321691639751442099L, -8.813735870195431822418551933572895326024e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-1025L, -0x1.0000000000001p0L, 1.570796326794896619231321691639751442099L, 8.813735870195431822418551933572895326024e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, 0x1.0000000000001p0L, 1.570796326794896619231321691639751442099L, -8.813735870195431822418551933572895326024e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, -0x1.0000000000001p0L, 1.570796326794896619231321691639751442099L, 8.813735870195431822418551933572895326024e-1L, 0 },
	TEST_c_c(cacos, 0x1.0000000000001p0L, 0x1.fp-1025L, 2.557178503953494342609835913586108008322e-301L, -2.107342425544701550354780375182800088393e-8L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1.0000000000001p0L, 0x1.fp-1025L, 3.141592653589793238462643383279502884197L, -2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(cacos, 0x1.0000000000001p0L, -0x1.fp-1025L, 2.557178503953494342609835913586108008322e-301L, 2.107342425544701550354780375182800088393e-8L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1.0000000000001p0L, -0x1.fp-1025L, 3.141592653589793238462643383279502884197L, 2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(cacos, 0.0L, 0x1.0000000000001p0L, 1.570796326794896619231321691639751442099L, -8.813735870195431822418551933572895326024e-1L, 0 },
	TEST_c_c(cacos, 0.0L, -0x1.0000000000001p0L, 1.570796326794896619231321691639751442099L, 8.813735870195431822418551933572895326024e-1L, 0 },
	TEST_c_c(cacos, -0.0L, 0x1.0000000000001p0L, 1.570796326794896619231321691639751442099L, -8.813735870195431822418551933572895326024e-1L, 0 },
	TEST_c_c(cacos, -0.0L, -0x1.0000000000001p0L, 1.570796326794896619231321691639751442099L, 8.813735870195431822418551933572895326024e-1L, 0 },
	TEST_c_c(cacos, 0x1.0000000000001p0L, 0.0L, 0.0L, -2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(cacos, -0x1.0000000000001p0L, 0.0L, 3.141592653589793238462643383279502884197L, -2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(cacos, 0x1.0000000000001p0L, -0.0L, 0.0L, 2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(cacos, -0x1.0000000000001p0L, -0.0L, 3.141592653589793238462643383279502884197L, 2.107342425544701550354780375182800088393e-8L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(cacos, 0x1p-63L, 0x1.0000000000000002p0L, 1.570796326794896619154657020805582738031L, -8.813735870195430253092739958139610131001e-1L, 0 },
	TEST_c_c(cacos, 0x1p-63L, -0x1.0000000000000002p0L, 1.570796326794896619154657020805582738031L, 8.813735870195430253092739958139610131001e-1L, 0 },
	TEST_c_c(cacos, -0x1p-63L, 0x1.0000000000000002p0L, 1.570796326794896619307986362473920146166L, -8.813735870195430253092739958139610131001e-1L, 0 },
	TEST_c_c(cacos, -0x1p-63L, -0x1.0000000000000002p0L, 1.570796326794896619307986362473920146166L, 8.813735870195430253092739958139610131001e-1L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000002p0L, 0x1p-63L, 2.119177303101063432592523199680782317447e-10L, -5.116146586219826555037807251857670783420e-10L, 0 },
	TEST_c_c(cacos, -0x1.0000000000000002p0L, 0x1p-63L, 3.141592653377875508152537040020250564229L, -5.116146586219826555037807251857670783420e-10L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000002p0L, -0x1p-63L, 2.119177303101063432592523199680782317447e-10L, 5.116146586219826555037807251857670783420e-10L, 0 },
	TEST_c_c(cacos, -0x1.0000000000000002p0L, -0x1p-63L, 3.141592653377875508152537040020250564229L, 5.116146586219826555037807251857670783420e-10L, 0 },
# if LDBL_MIN_EXP <= -16381
	TEST_c_c(cacos, 0x1.fp-16385L, 0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430253092739958139610130980e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-16385L, -0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430253092739958139610130980e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, 0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430253092739958139610130980e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, -0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430253092739958139610130980e-1L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000002p0L, 0x1.fp-16385L, 1.748608650034385653922359120438227480943e-4923L, -4.656612873077392578082927418388212703712e-10L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1.0000000000000002p0L, 0x1.fp-16385L, 3.141592653589793238462643383279502884197L, -4.656612873077392578082927418388212703712e-10L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000002p0L, -0x1.fp-16385L, 1.748608650034385653922359120438227480943e-4923L, 4.656612873077392578082927418388212703712e-10L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1.0000000000000002p0L, -0x1.fp-16385L, 3.141592653589793238462643383279502884197L, 4.656612873077392578082927418388212703712e-10L, 0 },
# endif
	TEST_c_c(cacos, 0.0L, 0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430253092739958139610130980e-1L, 0 },
	TEST_c_c(cacos, 0.0L, -0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430253092739958139610130980e-1L, 0 },
	TEST_c_c(cacos, -0.0L, 0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430253092739958139610130980e-1L, 0 },
	TEST_c_c(cacos, -0.0L, -0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430253092739958139610130980e-1L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000002p0L, 0.0L, 0.0L, -4.656612873077392578082927418388212703712e-10L, 0 },
	TEST_c_c(cacos, -0x1.0000000000000002p0L, 0.0L, 3.141592653589793238462643383279502884197L, -4.656612873077392578082927418388212703712e-10L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000002p0L, -0.0L, 0.0L, 4.656612873077392578082927418388212703712e-10L, 0 },
	TEST_c_c(cacos, -0x1.0000000000000002p0L, -0.0L, 3.141592653589793238462643383279502884197L, 4.656612873077392578082927418388212703712e-10L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(cacos, 0x1p-106L, 0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639742726335L, -8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, 0x1p-106L, -0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639742726335L, 8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, -0x1p-106L, 0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639760157863L, -8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, -0x1p-106L, -0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639760157863L, 8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, 0x1.000000000000000000000000008p0L, 0x1p-106L, 5.394221422390606848017034778914096659726e-17L, -2.285028863093221674154232933662774454211e-16L, 0 },
	TEST_c_c(cacos, -0x1.000000000000000000000000008p0L, 0x1p-106L, 3.141592653589793184520429159373434404027L, -2.285028863093221674154232933662774454211e-16L, 0 },
	TEST_c_c(cacos, 0x1.000000000000000000000000008p0L, -0x1p-106L, 5.394221422390606848017034778914096659726e-17L, 2.285028863093221674154232933662774454211e-16L, 0 },
	TEST_c_c(cacos, -0x1.000000000000000000000000008p0L, -0x1p-106L, 3.141592653589793184520429159373434404027L, 2.285028863093221674154232933662774454211e-16L, 0 },
	TEST_c_c(cacos, 0x1.fp-1025L, 0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-1025L, -0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, 0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, -0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, 0x1.000000000000000000000000008p0L, 0x1.fp-1025L, 2.426922623448365473354662093431821897807e-293L, -2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(cacos, -0x1.000000000000000000000000008p0L, 0x1.fp-1025L, 3.141592653589793238462643383279502884197L, -2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(cacos, 0x1.000000000000000000000000008p0L, -0x1.fp-1025L, 2.426922623448365473354662093431821897807e-293L, 2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(cacos, -0x1.000000000000000000000000008p0L, -0x1.fp-1025L, 3.141592653589793238462643383279502884197L, 2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(cacos, 0.0L, 0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, 0.0L, -0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, -0.0L, 0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, -0.0L, -0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(cacos, 0x1.000000000000000000000000008p0L, 0.0L, 0.0L, -2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(cacos, -0x1.000000000000000000000000008p0L, 0.0L, 3.141592653589793238462643383279502884197L, -2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(cacos, 0x1.000000000000000000000000008p0L, -0.0L, 0.0L, 2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(cacos, -0x1.000000000000000000000000008p0L, -0.0L, 3.141592653589793238462643383279502884197L, 2.220446049250313080847263336181636063482e-16L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(cacos, 0x1p-113L, 0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751374007L, -8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, 0x1p-113L, -0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751374007L, 8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, -0x1p-113L, 0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751510190L, -8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, -0x1p-113L, -0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751510190L, 8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000000000000000001p0L, 0x1p-113L, 4.767863183742677481693563511435642755521e-18L, -2.019699255375255198156433710951064632386e-17L, 0 },
	TEST_c_c(cacos, -0x1.0000000000000000000000000001p0L, 0x1p-113L, 3.141592653589793233694780199536825402504L, -2.019699255375255198156433710951064632386e-17L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000000000000000001p0L, -0x1p-113L, 4.767863183742677481693563511435642755521e-18L, 2.019699255375255198156433710951064632386e-17L, 0 },
	TEST_c_c(cacos, -0x1.0000000000000000000000000001p0L, -0x1p-113L, 3.141592653589793233694780199536825402504L, 2.019699255375255198156433710951064632386e-17L, 0 },
	TEST_c_c(cacos, 0x1.fp-16385L, 0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-16385L, -0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, 0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, -0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000000000000000001p0L, 0x1.fp-16385L, 4.148847925325683229178506116378864361396e-4916L, -1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(cacos, -0x1.0000000000000000000000000001p0L, 0x1.fp-16385L, 3.141592653589793238462643383279502884197L, -1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000000000000000001p0L, -0x1.fp-16385L, 4.148847925325683229178506116378864361396e-4916L, 1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(cacos, -0x1.0000000000000000000000000001p0L, -0x1.fp-16385L, 3.141592653589793238462643383279502884197L, 1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(cacos, 0.0L, 0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, 0.0L, -0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, -0.0L, 0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, -0.0L, -0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000000000000000001p0L, 0.0L, 0.0L, -1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(cacos, -0x1.0000000000000000000000000001p0L, 0.0L, 3.141592653589793238462643383279502884197L, -1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(cacos, 0x1.0000000000000000000000000001p0L, -0.0L, 0.0L, 1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(cacos, -0x1.0000000000000000000000000001p0L, -0.0L, 3.141592653589793238462643383279502884197L, 1.962615573354718824241727964954454332780e-17L, 0 },
#endif

	TEST_c_c(cacos, 0x1p-23L, 0x0.ffffffp0L, 1.570796242501197085295336701632142060969L, -8.813735448726963984495965873956465777250e-1L, 0 },
	TEST_c_c(cacos, 0x1p-23L, -0x0.ffffffp0L, 1.570796242501197085295336701632142060969L, 8.813735448726963984495965873956465777250e-1L, 0 },
	TEST_c_c(cacos, -0x1p-23L, 0x0.ffffffp0L, 1.570796411088596153167306681647360823228L, -8.813735448726963984495965873956465777250e-1L, 0 },
	TEST_c_c(cacos, -0x1p-23L, -0x0.ffffffp0L, 1.570796411088596153167306681647360823228L, 8.813735448726963984495965873956465777250e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffp0L, 0x1p-23L, 4.391863861910487109445187743978204002614e-4L, -2.714321200917194650737217746780928423385e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0x0.ffffffp0L, 0x1p-23L, 3.141153467203602189751698864505105063797L, -2.714321200917194650737217746780928423385e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 0x0.ffffffp0L, -0x1p-23L, 4.391863861910487109445187743978204002614e-4L, 2.714321200917194650737217746780928423385e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0x0.ffffffp0L, -0x1p-23L, 3.141153467203602189751698864505105063797L, 2.714321200917194650737217746780928423385e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 0x1.fp-129L, 0x0.ffffffp0L, 1.570796326794896619231321691639751442097L, -8.813735448726938863015878255140556727969e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-129L, -0x0.ffffffp0L, 1.570796326794896619231321691639751442097L, 8.813735448726938863015878255140556727969e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, 0x0.ffffffp0L, 1.570796326794896619231321691639751442101L, -8.813735448726938863015878255140556727969e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, -0x0.ffffffp0L, 1.570796326794896619231321691639751442101L, 8.813735448726938863015878255140556727969e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffp0L, 0x1.fp-129L, 3.452669847162035876032494826387364972849e-4L, -8.245504387859737323891843603996428534945e-36L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x0.ffffffp0L, 0x1.fp-129L, 3.141247386605077034875040133796864147700L, -8.245504387859737323891843603996428534945e-36L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x0.ffffffp0L, -0x1.fp-129L, 3.452669847162035876032494826387364972849e-4L, 8.245504387859737323891843603996428534945e-36L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x0.ffffffp0L, -0x1.fp-129L, 3.141247386605077034875040133796864147700L, 8.245504387859737323891843603996428534945e-36L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.0L, 0x0.ffffffp0L, 1.570796326794896619231321691639751442099L, -8.813735448726938863015878255140556727969e-1L, 0 },
	TEST_c_c(cacos, 0.0L, -0x0.ffffffp0L, 1.570796326794896619231321691639751442099L, 8.813735448726938863015878255140556727969e-1L, 0 },
	TEST_c_c(cacos, -0.0L, 0x0.ffffffp0L, 1.570796326794896619231321691639751442099L, -8.813735448726938863015878255140556727969e-1L, 0 },
	TEST_c_c(cacos, -0.0L, -0x0.ffffffp0L, 1.570796326794896619231321691639751442099L, 8.813735448726938863015878255140556727969e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffp0L, 0.0L, 3.452669847162035876032494826387364972849e-4L, -0.0L, 0|XFAIL_NATIVE_LINUXF|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x0.ffffffp0L, 0.0L, 3.141247386605077034875040133796864147700L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x0.ffffffp0L, -0.0L, 3.452669847162035876032494826387364972849e-4L, 0.0L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0x0.ffffffp0L, -0.0L, 3.141247386605077034875040133796864147700L, 0.0L, 0 },
	TEST_c_c(cacos, 0x1p-23L, 0.5L, 1.570796220170866625230343643673321150378L, -4.812118250596059896127318386463676808798e-1L, 0 },
	TEST_c_c(cacos, 0x1p-23L, -0.5L, 1.570796220170866625230343643673321150378L, 4.812118250596059896127318386463676808798e-1L, 0 },
	TEST_c_c(cacos, -0x1p-23L, 0.5L, 1.570796433418926613232299739606181733819L, -4.812118250596059896127318386463676808798e-1L, 0 },
	TEST_c_c(cacos, -0x1p-23L, -0.5L, 1.570796433418926613232299739606181733819L, 4.812118250596059896127318386463676808798e-1L, 0 },
	TEST_c_c(cacos, 0.5L, 0x1p-23L, 1.047197551196603215914744618665204532273L, -1.376510308240943236356088341381173571841e-7L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0.5L, 0x1p-23L, 2.094395102393190022547898764614298351924L, -1.376510308240943236356088341381173571841e-7L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 0.5L, -0x1p-23L, 1.047197551196603215914744618665204532273L, 1.376510308240943236356088341381173571841e-7L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0.5L, -0x1p-23L, 2.094395102393190022547898764614298351924L, 1.376510308240943236356088341381173571841e-7L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 0x1.fp-129L, 0.5L, 1.570796326794896619231321691639751442096L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-129L, -0.5L, 1.570796326794896619231321691639751442096L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, 0.5L, 1.570796326794896619231321691639751442101L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, -0.5L, 1.570796326794896619231321691639751442101L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, 0.5L, 0x1.fp-129L, 1.047197551196597746154214461093167628066L, -3.287317402534702257036015056278368274737e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, 0x1.fp-129L, 2.094395102393195492308428922186335256131L, -3.287317402534702257036015056278368274737e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.5L, -0x1.fp-129L, 1.047197551196597746154214461093167628066L, 3.287317402534702257036015056278368274737e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, -0x1.fp-129L, 2.094395102393195492308428922186335256131L, 3.287317402534702257036015056278368274737e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-23L, 0x1p-23L, 1.570796207585607068450636380271254316214L, -1.192092895507818146886315028596704749235e-7L, 0 },
	TEST_c_c(cacos, 0x1p-23L, -0x1p-23L, 1.570796207585607068450636380271254316214L, 1.192092895507818146886315028596704749235e-7L, 0 },
	TEST_c_c(cacos, -0x1p-23L, 0x1p-23L, 1.570796446004186170012007003008248567984L, -1.192092895507818146886315028596704749235e-7L, 0 },
	TEST_c_c(cacos, -0x1p-23L, -0x1p-23L, 1.570796446004186170012007003008248567984L, 1.192092895507818146886315028596704749235e-7L, 0 },
	TEST_c_c(cacos, 0x1.fp-129L, 0x1p-23L, 1.570796326794896619231321691639751442096L, -1.192092895507809676556842485683592032154e-7L, 0 },
	TEST_c_c(cacos, 0x1.fp-129L, -0x1p-23L, 1.570796326794896619231321691639751442096L, 1.192092895507809676556842485683592032154e-7L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, 0x1p-23L, 1.570796326794896619231321691639751442101L, -1.192092895507809676556842485683592032154e-7L, 0 },
	TEST_c_c(cacos, -0x1.fp-129L, -0x1p-23L, 1.570796326794896619231321691639751442101L, 1.192092895507809676556842485683592032154e-7L, 0 },
	TEST_c_c(cacos, 0x1p-23L, 0x1.fp-129L, 1.570796207585607068449789347324000006847L, -2.846900380897747786805634596726756660388e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-23L, 0x1.fp-129L, 1.570796446004186170012854035955502877351L, -2.846900380897747786805634596726756660388e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-23L, -0x1.fp-129L, 1.570796207585607068449789347324000006847L, 2.846900380897747786805634596726756660388e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-23L, -0x1.fp-129L, 1.570796446004186170012854035955502877351L, 2.846900380897747786805634596726756660388e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.0L, 0x1p-23L, 1.570796326794896619231321691639751442099L, -1.192092895507809676556842485683592032154e-7L, 0 },
	TEST_c_c(cacos, 0.0L, -0x1p-23L, 1.570796326794896619231321691639751442099L, 1.192092895507809676556842485683592032154e-7L, 0 },
	TEST_c_c(cacos, -0.0L, 0x1p-23L, 1.570796326794896619231321691639751442099L, -1.192092895507809676556842485683592032154e-7L, 0 },
	TEST_c_c(cacos, -0.0L, -0x1p-23L, 1.570796326794896619231321691639751442099L, 1.192092895507809676556842485683592032154e-7L, 0 },
	TEST_c_c(cacos, 0x1p-23L, 0.0L, 1.570796207585607068449789347324000006847L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-23L, 0.0L, 1.570796446004186170012854035955502877351L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-23L, -0.0L, 1.570796207585607068449789347324000006847L, 0.0L, 0 },
	TEST_c_c(cacos, -0x1p-23L, -0.0L, 1.570796446004186170012854035955502877351L, 0.0L, 0 },
	TEST_c_c(cacos, 0x1.fp-129L, 0x1.fp-129L, 1.570796326794896619231321691639751442096L, -2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1.fp-129L, -0x1.fp-129L, 1.570796326794896619231321691639751442096L, 2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1.fp-129L, 0x1.fp-129L, 1.570796326794896619231321691639751442101L, -2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1.fp-129L, -0x1.fp-129L, 1.570796326794896619231321691639751442101L, 2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(cacos, 0x1p-52L, 0x0.fffffffffffff8p0L, 1.570796326794896462222075823262236786996L, -8.813735870195429467279863907910458761820e-1L, 0 },
	TEST_c_c(cacos, 0x1p-52L, -0x0.fffffffffffff8p0L, 1.570796326794896462222075823262236786996L, 8.813735870195429467279863907910458761820e-1L, 0 },
	TEST_c_c(cacos, -0x1p-52L, 0x0.fffffffffffff8p0L, 1.570796326794896776240567560017266097201L, -8.813735870195429467279863907910458761820e-1L, 0 },
	TEST_c_c(cacos, -0x1p-52L, -0x0.fffffffffffff8p0L, 1.570796326794896776240567560017266097201L, 8.813735870195429467279863907910458761820e-1L, 0 },
	TEST_c_c(cacos, 0x0.fffffffffffff8p0L, 0x1p-52L, 1.895456983915074112227925127005564372844e-8L, -1.171456840272878582596796205397918831268e-8L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x0.fffffffffffff8p0L, 0x1p-52L, 3.141592634635223399311902261000251614142L, -1.171456840272878582596796205397918831268e-8L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x0.fffffffffffff8p0L, -0x1p-52L, 1.895456983915074112227925127005564372844e-8L, 1.171456840272878582596796205397918831268e-8L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x0.fffffffffffff8p0L, -0x1p-52L, 3.141592634635223399311902261000251614142L, 1.171456840272878582596796205397918831268e-8L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1.fp-1025L, 0x0.fffffffffffff8p0L, 1.570796326794896619231321691639751442099L, -8.813735870195429467279863907910371604180e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-1025L, -0x0.fffffffffffff8p0L, 1.570796326794896619231321691639751442099L, 8.813735870195429467279863907910371604180e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, 0x0.fffffffffffff8p0L, 1.570796326794896619231321691639751442099L, -8.813735870195429467279863907910371604180e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, -0x0.fffffffffffff8p0L, 1.570796326794896619231321691639751442099L, 8.813735870195429467279863907910371604180e-1L, 0 },
	TEST_c_c(cacos, 0x0.fffffffffffff8p0L, 0x1.fp-1025L, 1.490116119384765638786343542550461592240e-8L, -3.616396521699973256461764099945789620604e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x0.fffffffffffff8p0L, 0x1.fp-1025L, 3.141592638688632044614986995416067458693L, -3.616396521699973256461764099945789620604e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x0.fffffffffffff8p0L, -0x1.fp-1025L, 1.490116119384765638786343542550461592240e-8L, 3.616396521699973256461764099945789620604e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x0.fffffffffffff8p0L, -0x1.fp-1025L, 3.141592638688632044614986995416067458693L, 3.616396521699973256461764099945789620604e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.0L, 0x0.fffffffffffff8p0L, 1.570796326794896619231321691639751442099L, -8.813735870195429467279863907910371604180e-1L, 0 },
	TEST_c_c(cacos, 0.0L, -0x0.fffffffffffff8p0L, 1.570796326794896619231321691639751442099L, 8.813735870195429467279863907910371604180e-1L, 0 },
	TEST_c_c(cacos, -0.0L, 0x0.fffffffffffff8p0L, 1.570796326794896619231321691639751442099L, -8.813735870195429467279863907910371604180e-1L, 0 },
	TEST_c_c(cacos, -0.0L, -0x0.fffffffffffff8p0L, 1.570796326794896619231321691639751442099L, 8.813735870195429467279863907910371604180e-1L, 0 },
	TEST_c_c(cacos, 0x0.fffffffffffff8p0L, 0.0L, 1.490116119384765638786343542550461592240e-8L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x0.fffffffffffff8p0L, 0.0L, 3.141592638688632044614986995416067458693L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x0.fffffffffffff8p0L, -0.0L, 1.490116119384765638786343542550461592240e-8L, 0.0L, 0 },
	TEST_c_c(cacos, -0x0.fffffffffffff8p0L, -0.0L, 3.141592638688632044614986995416067458693L, 0.0L, 0 },
#endif
	TEST_c_c(cacos, 0x1p-52L, 0.5L, 1.570796326794896420628589431857911675917L, -4.812118250596034474977589134243772428682e-1L, 0 },
	TEST_c_c(cacos, 0x1p-52L, -0.5L, 1.570796326794896420628589431857911675917L, 4.812118250596034474977589134243772428682e-1L, 0 },
	TEST_c_c(cacos, -0x1p-52L, 0.5L, 1.570796326794896817834053951421591208280L, -4.812118250596034474977589134243772428682e-1L, 0 },
	TEST_c_c(cacos, -0x1p-52L, -0.5L, 1.570796326794896817834053951421591208280L, 4.812118250596034474977589134243772428682e-1L, 0 },
	TEST_c_c(cacos, 0.5L, 0x1p-52L, 1.047197551196597746154214461093186605110L, -2.563950248511418570403591756798643804971e-16L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, 0x1p-52L, 2.094395102393195492308428922186316279087L, -2.563950248511418570403591756798643804971e-16L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.5L, -0x1p-52L, 1.047197551196597746154214461093186605110L, 2.563950248511418570403591756798643804971e-16L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, -0x1p-52L, 2.094395102393195492308428922186316279087L, 2.563950248511418570403591756798643804971e-16L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(cacos, 0x1.fp-1025L, 0.5L, 1.570796326794896619231321691639751442099L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-1025L, -0.5L, 1.570796326794896619231321691639751442099L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, 0.5L, 1.570796326794896619231321691639751442099L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, -0.5L, 1.570796326794896619231321691639751442099L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, 0.5L, 0x1.fp-1025L, 1.047197551196597746154214461093167628066L, -6.222508863508420569166420770843207333493e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, 0x1.fp-1025L, 2.094395102393195492308428922186335256131L, -6.222508863508420569166420770843207333493e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.5L, -0x1.fp-1025L, 1.047197551196597746154214461093167628066L, 6.222508863508420569166420770843207333493e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, -0x1.fp-1025L, 2.094395102393195492308428922186335256131L, 6.222508863508420569166420770843207333493e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(cacos, 0x1p-52L, 0x1p-52L, 1.570796326794896397186716766608443357372L, -2.220446049250313080847263336181677117148e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 0x1p-52L, -0x1p-52L, 1.570796326794896397186716766608443357372L, 2.220446049250313080847263336181677117148e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0x1p-52L, 0x1p-52L, 1.570796326794896841275926616671059526825L, -2.220446049250313080847263336181677117148e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0x1p-52L, -0x1p-52L, 1.570796326794896841275926616671059526825L, 2.220446049250313080847263336181677117148e-16L, 0|XFAIL_NATIVE_LINUXF },
#ifndef TEST_FLOAT
	TEST_c_c(cacos, 0x1.fp-1025L, 0x1p-52L, 1.570796326794896619231321691639751442099L, -2.220446049250313080847263336181622378926e-16L, 0 },
	TEST_c_c(cacos, 0x1.fp-1025L, -0x1p-52L, 1.570796326794896619231321691639751442099L, 2.220446049250313080847263336181622378926e-16L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, 0x1p-52L, 1.570796326794896619231321691639751442099L, -2.220446049250313080847263336181622378926e-16L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, -0x1p-52L, 1.570796326794896619231321691639751442099L, 2.220446049250313080847263336181622378926e-16L, 0 },
	TEST_c_c(cacos, 0x1p-52L, 0x1.fp-1025L, 1.570796326794896397186716766608443357372L, -5.388850751072128349671657362289548938458e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-52L, 0x1.fp-1025L, 1.570796326794896841275926616671059526825L, -5.388850751072128349671657362289548938458e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-52L, -0x1.fp-1025L, 1.570796326794896397186716766608443357372L, 5.388850751072128349671657362289548938458e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-52L, -0x1.fp-1025L, 1.570796326794896841275926616671059526825L, 5.388850751072128349671657362289548938458e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(cacos, 0.0L, 0x1p-52L, 1.570796326794896619231321691639751442099L, -2.220446049250313080847263336181622378926e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 0.0L, -0x1p-52L, 1.570796326794896619231321691639751442099L, 2.220446049250313080847263336181622378926e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0.0L, 0x1p-52L, 1.570796326794896619231321691639751442099L, -2.220446049250313080847263336181622378926e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, -0.0L, -0x1p-52L, 1.570796326794896619231321691639751442099L, 2.220446049250313080847263336181622378926e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacos, 0x1p-52L, 0.0L, 1.570796326794896397186716766608443357372L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-52L, 0.0L, 1.570796326794896841275926616671059526825L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-52L, -0.0L, 1.570796326794896397186716766608443357372L, 0.0L, 0 },
	TEST_c_c(cacos, -0x1p-52L, -0.0L, 1.570796326794896841275926616671059526825L, 0.0L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(cacos, 0x1.fp-1025L, 0x1.fp-1025L, 1.570796326794896619231321691639751442099L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1.fp-1025L, -0x1.fp-1025L, 1.570796326794896619231321691639751442099L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1.fp-1025L, 0x1.fp-1025L, 1.570796326794896619231321691639751442099L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1.fp-1025L, -0x1.fp-1025L, 1.570796326794896619231321691639751442099L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(cacos, 0x1p-63L, 0x0.ffffffffffffffffp0L, 1.570796326794896619154657020805582738025L, -8.813735870195430251942769895627079569937e-1L, 0 },
	TEST_c_c(cacos, 0x1p-63L, -0x0.ffffffffffffffffp0L, 1.570796326794896619154657020805582738025L, 8.813735870195430251942769895627079569937e-1L, 0 },
	TEST_c_c(cacos, -0x1p-63L, 0x0.ffffffffffffffffp0L, 1.570796326794896619307986362473920146173L, -8.813735870195430251942769895627079569937e-1L, 0 },
	TEST_c_c(cacos, -0x1p-63L, -0x0.ffffffffffffffffp0L, 1.570796326794896619307986362473920146173L, 8.813735870195430251942769895627079569937e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffp0L, 0x1p-63L, 4.188407771167967636741951941902992986043e-10L, -2.588578361325995866221775673638805081337e-10L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x0.ffffffffffffffffp0L, 0x1p-63L, 3.141592653170952461345846619605307690007L, -2.588578361325995866221775673638805081337e-10L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 0x0.ffffffffffffffffp0L, -0x1p-63L, 4.188407771167967636741951941902992986043e-10L, 2.588578361325995866221775673638805081337e-10L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x0.ffffffffffffffffp0L, -0x1p-63L, 3.141592653170952461345846619605307690007L, 2.588578361325995866221775673638805081337e-10L, 0|XFAIL_NATIVE_LINUXL },
# if LDBL_MIN_EXP <= -16381
	TEST_c_c(cacos, 0x1.fp-16385L, 0x0.ffffffffffffffffp0L, 1.570796326794896619231321691639751442099L, -8.813735870195430251942769895627079569917e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-16385L, -0x0.ffffffffffffffffp0L, 1.570796326794896619231321691639751442099L, 8.813735870195430251942769895627079569917e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, 0x0.ffffffffffffffffp0L, 1.570796326794896619231321691639751442099L, -8.813735870195430251942769895627079569917e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, -0x0.ffffffffffffffffp0L, 1.570796326794896619231321691639751442099L, 8.813735870195430251942769895627079569917e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffp0L, 0x1.fp-16385L, 3.292722539913596233371825532007990724506e-10L, -2.472906068161537187835415298076415423459e-4923L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x0.ffffffffffffffffp0L, 0x1.fp-16385L, 3.141592653260520984471283759942320330996L, -2.472906068161537187835415298076415423459e-4923L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 0x0.ffffffffffffffffp0L, -0x1.fp-16385L, 3.292722539913596233371825532007990724506e-10L, 2.472906068161537187835415298076415423459e-4923L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x0.ffffffffffffffffp0L, -0x1.fp-16385L, 3.141592653260520984471283759942320330996L, 2.472906068161537187835415298076415423459e-4923L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
# endif
	TEST_c_c(cacos, 0.0L, 0x0.ffffffffffffffffp0L, 1.570796326794896619231321691639751442099L, -8.813735870195430251942769895627079569917e-1L, 0 },
	TEST_c_c(cacos, 0.0L, -0x0.ffffffffffffffffp0L, 1.570796326794896619231321691639751442099L, 8.813735870195430251942769895627079569917e-1L, 0 },
	TEST_c_c(cacos, -0.0L, 0x0.ffffffffffffffffp0L, 1.570796326794896619231321691639751442099L, -8.813735870195430251942769895627079569917e-1L, 0 },
	TEST_c_c(cacos, -0.0L, -0x0.ffffffffffffffffp0L, 1.570796326794896619231321691639751442099L, 8.813735870195430251942769895627079569917e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffp0L, 0.0L, 3.292722539913596233371825532007990724506e-10L, -0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x0.ffffffffffffffffp0L, 0.0L, 3.141592653260520984471283759942320330996L, -0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 0x0.ffffffffffffffffp0L, -0.0L, 3.292722539913596233371825532007990724506e-10L, 0.0L, 0 },
	TEST_c_c(cacos, -0x0.ffffffffffffffffp0L, -0.0L, 3.141592653260520984471283759942320330996L, 0.0L, 0 },
#endif
	TEST_c_c(cacos, 0x1p-63L, 0.5L, 1.570796326794896619134347701278529840650L, -4.812118250596034474977589134243684231373e-1L, 0 },
	TEST_c_c(cacos, 0x1p-63L, -0.5L, 1.570796326794896619134347701278529840650L, 4.812118250596034474977589134243684231373e-1L, 0 },
	TEST_c_c(cacos, -0x1p-63L, 0.5L, 1.570796326794896619328295682000973043547L, -4.812118250596034474977589134243684231373e-1L, 0 },
	TEST_c_c(cacos, -0x1p-63L, -0.5L, 1.570796326794896619328295682000973043547L, 4.812118250596034474977589134243684231373e-1L, 0 },
	TEST_c_c(cacos, 0.5L, 0x1p-63L, 1.047197551196597746154214461093167628070L, -1.251928832280966098829878787499365228659e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, 0x1p-63L, 2.094395102393195492308428922186335256127L, -1.251928832280966098829878787499365228659e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.5L, -0x1p-63L, 1.047197551196597746154214461093167628070L, 1.251928832280966098829878787499365228659e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, -0x1p-63L, 2.094395102393195492308428922186335256127L, 1.251928832280966098829878787499365228659e-19L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacos, 0x1.fp-16385L, 0.5L, 1.570796326794896619231321691639751442099L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-16385L, -0.5L, 1.570796326794896619231321691639751442099L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, 0.5L, 1.570796326794896619231321691639751442099L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, -0.5L, 1.570796326794896619231321691639751442099L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, 0.5L, 0x1.fp-16385L, 1.047197551196597746154214461093167628066L, -9.402257155670418112159334875199821342890e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0.5L, 0x1.fp-16385L, 2.094395102393195492308428922186335256131L, -9.402257155670418112159334875199821342890e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 0.5L, -0x1.fp-16385L, 1.047197551196597746154214461093167628066L, 9.402257155670418112159334875199821342890e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0.5L, -0x1.fp-16385L, 2.094395102393195492308428922186335256131L, 9.402257155670418112159334875199821342890e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(cacos, 0x1p-63L, 0x1p-63L, 1.570796326794896619122901474391200998698L, -1.084202172485504434007452800869941711430e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-63L, -0x1p-63L, 1.570796326794896619122901474391200998698L, 1.084202172485504434007452800869941711430e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-63L, 0x1p-63L, 1.570796326794896619339741908888301885499L, -1.084202172485504434007452800869941711430e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-63L, -0x1p-63L, 1.570796326794896619339741908888301885499L, 1.084202172485504434007452800869941711430e-19L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacos, 0x1.fp-16385L, 0x1p-63L, 1.570796326794896619231321691639751442099L, -1.084202172485504434007452800869941711424e-19L, 0 },
	TEST_c_c(cacos, 0x1.fp-16385L, -0x1p-63L, 1.570796326794896619231321691639751442099L, 1.084202172485504434007452800869941711424e-19L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, 0x1p-63L, 1.570796326794896619231321691639751442099L, -1.084202172485504434007452800869941711424e-19L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, -0x1p-63L, 1.570796326794896619231321691639751442099L, 1.084202172485504434007452800869941711424e-19L, 0 },
	TEST_c_c(cacos, 0x1p-63L, 0x1.fp-16385L, 1.570796326794896619122901474391200998698L, -8.142593549724601460479922838826119584465e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1p-63L, 0x1.fp-16385L, 1.570796326794896619339741908888301885499L, -8.142593549724601460479922838826119584465e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 0x1p-63L, -0x1.fp-16385L, 1.570796326794896619122901474391200998698L, 8.142593549724601460479922838826119584465e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1p-63L, -0x1.fp-16385L, 1.570796326794896619339741908888301885499L, 8.142593549724601460479922838826119584465e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(cacos, 0.0L, 0x1p-63L, 1.570796326794896619231321691639751442099L, -1.084202172485504434007452800869941711424e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.0L, -0x1p-63L, 1.570796326794896619231321691639751442099L, 1.084202172485504434007452800869941711424e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.0L, 0x1p-63L, 1.570796326794896619231321691639751442099L, -1.084202172485504434007452800869941711424e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.0L, -0x1p-63L, 1.570796326794896619231321691639751442099L, 1.084202172485504434007452800869941711424e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-63L, 0.0L, 1.570796326794896619122901474391200998698L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-63L, 0.0L, 1.570796326794896619339741908888301885499L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-63L, -0.0L, 1.570796326794896619122901474391200998698L, 0.0L, 0 },
	TEST_c_c(cacos, -0x1p-63L, -0.0L, 1.570796326794896619339741908888301885499L, 0.0L, 0 },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacos, 0x1.fp-16385L, 0x1.fp-16385L, 1.570796326794896619231321691639751442099L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 0x1.fp-16385L, -0x1.fp-16385L, 1.570796326794896619231321691639751442099L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1.fp-16385L, 0x1.fp-16385L, 1.570796326794896619231321691639751442099L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1.fp-16385L, -0x1.fp-16385L, 1.570796326794896619231321691639751442099L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(cacos, 0x1p-105L, 0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639734010571L, -8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, 0x1p-105L, -0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639734010571L, 8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, -0x1p-105L, 0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639768873627L, -8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, -0x1p-105L, -0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639768873627L, 8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffcp0L, 0x1p-105L, 1.997188458999618357060632049675702684031e-16L, -1.234330349600789959989661887846005659983e-16L, 0 },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffcp0L, 0x1p-105L, 3.141592653589793038743797483317667178134L, -1.234330349600789959989661887846005659983e-16L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffcp0L, -0x1p-105L, 1.997188458999618357060632049675702684031e-16L, 1.234330349600789959989661887846005659983e-16L, 0 },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffcp0L, -0x1p-105L, 3.141592653589793038743797483317667178134L, 1.234330349600789959989661887846005659983e-16L, 0 },
	TEST_c_c(cacos, 0x1.fp-1025L, 0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-1025L, -0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, 0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-1025L, -0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffcp0L, 0x1.fp-1025L, 1.570092458683775059393382371963565104164e-16L, -3.432186888910770473784103112009399360940e-293L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffcp0L, 0x1.fp-1025L, 3.141592653589793081453397514901996944859L, -3.432186888910770473784103112009399360940e-293L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffcp0L, -0x1.fp-1025L, 1.570092458683775059393382371963565104164e-16L, 3.432186888910770473784103112009399360940e-293L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffcp0L, -0x1.fp-1025L, 3.141592653589793081453397514901996944859L, 3.432186888910770473784103112009399360940e-293L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(cacos, 0.0L, 0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, 0.0L, -0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, -0.0L, 0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, -0.0L, -0x0.ffffffffffffffffffffffffffcp0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffcp0L, 0.0L, 1.570092458683775059393382371963565104164e-16L, -0.0L, 0 },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffcp0L, 0.0L, 3.141592653589793081453397514901996944859L, -0.0L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffcp0L, -0.0L, 1.570092458683775059393382371963565104164e-16L, 0.0L, 0 },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffcp0L, -0.0L, 3.141592653589793081453397514901996944859L, 0.0L, 0 },
#endif
	TEST_c_c(cacos, 0x1p-105L, 0.5L, 1.570796326794896619231321691639729392766L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, 0x1p-105L, -0.5L, 1.570796326794896619231321691639729392766L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, -0x1p-105L, 0.5L, 1.570796326794896619231321691639773491431L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, -0x1p-105L, -0.5L, 1.570796326794896619231321691639773491431L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, 0.5L, 0x1p-105L, 1.047197551196597746154214461093167628066L, -2.846556599890768890940941645338546615370e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, 0x1p-105L, 2.094395102393195492308428922186335256131L, -2.846556599890768890940941645338546615370e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.5L, -0x1p-105L, 1.047197551196597746154214461093167628066L, 2.846556599890768890940941645338546615370e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, -0x1p-105L, 2.094395102393195492308428922186335256131L, 2.846556599890768890940941645338546615370e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-105L, 0x1p-105L, 1.570796326794896619231321691639726790195L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-105L, -0x1p-105L, 1.570796326794896619231321691639726790195L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-105L, 0x1p-105L, 1.570796326794896619231321691639776094002L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-105L, -0x1p-105L, 1.570796326794896619231321691639776094002L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(cacos, 0x1.fp-1025L, 0x1p-105L, 1.570796326794896619231321691639751442099L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1.fp-1025L, -0x1p-105L, 1.570796326794896619231321691639751442099L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1.fp-1025L, 0x1p-105L, 1.570796326794896619231321691639751442099L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1.fp-1025L, -0x1p-105L, 1.570796326794896619231321691639751442099L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-105L, 0x1.fp-1025L, 1.570796326794896619231321691639726790195L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-105L, 0x1.fp-1025L, 1.570796326794896619231321691639776094002L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-105L, -0x1.fp-1025L, 1.570796326794896619231321691639726790195L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-105L, -0x1.fp-1025L, 1.570796326794896619231321691639776094002L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(cacos, 0.0L, 0x1p-105L, 1.570796326794896619231321691639751442099L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.0L, -0x1p-105L, 1.570796326794896619231321691639751442099L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.0L, 0x1p-105L, 1.570796326794896619231321691639751442099L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.0L, -0x1p-105L, 1.570796326794896619231321691639751442099L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-105L, 0.0L, 1.570796326794896619231321691639726790195L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-105L, 0.0L, 1.570796326794896619231321691639776094002L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-105L, -0.0L, 1.570796326794896619231321691639726790195L, 0.0L, 0 },
	TEST_c_c(cacos, -0x1p-105L, -0.0L, 1.570796326794896619231321691639776094002L, 0.0L, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(cacos, 0x1p-112L, 0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751305915L, -8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, 0x1p-112L, -0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751305915L, 8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, -0x1p-112L, 0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751578282L, -8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, -0x1p-112L, -0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751578282L, 8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-112L, 1.765281878332676474347460257221369818872e-17L, -1.091004200533850618345330909857103457781e-17L, 0 },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-112L, 3.141592653589793220809824599952738140723L, -1.091004200533850618345330909857103457781e-17L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-112L, 1.765281878332676474347460257221369818872e-17L, 1.091004200533850618345330909857103457781e-17L, 0 },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-112L, 3.141592653589793220809824599952738140723L, 1.091004200533850618345330909857103457781e-17L, 0 },
	TEST_c_c(cacos, 0x1.fp-16385L, 0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, 0x1.fp-16385L, -0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, 0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, -0x1.fp-16385L, -0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1.fp-16385L, 1.387778780781445675529539585113525401762e-17L, -5.867357004219059108790368268900490184505e-4916L, 0 },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1.fp-16385L, 3.141592653589793224584855575465046128902L, -5.867357004219059108790368268900490184505e-4916L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1.fp-16385L, 1.387778780781445675529539585113525401762e-17L, 5.867357004219059108790368268900490184505e-4916L, 0 },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1.fp-16385L, 3.141592653589793224584855575465046128902L, 5.867357004219059108790368268900490184505e-4916L, 0 },
	TEST_c_c(cacos, 0.0L, 0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, 0.0L, -0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, -0.0L, 0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751442099L, -8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, -0.0L, -0x0.ffffffffffffffffffffffffffff8p0L, 1.570796326794896619231321691639751442099L, 8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, 1.387778780781445675529539585113525401762e-17L, -0.0L, 0 },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, 3.141592653589793224584855575465046128902L, -0.0L, 0 },
	TEST_c_c(cacos, 0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, 1.387778780781445675529539585113525401762e-17L, 0.0L, 0 },
	TEST_c_c(cacos, -0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, 3.141592653589793224584855575465046128902L, 0.0L, 0 },
#endif
	TEST_c_c(cacos, 0x1p-112L, 0.5L, 1.570796326794896619231321691639751269838L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, 0x1p-112L, -0.5L, 1.570796326794896619231321691639751269838L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, -0x1p-112L, 0.5L, 1.570796326794896619231321691639751614359L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, -0x1p-112L, -0.5L, 1.570796326794896619231321691639751614359L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(cacos, 0.5L, 0x1p-112L, 1.047197551196597746154214461093167628066L, -2.223872343664663196047610660420739543258e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, 0x1p-112L, 2.094395102393195492308428922186335256131L, -2.223872343664663196047610660420739543258e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.5L, -0x1p-112L, 1.047197551196597746154214461093167628066L, 2.223872343664663196047610660420739543258e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.5L, -0x1p-112L, 2.094395102393195492308428922186335256131L, 2.223872343664663196047610660420739543258e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-112L, 0x1p-112L, 1.570796326794896619231321691639751249506L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-112L, -0x1p-112L, 1.570796326794896619231321691639751249506L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-112L, 0x1p-112L, 1.570796326794896619231321691639751634692L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-112L, -0x1p-112L, 1.570796326794896619231321691639751634692L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacos, 0x1.fp-16385L, 0x1p-112L, 1.570796326794896619231321691639751442099L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 0x1.fp-16385L, -0x1p-112L, 1.570796326794896619231321691639751442099L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1.fp-16385L, 0x1p-112L, 1.570796326794896619231321691639751442099L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1.fp-16385L, -0x1p-112L, 1.570796326794896619231321691639751442099L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 0x1p-112L, 0x1.fp-16385L, 1.570796326794896619231321691639751249506L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1p-112L, 0x1.fp-16385L, 1.570796326794896619231321691639751634692L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, 0x1p-112L, -0x1.fp-16385L, 1.570796326794896619231321691639751249506L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacos, -0x1p-112L, -0x1.fp-16385L, 1.570796326794896619231321691639751634692L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(cacos, 0.0L, 0x1p-112L, 1.570796326794896619231321691639751442099L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0.0L, -0x1p-112L, 1.570796326794896619231321691639751442099L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.0L, 0x1p-112L, 1.570796326794896619231321691639751442099L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0.0L, -0x1p-112L, 1.570796326794896619231321691639751442099L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-112L, 0.0L, 1.570796326794896619231321691639751249506L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, -0x1p-112L, 0.0L, 1.570796326794896619231321691639751634692L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacos, 0x1p-112L, -0.0L, 1.570796326794896619231321691639751249506L, 0.0L, 0 },
	TEST_c_c(cacos, -0x1p-112L, -0.0L, 1.570796326794896619231321691639751634692L, 0.0L, 0 },

	TEST_c_c(cacos, 0.75L, 1.25L, 1.11752014915610270578240049553777969L, -1.13239363160530819522266333696834467L, 0 },
	TEST_c_c(cacos, -2, -3, 2.1414491111159960199416055713254211L, 1.9833870299165354323470769028940395L, 0 },
}


static void test_cacosh(void)
{
	struct test *testp = lookup("cacosh" },

	TEST_c_c(cacosh, 0, 0, 0.0, M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_M, 0, 0.0, M_PI_2l, 0 },
	TEST_c_c(cacosh, 0, ZERO_M, 0.0, -M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_M, ZERO_M, 0.0, -M_PI_2l, 0 },
	TEST_c_c(cacosh, INF_M, INF_P, INF_P, M_PI_34l, INVALID_EXCEPTION_OK },
	TEST_c_c(cacosh, INF_M, INF_M, INF_P, -M_PI_34l, 0 },

	TEST_c_c(cacosh, INF_P, INF_P, INF_P, M_PI_4l, 0 },
	TEST_c_c(cacosh, INF_P, INF_M, INF_P, -M_PI_4l, 0 },

	TEST_c_c(cacosh, -10.0, INF_P, INF_P, M_PI_2l, 0 },
	TEST_c_c(cacosh, -10.0, INF_M, INF_P, -M_PI_2l, 0 },
	TEST_c_c(cacosh, 0, INF_P, INF_P, M_PI_2l, 0 },
	TEST_c_c(cacosh, 0, INF_M, INF_P, -M_PI_2l, 0 },
	TEST_c_c(cacosh, 0.1L, INF_P, INF_P, M_PI_2l, 0 },
	TEST_c_c(cacosh, 0.1L, INF_M, INF_P, -M_PI_2l, 0 },

	TEST_c_c(cacosh, INF_M, 0, INF_P, M_PIl, 0 },
	TEST_c_c(cacosh, INF_M, ZERO_M, INF_P, -M_PIl, 0 },
	TEST_c_c(cacosh, INF_M, 100, INF_P, M_PIl, 0 },
	TEST_c_c(cacosh, INF_M, -100, INF_P, -M_PIl, 0 },

	TEST_c_c(cacosh, INF_P, 0, INF_P, 0.0, 0 },
	TEST_c_c(cacosh, INF_P, ZERO_M, INF_P, ZERO_M, 0 },
	TEST_c_c(cacosh, INF_P, 0.5, INF_P, 0.0, 0 },
	TEST_c_c(cacosh, INF_P, -0.5, INF_P, ZERO_M, 0 },

	TEST_c_c(cacosh, INF_P, QNAN_P, INF_P, QNAN_P, 0 },
	TEST_c_c(cacosh, INF_M, QNAN_P, INF_P, QNAN_P, 0 },

	TEST_c_c(cacosh, 0, QNAN_P, QNAN_P, QNAN_P, 0 },
	TEST_c_c(cacosh, ZERO_M, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(cacosh, QNAN_P, INF_P, INF_P, QNAN_P, 0 },
	TEST_c_c(cacosh, QNAN_P, INF_M, INF_P, QNAN_P, 0 },

	TEST_c_c(cacosh, 10.5, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(cacosh, -10.5, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(cacosh, QNAN_P, 0.75, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(cacosh, QNAN_P, -0.75, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(cacosh, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(cacosh, ZERO_P, -1.5L, 1.194763217287109304111930828519090523536L, -M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_M, -1.5L, 1.194763217287109304111930828519090523536L, -M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_P, -1.0L, 0.8813735870195430252326093249797923090282L, -M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_M, -1.0L, 0.8813735870195430252326093249797923090282L, -M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_P, -0.5L, 0.4812118250596034474977589134243684231352L, -M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_M, -0.5L, 0.4812118250596034474977589134243684231352L, -M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_P, 0.5L, 0.4812118250596034474977589134243684231352L, M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_M, 0.5L, 0.4812118250596034474977589134243684231352L, M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_P, 1.0L, 0.8813735870195430252326093249797923090282L, M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_M, 1.0L, 0.8813735870195430252326093249797923090282L, M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_P, 1.5L, 1.194763217287109304111930828519090523536L, M_PI_2l, 0 },
	TEST_c_c(cacosh, ZERO_M, 1.5L, 1.194763217287109304111930828519090523536L, M_PI_2l, 0 },

	TEST_c_c(cacosh, -1.5L, ZERO_P, 0.9624236501192068949955178268487368462704L, M_PIl, 0 },
	TEST_c_c(cacosh, -1.5L, ZERO_M, 0.9624236501192068949955178268487368462704L, -M_PIl, 0 },
	TEST_c_c(cacosh, -1.0L, ZERO_P, ZERO_P, M_PIl, 0 },
	TEST_c_c(cacosh, -1.0L, ZERO_M, ZERO_P, -M_PIl, 0 },
	TEST_c_c(cacosh, -0.5L, ZERO_P, ZERO_P, 2.094395102393195492308428922186335256131L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, ZERO_M, ZERO_P, -2.094395102393195492308428922186335256131L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.5L, ZERO_P, ZERO_P, 1.047197551196597746154214461093167628066L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.5L, ZERO_M, ZERO_P, -1.047197551196597746154214461093167628066L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 1.0L, ZERO_P, ZERO_P, ZERO_P, 0 },
	TEST_c_c(cacosh, 1.0L, ZERO_M, ZERO_P, ZERO_M, 0 },
	TEST_c_c(cacosh, 1.5L, ZERO_P, 0.9624236501192068949955178268487368462704L, ZERO_P, 0 },
	TEST_c_c(cacosh, 1.5L, ZERO_M, 0.9624236501192068949955178268487368462704L, ZERO_M, 0 },

	TEST_c_c(cacosh, 0x1p50L, 1.0L, 3.535050620855721078027883819436720218708e1L, 8.881784197001252323389053344727730248720e-16L, 0 },
	TEST_c_c(cacosh, 0x1p50L, -1.0L, 3.535050620855721078027883819436720218708e1L, -8.881784197001252323389053344727730248720e-16L, 0 },
	TEST_c_c(cacosh, -0x1p50L, 1.0L, 3.535050620855721078027883819436720218708e1L, 3.141592653589792350284223683154270545292L, 0 },
	TEST_c_c(cacosh, -0x1p50L, -1.0L, 3.535050620855721078027883819436720218708e1L, -3.141592653589792350284223683154270545292L, 0 },
	TEST_c_c(cacosh, 1.0L, 0x1p50L, 3.535050620855721078027883819436759661753e1L, 1.570796326794895731052901991514519103193L, 0 },
	TEST_c_c(cacosh, -1.0L, 0x1p50L, 3.535050620855721078027883819436759661753e1L, 1.570796326794897507409741391764983781004L, 0 },
	TEST_c_c(cacosh, 1.0L, -0x1p50L, 3.535050620855721078027883819436759661753e1L, -1.570796326794895731052901991514519103193L, 0 },
	TEST_c_c(cacosh, -1.0L, -0x1p50L, 3.535050620855721078027883819436759661753e1L, -1.570796326794897507409741391764983781004L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(cacosh, 0x1p500L, 1.0L, 3.472667374605326000180332928505464606058e2L, 3.054936363499604682051979393213617699789e-151L, 0 },
	TEST_c_c(cacosh, 0x1p500L, -1.0L, 3.472667374605326000180332928505464606058e2L, -3.054936363499604682051979393213617699789e-151L, 0 },
	TEST_c_c(cacosh, -0x1p500L, 1.0L, 3.472667374605326000180332928505464606058e2L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, -0x1p500L, -1.0L, 3.472667374605326000180332928505464606058e2L, -3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 1.0L, 0x1p500L, 3.472667374605326000180332928505464606058e2L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -1.0L, 0x1p500L, 3.472667374605326000180332928505464606058e2L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 1.0L, -0x1p500L, 3.472667374605326000180332928505464606058e2L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -1.0L, -0x1p500L, 3.472667374605326000180332928505464606058e2L, -1.570796326794896619231321691639751442099L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(cacosh, 0x1p5000L, 1.0L, 3.466429049980286492395577839412341016946e3L, 7.079811261048172892385615158694057552948e-1506L, 0 },
	TEST_c_c(cacosh, 0x1p5000L, -1.0L, 3.466429049980286492395577839412341016946e3L, -7.079811261048172892385615158694057552948e-1506L, 0 },
	TEST_c_c(cacosh, -0x1p5000L, 1.0L, 3.466429049980286492395577839412341016946e3L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, -0x1p5000L, -1.0L, 3.466429049980286492395577839412341016946e3L, -3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 1.0L, 0x1p5000L, 3.466429049980286492395577839412341016946e3L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -1.0L, 0x1p5000L, 3.466429049980286492395577839412341016946e3L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 1.0L, -0x1p5000L, 3.466429049980286492395577839412341016946e3L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -1.0L, -0x1p5000L, 3.466429049980286492395577839412341016946e3L, -1.570796326794896619231321691639751442099L, 0 },
#endif
	TEST_c_c(cacosh, 0x1.fp127L, 0x1.fp127L, 8.973081118419833726837456344608533993585e1L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXF },
#ifndef TEST_FLOAT
	TEST_c_c(cacosh, 0x1.fp1023L, 0x1.fp1023L, 7.107906849659093345062145442726115449315e2L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(cacosh, 0x1.fp16383L, 0x1.fp16383L, 1.135753137836666928715489992987020363057e4L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(cacosh, 0x1.fp-129L, 1.5L, 1.194763217287109304111930828519090523536L, 1.570796326794896619231321691639751442097L, 0 },
	TEST_c_c(cacosh, 0x1.fp-129L, -1.5L, 1.194763217287109304111930828519090523536L, -1.570796326794896619231321691639751442097L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, 1.5L, 1.194763217287109304111930828519090523536L, 1.570796326794896619231321691639751442100L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, -1.5L, 1.194763217287109304111930828519090523536L, -1.570796326794896619231321691639751442100L, 0 },
	TEST_c_c(cacosh, 1.5L, 0x1.fp-129L, 9.624236501192068949955178268487368462704e-1L, 2.546345110742945032959687790021055102355e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(cacosh, -1.5L, 0x1.fp-129L, 9.624236501192068949955178268487368462704e-1L, 3.141592653589793238462643383279502884195L, 0 },
	TEST_c_c(cacosh, 1.5L, -0x1.fp-129L, 9.624236501192068949955178268487368462704e-1L, -2.546345110742945032959687790021055102355e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(cacosh, -1.5L, -0x1.fp-129L, 9.624236501192068949955178268487368462704e-1L, -3.141592653589793238462643383279502884195L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(cacosh, 0x1.fp-1025L, 1.5L, 1.194763217287109304111930828519090523536L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1025L, -1.5L, 1.194763217287109304111930828519090523536L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, 1.5L, 1.194763217287109304111930828519090523536L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, -1.5L, 1.194763217287109304111930828519090523536L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 1.5L, 0x1.fp-1025L, 9.624236501192068949955178268487368462704e-1L, 4.819934639999230680322935210539402497827e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(cacosh, -1.5L, 0x1.fp-1025L, 9.624236501192068949955178268487368462704e-1L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 1.5L, -0x1.fp-1025L, 9.624236501192068949955178268487368462704e-1L, -4.819934639999230680322935210539402497827e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(cacosh, -1.5L, -0x1.fp-1025L, 9.624236501192068949955178268487368462704e-1L, -3.141592653589793238462643383279502884197L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacosh, 0x1.fp-16385L, 1.5L, 1.194763217287109304111930828519090523536L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-16385L, -1.5L, 1.194763217287109304111930828519090523536L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, 1.5L, 1.194763217287109304111930828519090523536L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, -1.5L, 1.194763217287109304111930828519090523536L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 1.5L, 0x1.fp-16385L, 9.624236501192068949955178268487368462704e-1L, 7.282957076134209141226696333885150260319e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(cacosh, -1.5L, 0x1.fp-16385L, 9.624236501192068949955178268487368462704e-1L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 1.5L, -0x1.fp-16385L, 9.624236501192068949955178268487368462704e-1L, -7.282957076134209141226696333885150260319e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(cacosh, -1.5L, -0x1.fp-16385L, 9.624236501192068949955178268487368462704e-1L, -3.141592653589793238462643383279502884197L, 0 },
#endif
	TEST_c_c(cacosh, 0.5L, 1.0L, 9.261330313501824245501244453057873152694e-1L, 1.221357263937683325603909865564381489366L, 0 },
	TEST_c_c(cacosh, 0.5L, -1.0L, 9.261330313501824245501244453057873152694e-1L, -1.221357263937683325603909865564381489366L, 0 },
	TEST_c_c(cacosh, -0.5L, 1.0L, 9.261330313501824245501244453057873152694e-1L, 1.920235389652109912858733517715121394831L, 0 },
	TEST_c_c(cacosh, -0.5L, -1.0L, 9.261330313501824245501244453057873152694e-1L, -1.920235389652109912858733517715121394831L, 0 },
	TEST_c_c(cacosh, 1.0L, 0.5L, 7.328576759736452608886724437653071523305e-1L, 6.748888455860063801646649673121744318756e-1L, 0 },
	TEST_c_c(cacosh, -1.0L, 0.5L, 7.328576759736452608886724437653071523305e-1L, 2.466703808003786858297978415967328452322L, 0 },
	TEST_c_c(cacosh, 1.0L, -0.5L, 7.328576759736452608886724437653071523305e-1L, -6.748888455860063801646649673121744318756e-1L, 0 },
	TEST_c_c(cacosh, -1.0L, -0.5L, 7.328576759736452608886724437653071523305e-1L, -2.466703808003786858297978415967328452322L, 0 },
	TEST_c_c(cacosh, 0.25L, 1.0L, 8.924633639033482359562124741744951972772e-1L, 1.394493894017929688812643125003661339452L, 0 },
	TEST_c_c(cacosh, 0.25L, -1.0L, 8.924633639033482359562124741744951972772e-1L, -1.394493894017929688812643125003661339452L, 0 },
	TEST_c_c(cacosh, -0.25L, 1.0L, 8.924633639033482359562124741744951972772e-1L, 1.747098759571863549650000258275841544745L, 0 },
	TEST_c_c(cacosh, -0.25L, -1.0L, 8.924633639033482359562124741744951972772e-1L, -1.747098759571863549650000258275841544745L, 0 },
	TEST_c_c(cacosh, 1.0L, 0.25L, 5.097911466811016354623559941115413499164e-1L, 4.890443302710802929202843732146540079124e-1L, 0 },
	TEST_c_c(cacosh, -1.0L, 0.25L, 5.097911466811016354623559941115413499164e-1L, 2.652548323318712945542359010064848876285L, 0 },
	TEST_c_c(cacosh, 1.0L, -0.25L, 5.097911466811016354623559941115413499164e-1L, -4.890443302710802929202843732146540079124e-1L, 0 },
	TEST_c_c(cacosh, -1.0L, -0.25L, 5.097911466811016354623559941115413499164e-1L, -2.652548323318712945542359010064848876285L, 0 },
	TEST_c_c(cacosh, 0x1.fp-10L, 1.0L, 8.813742198809567991336704287826445879025e-1L, 1.569458417435338878318763342108699202986L, 0 },
	TEST_c_c(cacosh, 0x1.fp-10L, -1.0L, 8.813742198809567991336704287826445879025e-1L, -1.569458417435338878318763342108699202986L, 0 },
	TEST_c_c(cacosh, -0x1.fp-10L, 1.0L, 8.813742198809567991336704287826445879025e-1L, 1.572134236154454360143880041170803681211L, 0 },
	TEST_c_c(cacosh, -0x1.fp-10L, -1.0L, 8.813742198809567991336704287826445879025e-1L, -1.572134236154454360143880041170803681211L, 0 },
	TEST_c_c(cacosh, 1.0L, 0x1.fp-10L, 4.350501469856803800217957402220976497152e-2L, 4.349129763101882771258049954181971959031e-2L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -1.0L, 0x1.fp-10L, 4.350501469856803800217957402220976497152e-2L, 3.098101355958774410750062883737683164607L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 1.0L, -0x1.fp-10L, 4.350501469856803800217957402220976497152e-2L, -4.349129763101882771258049954181971959031e-2L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -1.0L, -0x1.fp-10L, 4.350501469856803800217957402220976497152e-2L, -3.098101355958774410750062883737683164607L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 0x1.fp-30L, 1.0L, 8.813735870195430258081932989769495326854e-1L, 1.570796325518966635014803151387033957091L, 0 },
	TEST_c_c(cacosh, 0x1.fp-30L, -1.0L, 8.813735870195430258081932989769495326854e-1L, -1.570796325518966635014803151387033957091L, 0 },
	TEST_c_c(cacosh, -0x1.fp-30L, 1.0L, 8.813735870195430258081932989769495326854e-1L, 1.570796328070826603447840231892468927106L, 0 },
	TEST_c_c(cacosh, -0x1.fp-30L, -1.0L, 8.813735870195430258081932989769495326854e-1L, -1.570796328070826603447840231892468927106L, 0 },
	TEST_c_c(cacosh, 1.0L, 0x1.fp-30L, 4.247867098745151888768727039216644758847e-5L, 4.247867097467650115899790787875186617316e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -1.0L, 0x1.fp-30L, 4.247867098745151888768727039216644758847e-5L, 3.141550174918818561961484385371624132331L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 1.0L, -0x1.fp-30L, 4.247867098745151888768727039216644758847e-5L, -4.247867097467650115899790787875186617316e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -1.0L, -0x1.fp-30L, 4.247867098745151888768727039216644758847e-5L, -3.141550174918818561961484385371624132331L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 0x1.fp-100L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691638670687364L, 0 },
	TEST_c_c(cacosh, 0x1.fp-100L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691638670687364L, 0 },
	TEST_c_c(cacosh, -0x1.fp-100L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691640832196834L, 0 },
	TEST_c_c(cacosh, -0x1.fp-100L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691640832196834L, 0 },
	TEST_c_c(cacosh, 1.0L, 0x1.fp-100L, 1.236292038260260888664514866457202186027e-15L, 1.236292038260260888664514866456887257525e-15L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -1.0L, 0x1.fp-100L, 1.236292038260260888664514866457202186027e-15L, 3.141592653589792002170605123018614219682L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 1.0L, -0x1.fp-100L, 1.236292038260260888664514866457202186027e-15L, -1.236292038260260888664514866456887257525e-15L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -1.0L, -0x1.fp-100L, 1.236292038260260888664514866457202186027e-15L, -3.141592653589792002170605123018614219682L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1.fp-129L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691639751442097L, 0 },
	TEST_c_c(cacosh, 0x1.fp-129L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691639751442097L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691639751442101L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691639751442101L, 0 },
	TEST_c_c(cacosh, 1.0L, 0x1.fp-129L, 5.335635276982233498398987585285818977933e-20L, 5.335635276982233498398987585285818977930e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -1.0L, 0x1.fp-129L, 5.335635276982233498398987585285818977933e-20L, 3.141592653589793238409287030509680549213L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 1.0L, -0x1.fp-129L, 5.335635276982233498398987585285818977933e-20L, -5.335635276982233498398987585285818977930e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -1.0L, -0x1.fp-129L, 5.335635276982233498398987585285818977933e-20L, -3.141592653589793238409287030509680549213L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(cacosh, 0x1.fp-1000L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1000L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1000L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1000L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 1.0L, 0x1.fp-1000L, 4.252291453851660175550490409247739011867e-151L, 4.252291453851660175550490409247739011867e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -1.0L, 0x1.fp-1000L, 4.252291453851660175550490409247739011867e-151L, 3.141592653589793238462643383279502884197L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 1.0L, -0x1.fp-1000L, 4.252291453851660175550490409247739011867e-151L, -4.252291453851660175550490409247739011867e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -1.0L, -0x1.fp-1000L, 4.252291453851660175550490409247739011867e-151L, -3.141592653589793238462643383279502884197L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1.fp-1025L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1025L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 1.0L, 0x1.fp-1025L, 7.340879205566679497036857179189356754017e-155L, 7.340879205566679497036857179189356754017e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -1.0L, 0x1.fp-1025L, 7.340879205566679497036857179189356754017e-155L, 3.141592653589793238462643383279502884197L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 1.0L, -0x1.fp-1025L, 7.340879205566679497036857179189356754017e-155L, -7.340879205566679497036857179189356754017e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -1.0L, -0x1.fp-1025L, 7.340879205566679497036857179189356754017e-155L, -3.141592653589793238462643383279502884197L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacosh, 0x1.fp-10000L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-10000L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-10000L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-10000L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 1.0L, 0x1.fp-10000L, 9.854680208706673586644342922051388714633e-1506L, 9.854680208706673586644342922051388714633e-1506L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -1.0L, 0x1.fp-10000L, 9.854680208706673586644342922051388714633e-1506L, 3.141592653589793238462643383279502884197L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, 1.0L, -0x1.fp-10000L, 9.854680208706673586644342922051388714633e-1506L, -9.854680208706673586644342922051388714633e-1506L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -1.0L, -0x1.fp-10000L, 9.854680208706673586644342922051388714633e-1506L, -3.141592653589793238462643383279502884197L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, 0x1.fp-16385L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-16385L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, 1.0L, 8.813735870195430252326093249797923090282e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, -1.0L, 8.813735870195430252326093249797923090282e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 1.0L, 0x1.fp-16385L, 9.023632056840860275214893047597614177639e-2467L, 9.023632056840860275214893047597614177639e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -1.0L, 0x1.fp-16385L, 9.023632056840860275214893047597614177639e-2467L, 3.141592653589793238462643383279502884197L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, 1.0L, -0x1.fp-16385L, 9.023632056840860275214893047597614177639e-2467L, -9.023632056840860275214893047597614177639e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -1.0L, -0x1.fp-16385L, 9.023632056840860275214893047597614177639e-2467L, -3.141592653589793238462643383279502884197L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(cacosh, 0x1p-23L, 0x1.000002p0L, 8.813736713132400470205730751186547909968e-1L, 1.570796242501204621739026081991856762292L, 0 },
	TEST_c_c(cacosh, 0x1p-23L, -0x1.000002p0L, 8.813736713132400470205730751186547909968e-1L, -1.570796242501204621739026081991856762292L, 0 },
	TEST_c_c(cacosh, -0x1p-23L, 0x1.000002p0L, 8.813736713132400470205730751186547909968e-1L, 1.570796411088588616723617301287646121905L, 0 },
	TEST_c_c(cacosh, -0x1p-23L, -0x1.000002p0L, 8.813736713132400470205730751186547909968e-1L, -1.570796411088588616723617301287646121905L, 0 },
	TEST_c_c(cacosh, 0x1.000002p0L, 0x1p-23L, 5.364668491573609633134147164031476452679e-4L, 2.222118384408546368406374049167636760903e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -0x1.000002p0L, 0x1p-23L, 5.364668491573609633134147164031476452679e-4L, 3.141370441751352383825802745874586120521L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 0x1.000002p0L, -0x1p-23L, 5.364668491573609633134147164031476452679e-4L, -2.222118384408546368406374049167636760903e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -0x1.000002p0L, -0x1p-23L, 5.364668491573609633134147164031476452679e-4L, -3.141370441751352383825802745874586120521L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 0x1.fp-129L, 0x1.000002p0L, 8.813736713132375348727889167749389235161e-1L, 1.570796326794896619231321691639751442097L, 0 },
	TEST_c_c(cacosh, 0x1.fp-129L, -0x1.000002p0L, 8.813736713132375348727889167749389235161e-1L, -1.570796326794896619231321691639751442097L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, 0x1.000002p0L, 8.813736713132375348727889167749389235161e-1L, 1.570796326794896619231321691639751442101L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, -0x1.000002p0L, 8.813736713132375348727889167749389235161e-1L, -1.570796326794896619231321691639751442101L, 0 },
	TEST_c_c(cacosh, 0x1.000002p0L, 0x1.fp-129L, 4.882812451493617206486388134172712975070e-4L, 5.830451806317544230969669308596361881467e-36L, 0 },
	TEST_c_c(cacosh, -0x1.000002p0L, 0x1.fp-129L, 4.882812451493617206486388134172712975070e-4L, 3.141592653589793238462643383279502878367L, 0 },
	TEST_c_c(cacosh, 0x1.000002p0L, -0x1.fp-129L, 4.882812451493617206486388134172712975070e-4L, -5.830451806317544230969669308596361881467e-36L, 0 },
	TEST_c_c(cacosh, -0x1.000002p0L, -0x1.fp-129L, 4.882812451493617206486388134172712975070e-4L, -3.141592653589793238462643383279502878367L, 0 },
	TEST_c_c(cacosh, 0.0L, 0x1.000002p0L, 8.813736713132375348727889167749389235161e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.0L, -0x1.000002p0L, 8.813736713132375348727889167749389235161e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, 0x1.000002p0L, 8.813736713132375348727889167749389235161e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, -0x1.000002p0L, 8.813736713132375348727889167749389235161e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.000002p0L, 0.0L, 4.882812451493617206486388134172712975070e-4L, 0.0L, 0 },
	TEST_c_c(cacosh, -0x1.000002p0L, 0.0L, 4.882812451493617206486388134172712975070e-4L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0x1.000002p0L, -0.0L, 4.882812451493617206486388134172712975070e-4L, -0.0L, 0 },
	TEST_c_c(cacosh, -0x1.000002p0L, -0.0L, 4.882812451493617206486388134172712975070e-4L, -3.141592653589793238462643383279502884197L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(cacosh, 0x1p-52L, 0x1.0000000000001p0L, 8.813735870195431822418551933572982483664e-1L, 1.570796326794896462222075823262262934288L, 0 },
	TEST_c_c(cacosh, 0x1p-52L, -0x1.0000000000001p0L, 8.813735870195431822418551933572982483664e-1L, -1.570796326794896462222075823262262934288L, 0 },
	TEST_c_c(cacosh, -0x1p-52L, 0x1.0000000000001p0L, 8.813735870195431822418551933572982483664e-1L, 1.570796326794896776240567560017239949909L, 0 },
	TEST_c_c(cacosh, -0x1p-52L, -0x1.0000000000001p0L, 8.813735870195431822418551933572982483664e-1L, -1.570796326794896776240567560017239949909L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000001p0L, 0x1p-52L, 2.315303644582684770975188768022139415020e-8L, 9.590301705980041385828904092662391018164e-9L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000001p0L, 0x1p-52L, 2.315303644582684770975188768022139415020e-8L, 3.141592643999491532482601997450598791535L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000001p0L, -0x1p-52L, 2.315303644582684770975188768022139415020e-8L, -9.590301705980041385828904092662391018164e-9L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000001p0L, -0x1p-52L, 2.315303644582684770975188768022139415020e-8L, -3.141592643999491532482601997450598791535L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1025L, 0x1.0000000000001p0L, 8.813735870195431822418551933572895326024e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1025L, -0x1.0000000000001p0L, 8.813735870195431822418551933572895326024e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, 0x1.0000000000001p0L, 8.813735870195431822418551933572895326024e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, -0x1.0000000000001p0L, 8.813735870195431822418551933572895326024e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000001p0L, 0x1.fp-1025L, 2.107342425544701550354780375182800088393e-8L, 2.557178503953494342609835913586108008322e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(cacosh, -0x1.0000000000001p0L, 0x1.fp-1025L, 2.107342425544701550354780375182800088393e-8L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000001p0L, -0x1.fp-1025L, 2.107342425544701550354780375182800088393e-8L, -2.557178503953494342609835913586108008322e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(cacosh, -0x1.0000000000001p0L, -0x1.fp-1025L, 2.107342425544701550354780375182800088393e-8L, -3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0.0L, 0x1.0000000000001p0L, 8.813735870195431822418551933572895326024e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.0L, -0x1.0000000000001p0L, 8.813735870195431822418551933572895326024e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, 0x1.0000000000001p0L, 8.813735870195431822418551933572895326024e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, -0x1.0000000000001p0L, 8.813735870195431822418551933572895326024e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000001p0L, 0.0L, 2.107342425544701550354780375182800088393e-8L, 0.0L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000001p0L, 0.0L, 2.107342425544701550354780375182800088393e-8L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000001p0L, -0.0L, 2.107342425544701550354780375182800088393e-8L, -0.0L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000001p0L, -0.0L, 2.107342425544701550354780375182800088393e-8L, -3.141592653589793238462643383279502884197L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(cacosh, 0x1p-63L, 0x1.0000000000000002p0L, 8.813735870195430253092739958139610131001e-1L, 1.570796326794896619154657020805582738031L, 0 },
	TEST_c_c(cacosh, 0x1p-63L, -0x1.0000000000000002p0L, 8.813735870195430253092739958139610131001e-1L, -1.570796326794896619154657020805582738031L, 0 },
	TEST_c_c(cacosh, -0x1p-63L, 0x1.0000000000000002p0L, 8.813735870195430253092739958139610131001e-1L, 1.570796326794896619307986362473920146166L, 0 },
	TEST_c_c(cacosh, -0x1p-63L, -0x1.0000000000000002p0L, 8.813735870195430253092739958139610131001e-1L, -1.570796326794896619307986362473920146166L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000002p0L, 0x1p-63L, 5.116146586219826555037807251857670783420e-10L, 2.119177303101063432592523199680782317447e-10L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000002p0L, 0x1p-63L, 5.116146586219826555037807251857670783420e-10L, 3.141592653377875508152537040020250564229L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000002p0L, -0x1p-63L, 5.116146586219826555037807251857670783420e-10L, -2.119177303101063432592523199680782317447e-10L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000002p0L, -0x1p-63L, 5.116146586219826555037807251857670783420e-10L, -3.141592653377875508152537040020250564229L, 0 },
# if LDBL_MIN_EXP <= -16381
	TEST_c_c(cacosh, 0x1.fp-16385L, 0x1.0000000000000002p0L, 8.813735870195430253092739958139610130980e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-16385L, -0x1.0000000000000002p0L, 8.813735870195430253092739958139610130980e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, 0x1.0000000000000002p0L, 8.813735870195430253092739958139610130980e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, -0x1.0000000000000002p0L, 8.813735870195430253092739958139610130980e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000002p0L, 0x1.fp-16385L, 4.656612873077392578082927418388212703712e-10L, 1.748608650034385653922359120438227480943e-4923L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000002p0L, 0x1.fp-16385L, 4.656612873077392578082927418388212703712e-10L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000002p0L, -0x1.fp-16385L, 4.656612873077392578082927418388212703712e-10L, -1.748608650034385653922359120438227480943e-4923L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000002p0L, -0x1.fp-16385L, 4.656612873077392578082927418388212703712e-10L, -3.141592653589793238462643383279502884197L, 0 },
# endif
	TEST_c_c(cacosh, 0.0L, 0x1.0000000000000002p0L, 8.813735870195430253092739958139610130980e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.0L, -0x1.0000000000000002p0L, 8.813735870195430253092739958139610130980e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, 0x1.0000000000000002p0L, 8.813735870195430253092739958139610130980e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, -0x1.0000000000000002p0L, 8.813735870195430253092739958139610130980e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000002p0L, 0.0L, 4.656612873077392578082927418388212703712e-10L, 0.0L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000002p0L, 0.0L, 4.656612873077392578082927418388212703712e-10L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000002p0L, -0.0L, 4.656612873077392578082927418388212703712e-10L, -0.0L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000002p0L, -0.0L, 4.656612873077392578082927418388212703712e-10L, -3.141592653589793238462643383279502884197L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(cacosh, 0x1p-106L, 0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, 1.570796326794896619231321691639742726335L, 0 },
	TEST_c_c(cacosh, 0x1p-106L, -0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, -1.570796326794896619231321691639742726335L, 0 },
	TEST_c_c(cacosh, -0x1p-106L, 0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, 1.570796326794896619231321691639760157863L, 0 },
	TEST_c_c(cacosh, -0x1p-106L, -0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, -1.570796326794896619231321691639760157863L, 0 },
	TEST_c_c(cacosh, 0x1.000000000000000000000000008p0L, 0x1p-106L, 2.285028863093221674154232933662774454211e-16L, 5.394221422390606848017034778914096659726e-17L, 0 },
	TEST_c_c(cacosh, -0x1.000000000000000000000000008p0L, 0x1p-106L, 2.285028863093221674154232933662774454211e-16L, 3.141592653589793184520429159373434404027L, 0 },
	TEST_c_c(cacosh, 0x1.000000000000000000000000008p0L, -0x1p-106L, 2.285028863093221674154232933662774454211e-16L, -5.394221422390606848017034778914096659726e-17L, 0 },
	TEST_c_c(cacosh, -0x1.000000000000000000000000008p0L, -0x1p-106L, 2.285028863093221674154232933662774454211e-16L, -3.141592653589793184520429159373434404027L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1025L, 0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1025L, -0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, 0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, -0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.000000000000000000000000008p0L, 0x1.fp-1025L, 2.220446049250313080847263336181636063482e-16L, 2.426922623448365473354662093431821897807e-293L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM, 0 },
	TEST_c_c(cacosh, -0x1.000000000000000000000000008p0L, 0x1.fp-1025L, 2.220446049250313080847263336181636063482e-16L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0x1.000000000000000000000000008p0L, -0x1.fp-1025L, 2.220446049250313080847263336181636063482e-16L, -2.426922623448365473354662093431821897807e-293L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM, 0 },
	TEST_c_c(cacosh, -0x1.000000000000000000000000008p0L, -0x1.fp-1025L, 2.220446049250313080847263336181636063482e-16L, -3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0.0L, 0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.0L, -0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, 0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, -0x1.000000000000000000000000008p0L, 8.813735870195430252326093249798097405561e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.000000000000000000000000008p0L, 0.0L, 2.220446049250313080847263336181636063482e-16L, 0.0L, 0 },
	TEST_c_c(cacosh, -0x1.000000000000000000000000008p0L, 0.0L, 2.220446049250313080847263336181636063482e-16L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0x1.000000000000000000000000008p0L, -0.0L, 2.220446049250313080847263336181636063482e-16L, -0.0L, 0 },
	TEST_c_c(cacosh, -0x1.000000000000000000000000008p0L, -0.0L, 2.220446049250313080847263336181636063482e-16L, -3.141592653589793238462643383279502884197L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(cacosh, 0x1p-113L, 0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, 1.570796326794896619231321691639751374007L, 0 },
	TEST_c_c(cacosh, 0x1p-113L, -0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, -1.570796326794896619231321691639751374007L, 0 },
	TEST_c_c(cacosh, -0x1p-113L, 0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, 1.570796326794896619231321691639751510190L, 0 },
	TEST_c_c(cacosh, -0x1p-113L, -0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, -1.570796326794896619231321691639751510190L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000000000000000001p0L, 0x1p-113L, 2.019699255375255198156433710951064632386e-17L, 4.767863183742677481693563511435642755521e-18L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000000000000000001p0L, 0x1p-113L, 2.019699255375255198156433710951064632386e-17L, 3.141592653589793233694780199536825402504L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000000000000000001p0L, -0x1p-113L, 2.019699255375255198156433710951064632386e-17L, -4.767863183742677481693563511435642755521e-18L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000000000000000001p0L, -0x1p-113L, 2.019699255375255198156433710951064632386e-17L, -3.141592653589793233694780199536825402504L, 0 },
	TEST_c_c(cacosh, 0x1.fp-16385L, 0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-16385L, -0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, 0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, -0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000000000000000001p0L, 0x1.fp-16385L, 1.962615573354718824241727964954454332780e-17L, 4.148847925325683229178506116378864361396e-4916L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000000000000000001p0L, 0x1.fp-16385L, 1.962615573354718824241727964954454332780e-17L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000000000000000001p0L, -0x1.fp-16385L, 1.962615573354718824241727964954454332780e-17L, -4.148847925325683229178506116378864361396e-4916L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000000000000000001p0L, -0x1.fp-16385L, 1.962615573354718824241727964954454332780e-17L, -3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0.0L, 0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.0L, -0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, 0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, -0x1.0000000000000000000000000001p0L, 8.813735870195430252326093249797924452120e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000000000000000001p0L, 0.0L, 1.962615573354718824241727964954454332780e-17L, 0.0L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000000000000000001p0L, 0.0L, 1.962615573354718824241727964954454332780e-17L, 3.141592653589793238462643383279502884197L, 0 },
	TEST_c_c(cacosh, 0x1.0000000000000000000000000001p0L, -0.0L, 1.962615573354718824241727964954454332780e-17L, -0.0L, 0 },
	TEST_c_c(cacosh, -0x1.0000000000000000000000000001p0L, -0.0L, 1.962615573354718824241727964954454332780e-17L, -3.141592653589793238462643383279502884197L, 0 },
#endif
	TEST_c_c(cacosh, 0x1p-23L, 0x0.ffffffp0L, 8.813735448726963984495965873956465777250e-1L, 1.570796242501197085295336701632142060969L, 0 },
	TEST_c_c(cacosh, 0x1p-23L, -0x0.ffffffp0L, 8.813735448726963984495965873956465777250e-1L, -1.570796242501197085295336701632142060969L, 0 },
	TEST_c_c(cacosh, -0x1p-23L, 0x0.ffffffp0L, 8.813735448726963984495965873956465777250e-1L, 1.570796411088596153167306681647360823228L, 0 },
	TEST_c_c(cacosh, -0x1p-23L, -0x0.ffffffp0L, 8.813735448726963984495965873956465777250e-1L, -1.570796411088596153167306681647360823228L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffp0L, 0x1p-23L, 2.714321200917194650737217746780928423385e-4L, 4.391863861910487109445187743978204002614e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -0x0.ffffffp0L, 0x1p-23L, 2.714321200917194650737217746780928423385e-4L, 3.141153467203602189751698864505105063797L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 0x0.ffffffp0L, -0x1p-23L, 2.714321200917194650737217746780928423385e-4L, -4.391863861910487109445187743978204002614e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -0x0.ffffffp0L, -0x1p-23L, 2.714321200917194650737217746780928423385e-4L, -3.141153467203602189751698864505105063797L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 0x1.fp-129L, 0x0.ffffffp0L, 8.813735448726938863015878255140556727969e-1L, 1.570796326794896619231321691639751442097L, 0 },
	TEST_c_c(cacosh, 0x1.fp-129L, -0x0.ffffffp0L, 8.813735448726938863015878255140556727969e-1L, -1.570796326794896619231321691639751442097L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, 0x0.ffffffp0L, 8.813735448726938863015878255140556727969e-1L, 1.570796326794896619231321691639751442101L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, -0x0.ffffffp0L, 8.813735448726938863015878255140556727969e-1L, -1.570796326794896619231321691639751442101L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffp0L, 0x1.fp-129L, 8.245504387859737323891843603996428534945e-36L, 3.452669847162035876032494826387364972849e-4L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x0.ffffffp0L, 0x1.fp-129L, 8.245504387859737323891843603996428534945e-36L, 3.141247386605077034875040133796864147700L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x0.ffffffp0L, -0x1.fp-129L, 8.245504387859737323891843603996428534945e-36L, -3.452669847162035876032494826387364972849e-4L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x0.ffffffp0L, -0x1.fp-129L, 8.245504387859737323891843603996428534945e-36L, -3.141247386605077034875040133796864147700L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.0L, 0x0.ffffffp0L, 8.813735448726938863015878255140556727969e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.0L, -0x0.ffffffp0L, 8.813735448726938863015878255140556727969e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, 0x0.ffffffp0L, 8.813735448726938863015878255140556727969e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, -0x0.ffffffp0L, 8.813735448726938863015878255140556727969e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffp0L, 0.0L, 0.0L, 3.452669847162035876032494826387364972849e-4L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffp0L, 0.0L, 0.0L, 3.141247386605077034875040133796864147700L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffp0L, -0.0L, 0.0L, -3.452669847162035876032494826387364972849e-4L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffp0L, -0.0L, 0.0L, -3.141247386605077034875040133796864147700L, 0 },
	TEST_c_c(cacosh, 0x1p-23L, 0.5L, 4.812118250596059896127318386463676808798e-1L, 1.570796220170866625230343643673321150378L, 0 },
	TEST_c_c(cacosh, 0x1p-23L, -0.5L, 4.812118250596059896127318386463676808798e-1L, -1.570796220170866625230343643673321150378L, 0 },
	TEST_c_c(cacosh, -0x1p-23L, 0.5L, 4.812118250596059896127318386463676808798e-1L, 1.570796433418926613232299739606181733819L, 0 },
	TEST_c_c(cacosh, -0x1p-23L, -0.5L, 4.812118250596059896127318386463676808798e-1L, -1.570796433418926613232299739606181733819L, 0 },
	TEST_c_c(cacosh, 0.5L, 0x1p-23L, 1.376510308240943236356088341381173571841e-7L, 1.047197551196603215914744618665204532273L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -0.5L, 0x1p-23L, 1.376510308240943236356088341381173571841e-7L, 2.094395102393190022547898764614298351924L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 0.5L, -0x1p-23L, 1.376510308240943236356088341381173571841e-7L, -1.047197551196603215914744618665204532273L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -0.5L, -0x1p-23L, 1.376510308240943236356088341381173571841e-7L, -2.094395102393190022547898764614298351924L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 0x1.fp-129L, 0.5L, 4.812118250596034474977589134243684231352e-1L, 1.570796326794896619231321691639751442096L, 0 },
	TEST_c_c(cacosh, 0x1.fp-129L, -0.5L, 4.812118250596034474977589134243684231352e-1L, -1.570796326794896619231321691639751442096L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, 0.5L, 4.812118250596034474977589134243684231352e-1L, 1.570796326794896619231321691639751442101L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, -0.5L, 4.812118250596034474977589134243684231352e-1L, -1.570796326794896619231321691639751442101L, 0 },
	TEST_c_c(cacosh, 0.5L, 0x1.fp-129L, 3.287317402534702257036015056278368274737e-39L, 1.047197551196597746154214461093167628066L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, 0x1.fp-129L, 3.287317402534702257036015056278368274737e-39L, 2.094395102393195492308428922186335256131L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.5L, -0x1.fp-129L, 3.287317402534702257036015056278368274737e-39L, -1.047197551196597746154214461093167628066L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, -0x1.fp-129L, 3.287317402534702257036015056278368274737e-39L, -2.094395102393195492308428922186335256131L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-23L, 0x1p-23L, 1.192092895507818146886315028596704749235e-7L, 1.570796207585607068450636380271254316214L, 0 },
	TEST_c_c(cacosh, 0x1p-23L, -0x1p-23L, 1.192092895507818146886315028596704749235e-7L, -1.570796207585607068450636380271254316214L, 0 },
	TEST_c_c(cacosh, -0x1p-23L, 0x1p-23L, 1.192092895507818146886315028596704749235e-7L, 1.570796446004186170012007003008248567984L, 0 },
	TEST_c_c(cacosh, -0x1p-23L, -0x1p-23L, 1.192092895507818146886315028596704749235e-7L, -1.570796446004186170012007003008248567984L, 0 },
	TEST_c_c(cacosh, 0x1.fp-129L, 0x1p-23L, 1.192092895507809676556842485683592032154e-7L, 1.570796326794896619231321691639751442096L, 0 },
	TEST_c_c(cacosh, 0x1.fp-129L, -0x1p-23L, 1.192092895507809676556842485683592032154e-7L, -1.570796326794896619231321691639751442096L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, 0x1p-23L, 1.192092895507809676556842485683592032154e-7L, 1.570796326794896619231321691639751442101L, 0 },
	TEST_c_c(cacosh, -0x1.fp-129L, -0x1p-23L, 1.192092895507809676556842485683592032154e-7L, -1.570796326794896619231321691639751442101L, 0 },
	TEST_c_c(cacosh, 0x1p-23L, 0x1.fp-129L, 2.846900380897747786805634596726756660388e-39L, 1.570796207585607068449789347324000006847L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-23L, 0x1.fp-129L, 2.846900380897747786805634596726756660388e-39L, 1.570796446004186170012854035955502877351L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-23L, -0x1.fp-129L, 2.846900380897747786805634596726756660388e-39L, -1.570796207585607068449789347324000006847L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-23L, -0x1.fp-129L, 2.846900380897747786805634596726756660388e-39L, -1.570796446004186170012854035955502877351L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.0L, 0x1p-23L, 1.192092895507809676556842485683592032154e-7L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.0L, -0x1p-23L, 1.192092895507809676556842485683592032154e-7L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, 0x1p-23L, 1.192092895507809676556842485683592032154e-7L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, -0x1p-23L, 1.192092895507809676556842485683592032154e-7L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1p-23L, 0.0L, 0.0L, 1.570796207585607068449789347324000006847L, 0 },
	TEST_c_c(cacosh, -0x1p-23L, 0.0L, 0.0L, 1.570796446004186170012854035955502877351L, 0 },
	TEST_c_c(cacosh, 0x1p-23L, -0.0L, 0.0L, -1.570796207585607068449789347324000006847L, 0 },
	TEST_c_c(cacosh, -0x1p-23L, -0.0L, 0.0L, -1.570796446004186170012854035955502877351L, 0 },
	TEST_c_c(cacosh, 0x1.fp-129L, 0x1.fp-129L, 2.846900380897727558361783801085126250967e-39L, 1.570796326794896619231321691639751442096L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1.fp-129L, -0x1.fp-129L, 2.846900380897727558361783801085126250967e-39L, -1.570796326794896619231321691639751442096L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1.fp-129L, 0x1.fp-129L, 2.846900380897727558361783801085126250967e-39L, 1.570796326794896619231321691639751442101L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1.fp-129L, -0x1.fp-129L, 2.846900380897727558361783801085126250967e-39L, -1.570796326794896619231321691639751442101L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(cacosh, 0x1p-52L, 0x0.fffffffffffff8p0L, 8.813735870195429467279863907910458761820e-1L, 1.570796326794896462222075823262236786996L, 0 },
	TEST_c_c(cacosh, 0x1p-52L, -0x0.fffffffffffff8p0L, 8.813735870195429467279863907910458761820e-1L, -1.570796326794896462222075823262236786996L, 0 },
	TEST_c_c(cacosh, -0x1p-52L, 0x0.fffffffffffff8p0L, 8.813735870195429467279863907910458761820e-1L, 1.570796326794896776240567560017266097201L, 0 },
	TEST_c_c(cacosh, -0x1p-52L, -0x0.fffffffffffff8p0L, 8.813735870195429467279863907910458761820e-1L, -1.570796326794896776240567560017266097201L, 0 },
	TEST_c_c(cacosh, 0x0.fffffffffffff8p0L, 0x1p-52L, 1.171456840272878582596796205397918831268e-8L, 1.895456983915074112227925127005564372844e-8L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x0.fffffffffffff8p0L, 0x1p-52L, 1.171456840272878582596796205397918831268e-8L, 3.141592634635223399311902261000251614142L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x0.fffffffffffff8p0L, -0x1p-52L, 1.171456840272878582596796205397918831268e-8L, -1.895456983915074112227925127005564372844e-8L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x0.fffffffffffff8p0L, -0x1p-52L, 1.171456840272878582596796205397918831268e-8L, -3.141592634635223399311902261000251614142L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1.fp-1025L, 0x0.fffffffffffff8p0L, 8.813735870195429467279863907910371604180e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1025L, -0x0.fffffffffffff8p0L, 8.813735870195429467279863907910371604180e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, 0x0.fffffffffffff8p0L, 8.813735870195429467279863907910371604180e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, -0x0.fffffffffffff8p0L, 8.813735870195429467279863907910371604180e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x0.fffffffffffff8p0L, 0x1.fp-1025L, 3.616396521699973256461764099945789620604e-301L, 1.490116119384765638786343542550461592240e-8L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x0.fffffffffffff8p0L, 0x1.fp-1025L, 3.616396521699973256461764099945789620604e-301L, 3.141592638688632044614986995416067458693L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x0.fffffffffffff8p0L, -0x1.fp-1025L, 3.616396521699973256461764099945789620604e-301L, -1.490116119384765638786343542550461592240e-8L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x0.fffffffffffff8p0L, -0x1.fp-1025L, 3.616396521699973256461764099945789620604e-301L, -3.141592638688632044614986995416067458693L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.0L, 0x0.fffffffffffff8p0L, 8.813735870195429467279863907910371604180e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.0L, -0x0.fffffffffffff8p0L, 8.813735870195429467279863907910371604180e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, 0x0.fffffffffffff8p0L, 8.813735870195429467279863907910371604180e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, -0x0.fffffffffffff8p0L, 8.813735870195429467279863907910371604180e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x0.fffffffffffff8p0L, 0.0L, 0.0L, 1.490116119384765638786343542550461592240e-8L, 0 },
	TEST_c_c(cacosh, -0x0.fffffffffffff8p0L, 0.0L, 0.0L, 3.141592638688632044614986995416067458693L, 0 },
	TEST_c_c(cacosh, 0x0.fffffffffffff8p0L, -0.0L, 0.0L, -1.490116119384765638786343542550461592240e-8L, 0 },
	TEST_c_c(cacosh, -0x0.fffffffffffff8p0L, -0.0L, 0.0L, -3.141592638688632044614986995416067458693L, 0 },
#endif
	TEST_c_c(cacosh, 0x1p-52L, 0.5L, 4.812118250596034474977589134243772428682e-1L, 1.570796326794896420628589431857911675917L, 0 },
	TEST_c_c(cacosh, 0x1p-52L, -0.5L, 4.812118250596034474977589134243772428682e-1L, -1.570796326794896420628589431857911675917L, 0 },
	TEST_c_c(cacosh, -0x1p-52L, 0.5L, 4.812118250596034474977589134243772428682e-1L, 1.570796326794896817834053951421591208280L, 0 },
	TEST_c_c(cacosh, -0x1p-52L, -0.5L, 4.812118250596034474977589134243772428682e-1L, -1.570796326794896817834053951421591208280L, 0 },
	TEST_c_c(cacosh, 0.5L, 0x1p-52L, 2.563950248511418570403591756798643804971e-16L, 1.047197551196597746154214461093186605110L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, 0x1p-52L, 2.563950248511418570403591756798643804971e-16L, 2.094395102393195492308428922186316279087L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.5L, -0x1p-52L, 2.563950248511418570403591756798643804971e-16L, -1.047197551196597746154214461093186605110L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, -0x1p-52L, 2.563950248511418570403591756798643804971e-16L, -2.094395102393195492308428922186316279087L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(cacosh, 0x1.fp-1025L, 0.5L, 4.812118250596034474977589134243684231352e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1025L, -0.5L, 4.812118250596034474977589134243684231352e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, 0.5L, 4.812118250596034474977589134243684231352e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, -0.5L, 4.812118250596034474977589134243684231352e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.5L, 0x1.fp-1025L, 6.222508863508420569166420770843207333493e-309L, 1.047197551196597746154214461093167628066L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, 0x1.fp-1025L, 6.222508863508420569166420770843207333493e-309L, 2.094395102393195492308428922186335256131L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.5L, -0x1.fp-1025L, 6.222508863508420569166420770843207333493e-309L, -1.047197551196597746154214461093167628066L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, -0x1.fp-1025L, 6.222508863508420569166420770843207333493e-309L, -2.094395102393195492308428922186335256131L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(cacosh, 0x1p-52L, 0x1p-52L, 2.220446049250313080847263336181677117148e-16L, 1.570796326794896397186716766608443357372L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 0x1p-52L, -0x1p-52L, 2.220446049250313080847263336181677117148e-16L, -1.570796326794896397186716766608443357372L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -0x1p-52L, 0x1p-52L, 2.220446049250313080847263336181677117148e-16L, 1.570796326794896841275926616671059526825L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -0x1p-52L, -0x1p-52L, 2.220446049250313080847263336181677117148e-16L, -1.570796326794896841275926616671059526825L, 0|XFAIL_NATIVE_LINUXF },
#ifndef TEST_FLOAT
	TEST_c_c(cacosh, 0x1.fp-1025L, 0x1p-52L, 2.220446049250313080847263336181622378926e-16L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1025L, -0x1p-52L, 2.220446049250313080847263336181622378926e-16L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, 0x1p-52L, 2.220446049250313080847263336181622378926e-16L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, -0x1p-52L, 2.220446049250313080847263336181622378926e-16L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1p-52L, 0x1.fp-1025L, 5.388850751072128349671657362289548938458e-309L, 1.570796326794896397186716766608443357372L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-52L, 0x1.fp-1025L, 5.388850751072128349671657362289548938458e-309L, 1.570796326794896841275926616671059526825L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-52L, -0x1.fp-1025L, 5.388850751072128349671657362289548938458e-309L, -1.570796326794896397186716766608443357372L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-52L, -0x1.fp-1025L, 5.388850751072128349671657362289548938458e-309L, -1.570796326794896841275926616671059526825L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(cacosh, 0.0L, 0x1p-52L, 2.220446049250313080847263336181622378926e-16L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 0.0L, -0x1p-52L, 2.220446049250313080847263336181622378926e-16L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -0.0L, 0x1p-52L, 2.220446049250313080847263336181622378926e-16L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, -0.0L, -0x1p-52L, 2.220446049250313080847263336181622378926e-16L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cacosh, 0x1p-52L, 0.0L, 0.0L, 1.570796326794896397186716766608443357372L, 0 },
	TEST_c_c(cacosh, -0x1p-52L, 0.0L, 0.0L, 1.570796326794896841275926616671059526825L, 0 },
	TEST_c_c(cacosh, 0x1p-52L, -0.0L, 0.0L, -1.570796326794896397186716766608443357372L, 0 },
	TEST_c_c(cacosh, -0x1p-52L, -0.0L, 0.0L, -1.570796326794896841275926616671059526825L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(cacosh, 0x1.fp-1025L, 0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1.fp-1025L, -0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1.fp-1025L, 0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1.fp-1025L, -0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(cacosh, 0x1p-63L, 0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569937e-1L, 1.570796326794896619154657020805582738025L, 0 },
	TEST_c_c(cacosh, 0x1p-63L, -0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569937e-1L, -1.570796326794896619154657020805582738025L, 0 },
	TEST_c_c(cacosh, -0x1p-63L, 0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569937e-1L, 1.570796326794896619307986362473920146173L, 0 },
	TEST_c_c(cacosh, -0x1p-63L, -0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569937e-1L, -1.570796326794896619307986362473920146173L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffp0L, 0x1p-63L, 2.588578361325995866221775673638805081337e-10L, 4.188407771167967636741951941902992986043e-10L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffp0L, 0x1p-63L, 2.588578361325995866221775673638805081337e-10L, 3.141592653170952461345846619605307690007L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffp0L, -0x1p-63L, 2.588578361325995866221775673638805081337e-10L, -4.188407771167967636741951941902992986043e-10L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffp0L, -0x1p-63L, 2.588578361325995866221775673638805081337e-10L, -3.141592653170952461345846619605307690007L, 0|XFAIL_NATIVE_LINUXL },
# if LDBL_MIN_EXP <= -16381
	TEST_c_c(cacosh, 0x1.fp-16385L, 0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569917e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-16385L, -0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569917e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, 0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569917e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, -0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569917e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffp0L, 0x1.fp-16385L, 2.472906068161537187835415298076415423459e-4923L, 3.292722539913596233371825532007990724506e-10L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffp0L, 0x1.fp-16385L, 2.472906068161537187835415298076415423459e-4923L, 3.141592653260520984471283759942320330996L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffp0L, -0x1.fp-16385L, 2.472906068161537187835415298076415423459e-4923L, -3.292722539913596233371825532007990724506e-10L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffp0L, -0x1.fp-16385L, 2.472906068161537187835415298076415423459e-4923L, -3.141592653260520984471283759942320330996L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
# endif
	TEST_c_c(cacosh, 0.0L, 0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569917e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.0L, -0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569917e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, 0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569917e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, -0x0.ffffffffffffffffp0L, 8.813735870195430251942769895627079569917e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffp0L, 0.0L, 0.0L, 3.292722539913596233371825532007990724506e-10L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffp0L, 0.0L, 0.0L, 3.141592653260520984471283759942320330996L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffp0L, -0.0L, 0.0L, -3.292722539913596233371825532007990724506e-10L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffp0L, -0.0L, 0.0L, -3.141592653260520984471283759942320330996L, 0 },
#endif
	TEST_c_c(cacosh, 0x1p-63L, 0.5L, 4.812118250596034474977589134243684231373e-1L, 1.570796326794896619134347701278529840650L, 0 },
	TEST_c_c(cacosh, 0x1p-63L, -0.5L, 4.812118250596034474977589134243684231373e-1L, -1.570796326794896619134347701278529840650L, 0 },
	TEST_c_c(cacosh, -0x1p-63L, 0.5L, 4.812118250596034474977589134243684231373e-1L, 1.570796326794896619328295682000973043547L, 0 },
	TEST_c_c(cacosh, -0x1p-63L, -0.5L, 4.812118250596034474977589134243684231373e-1L, -1.570796326794896619328295682000973043547L, 0 },
	TEST_c_c(cacosh, 0.5L, 0x1p-63L, 1.251928832280966098829878787499365228659e-19L, 1.047197551196597746154214461093167628070L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, 0x1p-63L, 1.251928832280966098829878787499365228659e-19L, 2.094395102393195492308428922186335256127L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.5L, -0x1p-63L, 1.251928832280966098829878787499365228659e-19L, -1.047197551196597746154214461093167628070L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, -0x1p-63L, 1.251928832280966098829878787499365228659e-19L, -2.094395102393195492308428922186335256127L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacosh, 0x1.fp-16385L, 0.5L, 4.812118250596034474977589134243684231352e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-16385L, -0.5L, 4.812118250596034474977589134243684231352e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, 0.5L, 4.812118250596034474977589134243684231352e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, -0.5L, 4.812118250596034474977589134243684231352e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.5L, 0x1.fp-16385L, 9.402257155670418112159334875199821342890e-4933L, 1.047197551196597746154214461093167628066L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0.5L, 0x1.fp-16385L, 9.402257155670418112159334875199821342890e-4933L, 2.094395102393195492308428922186335256131L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, 0.5L, -0x1.fp-16385L, 9.402257155670418112159334875199821342890e-4933L, -1.047197551196597746154214461093167628066L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0.5L, -0x1.fp-16385L, 9.402257155670418112159334875199821342890e-4933L, -2.094395102393195492308428922186335256131L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(cacosh, 0x1p-63L, 0x1p-63L, 1.084202172485504434007452800869941711430e-19L, 1.570796326794896619122901474391200998698L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-63L, -0x1p-63L, 1.084202172485504434007452800869941711430e-19L, -1.570796326794896619122901474391200998698L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-63L, 0x1p-63L, 1.084202172485504434007452800869941711430e-19L, 1.570796326794896619339741908888301885499L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-63L, -0x1p-63L, 1.084202172485504434007452800869941711430e-19L, -1.570796326794896619339741908888301885499L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacosh, 0x1.fp-16385L, 0x1p-63L, 1.084202172485504434007452800869941711424e-19L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-16385L, -0x1p-63L, 1.084202172485504434007452800869941711424e-19L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, 0x1p-63L, 1.084202172485504434007452800869941711424e-19L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, -0x1p-63L, 1.084202172485504434007452800869941711424e-19L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1p-63L, 0x1.fp-16385L, 8.142593549724601460479922838826119584465e-4933L, 1.570796326794896619122901474391200998698L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x1p-63L, 0x1.fp-16385L, 8.142593549724601460479922838826119584465e-4933L, 1.570796326794896619339741908888301885499L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, 0x1p-63L, -0x1.fp-16385L, 8.142593549724601460479922838826119584465e-4933L, -1.570796326794896619122901474391200998698L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x1p-63L, -0x1.fp-16385L, 8.142593549724601460479922838826119584465e-4933L, -1.570796326794896619339741908888301885499L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(cacosh, 0.0L, 0x1p-63L, 1.084202172485504434007452800869941711424e-19L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.0L, -0x1p-63L, 1.084202172485504434007452800869941711424e-19L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.0L, 0x1p-63L, 1.084202172485504434007452800869941711424e-19L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.0L, -0x1p-63L, 1.084202172485504434007452800869941711424e-19L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-63L, 0.0L, 0.0L, 1.570796326794896619122901474391200998698L, 0 },
	TEST_c_c(cacosh, -0x1p-63L, 0.0L, 0.0L, 1.570796326794896619339741908888301885499L, 0 },
	TEST_c_c(cacosh, 0x1p-63L, -0.0L, 0.0L, -1.570796326794896619122901474391200998698L, 0 },
	TEST_c_c(cacosh, -0x1p-63L, -0.0L, 0.0L, -1.570796326794896619339741908888301885499L, 0 },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacosh, 0x1.fp-16385L, 0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, 0x1.fp-16385L, -0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x1.fp-16385L, 0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x1.fp-16385L, -0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(cacosh, 0x1p-105L, 0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, 1.570796326794896619231321691639734010571L, 0 },
	TEST_c_c(cacosh, 0x1p-105L, -0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, -1.570796326794896619231321691639734010571L, 0 },
	TEST_c_c(cacosh, -0x1p-105L, 0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, 1.570796326794896619231321691639768873627L, 0 },
	TEST_c_c(cacosh, -0x1p-105L, -0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, -1.570796326794896619231321691639768873627L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffcp0L, 0x1p-105L, 1.234330349600789959989661887846005659983e-16L, 1.997188458999618357060632049675702684031e-16L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffcp0L, 0x1p-105L, 1.234330349600789959989661887846005659983e-16L, 3.141592653589793038743797483317667178134L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffcp0L, -0x1p-105L, 1.234330349600789959989661887846005659983e-16L, -1.997188458999618357060632049675702684031e-16L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffcp0L, -0x1p-105L, 1.234330349600789959989661887846005659983e-16L, -3.141592653589793038743797483317667178134L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1025L, 0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-1025L, -0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, 0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-1025L, -0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffcp0L, 0x1.fp-1025L, 3.432186888910770473784103112009399360940e-293L, 1.570092458683775059393382371963565104164e-16L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffcp0L, 0x1.fp-1025L, 3.432186888910770473784103112009399360940e-293L, 3.141592653589793081453397514901996944859L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffcp0L, -0x1.fp-1025L, 3.432186888910770473784103112009399360940e-293L, -1.570092458683775059393382371963565104164e-16L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffcp0L, -0x1.fp-1025L, 3.432186888910770473784103112009399360940e-293L, -3.141592653589793081453397514901996944859L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM, 0 },
	TEST_c_c(cacosh, 0.0L, 0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.0L, -0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, 0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, -0x0.ffffffffffffffffffffffffffcp0L, 8.813735870195430252326093249797835932642e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffcp0L, 0.0L, 0.0L, 1.570092458683775059393382371963565104164e-16L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffcp0L, 0.0L, 0.0L, 3.141592653589793081453397514901996944859L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffcp0L, -0.0L, 0.0L, -1.570092458683775059393382371963565104164e-16L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffcp0L, -0.0L, 0.0L, -3.141592653589793081453397514901996944859L, 0 },
#endif
	TEST_c_c(cacosh, 0x1p-105L, 0.5L, 4.812118250596034474977589134243684231352e-1L, 1.570796326794896619231321691639729392766L, 0 },
	TEST_c_c(cacosh, 0x1p-105L, -0.5L, 4.812118250596034474977589134243684231352e-1L, -1.570796326794896619231321691639729392766L, 0 },
	TEST_c_c(cacosh, -0x1p-105L, 0.5L, 4.812118250596034474977589134243684231352e-1L, 1.570796326794896619231321691639773491431L, 0 },
	TEST_c_c(cacosh, -0x1p-105L, -0.5L, 4.812118250596034474977589134243684231352e-1L, -1.570796326794896619231321691639773491431L, 0 },
	TEST_c_c(cacosh, 0.5L, 0x1p-105L, 2.846556599890768890940941645338546615370e-32L, 1.047197551196597746154214461093167628066L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, 0x1p-105L, 2.846556599890768890940941645338546615370e-32L, 2.094395102393195492308428922186335256131L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.5L, -0x1p-105L, 2.846556599890768890940941645338546615370e-32L, -1.047197551196597746154214461093167628066L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, -0x1p-105L, 2.846556599890768890940941645338546615370e-32L, -2.094395102393195492308428922186335256131L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-105L, 0x1p-105L, 2.465190328815661891911651766508706967729e-32L, 1.570796326794896619231321691639726790195L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-105L, -0x1p-105L, 2.465190328815661891911651766508706967729e-32L, -1.570796326794896619231321691639726790195L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-105L, 0x1p-105L, 2.465190328815661891911651766508706967729e-32L, 1.570796326794896619231321691639776094002L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-105L, -0x1p-105L, 2.465190328815661891911651766508706967729e-32L, -1.570796326794896619231321691639776094002L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(cacosh, 0x1.fp-1025L, 0x1p-105L, 2.465190328815661891911651766508706967729e-32L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1.fp-1025L, -0x1p-105L, 2.465190328815661891911651766508706967729e-32L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1.fp-1025L, 0x1p-105L, 2.465190328815661891911651766508706967729e-32L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1.fp-1025L, -0x1p-105L, 2.465190328815661891911651766508706967729e-32L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-105L, 0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, 1.570796326794896619231321691639726790195L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-105L, 0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, 1.570796326794896619231321691639776094002L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-105L, -0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, -1.570796326794896619231321691639726790195L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-105L, -0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, -1.570796326794896619231321691639776094002L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(cacosh, 0.0L, 0x1p-105L, 2.465190328815661891911651766508706967729e-32L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.0L, -0x1p-105L, 2.465190328815661891911651766508706967729e-32L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.0L, 0x1p-105L, 2.465190328815661891911651766508706967729e-32L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.0L, -0x1p-105L, 2.465190328815661891911651766508706967729e-32L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-105L, 0.0L, 0.0L, 1.570796326794896619231321691639726790195L, 0 },
	TEST_c_c(cacosh, -0x1p-105L, 0.0L, 0.0L, 1.570796326794896619231321691639776094002L, 0 },
	TEST_c_c(cacosh, 0x1p-105L, -0.0L, 0.0L, -1.570796326794896619231321691639726790195L, 0 },
	TEST_c_c(cacosh, -0x1p-105L, -0.0L, 0.0L, -1.570796326794896619231321691639776094002L, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(cacosh, 0x1p-112L, 0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, 1.570796326794896619231321691639751305915L, 0 },
	TEST_c_c(cacosh, 0x1p-112L, -0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, -1.570796326794896619231321691639751305915L, 0 },
	TEST_c_c(cacosh, -0x1p-112L, 0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, 1.570796326794896619231321691639751578282L, 0 },
	TEST_c_c(cacosh, -0x1p-112L, -0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, -1.570796326794896619231321691639751578282L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-112L, 1.091004200533850618345330909857103457781e-17L, 1.765281878332676474347460257221369818872e-17L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-112L, 1.091004200533850618345330909857103457781e-17L, 3.141592653589793220809824599952738140723L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-112L, 1.091004200533850618345330909857103457781e-17L, -1.765281878332676474347460257221369818872e-17L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-112L, 1.091004200533850618345330909857103457781e-17L, -3.141592653589793220809824599952738140723L, 0 },
	TEST_c_c(cacosh, 0x1.fp-16385L, 0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x1.fp-16385L, -0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, 0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0x1.fp-16385L, -0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1.fp-16385L, 5.867357004219059108790368268900490184505e-4916L, 1.387778780781445675529539585113525401762e-17L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1.fp-16385L, 5.867357004219059108790368268900490184505e-4916L, 3.141592653589793224584855575465046128902L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1.fp-16385L, 5.867357004219059108790368268900490184505e-4916L, -1.387778780781445675529539585113525401762e-17L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1.fp-16385L, 5.867357004219059108790368268900490184505e-4916L, -3.141592653589793224584855575465046128902L, 0 },
	TEST_c_c(cacosh, 0.0L, 0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0.0L, -0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, 0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, -0.0L, -0x0.ffffffffffffffffffffffffffff8p0L, 8.813735870195430252326093249797922409363e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, 0.0L, 1.387778780781445675529539585113525401762e-17L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, 0.0L, 3.141592653589793224584855575465046128902L, 0 },
	TEST_c_c(cacosh, 0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, 0.0L, -1.387778780781445675529539585113525401762e-17L, 0 },
	TEST_c_c(cacosh, -0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, 0.0L, -3.141592653589793224584855575465046128902L, 0 },
#endif
	TEST_c_c(cacosh, 0x1p-112L, 0.5L, 4.812118250596034474977589134243684231352e-1L, 1.570796326794896619231321691639751269838L, 0 },
	TEST_c_c(cacosh, 0x1p-112L, -0.5L, 4.812118250596034474977589134243684231352e-1L, -1.570796326794896619231321691639751269838L, 0 },
	TEST_c_c(cacosh, -0x1p-112L, 0.5L, 4.812118250596034474977589134243684231352e-1L, 1.570796326794896619231321691639751614359L, 0 },
	TEST_c_c(cacosh, -0x1p-112L, -0.5L, 4.812118250596034474977589134243684231352e-1L, -1.570796326794896619231321691639751614359L, 0 },
	TEST_c_c(cacosh, 0.5L, 0x1p-112L, 2.223872343664663196047610660420739543258e-34L, 1.047197551196597746154214461093167628066L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, 0x1p-112L, 2.223872343664663196047610660420739543258e-34L, 2.094395102393195492308428922186335256131L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.5L, -0x1p-112L, 2.223872343664663196047610660420739543258e-34L, -1.047197551196597746154214461093167628066L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.5L, -0x1p-112L, 2.223872343664663196047610660420739543258e-34L, -2.094395102393195492308428922186335256131L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-112L, 0x1p-112L, 1.925929944387235853055977942584927318538e-34L, 1.570796326794896619231321691639751249506L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-112L, -0x1p-112L, 1.925929944387235853055977942584927318538e-34L, -1.570796326794896619231321691639751249506L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-112L, 0x1p-112L, 1.925929944387235853055977942584927318538e-34L, 1.570796326794896619231321691639751634692L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0x1p-112L, -0x1p-112L, 1.925929944387235853055977942584927318538e-34L, -1.570796326794896619231321691639751634692L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(cacosh, 0x1.fp-16385L, 0x1p-112L, 1.925929944387235853055977942584927318538e-34L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, 0x1.fp-16385L, -0x1p-112L, 1.925929944387235853055977942584927318538e-34L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x1.fp-16385L, 0x1p-112L, 1.925929944387235853055977942584927318538e-34L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x1.fp-16385L, -0x1p-112L, 1.925929944387235853055977942584927318538e-34L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, 0x1p-112L, 0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, 1.570796326794896619231321691639751249506L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x1p-112L, 0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, 1.570796326794896619231321691639751634692L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, 0x1p-112L, -0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, -1.570796326794896619231321691639751249506L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cacosh, -0x1p-112L, -0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, -1.570796326794896619231321691639751634692L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(cacosh, 0.0L, 0x1p-112L, 1.925929944387235853055977942584927318538e-34L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0.0L, -0x1p-112L, 1.925929944387235853055977942584927318538e-34L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.0L, 0x1p-112L, 1.925929944387235853055977942584927318538e-34L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, -0.0L, -0x1p-112L, 1.925929944387235853055977942584927318538e-34L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cacosh, 0x1p-112L, 0.0L, 0.0L, 1.570796326794896619231321691639751249506L, 0 },
	TEST_c_c(cacosh, -0x1p-112L, 0.0L, 0.0L, 1.570796326794896619231321691639751634692L, 0 },
	TEST_c_c(cacosh, 0x1p-112L, -0.0L, 0.0L, -1.570796326794896619231321691639751249506L, 0 },
	TEST_c_c(cacosh, -0x1p-112L, -0.0L, 0.0L, -1.570796326794896619231321691639751634692L, 0 },

	TEST_c_c(cacosh, 0.75L, 1.25L, 1.13239363160530819522266333696834467L, 1.11752014915610270578240049553777969L, 0 },
	TEST_c_c(cacosh, -2, -3, 1.9833870299165354323470769028940395L, -2.1414491111159960199416055713254211L, 0 },
}


static void test_casin(void)
{
	struct test *testp = lookup("casin" },

	TEST_c_c(casin, 1.0, 2.0, 4.270785786211490631e-01, 1.528570890426635742e00, 0 },
	TEST_c_c(casin, 0.0, 2.0, 0.000000000000000000e00, 1.443635478615760803e00, 0 },
	TEST_c_c(casin, 1.0, 0.0, 1.570796325802803039e00, ZERO_P, 0 },
	TEST_c_c(casin, -2.0, 2.0, -7.542491331696510314e-01, 1.734324455261230468e00, 0 },
	TEST_c_c(casin, -1.0, -2.0, -4.270785823464393615e-01, -1.528570920228958129e00, 0 },
	TEST_c_c(casin, 1.0, -2.0, 4.270785823464393615e-01, -1.528570920228958129e00, 0 },

	TEST_c_c(casin, 0, 0, 0.0, 0.0, 0 },
	TEST_c_c(casin, ZERO_M, 0, ZERO_M, 0.0, 0 },
	TEST_c_c(casin, 0, ZERO_M, 0.0, ZERO_M, 0 },
	TEST_c_c(casin, ZERO_M, ZERO_M, ZERO_M, ZERO_M, 0 },

	TEST_c_c(casin, INF_P, INF_P, M_PI_4l, INF_P, 0 },
	TEST_c_c(casin, INF_P, INF_M, M_PI_4l, INF_M, 0 },
	TEST_c_c(casin, INF_M, INF_P, -M_PI_4l, INF_P, 0 },
	TEST_c_c(casin, INF_M, INF_M, -M_PI_4l, INF_M, 0 },

	TEST_c_c(casin, -10.0, INF_P, ZERO_M, INF_P, 0 },
	TEST_c_c(casin, -10.0, INF_M, ZERO_M, INF_M, 0 },
	TEST_c_c(casin, 0, INF_P, 0.0, INF_P, 0 },
	TEST_c_c(casin, 0, INF_M, 0.0, INF_M, 0 },
	TEST_c_c(casin, ZERO_M, INF_P, ZERO_M, INF_P, 0 },
	TEST_c_c(casin, ZERO_M, INF_M, ZERO_M, INF_M, 0 },
	TEST_c_c(casin, 0.1L, INF_P, 0.0, INF_P, 0 },
	TEST_c_c(casin, 0.1L, INF_M, 0.0, INF_M, 0 },

	TEST_c_c(casin, INF_M, 0, -M_PI_2l, INF_P, 0 },
	TEST_c_c(casin, INF_M, ZERO_M, -M_PI_2l, INF_M, 0 },
	TEST_c_c(casin, INF_M, 100, -M_PI_2l, INF_P, 0 },
	TEST_c_c(casin, INF_M, -100, -M_PI_2l, INF_M, 0 },

	TEST_c_c(casin, INF_P, 0, M_PI_2l, INF_P, 0 },
	TEST_c_c(casin, INF_P, ZERO_M, M_PI_2l, INF_M, 0 },
	TEST_c_c(casin, INF_P, 0.5, M_PI_2l, INF_P, 0 },
	TEST_c_c(casin, INF_P, -0.5, M_PI_2l, INF_M, 0 },

	TEST_c_c(casin, QNAN_P, INF_P, QNAN_P, INF_P, 0 },
	TEST_c_c(casin, QNAN_P, INF_M, QNAN_P, INF_M, 0 },

	TEST_c_c(casin, 0.0, QNAN_P, 0.0, QNAN_P, 0 },
	TEST_c_c(casin, ZERO_M, QNAN_P, ZERO_M, QNAN_P, 0 },

	TEST_c_c(casin, INF_P, QNAN_P, QNAN_P, INF_P, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(casin, INF_M, QNAN_P, QNAN_P, INF_P, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(casin, QNAN_P, 10.5, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(casin, QNAN_P, -10.5, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(casin, 0.75, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(casin, -0.75, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(casin, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(casin, ZERO_P, -1.5L, ZERO_P, -1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(casin, ZERO_M, -1.5L, ZERO_M, -1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(casin, ZERO_P, -1.0L, ZERO_P, -0.8813735870195430252326093249797923090282L, 0 },
	TEST_c_c(casin, ZERO_M, -1.0L, ZERO_M, -0.8813735870195430252326093249797923090282L, 0 },
	TEST_c_c(casin, ZERO_P, -0.5L, ZERO_P, -0.4812118250596034474977589134243684231352L, 0 },
	TEST_c_c(casin, ZERO_M, -0.5L, ZERO_M, -0.4812118250596034474977589134243684231352L, 0 },
	TEST_c_c(casin, ZERO_P, 0.5L, ZERO_P, 0.4812118250596034474977589134243684231352L, 0 },
	TEST_c_c(casin, ZERO_M, 0.5L, ZERO_M, 0.4812118250596034474977589134243684231352L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, ZERO_P, 1.0L, ZERO_P, 0.8813735870195430252326093249797923090282L, 0 },
	TEST_c_c(casin, ZERO_M, 1.0L, ZERO_M, 0.8813735870195430252326093249797923090282L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, ZERO_P, 1.5L, ZERO_P, 1.194763217287109304111930828519090523536L, 0 },
	TEST_c_c(casin, ZERO_M, 1.5L, ZERO_M, 1.194763217287109304111930828519090523536L, 0|XFAIL_NATIVE_LINUX },

	TEST_c_c(casin, -1.5L, ZERO_P, -M_PI_2l, 0.9624236501192068949955178268487368462704L, 0 },
	TEST_c_c(casin, -1.5L, ZERO_M, -M_PI_2l, -0.9624236501192068949955178268487368462704L, 0 },
	TEST_c_c(casin, -1.0L, ZERO_P, -M_PI_2l, ZERO_P, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, ZERO_M, -M_PI_2l, ZERO_M, 0 },
	TEST_c_c(casin, -0.5L, ZERO_P, -0.5235987755982988730771072305465838140329L, ZERO_P, 0|XFAIL_NATIVE_LINUXL|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -0.5L, ZERO_M, -0.5235987755982988730771072305465838140329L, ZERO_M, 0 },
	TEST_c_c(casin, 0.5L, ZERO_P, 0.5235987755982988730771072305465838140329L, ZERO_P, 0|XFAIL_NATIVE_LINUXL|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0.5L, ZERO_M, 0.5235987755982988730771072305465838140329L, ZERO_M, 0 },
	TEST_c_c(casin, 1.0L, ZERO_P, M_PI_2l, ZERO_P, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 1.0L, ZERO_M, M_PI_2l, ZERO_M, 0 },
	TEST_c_c(casin, 1.5L, ZERO_P, M_PI_2l, 0.9624236501192068949955178268487368462704L, 0 },
	TEST_c_c(casin, 1.5L, ZERO_M, M_PI_2l, -0.9624236501192068949955178268487368462704L, 0 },

	TEST_c_c(casin, 0x1p50L, 1.0L, 1.570796326794895731052901991514519103193L, 3.535050620855721078027883819436720218708e1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p50L, -1.0L, 1.570796326794895731052901991514519103193L, -3.535050620855721078027883819436720218708e1L, 0 },
	TEST_c_c(casin, -0x1p50L, 1.0L, -1.570796326794895731052901991514519103193L, 3.535050620855721078027883819436720218708e1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p50L, -1.0L, -1.570796326794895731052901991514519103193L, -3.535050620855721078027883819436720218708e1L, 0 },
	TEST_c_c(casin, 1.0L, 0x1p50L, 8.881784197001252323389053344720723756399e-16L, 3.535050620855721078027883819436759661753e1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, 0x1p50L, -8.881784197001252323389053344720723756399e-16L, 3.535050620855721078027883819436759661753e1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 1.0L, -0x1p50L, 8.881784197001252323389053344720723756399e-16L, -3.535050620855721078027883819436759661753e1L, 0 },
	TEST_c_c(casin, -1.0L, -0x1p50L, -8.881784197001252323389053344720723756399e-16L, -3.535050620855721078027883819436759661753e1L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(casin, 0x1p500L, 1.0L, 1.570796326794896619231321691639751442099L, 3.472667374605326000180332928505464606058e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p500L, -1.0L, 1.570796326794896619231321691639751442099L, -3.472667374605326000180332928505464606058e2L, 0 },
	TEST_c_c(casin, -0x1p500L, 1.0L, -1.570796326794896619231321691639751442099L, 3.472667374605326000180332928505464606058e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p500L, -1.0L, -1.570796326794896619231321691639751442099L, -3.472667374605326000180332928505464606058e2L, 0 },
	TEST_c_c(casin, 1.0L, 0x1p500L, 3.054936363499604682051979393213617699789e-151L, 3.472667374605326000180332928505464606058e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, 0x1p500L, -3.054936363499604682051979393213617699789e-151L, 3.472667374605326000180332928505464606058e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 1.0L, -0x1p500L, 3.054936363499604682051979393213617699789e-151L, -3.472667374605326000180332928505464606058e2L, 0 },
	TEST_c_c(casin, -1.0L, -0x1p500L, -3.054936363499604682051979393213617699789e-151L, -3.472667374605326000180332928505464606058e2L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(casin, 0x1p5000L, 1.0L, 1.570796326794896619231321691639751442099L, 3.466429049980286492395577839412341016946e3L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p5000L, -1.0L, 1.570796326794896619231321691639751442099L, -3.466429049980286492395577839412341016946e3L, 0 },
	TEST_c_c(casin, -0x1p5000L, 1.0L, -1.570796326794896619231321691639751442099L, 3.466429049980286492395577839412341016946e3L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p5000L, -1.0L, -1.570796326794896619231321691639751442099L, -3.466429049980286492395577839412341016946e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casin, 1.0L, 0x1p5000L, 7.079811261048172892385615158694057552948e-1506L, 3.466429049980286492395577839412341016946e3L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, 0x1p5000L, -7.079811261048172892385615158694057552948e-1506L, 3.466429049980286492395577839412341016946e3L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 1.0L, -0x1p5000L, 7.079811261048172892385615158694057552948e-1506L, -3.466429049980286492395577839412341016946e3L, 0 },
	TEST_c_c(casin, -1.0L, -0x1p5000L, -7.079811261048172892385615158694057552948e-1506L, -3.466429049980286492395577839412341016946e3L, 0 },
#endif

	TEST_c_c(casin, 0x1.fp127L, 0x1.fp127L, 7.853981633974483096156608458198757210493e-1L, 8.973081118419833726837456344608533993585e1L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(casin, 0x1.fp1023L, 0x1.fp1023L, 7.853981633974483096156608458198757210493e-1L, 7.107906849659093345062145442726115449315e2L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(casin, 0x1.fp16383L, 0x1.fp16383L, 7.853981633974483096156608458198757210493e-1L, 1.135753137836666928715489992987020363057e4L, 0|XFAIL_NATIVE_LINUX },
#endif

	TEST_c_c(casin, 0x1.fp-129L, 1.5L, 1.579176199917649005841160751101628985741e-39L, 1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1.fp-129L, -1.5L, 1.579176199917649005841160751101628985741e-39L, -1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-129L, 1.5L, -1.579176199917649005841160751101628985741e-39L, 1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-129L, -1.5L, -1.579176199917649005841160751101628985741e-39L, -1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 1.5L, 0x1.fp-129L, 1.570796326794896619231321691639751442096L, 9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(casin, -1.5L, 0x1.fp-129L, -1.570796326794896619231321691639751442096L, 9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(casin, 1.5L, -0x1.fp-129L, 1.570796326794896619231321691639751442096L, -9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(casin, -1.5L, -0x1.fp-129L, -1.570796326794896619231321691639751442096L, -9.624236501192068949955178268487368462704e-1L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(casin, 0x1.fp-1025L, 1.5L, 2.989196569048182929051881765490354365918e-309L, 1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0x1.fp-1025L, -1.5L, 2.989196569048182929051881765490354365918e-309L, -1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, 1.5L, -2.989196569048182929051881765490354365918e-309L, 1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, -1.5L, -2.989196569048182929051881765490354365918e-309L, -1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 1.5L, 0x1.fp-1025L, 1.570796326794896619231321691639751442099L, 9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(casin, -1.5L, 0x1.fp-1025L, -1.570796326794896619231321691639751442099L, 9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(casin, 1.5L, -0x1.fp-1025L, 1.570796326794896619231321691639751442099L, -9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(casin, -1.5L, -0x1.fp-1025L, -1.570796326794896619231321691639751442099L, -9.624236501192068949955178268487368462704e-1L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(casin, 0x1.fp-16385L, 1.5L, 4.516698239814521372306784062043266700598e-4933L, 1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0x1.fp-16385L, -1.5L, 4.516698239814521372306784062043266700598e-4933L, -1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, 1.5L, -4.516698239814521372306784062043266700598e-4933L, 1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, -1.5L, -4.516698239814521372306784062043266700598e-4933L, -1.194763217287109304111930828519090523536L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 1.5L, 0x1.fp-16385L, 1.570796326794896619231321691639751442099L, 9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(casin, -1.5L, 0x1.fp-16385L, -1.570796326794896619231321691639751442099L, 9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(casin, 1.5L, -0x1.fp-16385L, 1.570796326794896619231321691639751442099L, -9.624236501192068949955178268487368462704e-1L, 0 },
	TEST_c_c(casin, -1.5L, -0x1.fp-16385L, -1.570796326794896619231321691639751442099L, -9.624236501192068949955178268487368462704e-1L, 0 },
#endif

	TEST_c_c(casin, 0.5L, 1.0L, 3.494390628572132936274118260753699527325e-1L, 9.261330313501824245501244453057873152694e-1L, 0 },
	TEST_c_c(casin, 0.5L, -1.0L, 3.494390628572132936274118260753699527325e-1L, -9.261330313501824245501244453057873152694e-1L, 0 },
	TEST_c_c(casin, -0.5L, 1.0L, -3.494390628572132936274118260753699527325e-1L, 9.261330313501824245501244453057873152694e-1L, 0 },
	TEST_c_c(casin, -0.5L, -1.0L, -3.494390628572132936274118260753699527325e-1L, -9.261330313501824245501244453057873152694e-1L, 0 },
	TEST_c_c(casin, 1.0L, 0.5L, 8.959074812088902390666567243275770102229e-1L, 7.328576759736452608886724437653071523305e-1L, 0 },
	TEST_c_c(casin, -1.0L, 0.5L, -8.959074812088902390666567243275770102229e-1L, 7.328576759736452608886724437653071523305e-1L, 0 },
	TEST_c_c(casin, 1.0L, -0.5L, 8.959074812088902390666567243275770102229e-1L, -7.328576759736452608886724437653071523305e-1L, 0 },
	TEST_c_c(casin, -1.0L, -0.5L, -8.959074812088902390666567243275770102229e-1L, -7.328576759736452608886724437653071523305e-1L, 0 },
	TEST_c_c(casin, 0.25L, 1.0L, 1.763024327769669304186785666360901026468e-1L, 8.924633639033482359562124741744951972772e-1L, 0 },
	TEST_c_c(casin, 0.25L, -1.0L, 1.763024327769669304186785666360901026468e-1L, -8.924633639033482359562124741744951972772e-1L, 0 },
	TEST_c_c(casin, -0.25L, 1.0L, -1.763024327769669304186785666360901026468e-1L, 8.924633639033482359562124741744951972772e-1L, 0 },
	TEST_c_c(casin, -0.25L, -1.0L, -1.763024327769669304186785666360901026468e-1L, -8.924633639033482359562124741744951972772e-1L, 0 },
	TEST_c_c(casin, 1.0L, 0.25L, 1.081751996523816326311037318425097434186L, 5.097911466811016354623559941115413499164e-1L, 0 },
	TEST_c_c(casin, -1.0L, 0.25L, -1.081751996523816326311037318425097434186L, 5.097911466811016354623559941115413499164e-1L, 0 },
	TEST_c_c(casin, 1.0L, -0.25L, 1.081751996523816326311037318425097434186L, -5.097911466811016354623559941115413499164e-1L, 0 },
	TEST_c_c(casin, -1.0L, -0.25L, -1.081751996523816326311037318425097434186L, -5.097911466811016354623559941115413499164e-1L, 0 },
	TEST_c_c(casin, 0x1.fp-10L, 1.0L, 1.337909359557740912558349531052239112857e-3L, 8.813742198809567991336704287826445879025e-1L, 0 },
	TEST_c_c(casin, 0x1.fp-10L, -1.0L, 1.337909359557740912558349531052239112857e-3L, -8.813742198809567991336704287826445879025e-1L, 0 },
	TEST_c_c(casin, -0x1.fp-10L, 1.0L, -1.337909359557740912558349531052239112857e-3L, 8.813742198809567991336704287826445879025e-1L, 0 },
	TEST_c_c(casin, -0x1.fp-10L, -1.0L, -1.337909359557740912558349531052239112857e-3L, -8.813742198809567991336704287826445879025e-1L, 0 },
	TEST_c_c(casin, 1.0L, 0x1.fp-10L, 1.527305029163877791518741192097931722508L, 4.350501469856803800217957402220976497152e-2L, 0 },
	TEST_c_c(casin, -1.0L, 0x1.fp-10L, -1.527305029163877791518741192097931722508L, 4.350501469856803800217957402220976497152e-2L, 0 },
	TEST_c_c(casin, 1.0L, -0x1.fp-10L, 1.527305029163877791518741192097931722508L, -4.350501469856803800217957402220976497152e-2L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -1.0L, -0x1.fp-10L, -1.527305029163877791518741192097931722508L, -4.350501469856803800217957402220976497152e-2L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0x1.fp-30L, 1.0L, 1.275929984216518540252717485007112529021e-9L, 8.813735870195430258081932989769495326854e-1L, 0 },
	TEST_c_c(casin, 0x1.fp-30L, -1.0L, 1.275929984216518540252717485007112529021e-9L, -8.813735870195430258081932989769495326854e-1L, 0 },
	TEST_c_c(casin, -0x1.fp-30L, 1.0L, -1.275929984216518540252717485007112529021e-9L, 8.813735870195430258081932989769495326854e-1L, 0 },
	TEST_c_c(casin, -0x1.fp-30L, -1.0L, -1.275929984216518540252717485007112529021e-9L, -8.813735870195430258081932989769495326854e-1L, 0 },
	TEST_c_c(casin, 1.0L, 0x1.fp-30L, 1.570753848123921942730162693731872690232L, 4.247867098745151888768727039216644758847e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -1.0L, 0x1.fp-30L, -1.570753848123921942730162693731872690232L, 4.247867098745151888768727039216644758847e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 1.0L, -0x1.fp-30L, 1.570753848123921942730162693731872690232L, -4.247867098745151888768727039216644758847e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -1.0L, -0x1.fp-30L, -1.570753848123921942730162693731872690232L, -4.247867098745151888768727039216644758847e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0x1.fp-100L, 1.0L, 1.080754735021050612990719608916167354321e-30L, 8.813735870195430252326093249797923090282e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1.fp-100L, -1.0L, 1.080754735021050612990719608916167354321e-30L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(casin, -0x1.fp-100L, 1.0L, -1.080754735021050612990719608916167354321e-30L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(casin, -0x1.fp-100L, -1.0L, -1.080754735021050612990719608916167354321e-30L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(casin, 1.0L, 0x1.fp-100L, 1.570796326794895382939283431378862777584L, 1.236292038260260888664514866457202186027e-15L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, 0x1.fp-100L, -1.570796326794895382939283431378862777584L, 1.236292038260260888664514866457202186027e-15L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 1.0L, -0x1.fp-100L, 1.570796326794895382939283431378862777584L, -1.236292038260260888664514866457202186027e-15L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, -0x1.fp-100L, -1.570796326794895382939283431378862777584L, -1.236292038260260888664514866457202186027e-15L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1.fp-129L, 1.0L, 2.013062564695348242280482517399205554874e-39L, 8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0x1.fp-129L, -1.0L, 2.013062564695348242280482517399205554874e-39L, -8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, -0x1.fp-129L, 1.0L, -2.013062564695348242280482517399205554874e-39L, 8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -0x1.fp-129L, -1.0L, -2.013062564695348242280482517399205554874e-39L, -8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, 1.0L, 0x1.fp-129L, 1.570796326794896619177965338869929107115L, 5.335635276982233498398987585285818977933e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, 0x1.fp-129L, -1.570796326794896619177965338869929107115L, 5.335635276982233498398987585285818977933e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 1.0L, -0x1.fp-129L, 1.570796326794896619177965338869929107115L, -5.335635276982233498398987585285818977933e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, -0x1.fp-129L, -1.570796326794896619177965338869929107115L, -5.335635276982233498398987585285818977933e-20L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(casin, 0x1.fp-1000L, 1.0L, 1.278589251976747242280879285935084814093e-301L, 8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casin, 0x1.fp-1000L, -1.0L, 1.278589251976747242280879285935084814093e-301L, -8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casin, -0x1.fp-1000L, 1.0L, -1.278589251976747242280879285935084814093e-301L, 8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casin, -0x1.fp-1000L, -1.0L, -1.278589251976747242280879285935084814093e-301L, -8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casin, 1.0L, 0x1.fp-1000L, 1.570796326794896619231321691639751442099L, 4.252291453851660175550490409247739011867e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, 0x1.fp-1000L, -1.570796326794896619231321691639751442099L, 4.252291453851660175550490409247739011867e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 1.0L, -0x1.fp-1000L, 1.570796326794896619231321691639751442099L, -4.252291453851660175550490409247739011867e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, -0x1.fp-1000L, -1.570796326794896619231321691639751442099L, -4.252291453851660175550490409247739011867e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1.fp-1025L, 1.0L, 3.810492908885321743133304375216617626230e-309L, 8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0x1.fp-1025L, -1.0L, 3.810492908885321743133304375216617626230e-309L, -8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, 1.0L, -3.810492908885321743133304375216617626230e-309L, 8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, -1.0L, -3.810492908885321743133304375216617626230e-309L, -8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 1.0L, 0x1.fp-1025L, 1.570796326794896619231321691639751442099L, 7.340879205566679497036857179189356754017e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, 0x1.fp-1025L, -1.570796326794896619231321691639751442099L, 7.340879205566679497036857179189356754017e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 1.0L, -0x1.fp-1025L, 1.570796326794896619231321691639751442099L, -7.340879205566679497036857179189356754017e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, -0x1.fp-1025L, -1.570796326794896619231321691639751442099L, -7.340879205566679497036857179189356754017e-155L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(casin, 0x1.fp-10000L, 1.0L, 6.867047849047171855399183659351043150871e-3011L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(casin, 0x1.fp-10000L, -1.0L, 6.867047849047171855399183659351043150871e-3011L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(casin, -0x1.fp-10000L, 1.0L, -6.867047849047171855399183659351043150871e-3011L, 8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(casin, -0x1.fp-10000L, -1.0L, -6.867047849047171855399183659351043150871e-3011L, -8.813735870195430252326093249797923090282e-1L, 0 },
	TEST_c_c(casin, 1.0L, 0x1.fp-10000L, 1.570796326794896619231321691639751442099L, 9.854680208706673586644342922051388714633e-1506L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, 0x1.fp-10000L, -1.570796326794896619231321691639751442099L, 9.854680208706673586644342922051388714633e-1506L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 1.0L, -0x1.fp-10000L, 1.570796326794896619231321691639751442099L, -9.854680208706673586644342922051388714633e-1506L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, -0x1.fp-10000L, -1.570796326794896619231321691639751442099L, -9.854680208706673586644342922051388714633e-1506L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1.fp-16385L, 1.0L, 5.757683115456107044131264955348448954458e-4933L, 8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0x1.fp-16385L, -1.0L, 5.757683115456107044131264955348448954458e-4933L, -8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, 1.0L, -5.757683115456107044131264955348448954458e-4933L, 8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, -1.0L, -5.757683115456107044131264955348448954458e-4933L, -8.813735870195430252326093249797923090282e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 1.0L, 0x1.fp-16385L, 1.570796326794896619231321691639751442099L, 9.023632056840860275214893047597614177639e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, 0x1.fp-16385L, -1.570796326794896619231321691639751442099L, 9.023632056840860275214893047597614177639e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 1.0L, -0x1.fp-16385L, 1.570796326794896619231321691639751442099L, -9.023632056840860275214893047597614177639e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -1.0L, -0x1.fp-16385L, -1.570796326794896619231321691639751442099L, -9.023632056840860275214893047597614177639e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
#endif

	TEST_c_c(casin, 0x1p-23L, 0x1.000002p0L, 8.429369199749229560964789467980644296420e-8L, 8.813736713132400470205730751186547909968e-1L, 0 },
	TEST_c_c(casin, 0x1p-23L, -0x1.000002p0L, 8.429369199749229560964789467980644296420e-8L, -8.813736713132400470205730751186547909968e-1L, 0 },
	TEST_c_c(casin, -0x1p-23L, 0x1.000002p0L, -8.429369199749229560964789467980644296420e-8L, 8.813736713132400470205730751186547909968e-1L, 0 },
	TEST_c_c(casin, -0x1p-23L, -0x1.000002p0L, -8.429369199749229560964789467980644296420e-8L, -8.813736713132400470205730751186547909968e-1L, 0 },
	TEST_c_c(casin, 0x1.000002p0L, 0x1p-23L, 1.570574114956455764594481054234834678422L, 5.364668491573609633134147164031476452679e-4L, 0 },
	TEST_c_c(casin, -0x1.000002p0L, 0x1p-23L, -1.570574114956455764594481054234834678422L, 5.364668491573609633134147164031476452679e-4L, 0 },
	TEST_c_c(casin, 0x1.000002p0L, -0x1p-23L, 1.570574114956455764594481054234834678422L, -5.364668491573609633134147164031476452679e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -0x1.000002p0L, -0x1p-23L, -1.570574114956455764594481054234834678422L, -5.364668491573609633134147164031476452679e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0x1.fp-129L, 0x1.000002p0L, 2.013062444707472738895109955455676357057e-39L, 8.813736713132375348727889167749389235161e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0x1.fp-129L, -0x1.000002p0L, 2.013062444707472738895109955455676357057e-39L, -8.813736713132375348727889167749389235161e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, -0x1.fp-129L, 0x1.000002p0L, -2.013062444707472738895109955455676357057e-39L, 8.813736713132375348727889167749389235161e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -0x1.fp-129L, -0x1.000002p0L, -2.013062444707472738895109955455676357057e-39L, -8.813736713132375348727889167749389235161e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, 0x1.000002p0L, 0x1.fp-129L, 1.570796326794896619231321691639751436268L, 4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(casin, -0x1.000002p0L, 0x1.fp-129L, -1.570796326794896619231321691639751436268L, 4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(casin, 0x1.000002p0L, -0x1.fp-129L, 1.570796326794896619231321691639751436268L, -4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(casin, -0x1.000002p0L, -0x1.fp-129L, -1.570796326794896619231321691639751436268L, -4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(casin, 0.0L, 0x1.000002p0L, 0.0L, 8.813736713132375348727889167749389235161e-1L, 0 },
	TEST_c_c(casin, 0.0L, -0x1.000002p0L, 0.0L, -8.813736713132375348727889167749389235161e-1L, 0 },
	TEST_c_c(casin, -0.0L, 0x1.000002p0L, -0.0L, 8.813736713132375348727889167749389235161e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, -0x1.000002p0L, -0.0L, -8.813736713132375348727889167749389235161e-1L, 0 },
	TEST_c_c(casin, 0x1.000002p0L, 0.0L, 1.570796326794896619231321691639751442099L, 4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(casin, -0x1.000002p0L, 0.0L, -1.570796326794896619231321691639751442099L, 4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(casin, 0x1.000002p0L, -0.0L, 1.570796326794896619231321691639751442099L, -4.882812451493617206486388134172712975070e-4L, 0 },
	TEST_c_c(casin, -0x1.000002p0L, -0.0L, -1.570796326794896619231321691639751442099L, -4.882812451493617206486388134172712975070e-4L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(casin, 0x1p-52L, 0x1.0000000000001p0L, 1.570092458683774885078102529858632363236e-16L, 8.813735870195431822418551933572982483664e-1L, 0 },
	TEST_c_c(casin, 0x1p-52L, -0x1.0000000000001p0L, 1.570092458683774885078102529858632363236e-16L, -8.813735870195431822418551933572982483664e-1L, 0 },
	TEST_c_c(casin, -0x1p-52L, 0x1.0000000000001p0L, -1.570092458683774885078102529858632363236e-16L, 8.813735870195431822418551933572982483664e-1L, 0 },
	TEST_c_c(casin, -0x1p-52L, -0x1.0000000000001p0L, -1.570092458683774885078102529858632363236e-16L, -8.813735870195431822418551933572982483664e-1L, 0 },
	TEST_c_c(casin, 0x1.0000000000001p0L, 0x1p-52L, 1.570796317204594913251280305810847349436L, 2.315303644582684770975188768022139415020e-8L, 0 },
	TEST_c_c(casin, -0x1.0000000000001p0L, 0x1p-52L, -1.570796317204594913251280305810847349436L, 2.315303644582684770975188768022139415020e-8L, 0 },
	TEST_c_c(casin, 0x1.0000000000001p0L, -0x1p-52L, 1.570796317204594913251280305810847349436L, -2.315303644582684770975188768022139415020e-8L, 0 },
	TEST_c_c(casin, -0x1.0000000000001p0L, -0x1p-52L, -1.570796317204594913251280305810847349436L, -2.315303644582684770975188768022139415020e-8L, 0 },
	TEST_c_c(casin, 0x1.fp-1025L, 0x1.0000000000001p0L, 3.810492908885321320083608113679347200012e-309L, 8.813735870195431822418551933572895326024e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0x1.fp-1025L, -0x1.0000000000001p0L, 3.810492908885321320083608113679347200012e-309L, -8.813735870195431822418551933572895326024e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, 0x1.0000000000001p0L, -3.810492908885321320083608113679347200012e-309L, 8.813735870195431822418551933572895326024e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, -0x1.0000000000001p0L, -3.810492908885321320083608113679347200012e-309L, -8.813735870195431822418551933572895326024e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0x1.0000000000001p0L, 0x1.fp-1025L, 1.570796326794896619231321691639751442099L, 2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(casin, -0x1.0000000000001p0L, 0x1.fp-1025L, -1.570796326794896619231321691639751442099L, 2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(casin, 0x1.0000000000001p0L, -0x1.fp-1025L, 1.570796326794896619231321691639751442099L, -2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(casin, -0x1.0000000000001p0L, -0x1.fp-1025L, -1.570796326794896619231321691639751442099L, -2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(casin, 0.0L, 0x1.0000000000001p0L, 0.0L, 8.813735870195431822418551933572895326024e-1L, 0 },
	TEST_c_c(casin, 0.0L, -0x1.0000000000001p0L, 0.0L, -8.813735870195431822418551933572895326024e-1L, 0 },
	TEST_c_c(casin, -0.0L, 0x1.0000000000001p0L, -0.0L, 8.813735870195431822418551933572895326024e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, -0x1.0000000000001p0L, -0.0L, -8.813735870195431822418551933572895326024e-1L, 0 },
	TEST_c_c(casin, 0x1.0000000000001p0L, 0.0L, 1.570796326794896619231321691639751442099L, 2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(casin, -0x1.0000000000001p0L, 0.0L, -1.570796326794896619231321691639751442099L, 2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(casin, 0x1.0000000000001p0L, -0.0L, 1.570796326794896619231321691639751442099L, -2.107342425544701550354780375182800088393e-8L, 0 },
	TEST_c_c(casin, -0x1.0000000000001p0L, -0.0L, -1.570796326794896619231321691639751442099L, -2.107342425544701550354780375182800088393e-8L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(casin, 0x1p-63L, 0x1.0000000000000002p0L, 7.666467083416870406778649849746878368519e-20L, 8.813735870195430253092739958139610131001e-1L, 0 },
	TEST_c_c(casin, 0x1p-63L, -0x1.0000000000000002p0L, 7.666467083416870406778649849746878368519e-20L, -8.813735870195430253092739958139610131001e-1L, 0 },
	TEST_c_c(casin, -0x1p-63L, 0x1.0000000000000002p0L, -7.666467083416870406778649849746878368519e-20L, 8.813735870195430253092739958139610131001e-1L, 0 },
	TEST_c_c(casin, -0x1p-63L, -0x1.0000000000000002p0L, -7.666467083416870406778649849746878368519e-20L, -8.813735870195430253092739958139610131001e-1L, 0 },
	TEST_c_c(casin, 0x1.0000000000000002p0L, 0x1p-63L, 1.570796326582978888921215348380499122131L, 5.116146586219826555037807251857670783420e-10L, 0 },
	TEST_c_c(casin, -0x1.0000000000000002p0L, 0x1p-63L, -1.570796326582978888921215348380499122131L, 5.116146586219826555037807251857670783420e-10L, 0 },
	TEST_c_c(casin, 0x1.0000000000000002p0L, -0x1p-63L, 1.570796326582978888921215348380499122131L, -5.116146586219826555037807251857670783420e-10L, 0 },
	TEST_c_c(casin, -0x1.0000000000000002p0L, -0x1p-63L, -1.570796326582978888921215348380499122131L, -5.116146586219826555037807251857670783420e-10L, 0 },
# if LDBL_MIN_EXP <= -16381
	TEST_c_c(casin, 0x1.fp-16385L, 0x1.0000000000000002p0L, 5.757683115456107043819140328235418018963e-4933L, 8.813735870195430253092739958139610130980e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0x1.fp-16385L, -0x1.0000000000000002p0L, 5.757683115456107043819140328235418018963e-4933L, -8.813735870195430253092739958139610130980e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, 0x1.0000000000000002p0L, -5.757683115456107043819140328235418018963e-4933L, 8.813735870195430253092739958139610130980e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, -0x1.0000000000000002p0L, -5.757683115456107043819140328235418018963e-4933L, -8.813735870195430253092739958139610130980e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0x1.0000000000000002p0L, 0x1.fp-16385L, 1.570796326794896619231321691639751442099L, 4.656612873077392578082927418388212703712e-10L, 0 },
	TEST_c_c(casin, -0x1.0000000000000002p0L, 0x1.fp-16385L, -1.570796326794896619231321691639751442099L, 4.656612873077392578082927418388212703712e-10L, 0 },
	TEST_c_c(casin, 0x1.0000000000000002p0L, -0x1.fp-16385L, 1.570796326794896619231321691639751442099L, -4.656612873077392578082927418388212703712e-10L, 0 },
	TEST_c_c(casin, -0x1.0000000000000002p0L, -0x1.fp-16385L, -1.570796326794896619231321691639751442099L, -4.656612873077392578082927418388212703712e-10L, 0 },
# endif
	TEST_c_c(casin, 0.0L, 0x1.0000000000000002p0L, 0.0L, 8.813735870195430253092739958139610130980e-1L, 0 },
	TEST_c_c(casin, 0.0L, -0x1.0000000000000002p0L, 0.0L, -8.813735870195430253092739958139610130980e-1L, 0 },
	TEST_c_c(casin, -0.0L, 0x1.0000000000000002p0L, -0.0L, 8.813735870195430253092739958139610130980e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casin, -0.0L, -0x1.0000000000000002p0L, -0.0L, -8.813735870195430253092739958139610130980e-1L, 0 },
	TEST_c_c(casin, 0x1.0000000000000002p0L, 0.0L, 1.570796326794896619231321691639751442099L, 4.656612873077392578082927418388212703712e-10L, 0 },
	TEST_c_c(casin, -0x1.0000000000000002p0L, 0.0L, -1.570796326794896619231321691639751442099L, 4.656612873077392578082927418388212703712e-10L, 0 },
	TEST_c_c(casin, 0x1.0000000000000002p0L, -0.0L, 1.570796326794896619231321691639751442099L, -4.656612873077392578082927418388212703712e-10L, 0 },
	TEST_c_c(casin, -0x1.0000000000000002p0L, -0.0L, -1.570796326794896619231321691639751442099L, -4.656612873077392578082927418388212703712e-10L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(casin, 0x1p-106L, 0x1.000000000000000000000000008p0L, 8.715763992105246878957416200936726072500e-33L, 8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(casin, 0x1p-106L, -0x1.000000000000000000000000008p0L, 8.715763992105246878957416200936726072500e-33L, -8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(casin, -0x1p-106L, 0x1.000000000000000000000000008p0L, -8.715763992105246878957416200936726072500e-33L, 8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(casin, -0x1p-106L, -0x1.000000000000000000000000008p0L, -8.715763992105246878957416200936726072500e-33L, -8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(casin, 0x1.000000000000000000000000008p0L, 0x1p-106L, 1.570796326794896565289107467733682961928L, 2.285028863093221674154232933662774454211e-16L, 0 },
	TEST_c_c(casin, -0x1.000000000000000000000000008p0L, 0x1p-106L, -1.570796326794896565289107467733682961928L, 2.285028863093221674154232933662774454211e-16L, 0 },
	TEST_c_c(casin, 0x1.000000000000000000000000008p0L, -0x1p-106L, 1.570796326794896565289107467733682961928L, -2.285028863093221674154232933662774454211e-16L, 0 },
	TEST_c_c(casin, -0x1.000000000000000000000000008p0L, -0x1p-106L, -1.570796326794896565289107467733682961928L, -2.285028863093221674154232933662774454211e-16L, 0 },
	TEST_c_c(casin, 0x1.fp-1025L, 0x1.000000000000000000000000008p0L, 3.810492908885321743133304375216570658278e-309L, 8.813735870195430252326093249798097405561e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0x1.fp-1025L, -0x1.000000000000000000000000008p0L, 3.810492908885321743133304375216570658278e-309L, -8.813735870195430252326093249798097405561e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, 0x1.000000000000000000000000008p0L, -3.810492908885321743133304375216570658278e-309L, 8.813735870195430252326093249798097405561e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, -0x1.000000000000000000000000008p0L, -3.810492908885321743133304375216570658278e-309L, -8.813735870195430252326093249798097405561e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0x1.000000000000000000000000008p0L, 0x1.fp-1025L, 1.570796326794896619231321691639751442099L, 2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(casin, -0x1.000000000000000000000000008p0L, 0x1.fp-1025L, -1.570796326794896619231321691639751442099L, 2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(casin, 0x1.000000000000000000000000008p0L, -0x1.fp-1025L, 1.570796326794896619231321691639751442099L, -2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(casin, -0x1.000000000000000000000000008p0L, -0x1.fp-1025L, -1.570796326794896619231321691639751442099L, -2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(casin, 0.0L, 0x1.000000000000000000000000008p0L, 0.0L, 8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(casin, 0.0L, -0x1.000000000000000000000000008p0L, 0.0L, -8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(casin, -0.0L, 0x1.000000000000000000000000008p0L, -0.0L, 8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(casin, -0.0L, -0x1.000000000000000000000000008p0L, -0.0L, -8.813735870195430252326093249798097405561e-1L, 0 },
	TEST_c_c(casin, 0x1.000000000000000000000000008p0L, 0.0L, 1.570796326794896619231321691639751442099L, 2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(casin, -0x1.000000000000000000000000008p0L, 0.0L, -1.570796326794896619231321691639751442099L, 2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(casin, 0x1.000000000000000000000000008p0L, -0.0L, 1.570796326794896619231321691639751442099L, -2.220446049250313080847263336181636063482e-16L, 0 },
	TEST_c_c(casin, -0x1.000000000000000000000000008p0L, -0.0L, -1.570796326794896619231321691639751442099L, -2.220446049250313080847263336181636063482e-16L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(casin, 0x1p-113L, 0x1.0000000000000000000000000001p0L, 6.809190618832224124185481406981900518193e-35L, 8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(casin, 0x1p-113L, -0x1.0000000000000000000000000001p0L, 6.809190618832224124185481406981900518193e-35L, -8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(casin, -0x1p-113L, 0x1.0000000000000000000000000001p0L, -6.809190618832224124185481406981900518193e-35L, 8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(casin, -0x1p-113L, -0x1.0000000000000000000000000001p0L, -6.809190618832224124185481406981900518193e-35L, -8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(casin, 0x1.0000000000000000000000000001p0L, 0x1p-113L, 1.570796326794896614463458507897073960405L, 2.019699255375255198156433710951064632386e-17L, 0 },
	TEST_c_c(casin, -0x1.0000000000000000000000000001p0L, 0x1p-113L, -1.570796326794896614463458507897073960405L, 2.019699255375255198156433710951064632386e-17L, 0 },
	TEST_c_c(casin, 0x1.0000000000000000000000000001p0L, -0x1p-113L, 1.570796326794896614463458507897073960405L, -2.019699255375255198156433710951064632386e-17L, 0 },
	TEST_c_c(casin, -0x1.0000000000000000000000000001p0L, -0x1p-113L, -1.570796326794896614463458507897073960405L, -2.019699255375255198156433710951064632386e-17L, 0 },
	TEST_c_c(casin, 0x1.fp-16385L, 0x1.0000000000000000000000000001p0L, 5.757683115456107044131264955348448400014e-4933L, 8.813735870195430252326093249797924452120e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0x1.fp-16385L, -0x1.0000000000000000000000000001p0L, 5.757683115456107044131264955348448400014e-4933L, -8.813735870195430252326093249797924452120e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, 0x1.0000000000000000000000000001p0L, -5.757683115456107044131264955348448400014e-4933L, 8.813735870195430252326093249797924452120e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, -0x1.0000000000000000000000000001p0L, -5.757683115456107044131264955348448400014e-4933L, -8.813735870195430252326093249797924452120e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0x1.0000000000000000000000000001p0L, 0x1.fp-16385L, 1.570796326794896619231321691639751442099L, 1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(casin, -0x1.0000000000000000000000000001p0L, 0x1.fp-16385L, -1.570796326794896619231321691639751442099L, 1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(casin, 0x1.0000000000000000000000000001p0L, -0x1.fp-16385L, 1.570796326794896619231321691639751442099L, -1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(casin, -0x1.0000000000000000000000000001p0L, -0x1.fp-16385L, -1.570796326794896619231321691639751442099L, -1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(casin, 0.0L, 0x1.0000000000000000000000000001p0L, 0.0L, 8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(casin, 0.0L, -0x1.0000000000000000000000000001p0L, 0.0L, -8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(casin, -0.0L, 0x1.0000000000000000000000000001p0L, -0.0L, 8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(casin, -0.0L, -0x1.0000000000000000000000000001p0L, -0.0L, -8.813735870195430252326093249797924452120e-1L, 0 },
	TEST_c_c(casin, 0x1.0000000000000000000000000001p0L, 0.0L, 1.570796326794896619231321691639751442099L, 1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(casin, -0x1.0000000000000000000000000001p0L, 0.0L, -1.570796326794896619231321691639751442099L, 1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(casin, 0x1.0000000000000000000000000001p0L, -0.0L, 1.570796326794896619231321691639751442099L, -1.962615573354718824241727964954454332780e-17L, 0 },
	TEST_c_c(casin, -0x1.0000000000000000000000000001p0L, -0.0L, -1.570796326794896619231321691639751442099L, -1.962615573354718824241727964954454332780e-17L, 0 },
#endif

	TEST_c_c(casin, 0x1p-23L, 0x0.ffffffp0L, 8.429369953393598499000760938112968755257e-8L, 8.813735448726963984495965873956465777250e-1L, 0 },
	TEST_c_c(casin, 0x1p-23L, -0x0.ffffffp0L, 8.429369953393598499000760938112968755257e-8L, -8.813735448726963984495965873956465777250e-1L, 0 },
	TEST_c_c(casin, -0x1p-23L, 0x0.ffffffp0L, -8.429369953393598499000760938112968755257e-8L, 8.813735448726963984495965873956465777250e-1L, 0 },
	TEST_c_c(casin, -0x1p-23L, -0x0.ffffffp0L, -8.429369953393598499000760938112968755257e-8L, -8.813735448726963984495965873956465777250e-1L, 0 },
	TEST_c_c(casin, 0x0.ffffffp0L, 0x1p-23L, 1.570357140408705570520377172865353621698L, 2.714321200917194650737217746780928423385e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -0x0.ffffffp0L, 0x1p-23L, -1.570357140408705570520377172865353621698L, 2.714321200917194650737217746780928423385e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0x0.ffffffp0L, -0x1p-23L, 1.570357140408705570520377172865353621698L, -2.714321200917194650737217746780928423385e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -0x0.ffffffp0L, -0x1p-23L, -1.570357140408705570520377172865353621698L, -2.714321200917194650737217746780928423385e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0x1.fp-129L, 0x0.ffffffp0L, 2.013062624689288675911339929224634593425e-39L, 8.813735448726938863015878255140556727969e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, 0x1.fp-129L, -0x0.ffffffp0L, 2.013062624689288675911339929224634593425e-39L, -8.813735448726938863015878255140556727969e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, -0x1.fp-129L, 0x0.ffffffp0L, -2.013062624689288675911339929224634593425e-39L, 8.813735448726938863015878255140556727969e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, -0x1.fp-129L, -0x0.ffffffp0L, -2.013062624689288675911339929224634593425e-39L, -8.813735448726938863015878255140556727969e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, 0x0.ffffffp0L, 0x1.fp-129L, 1.570451059810180415643718442157112705601L, 8.245504387859737323891843603996428534945e-36L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.ffffffp0L, 0x1.fp-129L, -1.570451059810180415643718442157112705601L, 8.245504387859737323891843603996428534945e-36L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x0.ffffffp0L, -0x1.fp-129L, 1.570451059810180415643718442157112705601L, -8.245504387859737323891843603996428534945e-36L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.ffffffp0L, -0x1.fp-129L, -1.570451059810180415643718442157112705601L, -8.245504387859737323891843603996428534945e-36L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.0L, 0x0.ffffffp0L, 0.0L, 8.813735448726938863015878255140556727969e-1L, 0 },
	TEST_c_c(casin, 0.0L, -0x0.ffffffp0L, 0.0L, -8.813735448726938863015878255140556727969e-1L, 0 },
	TEST_c_c(casin, -0.0L, 0x0.ffffffp0L, -0.0L, 8.813735448726938863015878255140556727969e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, -0x0.ffffffp0L, -0.0L, -8.813735448726938863015878255140556727969e-1L, 0 },
	TEST_c_c(casin, 0x0.ffffffp0L, 0.0L, 1.570451059810180415643718442157112705601L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.ffffffp0L, 0.0L, -1.570451059810180415643718442157112705601L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x0.ffffffp0L, -0.0L, 1.570451059810180415643718442157112705601L, -0.0L, 0 },
	TEST_c_c(casin, -0x0.ffffffp0L, -0.0L, -1.570451059810180415643718442157112705601L, -0.0L, 0 },
	TEST_c_c(casin, 0x1p-23L, 0.5L, 1.066240299940009780479664302917201696886e-7L, 4.812118250596059896127318386463676808798e-1L, 0 },
	TEST_c_c(casin, 0x1p-23L, -0.5L, 1.066240299940009780479664302917201696886e-7L, -4.812118250596059896127318386463676808798e-1L, 0 },
	TEST_c_c(casin, -0x1p-23L, 0.5L, -1.066240299940009780479664302917201696886e-7L, 4.812118250596059896127318386463676808798e-1L, 0 },
	TEST_c_c(casin, -0x1p-23L, -0.5L, -1.066240299940009780479664302917201696886e-7L, -4.812118250596059896127318386463676808798e-1L, 0 },
	TEST_c_c(casin, 0.5L, 0x1p-23L, 5.235987755982934033165770729745469098254e-1L, 1.376510308240943236356088341381173571841e-7L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -0.5L, 0x1p-23L, -5.235987755982934033165770729745469098254e-1L, 1.376510308240943236356088341381173571841e-7L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0.5L, -0x1p-23L, 5.235987755982934033165770729745469098254e-1L, -1.376510308240943236356088341381173571841e-7L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -0.5L, -0x1p-23L, -5.235987755982934033165770729745469098254e-1L, -1.376510308240943236356088341381173571841e-7L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0x1.fp-129L, 0.5L, 2.546345110742945032959687790021055102355e-39L, 4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, 0x1.fp-129L, -0.5L, 2.546345110742945032959687790021055102355e-39L, -4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, -0x1.fp-129L, 0.5L, -2.546345110742945032959687790021055102355e-39L, 4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, -0x1.fp-129L, -0.5L, -2.546345110742945032959687790021055102355e-39L, -4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casin, 0.5L, 0x1.fp-129L, 5.235987755982988730771072305465838140329e-1L, 3.287317402534702257036015056278368274737e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, 0x1.fp-129L, -5.235987755982988730771072305465838140329e-1L, 3.287317402534702257036015056278368274737e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.5L, -0x1.fp-129L, 5.235987755982988730771072305465838140329e-1L, -3.287317402534702257036015056278368274737e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, -0x1.fp-129L, -5.235987755982988730771072305465838140329e-1L, -3.287317402534702257036015056278368274737e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-23L, 0x1p-23L, 1.192092895507806853113684971258850504935e-7L, 1.192092895507818146886315028596704749235e-7L, 0 },
	TEST_c_c(casin, 0x1p-23L, -0x1p-23L, 1.192092895507806853113684971258850504935e-7L, -1.192092895507818146886315028596704749235e-7L, 0 },
	TEST_c_c(casin, -0x1p-23L, 0x1p-23L, -1.192092895507806853113684971258850504935e-7L, 1.192092895507818146886315028596704749235e-7L, 0 },
	TEST_c_c(casin, -0x1p-23L, -0x1p-23L, -1.192092895507806853113684971258850504935e-7L, -1.192092895507818146886315028596704749235e-7L, 0 },
	/* Bug 15319: underflow exception may be missing.  */
	TEST_c_c(casin, 0x1.fp-129L, 0x1p-23L, 2.846900380897707329917933005874691056564e-39L, 1.192092895507809676556842485683592032154e-7L, UNDERFLOW_EXCEPTION_OK_FLOAT },
	TEST_c_c(casin, 0x1.fp-129L, -0x1p-23L, 2.846900380897707329917933005874691056564e-39L, -1.192092895507809676556842485683592032154e-7L, UNDERFLOW_EXCEPTION_OK_FLOAT },
	TEST_c_c(casin, -0x1.fp-129L, 0x1p-23L, -2.846900380897707329917933005874691056564e-39L, 1.192092895507809676556842485683592032154e-7L, UNDERFLOW_EXCEPTION_OK_FLOAT },
	TEST_c_c(casin, -0x1.fp-129L, -0x1p-23L, -2.846900380897707329917933005874691056564e-39L, -1.192092895507809676556842485683592032154e-7L, UNDERFLOW_EXCEPTION_OK_FLOAT },
	TEST_c_c(casin, 0x1p-23L, 0x1.fp-129L, 1.192092895507815323443157514352519154304e-7L, 2.846900380897747786805634596726756660388e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-23L, 0x1.fp-129L, -1.192092895507815323443157514352519154304e-7L, 2.846900380897747786805634596726756660388e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-23L, -0x1.fp-129L, 1.192092895507815323443157514352519154304e-7L, -2.846900380897747786805634596726756660388e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-23L, -0x1.fp-129L, -1.192092895507815323443157514352519154304e-7L, -2.846900380897747786805634596726756660388e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.0L, 0x1p-23L, 0.0L, 1.192092895507809676556842485683592032154e-7L, 0 },
	TEST_c_c(casin, 0.0L, -0x1p-23L, 0.0L, -1.192092895507809676556842485683592032154e-7L, 0 },
	TEST_c_c(casin, -0.0L, 0x1p-23L, -0.0L, 1.192092895507809676556842485683592032154e-7L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, -0x1p-23L, -0.0L, -1.192092895507809676556842485683592032154e-7L, 0 },
	TEST_c_c(casin, 0x1p-23L, 0.0L, 1.192092895507815323443157514352519154304e-7L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-23L, 0.0L, -1.192092895507815323443157514352519154304e-7L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-23L, -0.0L, 1.192092895507815323443157514352519154304e-7L, -0.0L, 0 },
	TEST_c_c(casin, -0x1p-23L, -0.0L, -1.192092895507815323443157514352519154304e-7L, -0.0L, 0 },
	TEST_c_c(casin, 0x1.fp-129L, 0x1.fp-129L, 2.846900380897727558361783801085126250967e-39L, 2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1.fp-129L, -0x1.fp-129L, 2.846900380897727558361783801085126250967e-39L, -2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-129L, 0x1.fp-129L, -2.846900380897727558361783801085126250967e-39L, 2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-129L, -0x1.fp-129L, -2.846900380897727558361783801085126250967e-39L, -2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(casin, 0x1p-52L, 0x0.fffffffffffff8p0L, 1.570092458683775146551022293016031474627e-16L, 8.813735870195429467279863907910458761820e-1L, 0 },
	TEST_c_c(casin, 0x1p-52L, -0x0.fffffffffffff8p0L, 1.570092458683775146551022293016031474627e-16L, -8.813735870195429467279863907910458761820e-1L, 0 },
	TEST_c_c(casin, -0x1p-52L, 0x0.fffffffffffff8p0L, -1.570092458683775146551022293016031474627e-16L, 8.813735870195429467279863907910458761820e-1L, 0 },
	TEST_c_c(casin, -0x1p-52L, -0x0.fffffffffffff8p0L, -1.570092458683775146551022293016031474627e-16L, -8.813735870195429467279863907910458761820e-1L, 0 },
	TEST_c_c(casin, 0x0.fffffffffffff8p0L, 0x1p-52L, 1.570796307840326780080580569360500172043L, 1.171456840272878582596796205397918831268e-8L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.fffffffffffff8p0L, 0x1p-52L, -1.570796307840326780080580569360500172043L, 1.171456840272878582596796205397918831268e-8L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x0.fffffffffffff8p0L, -0x1p-52L, 1.570796307840326780080580569360500172043L, -1.171456840272878582596796205397918831268e-8L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.fffffffffffff8p0L, -0x1p-52L, -1.570796307840326780080580569360500172043L, -1.171456840272878582596796205397918831268e-8L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1.fp-1025L, 0x0.fffffffffffff8p0L, 3.810492908885321954658152505985270452320e-309L, 8.813735870195429467279863907910371604180e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0x1.fp-1025L, -0x0.fffffffffffff8p0L, 3.810492908885321954658152505985270452320e-309L, -8.813735870195429467279863907910371604180e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, 0x0.fffffffffffff8p0L, -3.810492908885321954658152505985270452320e-309L, 8.813735870195429467279863907910371604180e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, -0x0.fffffffffffff8p0L, -3.810492908885321954658152505985270452320e-309L, -8.813735870195429467279863907910371604180e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0x0.fffffffffffff8p0L, 0x1.fp-1025L, 1.570796311893735425383665303776316016594L, 3.616396521699973256461764099945789620604e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.fffffffffffff8p0L, 0x1.fp-1025L, -1.570796311893735425383665303776316016594L, 3.616396521699973256461764099945789620604e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x0.fffffffffffff8p0L, -0x1.fp-1025L, 1.570796311893735425383665303776316016594L, -3.616396521699973256461764099945789620604e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.fffffffffffff8p0L, -0x1.fp-1025L, -1.570796311893735425383665303776316016594L, -3.616396521699973256461764099945789620604e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.0L, 0x0.fffffffffffff8p0L, 0.0L, 8.813735870195429467279863907910371604180e-1L, 0 },
	TEST_c_c(casin, 0.0L, -0x0.fffffffffffff8p0L, 0.0L, -8.813735870195429467279863907910371604180e-1L, 0 },
	TEST_c_c(casin, -0.0L, 0x0.fffffffffffff8p0L, -0.0L, 8.813735870195429467279863907910371604180e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, -0x0.fffffffffffff8p0L, -0.0L, -8.813735870195429467279863907910371604180e-1L, 0 },
	TEST_c_c(casin, 0x0.fffffffffffff8p0L, 0.0L, 1.570796311893735425383665303776316016594L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.fffffffffffff8p0L, 0.0L, -1.570796311893735425383665303776316016594L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x0.fffffffffffff8p0L, -0.0L, 1.570796311893735425383665303776316016594L, -0.0L, 0 },
	TEST_c_c(casin, -0x0.fffffffffffff8p0L, -0.0L, -1.570796311893735425383665303776316016594L, -0.0L, 0 },
#endif
	TEST_c_c(casin, 0x1p-52L, 0.5L, 1.986027322597818397661816253158024383707e-16L, 4.812118250596034474977589134243772428682e-1L, 0 },
	TEST_c_c(casin, 0x1p-52L, -0.5L, 1.986027322597818397661816253158024383707e-16L, -4.812118250596034474977589134243772428682e-1L, 0 },
	TEST_c_c(casin, -0x1p-52L, 0.5L, -1.986027322597818397661816253158024383707e-16L, 4.812118250596034474977589134243772428682e-1L, 0 },
	TEST_c_c(casin, -0x1p-52L, -0.5L, -1.986027322597818397661816253158024383707e-16L, -4.812118250596034474977589134243772428682e-1L, 0 },
	TEST_c_c(casin, 0.5L, 0x1p-52L, 5.235987755982988730771072305465648369889e-1L, 2.563950248511418570403591756798643804971e-16L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, 0x1p-52L, -5.235987755982988730771072305465648369889e-1L, 2.563950248511418570403591756798643804971e-16L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.5L, -0x1p-52L, 5.235987755982988730771072305465648369889e-1L, -2.563950248511418570403591756798643804971e-16L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, -0x1p-52L, -5.235987755982988730771072305465648369889e-1L, -2.563950248511418570403591756798643804971e-16L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(casin, 0x1.fp-1025L, 0.5L, 4.819934639999230680322935210539402497827e-309L, 4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0x1.fp-1025L, -0.5L, 4.819934639999230680322935210539402497827e-309L, -4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, 0.5L, -4.819934639999230680322935210539402497827e-309L, 4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, -0.5L, -4.819934639999230680322935210539402497827e-309L, -4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0.5L, 0x1.fp-1025L, 5.235987755982988730771072305465838140329e-1L, 6.222508863508420569166420770843207333493e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, 0x1.fp-1025L, -5.235987755982988730771072305465838140329e-1L, 6.222508863508420569166420770843207333493e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.5L, -0x1.fp-1025L, 5.235987755982988730771072305465838140329e-1L, -6.222508863508420569166420770843207333493e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, -0x1.fp-1025L, -5.235987755982988730771072305465838140329e-1L, -6.222508863508420569166420770843207333493e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(casin, 0x1p-52L, 0x1p-52L, 2.220446049250313080847263336181604132852e-16L, 2.220446049250313080847263336181677117148e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0x1p-52L, -0x1p-52L, 2.220446049250313080847263336181604132852e-16L, -2.220446049250313080847263336181677117148e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -0x1p-52L, 0x1p-52L, -2.220446049250313080847263336181604132852e-16L, 2.220446049250313080847263336181677117148e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -0x1p-52L, -0x1p-52L, -2.220446049250313080847263336181604132852e-16L, -2.220446049250313080847263336181677117148e-16L, 0|XFAIL_NATIVE_LINUXF },
#ifndef TEST_FLOAT
	/* Bug 15319: underflow exception may be missing.  */
	TEST_c_c(casin, 0x1.fp-1025L, 0x1p-52L, 5.388850751072128349671657362289283247603e-309L, 2.220446049250313080847263336181622378926e-16L, UNDERFLOW_EXCEPTION_OK_DOUBLE },
	TEST_c_c(casin, 0x1.fp-1025L, -0x1p-52L, 5.388850751072128349671657362289283247603e-309L, -2.220446049250313080847263336181622378926e-16L, UNDERFLOW_EXCEPTION_OK_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, 0x1p-52L, -5.388850751072128349671657362289283247603e-309L, 2.220446049250313080847263336181622378926e-16L, UNDERFLOW_EXCEPTION_OK_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, -0x1p-52L, -5.388850751072128349671657362289283247603e-309L, -2.220446049250313080847263336181622378926e-16L, UNDERFLOW_EXCEPTION_OK_DOUBLE },
	TEST_c_c(casin, 0x1p-52L, 0x1.fp-1025L, 2.220446049250313080847263336181658871074e-16L, 5.388850751072128349671657362289548938458e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-52L, 0x1.fp-1025L, -2.220446049250313080847263336181658871074e-16L, 5.388850751072128349671657362289548938458e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-52L, -0x1.fp-1025L, 2.220446049250313080847263336181658871074e-16L, -5.388850751072128349671657362289548938458e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-52L, -0x1.fp-1025L, -2.220446049250313080847263336181658871074e-16L, -5.388850751072128349671657362289548938458e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(casin, 0.0L, 0x1p-52L, 0.0L, 2.220446049250313080847263336181622378926e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0.0L, -0x1p-52L, 0.0L, -2.220446049250313080847263336181622378926e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, -0.0L, 0x1p-52L, -0.0L, 2.220446049250313080847263336181622378926e-16L, 0|XFAIL_NATIVE_LINUXF|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, -0x1p-52L, -0.0L, -2.220446049250313080847263336181622378926e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casin, 0x1p-52L, 0.0L, 2.220446049250313080847263336181658871074e-16L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-52L, 0.0L, -2.220446049250313080847263336181658871074e-16L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-52L, -0.0L, 2.220446049250313080847263336181658871074e-16L, -0.0L, 0 },
	TEST_c_c(casin, -0x1p-52L, -0.0L, -2.220446049250313080847263336181658871074e-16L, -0.0L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(casin, 0x1.fp-1025L, 0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1.fp-1025L, -0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-1025L, 0x1.fp-1025L, -5.388850751072128349671657362289416093031e-309L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-1025L, -0x1.fp-1025L, -5.388850751072128349671657362289416093031e-309L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(casin, 0x1p-63L, 0x0.ffffffffffffffffp0L, 7.666467083416870407402049869781566856442e-20L, 8.813735870195430251942769895627079569937e-1L, 0 },
	TEST_c_c(casin, 0x1p-63L, -0x0.ffffffffffffffffp0L, 7.666467083416870407402049869781566856442e-20L, -8.813735870195430251942769895627079569937e-1L, 0 },
	TEST_c_c(casin, -0x1p-63L, 0x0.ffffffffffffffffp0L, -7.666467083416870407402049869781566856442e-20L, 8.813735870195430251942769895627079569937e-1L, 0 },
	TEST_c_c(casin, -0x1p-63L, -0x0.ffffffffffffffffp0L, -7.666467083416870407402049869781566856442e-20L, -8.813735870195430251942769895627079569937e-1L, 0 },
	TEST_c_c(casin, 0x0.ffffffffffffffffp0L, 0x1p-63L, 1.570796326376055842114524927965556247908L, 2.588578361325995866221775673638805081337e-10L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.ffffffffffffffffp0L, 0x1p-63L, -1.570796326376055842114524927965556247908L, 2.588578361325995866221775673638805081337e-10L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x0.ffffffffffffffffp0L, -0x1p-63L, 1.570796326376055842114524927965556247908L, -2.588578361325995866221775673638805081337e-10L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.ffffffffffffffffp0L, -0x1p-63L, -1.570796326376055842114524927965556247908L, -2.588578361325995866221775673638805081337e-10L, 0|XFAIL_NATIVE_LINUX },
# if LDBL_MIN_EXP <= -16381
	TEST_c_c(casin, 0x1.fp-16385L, 0x0.ffffffffffffffffp0L, 5.757683115456107044287327268904964422212e-4933L, 8.813735870195430251942769895627079569917e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0x1.fp-16385L, -0x0.ffffffffffffffffp0L, 5.757683115456107044287327268904964422212e-4933L, -8.813735870195430251942769895627079569917e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, 0x0.ffffffffffffffffp0L, -5.757683115456107044287327268904964422212e-4933L, 8.813735870195430251942769895627079569917e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, -0x0.ffffffffffffffffp0L, -5.757683115456107044287327268904964422212e-4933L, -8.813735870195430251942769895627079569917e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0x0.ffffffffffffffffp0L, 0x1.fp-16385L, 1.570796326465624365239962068302568888898L, 2.472906068161537187835415298076415423459e-4923L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.ffffffffffffffffp0L, 0x1.fp-16385L, -1.570796326465624365239962068302568888898L, 2.472906068161537187835415298076415423459e-4923L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x0.ffffffffffffffffp0L, -0x1.fp-16385L, 1.570796326465624365239962068302568888898L, -2.472906068161537187835415298076415423459e-4923L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x0.ffffffffffffffffp0L, -0x1.fp-16385L, -1.570796326465624365239962068302568888898L, -2.472906068161537187835415298076415423459e-4923L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
# endif
	TEST_c_c(casin, 0.0L, 0x0.ffffffffffffffffp0L, 0.0L, 8.813735870195430251942769895627079569917e-1L, 0 },
	TEST_c_c(casin, 0.0L, -0x0.ffffffffffffffffp0L, 0.0L, -8.813735870195430251942769895627079569917e-1L, 0 },
	TEST_c_c(casin, -0.0L, 0x0.ffffffffffffffffp0L, -0.0L, 8.813735870195430251942769895627079569917e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casin, -0.0L, -0x0.ffffffffffffffffp0L, -0.0L, -8.813735870195430251942769895627079569917e-1L, 0 },
	TEST_c_c(casin, 0x0.ffffffffffffffffp0L, 0.0L, 1.570796326465624365239962068302568888898L, 0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casin, -0x0.ffffffffffffffffp0L, 0.0L, -1.570796326465624365239962068302568888898L, 0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casin, 0x0.ffffffffffffffffp0L, -0.0L, 1.570796326465624365239962068302568888898L, -0.0L, 0 },
	TEST_c_c(casin, -0x0.ffffffffffffffffp0L, -0.0L, -1.570796326465624365239962068302568888898L, -0.0L, 0 },
#endif
	TEST_c_c(casin, 0x1p-63L, 0.5L, 9.697399036122160144833087173623140436413e-20L, 4.812118250596034474977589134243684231373e-1L, 0 },
	TEST_c_c(casin, 0x1p-63L, -0.5L, 9.697399036122160144833087173623140436413e-20L, -4.812118250596034474977589134243684231373e-1L, 0 },
	TEST_c_c(casin, -0x1p-63L, 0.5L, -9.697399036122160144833087173623140436413e-20L, 4.812118250596034474977589134243684231373e-1L, 0 },
	TEST_c_c(casin, -0x1p-63L, -0.5L, -9.697399036122160144833087173623140436413e-20L, -4.812118250596034474977589134243684231373e-1L, 0 },
	TEST_c_c(casin, 0.5L, 0x1p-63L, 5.235987755982988730771072305465838140283e-1L, 1.251928832280966098829878787499365228659e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, 0x1p-63L, -5.235987755982988730771072305465838140283e-1L, 1.251928832280966098829878787499365228659e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.5L, -0x1p-63L, 5.235987755982988730771072305465838140283e-1L, -1.251928832280966098829878787499365228659e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, -0x1p-63L, -5.235987755982988730771072305465838140283e-1L, -1.251928832280966098829878787499365228659e-19L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(casin, 0x1.fp-16385L, 0.5L, 7.282957076134209141226696333885150260319e-4933L, 4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0x1.fp-16385L, -0.5L, 7.282957076134209141226696333885150260319e-4933L, -4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, 0.5L, -7.282957076134209141226696333885150260319e-4933L, 4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, -0.5L, -7.282957076134209141226696333885150260319e-4933L, -4.812118250596034474977589134243684231352e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0.5L, 0x1.fp-16385L, 5.235987755982988730771072305465838140329e-1L, 9.402257155670418112159334875199821342890e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, 0x1.fp-16385L, -5.235987755982988730771072305465838140329e-1L, 9.402257155670418112159334875199821342890e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.5L, -0x1.fp-16385L, 5.235987755982988730771072305465838140329e-1L, -9.402257155670418112159334875199821342890e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, -0x1.fp-16385L, -5.235987755982988730771072305465838140329e-1L, -9.402257155670418112159334875199821342890e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(casin, 0x1p-63L, 0x1p-63L, 1.084202172485504434007452800869941711422e-19L, 1.084202172485504434007452800869941711430e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-63L, -0x1p-63L, 1.084202172485504434007452800869941711422e-19L, -1.084202172485504434007452800869941711430e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-63L, 0x1p-63L, -1.084202172485504434007452800869941711422e-19L, 1.084202172485504434007452800869941711430e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-63L, -0x1p-63L, -1.084202172485504434007452800869941711422e-19L, -1.084202172485504434007452800869941711430e-19L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	/* Bug 15319: underflow exception may be missing.  */
	TEST_c_c(casin, 0x1.fp-16385L, 0x1p-63L, 8.142593549724601460479922838826119584369e-4933L, 1.084202172485504434007452800869941711424e-19L, UNDERFLOW_EXCEPTION_OK },
	TEST_c_c(casin, 0x1.fp-16385L, -0x1p-63L, 8.142593549724601460479922838826119584369e-4933L, -1.084202172485504434007452800869941711424e-19L, UNDERFLOW_EXCEPTION_OK },
	TEST_c_c(casin, -0x1.fp-16385L, 0x1p-63L, -8.142593549724601460479922838826119584369e-4933L, 1.084202172485504434007452800869941711424e-19L, UNDERFLOW_EXCEPTION_OK },
	TEST_c_c(casin, -0x1.fp-16385L, -0x1p-63L, -8.142593549724601460479922838826119584369e-4933L, -1.084202172485504434007452800869941711424e-19L, UNDERFLOW_EXCEPTION_OK },
	TEST_c_c(casin, 0x1p-63L, 0x1.fp-16385L, 1.084202172485504434007452800869941711428e-19L, 8.142593549724601460479922838826119584465e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-63L, 0x1.fp-16385L, -1.084202172485504434007452800869941711428e-19L, 8.142593549724601460479922838826119584465e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-63L, -0x1.fp-16385L, 1.084202172485504434007452800869941711428e-19L, -8.142593549724601460479922838826119584465e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-63L, -0x1.fp-16385L, -1.084202172485504434007452800869941711428e-19L, -8.142593549724601460479922838826119584465e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(casin, 0.0L, 0x1p-63L, 0.0L, 1.084202172485504434007452800869941711424e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.0L, -0x1p-63L, 0.0L, -1.084202172485504434007452800869941711424e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, 0x1p-63L, -0.0L, 1.084202172485504434007452800869941711424e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, -0x1p-63L, -0.0L, -1.084202172485504434007452800869941711424e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-63L, 0.0L, 1.084202172485504434007452800869941711428e-19L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-63L, 0.0L, -1.084202172485504434007452800869941711428e-19L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-63L, -0.0L, 1.084202172485504434007452800869941711428e-19L, -0.0L, 0 },
	TEST_c_c(casin, -0x1p-63L, -0.0L, -1.084202172485504434007452800869941711428e-19L, -0.0L, 0 },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(casin, 0x1.fp-16385L, 0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1.fp-16385L, -0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-16385L, 0x1.fp-16385L, -8.142593549724601460479922838826119584417e-4933L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-16385L, -0x1.fp-16385L, -8.142593549724601460479922838826119584417e-4933L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(casin, 0x1p-105L, 0x0.ffffffffffffffffffffffffffcp0L, 1.743152798421049375791483240187377443526e-32L, 8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(casin, 0x1p-105L, -0x0.ffffffffffffffffffffffffffcp0L, 1.743152798421049375791483240187377443526e-32L, -8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(casin, -0x1p-105L, 0x0.ffffffffffffffffffffffffffcp0L, -1.743152798421049375791483240187377443526e-32L, 8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(casin, -0x1p-105L, -0x0.ffffffffffffffffffffffffffcp0L, -1.743152798421049375791483240187377443526e-32L, -8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffcp0L, 0x1p-105L, 1.570796326794896419512475791677915736035L, 1.234330349600789959989661887846005659983e-16L, 0 },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffcp0L, 0x1p-105L, -1.570796326794896419512475791677915736035L, 1.234330349600789959989661887846005659983e-16L, 0 },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffcp0L, -0x1p-105L, 1.570796326794896419512475791677915736035L, -1.234330349600789959989661887846005659983e-16L, 0 },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffcp0L, -0x1p-105L, -1.570796326794896419512475791677915736035L, -1.234330349600789959989661887846005659983e-16L, 0 },
	TEST_c_c(casin, 0x1.fp-1025L, 0x0.ffffffffffffffffffffffffffcp0L, 3.810492908885321743133304375216641110205e-309L, 8.813735870195430252326093249797835932642e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0x1.fp-1025L, -0x0.ffffffffffffffffffffffffffcp0L, 3.810492908885321743133304375216641110205e-309L, -8.813735870195430252326093249797835932642e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, 0x0.ffffffffffffffffffffffffffcp0L, -3.810492908885321743133304375216641110205e-309L, 8.813735870195430252326093249797835932642e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, -0x1.fp-1025L, -0x0.ffffffffffffffffffffffffffcp0L, -3.810492908885321743133304375216641110205e-309L, -8.813735870195430252326093249797835932642e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffcp0L, 0x1.fp-1025L, 1.570796326794896462222075823262245502760L, 3.432186888910770473784103112009399360940e-293L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffcp0L, 0x1.fp-1025L, -1.570796326794896462222075823262245502760L, 3.432186888910770473784103112009399360940e-293L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffcp0L, -0x1.fp-1025L, 1.570796326794896462222075823262245502760L, -3.432186888910770473784103112009399360940e-293L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffcp0L, -0x1.fp-1025L, -1.570796326794896462222075823262245502760L, -3.432186888910770473784103112009399360940e-293L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casin, 0.0L, 0x0.ffffffffffffffffffffffffffcp0L, 0.0L, 8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(casin, 0.0L, -0x0.ffffffffffffffffffffffffffcp0L, 0.0L, -8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(casin, -0.0L, 0x0.ffffffffffffffffffffffffffcp0L, -0.0L, 8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(casin, -0.0L, -0x0.ffffffffffffffffffffffffffcp0L, -0.0L, -8.813735870195430252326093249797835932642e-1L, 0 },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffcp0L, 0.0L, 1.570796326794896462222075823262245502760L, 0.0L, 0 },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffcp0L, 0.0L, -1.570796326794896462222075823262245502760L, 0.0L, 0 },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffcp0L, -0.0L, 1.570796326794896462222075823262245502760L, -0.0L, 0 },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffcp0L, -0.0L, -1.570796326794896462222075823262245502760L, -0.0L, 0 },
#endif
	TEST_c_c(casin, 0x1p-105L, 0.5L, 2.204933261082751447436996198490110245167e-32L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(casin, 0x1p-105L, -0.5L, 2.204933261082751447436996198490110245167e-32L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(casin, -0x1p-105L, 0.5L, -2.204933261082751447436996198490110245167e-32L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(casin, -0x1p-105L, -0.5L, -2.204933261082751447436996198490110245167e-32L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(casin, 0.5L, 0x1p-105L, 5.235987755982988730771072305465838140329e-1L, 2.846556599890768890940941645338546615370e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, 0x1p-105L, -5.235987755982988730771072305465838140329e-1L, 2.846556599890768890940941645338546615370e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.5L, -0x1p-105L, 5.235987755982988730771072305465838140329e-1L, -2.846556599890768890940941645338546615370e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, -0x1p-105L, -5.235987755982988730771072305465838140329e-1L, -2.846556599890768890940941645338546615370e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-105L, 0x1p-105L, 2.465190328815661891911651766508706967729e-32L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-105L, -0x1p-105L, 2.465190328815661891911651766508706967729e-32L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-105L, 0x1p-105L, -2.465190328815661891911651766508706967729e-32L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-105L, -0x1p-105L, -2.465190328815661891911651766508706967729e-32L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	/* Bug 15319: underflow exception may be missing.  */
	TEST_c_c(casin, 0x1.fp-1025L, 0x1p-105L, 5.388850751072128349671657362289416093031e-309L, 2.465190328815661891911651766508706967729e-32L, UNDERFLOW_EXCEPTION_OK_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1.fp-1025L, -0x1p-105L, 5.388850751072128349671657362289416093031e-309L, -2.465190328815661891911651766508706967729e-32L, UNDERFLOW_EXCEPTION_OK_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-1025L, 0x1p-105L, -5.388850751072128349671657362289416093031e-309L, 2.465190328815661891911651766508706967729e-32L, UNDERFLOW_EXCEPTION_OK_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-1025L, -0x1p-105L, -5.388850751072128349671657362289416093031e-309L, -2.465190328815661891911651766508706967729e-32L, UNDERFLOW_EXCEPTION_OK_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-105L, 0x1.fp-1025L, 2.465190328815661891911651766508706967729e-32L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-105L, 0x1.fp-1025L, -2.465190328815661891911651766508706967729e-32L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-105L, -0x1.fp-1025L, 2.465190328815661891911651766508706967729e-32L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-105L, -0x1.fp-1025L, -2.465190328815661891911651766508706967729e-32L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(casin, 0.0L, 0x1p-105L, 0.0L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.0L, -0x1p-105L, 0.0L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, 0x1p-105L, -0.0L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, -0x1p-105L, -0.0L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-105L, 0.0L, 2.465190328815661891911651766508706967729e-32L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-105L, 0.0L, -2.465190328815661891911651766508706967729e-32L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-105L, -0.0L, 2.465190328815661891911651766508706967729e-32L, -0.0L, 0 },
	TEST_c_c(casin, -0x1p-105L, -0.0L, -2.465190328815661891911651766508706967729e-32L, -0.0L, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(casin, 0x1p-112L, 0x0.ffffffffffffffffffffffffffff8p0L, 1.361838123766444824837096281396380300349e-34L, 8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(casin, 0x1p-112L, -0x0.ffffffffffffffffffffffffffff8p0L, 1.361838123766444824837096281396380300349e-34L, -8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(casin, -0x1p-112L, 0x0.ffffffffffffffffffffffffffff8p0L, -1.361838123766444824837096281396380300349e-34L, 8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(casin, -0x1p-112L, -0x0.ffffffffffffffffffffffffffff8p0L, -1.361838123766444824837096281396380300349e-34L, -8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-112L, 1.570796326794896601578502908312986698624L, 1.091004200533850618345330909857103457781e-17L, 0 },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-112L, -1.570796326794896601578502908312986698624L, 1.091004200533850618345330909857103457781e-17L, 0 },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-112L, 1.570796326794896601578502908312986698624L, -1.091004200533850618345330909857103457781e-17L, 0 },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-112L, -1.570796326794896601578502908312986698624L, -1.091004200533850618345330909857103457781e-17L, 0 },
	TEST_c_c(casin, 0x1.fp-16385L, 0x0.ffffffffffffffffffffffffffff8p0L, 5.757683115456107044131264955348449231681e-4933L, 8.813735870195430252326093249797922409363e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0x1.fp-16385L, -0x0.ffffffffffffffffffffffffffff8p0L, 5.757683115456107044131264955348449231681e-4933L, -8.813735870195430252326093249797922409363e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, 0x0.ffffffffffffffffffffffffffff8p0L, -5.757683115456107044131264955348449231681e-4933L, 8.813735870195430252326093249797922409363e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, -0x1.fp-16385L, -0x0.ffffffffffffffffffffffffffff8p0L, -5.757683115456107044131264955348449231681e-4933L, -8.813735870195430252326093249797922409363e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1.fp-16385L, 1.570796326794896605353533883825294686803L, 5.867357004219059108790368268900490184505e-4916L, 0 },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1.fp-16385L, -1.570796326794896605353533883825294686803L, 5.867357004219059108790368268900490184505e-4916L, 0 },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1.fp-16385L, 1.570796326794896605353533883825294686803L, -5.867357004219059108790368268900490184505e-4916L, 0 },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1.fp-16385L, -1.570796326794896605353533883825294686803L, -5.867357004219059108790368268900490184505e-4916L, 0 },
	TEST_c_c(casin, 0.0L, 0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, 8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(casin, 0.0L, -0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, -8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(casin, -0.0L, 0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, 8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(casin, -0.0L, -0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, -8.813735870195430252326093249797922409363e-1L, 0 },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, 1.570796326794896605353533883825294686803L, 0.0L, 0 },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, -1.570796326794896605353533883825294686803L, 0.0L, 0 },
	TEST_c_c(casin, 0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, 1.570796326794896605353533883825294686803L, -0.0L, 0 },
	TEST_c_c(casin, -0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, -1.570796326794896605353533883825294686803L, -0.0L, 0 },
#endif
	TEST_c_c(casin, 0x1p-112L, 0.5L, 1.722604110220899568310153280070398629037e-34L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(casin, 0x1p-112L, -0.5L, 1.722604110220899568310153280070398629037e-34L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(casin, -0x1p-112L, 0.5L, -1.722604110220899568310153280070398629037e-34L, 4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(casin, -0x1p-112L, -0.5L, -1.722604110220899568310153280070398629037e-34L, -4.812118250596034474977589134243684231352e-1L, 0 },
	TEST_c_c(casin, 0.5L, 0x1p-112L, 5.235987755982988730771072305465838140329e-1L, 2.223872343664663196047610660420739543258e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, 0x1p-112L, -5.235987755982988730771072305465838140329e-1L, 2.223872343664663196047610660420739543258e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.5L, -0x1p-112L, 5.235987755982988730771072305465838140329e-1L, -2.223872343664663196047610660420739543258e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.5L, -0x1p-112L, -5.235987755982988730771072305465838140329e-1L, -2.223872343664663196047610660420739543258e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-112L, 0x1p-112L, 1.925929944387235853055977942584927318538e-34L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-112L, -0x1p-112L, 1.925929944387235853055977942584927318538e-34L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-112L, 0x1p-112L, -1.925929944387235853055977942584927318538e-34L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-112L, -0x1p-112L, -1.925929944387235853055977942584927318538e-34L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	/* Bug 15319: underflow exception may be missing.  */
	TEST_c_c(casin, 0x1.fp-16385L, 0x1p-112L, 8.142593549724601460479922838826119584417e-4933L, 1.925929944387235853055977942584927318538e-34L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1.fp-16385L, -0x1p-112L, 8.142593549724601460479922838826119584417e-4933L, -1.925929944387235853055977942584927318538e-34L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-16385L, 0x1p-112L, -8.142593549724601460479922838826119584417e-4933L, 1.925929944387235853055977942584927318538e-34L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1.fp-16385L, -0x1p-112L, -8.142593549724601460479922838826119584417e-4933L, -1.925929944387235853055977942584927318538e-34L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-112L, 0x1.fp-16385L, 1.925929944387235853055977942584927318538e-34L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-112L, 0x1.fp-16385L, -1.925929944387235853055977942584927318538e-34L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-112L, -0x1.fp-16385L, 1.925929944387235853055977942584927318538e-34L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-112L, -0x1.fp-16385L, -1.925929944387235853055977942584927318538e-34L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(casin, 0.0L, 0x1p-112L, 0.0L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0.0L, -0x1p-112L, 0.0L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, 0x1p-112L, -0.0L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0.0L, -0x1p-112L, -0.0L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-112L, 0.0L, 1.925929944387235853055977942584927318538e-34L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, -0x1p-112L, 0.0L, -1.925929944387235853055977942584927318538e-34L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casin, 0x1p-112L, -0.0L, 1.925929944387235853055977942584927318538e-34L, -0.0L, 0 },
	TEST_c_c(casin, -0x1p-112L, -0.0L, -1.925929944387235853055977942584927318538e-34L, -0.0L, 0 },

	TEST_c_c(casin, 0.75L, 1.25L, 0.453276177638793913448921196101971749L, 1.13239363160530819522266333696834467L, 0 },
	TEST_c_c(casin, -2, -3, -0.57065278432109940071028387968566963L, -1.9833870299165354323470769028940395L, 0 },
}


static void test_casinh(void)
{
	struct test *testp = lookup("casinh" },

	TEST_c_c(casinh, 0, 0, 0.0, 0.0, 0 },
	TEST_c_c(casinh, ZERO_M, 0, ZERO_M, 0, 0 },
	TEST_c_c(casinh, 0, ZERO_M, 0.0, ZERO_M, 0 },
	TEST_c_c(casinh, ZERO_M, ZERO_M, ZERO_M, ZERO_M, 0 },

	TEST_c_c(casinh, INF_P, INF_P, INF_P, M_PI_4l, 0 },
	TEST_c_c(casinh, INF_P, INF_M, INF_P, -M_PI_4l, 0 },
	TEST_c_c(casinh, INF_M, INF_P, INF_M, M_PI_4l, 0 },
	TEST_c_c(casinh, INF_M, INF_M, INF_M, -M_PI_4l, 0 },

	TEST_c_c(casinh, -10.0, INF_P, INF_M, M_PI_2l, 0 },
	TEST_c_c(casinh, -10.0, INF_M, INF_M, -M_PI_2l, 0 },
	TEST_c_c(casinh, 0, INF_P, INF_P, M_PI_2l, 0 },
	TEST_c_c(casinh, 0, INF_M, INF_P, -M_PI_2l, 0 },
	TEST_c_c(casinh, ZERO_M, INF_P, INF_M, M_PI_2l, 0 },
	TEST_c_c(casinh, ZERO_M, INF_M, INF_M, -M_PI_2l, 0 },
	TEST_c_c(casinh, 0.1L, INF_P, INF_P, M_PI_2l, 0 },
	TEST_c_c(casinh, 0.1L, INF_M, INF_P, -M_PI_2l, 0 },

	TEST_c_c(casinh, INF_M, 0, INF_M, 0.0, 0 },
	TEST_c_c(casinh, INF_M, ZERO_M, INF_M, ZERO_M, 0 },
	TEST_c_c(casinh, INF_M, 100, INF_M, 0.0, 0 },
	TEST_c_c(casinh, INF_M, -100, INF_M, ZERO_M, 0 },

	TEST_c_c(casinh, INF_P, 0, INF_P, 0.0, 0 },
	TEST_c_c(casinh, INF_P, ZERO_M, INF_P, ZERO_M, 0 },
	TEST_c_c(casinh, INF_P, 0.5, INF_P, 0.0, 0 },
	TEST_c_c(casinh, INF_P, -0.5, INF_P, ZERO_M, 0 },

	TEST_c_c(casinh, INF_P, QNAN_P, INF_P, QNAN_P, 0 },
	TEST_c_c(casinh, INF_M, QNAN_P, INF_M, QNAN_P, 0 },

	TEST_c_c(casinh, QNAN_P, 0, QNAN_P, 0.0, 0 },
	TEST_c_c(casinh, QNAN_P, ZERO_M, QNAN_P, ZERO_M, 0 },

	TEST_c_c(casinh, QNAN_P, INF_P, INF_P, QNAN_P, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(casinh, QNAN_P, INF_M, INF_P, QNAN_P, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(casinh, 10.5, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(casinh, -10.5, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(casinh, QNAN_P, 0.75, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(casinh, -0.75, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(casinh, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(casinh, ZERO_P, -1.5L, 0.9624236501192068949955178268487368462704L, -M_PI_2l, 0 },
	TEST_c_c(casinh, ZERO_M, -1.5L, -0.9624236501192068949955178268487368462704L, -M_PI_2l, 0 },
	TEST_c_c(casinh, ZERO_P, -1.0L, ZERO_P, -M_PI_2l, 0 },
	TEST_c_c(casinh, ZERO_M, -1.0L, ZERO_M, -M_PI_2l, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, ZERO_P, -0.5L, ZERO_P, -0.5235987755982988730771072305465838140329L, 0 },
	TEST_c_c(casinh, ZERO_M, -0.5L, ZERO_M, -0.5235987755982988730771072305465838140329L, 0|XFAIL_NATIVE_LINUXL|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, ZERO_P, 0.5L, ZERO_P, 0.5235987755982988730771072305465838140329L, 0 },
	TEST_c_c(casinh, ZERO_M, 0.5L, ZERO_M, 0.5235987755982988730771072305465838140329L, 0|XFAIL_NATIVE_LINUXL|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, ZERO_P, 1.0L, ZERO_P, M_PI_2l, 0 },
	TEST_c_c(casinh, ZERO_M, 1.0L, ZERO_M, M_PI_2l, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, ZERO_P, 1.5L, 0.9624236501192068949955178268487368462704L, M_PI_2l, 0 },
	TEST_c_c(casinh, ZERO_M, 1.5L, -0.9624236501192068949955178268487368462704L, M_PI_2l, 0 },

	TEST_c_c(casinh, -1.5L, ZERO_P, -1.194763217287109304111930828519090523536L, ZERO_P, 0 },
	TEST_c_c(casinh, -1.5L, ZERO_M, -1.194763217287109304111930828519090523536L, ZERO_M, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -1.0L, ZERO_P, -0.8813735870195430252326093249797923090282L, ZERO_P, 0 },
	TEST_c_c(casinh, -1.0L, ZERO_M, -0.8813735870195430252326093249797923090282L, ZERO_M, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0.5L, ZERO_P, -0.4812118250596034474977589134243684231352L, ZERO_P, 0 },
	TEST_c_c(casinh, -0.5L, ZERO_M, -0.4812118250596034474977589134243684231352L, ZERO_M, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0.5L, ZERO_P, 0.4812118250596034474977589134243684231352L, ZERO_P, 0 },
	TEST_c_c(casinh, 0.5L, ZERO_M, 0.4812118250596034474977589134243684231352L, ZERO_M, 0 },
	TEST_c_c(casinh, 1.0L, ZERO_P, 0.8813735870195430252326093249797923090282L, ZERO_P, 0 },
	TEST_c_c(casinh, 1.0L, ZERO_M, 0.8813735870195430252326093249797923090282L, ZERO_M, 0 },
	TEST_c_c(casinh, 1.5L, ZERO_P, 1.194763217287109304111930828519090523536L, ZERO_P, 0 },
	TEST_c_c(casinh, 1.5L, ZERO_M, 1.194763217287109304111930828519090523536L, ZERO_M, 0 },

	TEST_c_c(casinh, 0x1p50L, 1.0L, 3.535050620855721078027883819436759661753e1L, 8.881784197001252323389053344720723756399e-16L, 0 },
	TEST_c_c(casinh, 0x1p50L, -1.0L, 3.535050620855721078027883819436759661753e1L, -8.881784197001252323389053344720723756399e-16L, 0 },
	TEST_c_c(casinh, -0x1p50L, 1.0L, -3.535050620855721078027883819436759661753e1L, 8.881784197001252323389053344720723756399e-16L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p50L, -1.0L, -3.535050620855721078027883819436759661753e1L, -8.881784197001252323389053344720723756399e-16L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 1.0L, 0x1p50L, 3.535050620855721078027883819436720218708e1L, 1.570796326794895731052901991514519103193L, 0 },
	TEST_c_c(casinh, -1.0L, 0x1p50L, -3.535050620855721078027883819436720218708e1L, 1.570796326794895731052901991514519103193L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 1.0L, -0x1p50L, 3.535050620855721078027883819436720218708e1L, -1.570796326794895731052901991514519103193L, 0 },
	TEST_c_c(casinh, -1.0L, -0x1p50L, -3.535050620855721078027883819436720218708e1L, -1.570796326794895731052901991514519103193L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(casinh, 0x1p500L, 1.0L, 3.472667374605326000180332928505464606058e2L, 3.054936363499604682051979393213617699789e-151L, 0 },
	TEST_c_c(casinh, 0x1p500L, -1.0L, 3.472667374605326000180332928505464606058e2L, -3.054936363499604682051979393213617699789e-151L, 0 },
	TEST_c_c(casinh, -0x1p500L, 1.0L, -3.472667374605326000180332928505464606058e2L, 3.054936363499604682051979393213617699789e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p500L, -1.0L, -3.472667374605326000180332928505464606058e2L, -3.054936363499604682051979393213617699789e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 1.0L, 0x1p500L, 3.472667374605326000180332928505464606058e2L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -1.0L, 0x1p500L, -3.472667374605326000180332928505464606058e2L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 1.0L, -0x1p500L, 3.472667374605326000180332928505464606058e2L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -1.0L, -0x1p500L, -3.472667374605326000180332928505464606058e2L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(casinh, 0x1p5000L, 1.0L, 3.466429049980286492395577839412341016946e3L, 7.079811261048172892385615158694057552948e-1506L, 0 },
	TEST_c_c(casinh, 0x1p5000L, -1.0L, 3.466429049980286492395577839412341016946e3L, -7.079811261048172892385615158694057552948e-1506L, 0 },
	TEST_c_c(casinh, -0x1p5000L, 1.0L, -3.466429049980286492395577839412341016946e3L, 7.079811261048172892385615158694057552948e-1506L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1p5000L, -1.0L, -3.466429049980286492395577839412341016946e3L, -7.079811261048172892385615158694057552948e-1506L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 1.0L, 0x1p5000L, 3.466429049980286492395577839412341016946e3L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -1.0L, 0x1p5000L, -3.466429049980286492395577839412341016946e3L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 1.0L, -0x1p5000L, 3.466429049980286492395577839412341016946e3L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -1.0L, -0x1p5000L, -3.466429049980286492395577839412341016946e3L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(casinh, 0x1.fp127L, 0x1.fp127L, 8.973081118419833726837456344608533993585e1L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXF },
#ifndef TEST_FLOAT
	TEST_c_c(casinh, 0x1.fp1023L, 0x1.fp1023L, 7.107906849659093345062145442726115449315e2L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(casinh, 0x1.fp16383L, 0x1.fp16383L, 1.135753137836666928715489992987020363057e4L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(casinh, 0x1.fp-129L, 1.5L, 9.624236501192068949955178268487368462704e-1L, 1.570796326794896619231321691639751442096L, 0 },
	TEST_c_c(casinh, 0x1.fp-129L, -1.5L, 9.624236501192068949955178268487368462704e-1L, -1.570796326794896619231321691639751442096L, 0 },
	TEST_c_c(casinh, -0x1.fp-129L, 1.5L, -9.624236501192068949955178268487368462704e-1L, 1.570796326794896619231321691639751442096L, 0 },
	TEST_c_c(casinh, -0x1.fp-129L, -1.5L, -9.624236501192068949955178268487368462704e-1L, -1.570796326794896619231321691639751442096L, 0 },
	TEST_c_c(casinh, 1.5L, 0x1.fp-129L, 1.194763217287109304111930828519090523536L, 1.579176199917649005841160751101628985741e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, -1.5L, 0x1.fp-129L, -1.194763217287109304111930828519090523536L, 1.579176199917649005841160751101628985741e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, 1.5L, -0x1.fp-129L, 1.194763217287109304111930828519090523536L, -1.579176199917649005841160751101628985741e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, -1.5L, -0x1.fp-129L, -1.194763217287109304111930828519090523536L, -1.579176199917649005841160751101628985741e-39L, UNDERFLOW_EXCEPTION_FLOAT },
#ifndef TEST_FLOAT
	TEST_c_c(casinh, 0x1.fp-1025L, 1.5L, 9.624236501192068949955178268487368462704e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.fp-1025L, -1.5L, 9.624236501192068949955178268487368462704e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-1025L, 1.5L, -9.624236501192068949955178268487368462704e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-1025L, -1.5L, -9.624236501192068949955178268487368462704e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 1.5L, 0x1.fp-1025L, 1.194763217287109304111930828519090523536L, 2.989196569048182929051881765490354365918e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -1.5L, 0x1.fp-1025L, -1.194763217287109304111930828519090523536L, 2.989196569048182929051881765490354365918e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, 1.5L, -0x1.fp-1025L, 1.194763217287109304111930828519090523536L, -2.989196569048182929051881765490354365918e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -1.5L, -0x1.fp-1025L, -1.194763217287109304111930828519090523536L, -2.989196569048182929051881765490354365918e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
#endif
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(casinh, 0x1.fp-16385L, 1.5L, 9.624236501192068949955178268487368462704e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.fp-16385L, -1.5L, 9.624236501192068949955178268487368462704e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-16385L, 1.5L, -9.624236501192068949955178268487368462704e-1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-16385L, -1.5L, -9.624236501192068949955178268487368462704e-1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 1.5L, 0x1.fp-16385L, 1.194763217287109304111930828519090523536L, 4.516698239814521372306784062043266700598e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -1.5L, 0x1.fp-16385L, -1.194763217287109304111930828519090523536L, 4.516698239814521372306784062043266700598e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, 1.5L, -0x1.fp-16385L, 1.194763217287109304111930828519090523536L, -4.516698239814521372306784062043266700598e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -1.5L, -0x1.fp-16385L, -1.194763217287109304111930828519090523536L, -4.516698239814521372306784062043266700598e-4933L, UNDERFLOW_EXCEPTION },
#endif

	TEST_c_c(casinh, 0.5L, 1.0L, 7.328576759736452608886724437653071523305e-1L, 8.959074812088902390666567243275770102229e-1L, 0 },
	TEST_c_c(casinh, 0.5L, -1.0L, 7.328576759736452608886724437653071523305e-1L, -8.959074812088902390666567243275770102229e-1L, 0 },
	TEST_c_c(casinh, -0.5L, 1.0L, -7.328576759736452608886724437653071523305e-1L, 8.959074812088902390666567243275770102229e-1L, 0 },
	TEST_c_c(casinh, -0.5L, -1.0L, -7.328576759736452608886724437653071523305e-1L, -8.959074812088902390666567243275770102229e-1L, 0 },
	TEST_c_c(casinh, 1.0L, 0.5L, 9.261330313501824245501244453057873152694e-1L, 3.494390628572132936274118260753699527325e-1L, 0 },
	TEST_c_c(casinh, -1.0L, 0.5L, -9.261330313501824245501244453057873152694e-1L, 3.494390628572132936274118260753699527325e-1L, 0 },
	TEST_c_c(casinh, 1.0L, -0.5L, 9.261330313501824245501244453057873152694e-1L, -3.494390628572132936274118260753699527325e-1L, 0 },
	TEST_c_c(casinh, -1.0L, -0.5L, -9.261330313501824245501244453057873152694e-1L, -3.494390628572132936274118260753699527325e-1L, 0 },
	TEST_c_c(casinh, 0.25L, 1.0L, 5.097911466811016354623559941115413499164e-1L, 1.081751996523816326311037318425097434186L, 0 },
	TEST_c_c(casinh, 0.25L, -1.0L, 5.097911466811016354623559941115413499164e-1L, -1.081751996523816326311037318425097434186L, 0 },
	TEST_c_c(casinh, -0.25L, 1.0L, -5.097911466811016354623559941115413499164e-1L, 1.081751996523816326311037318425097434186L, 0 },
	TEST_c_c(casinh, -0.25L, -1.0L, -5.097911466811016354623559941115413499164e-1L, -1.081751996523816326311037318425097434186L, 0 },
	TEST_c_c(casinh, 1.0L, 0.25L, 8.924633639033482359562124741744951972772e-1L, 1.763024327769669304186785666360901026468e-1L, 0 },
	TEST_c_c(casinh, -1.0L, 0.25L, -8.924633639033482359562124741744951972772e-1L, 1.763024327769669304186785666360901026468e-1L, 0 },
	TEST_c_c(casinh, 1.0L, -0.25L, 8.924633639033482359562124741744951972772e-1L, -1.763024327769669304186785666360901026468e-1L, 0 },
	TEST_c_c(casinh, -1.0L, -0.25L, -8.924633639033482359562124741744951972772e-1L, -1.763024327769669304186785666360901026468e-1L, 0 },
	TEST_c_c(casinh, 0x1.fp-10L, 1.0L, 4.350501469856803800217957402220976497152e-2L, 1.527305029163877791518741192097931722508L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 0x1.fp-10L, -1.0L, 4.350501469856803800217957402220976497152e-2L, -1.527305029163877791518741192097931722508L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1.fp-10L, 1.0L, -4.350501469856803800217957402220976497152e-2L, 1.527305029163877791518741192097931722508L, 0 },
	TEST_c_c(casinh, -0x1.fp-10L, -1.0L, -4.350501469856803800217957402220976497152e-2L, -1.527305029163877791518741192097931722508L, 0 },
	TEST_c_c(casinh, 1.0L, 0x1.fp-10L, 8.813742198809567991336704287826445879025e-1L, 1.337909359557740912558349531052239112857e-3L, 0 },
	TEST_c_c(casinh, -1.0L, 0x1.fp-10L, -8.813742198809567991336704287826445879025e-1L, 1.337909359557740912558349531052239112857e-3L, 0 },
	TEST_c_c(casinh, 1.0L, -0x1.fp-10L, 8.813742198809567991336704287826445879025e-1L, -1.337909359557740912558349531052239112857e-3L, 0 },
	TEST_c_c(casinh, -1.0L, -0x1.fp-10L, -8.813742198809567991336704287826445879025e-1L, -1.337909359557740912558349531052239112857e-3L, 0 },
	TEST_c_c(casinh, 0x1.fp-30L, 1.0L, 4.247867098745151888768727039216644758847e-5L, 1.570753848123921942730162693731872690232L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 0x1.fp-30L, -1.0L, 4.247867098745151888768727039216644758847e-5L, -1.570753848123921942730162693731872690232L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1.fp-30L, 1.0L, -4.247867098745151888768727039216644758847e-5L, 1.570753848123921942730162693731872690232L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1.fp-30L, -1.0L, -4.247867098745151888768727039216644758847e-5L, -1.570753848123921942730162693731872690232L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 1.0L, 0x1.fp-30L, 8.813735870195430258081932989769495326854e-1L, 1.275929984216518540252717485007112529021e-9L, 0 },
	TEST_c_c(casinh, -1.0L, 0x1.fp-30L, -8.813735870195430258081932989769495326854e-1L, 1.275929984216518540252717485007112529021e-9L, 0 },
	TEST_c_c(casinh, 1.0L, -0x1.fp-30L, 8.813735870195430258081932989769495326854e-1L, -1.275929984216518540252717485007112529021e-9L, 0 },
	TEST_c_c(casinh, -1.0L, -0x1.fp-30L, -8.813735870195430258081932989769495326854e-1L, -1.275929984216518540252717485007112529021e-9L, 0 },
	TEST_c_c(casinh, 0x1.fp-100L, 1.0L, 1.236292038260260888664514866457202186027e-15L, 1.570796326794895382939283431378862777584L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-100L, -1.0L, 1.236292038260260888664514866457202186027e-15L, -1.570796326794895382939283431378862777584L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-100L, 1.0L, -1.236292038260260888664514866457202186027e-15L, 1.570796326794895382939283431378862777584L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-100L, -1.0L, -1.236292038260260888664514866457202186027e-15L, -1.570796326794895382939283431378862777584L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 1.0L, 0x1.fp-100L, 8.813735870195430252326093249797923090282e-1L, 1.080754735021050612990719608916167354321e-30L, 0 },
	TEST_c_c(casinh, -1.0L, 0x1.fp-100L, -8.813735870195430252326093249797923090282e-1L, 1.080754735021050612990719608916167354321e-30L, 0 },
	TEST_c_c(casinh, 1.0L, -0x1.fp-100L, 8.813735870195430252326093249797923090282e-1L, -1.080754735021050612990719608916167354321e-30L, 0 },
	TEST_c_c(casinh, -1.0L, -0x1.fp-100L, -8.813735870195430252326093249797923090282e-1L, -1.080754735021050612990719608916167354321e-30L, 0 },
	TEST_c_c(casinh, 0x1.fp-129L, 1.0L, 5.335635276982233498398987585285818977933e-20L, 1.570796326794896619177965338869929107115L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-129L, -1.0L, 5.335635276982233498398987585285818977933e-20L, -1.570796326794896619177965338869929107115L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-129L, 1.0L, -5.335635276982233498398987585285818977933e-20L, 1.570796326794896619177965338869929107115L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-129L, -1.0L, -5.335635276982233498398987585285818977933e-20L, -1.570796326794896619177965338869929107115L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 1.0L, 0x1.fp-129L, 8.813735870195430252326093249797923090282e-1L, 2.013062564695348242280482517399205554874e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, -1.0L, 0x1.fp-129L, -8.813735870195430252326093249797923090282e-1L, 2.013062564695348242280482517399205554874e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 1.0L, -0x1.fp-129L, 8.813735870195430252326093249797923090282e-1L, -2.013062564695348242280482517399205554874e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, -1.0L, -0x1.fp-129L, -8.813735870195430252326093249797923090282e-1L, -2.013062564695348242280482517399205554874e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },
#ifndef TEST_FLOAT
	TEST_c_c(casinh, 0x1.fp-1000L, 1.0L, 4.252291453851660175550490409247739011867e-151L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-1000L, -1.0L, 4.252291453851660175550490409247739011867e-151L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1000L, 1.0L, -4.252291453851660175550490409247739011867e-151L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1000L, -1.0L, -4.252291453851660175550490409247739011867e-151L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 1.0L, 0x1.fp-1000L, 8.813735870195430252326093249797923090282e-1L, 1.278589251976747242280879285935084814093e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casinh, -1.0L, 0x1.fp-1000L, -8.813735870195430252326093249797923090282e-1L, 1.278589251976747242280879285935084814093e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casinh, 1.0L, -0x1.fp-1000L, 8.813735870195430252326093249797923090282e-1L, -1.278589251976747242280879285935084814093e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casinh, -1.0L, -0x1.fp-1000L, -8.813735870195430252326093249797923090282e-1L, -1.278589251976747242280879285935084814093e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casinh, 0x1.fp-1025L, 1.0L, 7.340879205566679497036857179189356754017e-155L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-1025L, -1.0L, 7.340879205566679497036857179189356754017e-155L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, 1.0L, -7.340879205566679497036857179189356754017e-155L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, -1.0L, -7.340879205566679497036857179189356754017e-155L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 1.0L, 0x1.fp-1025L, 8.813735870195430252326093249797923090282e-1L, 3.810492908885321743133304375216617626230e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -1.0L, 0x1.fp-1025L, -8.813735870195430252326093249797923090282e-1L, 3.810492908885321743133304375216617626230e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, 1.0L, -0x1.fp-1025L, 8.813735870195430252326093249797923090282e-1L, -3.810492908885321743133304375216617626230e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -1.0L, -0x1.fp-1025L, -8.813735870195430252326093249797923090282e-1L, -3.810492908885321743133304375216617626230e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
#endif
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(casinh, 0x1.fp-10000L, 1.0L, 9.854680208706673586644342922051388714633e-1506L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x1.fp-10000L, -1.0L, 9.854680208706673586644342922051388714633e-1506L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-10000L, 1.0L, -9.854680208706673586644342922051388714633e-1506L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-10000L, -1.0L, -9.854680208706673586644342922051388714633e-1506L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 1.0L, 0x1.fp-10000L, 8.813735870195430252326093249797923090282e-1L, 6.867047849047171855399183659351043150871e-3011L, 0 },
	TEST_c_c(casinh, -1.0L, 0x1.fp-10000L, -8.813735870195430252326093249797923090282e-1L, 6.867047849047171855399183659351043150871e-3011L, 0 },
	TEST_c_c(casinh, 1.0L, -0x1.fp-10000L, 8.813735870195430252326093249797923090282e-1L, -6.867047849047171855399183659351043150871e-3011L, 0 },
	TEST_c_c(casinh, -1.0L, -0x1.fp-10000L, -8.813735870195430252326093249797923090282e-1L, -6.867047849047171855399183659351043150871e-3011L, 0 },
	TEST_c_c(casinh, 0x1.fp-16385L, 1.0L, 9.023632056840860275214893047597614177639e-2467L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x1.fp-16385L, -1.0L, 9.023632056840860275214893047597614177639e-2467L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, 1.0L, -9.023632056840860275214893047597614177639e-2467L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, -1.0L, -9.023632056840860275214893047597614177639e-2467L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 1.0L, 0x1.fp-16385L, 8.813735870195430252326093249797923090282e-1L, 5.757683115456107044131264955348448954458e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -1.0L, 0x1.fp-16385L, -8.813735870195430252326093249797923090282e-1L, 5.757683115456107044131264955348448954458e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, 1.0L, -0x1.fp-16385L, 8.813735870195430252326093249797923090282e-1L, -5.757683115456107044131264955348448954458e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -1.0L, -0x1.fp-16385L, -8.813735870195430252326093249797923090282e-1L, -5.757683115456107044131264955348448954458e-4933L, UNDERFLOW_EXCEPTION },
#endif

	TEST_c_c(casinh, 0x1p-23L, 0x1.000002p0L, 5.364668491573609633134147164031476452679e-4L, 1.570574114956455764594481054234834678422L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 0x1p-23L, -0x1.000002p0L, 5.364668491573609633134147164031476452679e-4L, -1.570574114956455764594481054234834678422L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1p-23L, 0x1.000002p0L, -5.364668491573609633134147164031476452679e-4L, 1.570574114956455764594481054234834678422L, 0 },
	TEST_c_c(casinh, -0x1p-23L, -0x1.000002p0L, -5.364668491573609633134147164031476452679e-4L, -1.570574114956455764594481054234834678422L, 0 },
	TEST_c_c(casinh, 0x1.000002p0L, 0x1p-23L, 8.813736713132400470205730751186547909968e-1L, 8.429369199749229560964789467980644296420e-8L, 0 },
	TEST_c_c(casinh, -0x1.000002p0L, 0x1p-23L, -8.813736713132400470205730751186547909968e-1L, 8.429369199749229560964789467980644296420e-8L, 0 },
	TEST_c_c(casinh, 0x1.000002p0L, -0x1p-23L, 8.813736713132400470205730751186547909968e-1L, -8.429369199749229560964789467980644296420e-8L, 0 },
	TEST_c_c(casinh, -0x1.000002p0L, -0x1p-23L, -8.813736713132400470205730751186547909968e-1L, -8.429369199749229560964789467980644296420e-8L, 0 },
	TEST_c_c(casinh, 0x1.fp-129L, 0x1.000002p0L, 4.882812451493617206486388134172712975070e-4L, 1.570796326794896619231321691639751436268L, 0 },
	TEST_c_c(casinh, 0x1.fp-129L, -0x1.000002p0L, 4.882812451493617206486388134172712975070e-4L, -1.570796326794896619231321691639751436268L, 0 },
	TEST_c_c(casinh, -0x1.fp-129L, 0x1.000002p0L, -4.882812451493617206486388134172712975070e-4L, 1.570796326794896619231321691639751436268L, 0 },
	TEST_c_c(casinh, -0x1.fp-129L, -0x1.000002p0L, -4.882812451493617206486388134172712975070e-4L, -1.570796326794896619231321691639751436268L, 0 },
	TEST_c_c(casinh, 0x1.000002p0L, 0x1.fp-129L, 8.813736713132375348727889167749389235161e-1L, 2.013062444707472738895109955455676357057e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, -0x1.000002p0L, 0x1.fp-129L, -8.813736713132375348727889167749389235161e-1L, 2.013062444707472738895109955455676357057e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 0x1.000002p0L, -0x1.fp-129L, 8.813736713132375348727889167749389235161e-1L, -2.013062444707472738895109955455676357057e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, -0x1.000002p0L, -0x1.fp-129L, -8.813736713132375348727889167749389235161e-1L, -2.013062444707472738895109955455676357057e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 0.0L, 0x1.000002p0L, 4.882812451493617206486388134172712975070e-4L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0.0L, -0x1.000002p0L, 4.882812451493617206486388134172712975070e-4L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0.0L, 0x1.000002p0L, -4.882812451493617206486388134172712975070e-4L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0.0L, -0x1.000002p0L, -4.882812451493617206486388134172712975070e-4L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.000002p0L, 0.0L, 8.813736713132375348727889167749389235161e-1L, 0.0L, 0 },
	TEST_c_c(casinh, -0x1.000002p0L, 0.0L, -8.813736713132375348727889167749389235161e-1L, 0.0L, 0 },
	TEST_c_c(casinh, 0x1.000002p0L, -0.0L, 8.813736713132375348727889167749389235161e-1L, -0.0L, 0 },
	TEST_c_c(casinh, -0x1.000002p0L, -0.0L, -8.813736713132375348727889167749389235161e-1L, -0.0L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(casinh, 0x1p-52L, 0x1.0000000000001p0L, 2.315303644582684770975188768022139415020e-8L, 1.570796317204594913251280305810847349436L, 0 },
	TEST_c_c(casinh, 0x1p-52L, -0x1.0000000000001p0L, 2.315303644582684770975188768022139415020e-8L, -1.570796317204594913251280305810847349436L, 0 },
	TEST_c_c(casinh, -0x1p-52L, 0x1.0000000000001p0L, -2.315303644582684770975188768022139415020e-8L, 1.570796317204594913251280305810847349436L, 0 },
	TEST_c_c(casinh, -0x1p-52L, -0x1.0000000000001p0L, -2.315303644582684770975188768022139415020e-8L, -1.570796317204594913251280305810847349436L, 0 },
	TEST_c_c(casinh, 0x1.0000000000001p0L, 0x1p-52L, 8.813735870195431822418551933572982483664e-1L, 1.570092458683774885078102529858632363236e-16L, 0 },
	TEST_c_c(casinh, -0x1.0000000000001p0L, 0x1p-52L, -8.813735870195431822418551933572982483664e-1L, 1.570092458683774885078102529858632363236e-16L, 0 },
	TEST_c_c(casinh, 0x1.0000000000001p0L, -0x1p-52L, 8.813735870195431822418551933572982483664e-1L, -1.570092458683774885078102529858632363236e-16L, 0 },
	TEST_c_c(casinh, -0x1.0000000000001p0L, -0x1p-52L, -8.813735870195431822418551933572982483664e-1L, -1.570092458683774885078102529858632363236e-16L, 0 },
	TEST_c_c(casinh, 0x1.fp-1025L, 0x1.0000000000001p0L, 2.107342425544701550354780375182800088393e-8L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.fp-1025L, -0x1.0000000000001p0L, 2.107342425544701550354780375182800088393e-8L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-1025L, 0x1.0000000000001p0L, -2.107342425544701550354780375182800088393e-8L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-1025L, -0x1.0000000000001p0L, -2.107342425544701550354780375182800088393e-8L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.0000000000001p0L, 0x1.fp-1025L, 8.813735870195431822418551933572895326024e-1L, 3.810492908885321320083608113679347200012e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -0x1.0000000000001p0L, 0x1.fp-1025L, -8.813735870195431822418551933572895326024e-1L, 3.810492908885321320083608113679347200012e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, 0x1.0000000000001p0L, -0x1.fp-1025L, 8.813735870195431822418551933572895326024e-1L, -3.810492908885321320083608113679347200012e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -0x1.0000000000001p0L, -0x1.fp-1025L, -8.813735870195431822418551933572895326024e-1L, -3.810492908885321320083608113679347200012e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, 0.0L, 0x1.0000000000001p0L, 2.107342425544701550354780375182800088393e-8L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0.0L, -0x1.0000000000001p0L, 2.107342425544701550354780375182800088393e-8L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0.0L, 0x1.0000000000001p0L, -2.107342425544701550354780375182800088393e-8L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0.0L, -0x1.0000000000001p0L, -2.107342425544701550354780375182800088393e-8L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.0000000000001p0L, 0.0L, 8.813735870195431822418551933572895326024e-1L, 0.0L, 0 },
	TEST_c_c(casinh, -0x1.0000000000001p0L, 0.0L, -8.813735870195431822418551933572895326024e-1L, 0.0L, 0 },
	TEST_c_c(casinh, 0x1.0000000000001p0L, -0.0L, 8.813735870195431822418551933572895326024e-1L, -0.0L, 0 },
	TEST_c_c(casinh, -0x1.0000000000001p0L, -0.0L, -8.813735870195431822418551933572895326024e-1L, -0.0L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(casinh, 0x1p-63L, 0x1.0000000000000002p0L, 5.116146586219826555037807251857670783420e-10L, 1.570796326582978888921215348380499122131L, 0 },
	TEST_c_c(casinh, 0x1p-63L, -0x1.0000000000000002p0L, 5.116146586219826555037807251857670783420e-10L, -1.570796326582978888921215348380499122131L, 0 },
	TEST_c_c(casinh, -0x1p-63L, 0x1.0000000000000002p0L, -5.116146586219826555037807251857670783420e-10L, 1.570796326582978888921215348380499122131L, 0 },
	TEST_c_c(casinh, -0x1p-63L, -0x1.0000000000000002p0L, -5.116146586219826555037807251857670783420e-10L, -1.570796326582978888921215348380499122131L, 0 },
	TEST_c_c(casinh, 0x1.0000000000000002p0L, 0x1p-63L, 8.813735870195430253092739958139610131001e-1L, 7.666467083416870406778649849746878368519e-20L, 0 },
	TEST_c_c(casinh, -0x1.0000000000000002p0L, 0x1p-63L, -8.813735870195430253092739958139610131001e-1L, 7.666467083416870406778649849746878368519e-20L, 0 },
	TEST_c_c(casinh, 0x1.0000000000000002p0L, -0x1p-63L, 8.813735870195430253092739958139610131001e-1L, -7.666467083416870406778649849746878368519e-20L, 0 },
	TEST_c_c(casinh, -0x1.0000000000000002p0L, -0x1p-63L, -8.813735870195430253092739958139610131001e-1L, -7.666467083416870406778649849746878368519e-20L, 0 },
# if LDBL_MIN_EXP <= -16381
	TEST_c_c(casinh, 0x1.fp-16385L, 0x1.0000000000000002p0L, 4.656612873077392578082927418388212703712e-10L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.fp-16385L, -0x1.0000000000000002p0L, 4.656612873077392578082927418388212703712e-10L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-16385L, 0x1.0000000000000002p0L, -4.656612873077392578082927418388212703712e-10L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-16385L, -0x1.0000000000000002p0L, -4.656612873077392578082927418388212703712e-10L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.0000000000000002p0L, 0x1.fp-16385L, 8.813735870195430253092739958139610130980e-1L, 5.757683115456107043819140328235418018963e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -0x1.0000000000000002p0L, 0x1.fp-16385L, -8.813735870195430253092739958139610130980e-1L, 5.757683115456107043819140328235418018963e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, 0x1.0000000000000002p0L, -0x1.fp-16385L, 8.813735870195430253092739958139610130980e-1L, -5.757683115456107043819140328235418018963e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -0x1.0000000000000002p0L, -0x1.fp-16385L, -8.813735870195430253092739958139610130980e-1L, -5.757683115456107043819140328235418018963e-4933L, UNDERFLOW_EXCEPTION },
# endif
	TEST_c_c(casinh, 0.0L, 0x1.0000000000000002p0L, 4.656612873077392578082927418388212703712e-10L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0.0L, -0x1.0000000000000002p0L, 4.656612873077392578082927418388212703712e-10L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0.0L, 0x1.0000000000000002p0L, -4.656612873077392578082927418388212703712e-10L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0.0L, -0x1.0000000000000002p0L, -4.656612873077392578082927418388212703712e-10L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.0000000000000002p0L, 0.0L, 8.813735870195430253092739958139610130980e-1L, 0.0L, 0 },
	TEST_c_c(casinh, -0x1.0000000000000002p0L, 0.0L, -8.813735870195430253092739958139610130980e-1L, 0.0L, 0 },
	TEST_c_c(casinh, 0x1.0000000000000002p0L, -0.0L, 8.813735870195430253092739958139610130980e-1L, -0.0L, 0 },
	TEST_c_c(casinh, -0x1.0000000000000002p0L, -0.0L, -8.813735870195430253092739958139610130980e-1L, -0.0L, 0|XFAIL_NATIVE_LINUXL },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(casinh, 0x1p-106L, 0x1.000000000000000000000000008p0L, 2.285028863093221674154232933662774454211e-16L, 1.570796326794896565289107467733682961928L, 0 },
	TEST_c_c(casinh, 0x1p-106L, -0x1.000000000000000000000000008p0L, 2.285028863093221674154232933662774454211e-16L, -1.570796326794896565289107467733682961928L, 0 },
	TEST_c_c(casinh, -0x1p-106L, 0x1.000000000000000000000000008p0L, -2.285028863093221674154232933662774454211e-16L, 1.570796326794896565289107467733682961928L, 0 },
	TEST_c_c(casinh, -0x1p-106L, -0x1.000000000000000000000000008p0L, -2.285028863093221674154232933662774454211e-16L, -1.570796326794896565289107467733682961928L, 0 },
	TEST_c_c(casinh, 0x1.000000000000000000000000008p0L, 0x1p-106L, 8.813735870195430252326093249798097405561e-1L, 8.715763992105246878957416200936726072500e-33L, 0 },
	TEST_c_c(casinh, -0x1.000000000000000000000000008p0L, 0x1p-106L, -8.813735870195430252326093249798097405561e-1L, 8.715763992105246878957416200936726072500e-33L, 0 },
	TEST_c_c(casinh, 0x1.000000000000000000000000008p0L, -0x1p-106L, 8.813735870195430252326093249798097405561e-1L, -8.715763992105246878957416200936726072500e-33L, 0 },
	TEST_c_c(casinh, -0x1.000000000000000000000000008p0L, -0x1p-106L, -8.813735870195430252326093249798097405561e-1L, -8.715763992105246878957416200936726072500e-33L, 0 },
	TEST_c_c(casinh, 0x1.fp-1025L, 0x1.000000000000000000000000008p0L, 2.220446049250313080847263336181636063482e-16L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.fp-1025L, -0x1.000000000000000000000000008p0L, 2.220446049250313080847263336181636063482e-16L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-1025L, 0x1.000000000000000000000000008p0L, -2.220446049250313080847263336181636063482e-16L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-1025L, -0x1.000000000000000000000000008p0L, -2.220446049250313080847263336181636063482e-16L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.000000000000000000000000008p0L, 0x1.fp-1025L, 8.813735870195430252326093249798097405561e-1L, 3.810492908885321743133304375216570658278e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -0x1.000000000000000000000000008p0L, 0x1.fp-1025L, -8.813735870195430252326093249798097405561e-1L, 3.810492908885321743133304375216570658278e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, 0x1.000000000000000000000000008p0L, -0x1.fp-1025L, 8.813735870195430252326093249798097405561e-1L, -3.810492908885321743133304375216570658278e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -0x1.000000000000000000000000008p0L, -0x1.fp-1025L, -8.813735870195430252326093249798097405561e-1L, -3.810492908885321743133304375216570658278e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, 0.0L, 0x1.000000000000000000000000008p0L, 2.220446049250313080847263336181636063482e-16L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0.0L, -0x1.000000000000000000000000008p0L, 2.220446049250313080847263336181636063482e-16L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0.0L, 0x1.000000000000000000000000008p0L, -2.220446049250313080847263336181636063482e-16L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0.0L, -0x1.000000000000000000000000008p0L, -2.220446049250313080847263336181636063482e-16L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.000000000000000000000000008p0L, 0.0L, 8.813735870195430252326093249798097405561e-1L, 0.0L, 0 },
	TEST_c_c(casinh, -0x1.000000000000000000000000008p0L, 0.0L, -8.813735870195430252326093249798097405561e-1L, 0.0L, 0 },
	TEST_c_c(casinh, 0x1.000000000000000000000000008p0L, -0.0L, 8.813735870195430252326093249798097405561e-1L, -0.0L, 0 },
	TEST_c_c(casinh, -0x1.000000000000000000000000008p0L, -0.0L, -8.813735870195430252326093249798097405561e-1L, -0.0L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(casinh, 0x1p-113L, 0x1.0000000000000000000000000001p0L, 2.019699255375255198156433710951064632386e-17L, 1.570796326794896614463458507897073960405L, 0 },
	TEST_c_c(casinh, 0x1p-113L, -0x1.0000000000000000000000000001p0L, 2.019699255375255198156433710951064632386e-17L, -1.570796326794896614463458507897073960405L, 0 },
	TEST_c_c(casinh, -0x1p-113L, 0x1.0000000000000000000000000001p0L, -2.019699255375255198156433710951064632386e-17L, 1.570796326794896614463458507897073960405L, 0 },
	TEST_c_c(casinh, -0x1p-113L, -0x1.0000000000000000000000000001p0L, -2.019699255375255198156433710951064632386e-17L, -1.570796326794896614463458507897073960405L, 0 },
	TEST_c_c(casinh, 0x1.0000000000000000000000000001p0L, 0x1p-113L, 8.813735870195430252326093249797924452120e-1L, 6.809190618832224124185481406981900518193e-35L, 0 },
	TEST_c_c(casinh, -0x1.0000000000000000000000000001p0L, 0x1p-113L, -8.813735870195430252326093249797924452120e-1L, 6.809190618832224124185481406981900518193e-35L, 0 },
	TEST_c_c(casinh, 0x1.0000000000000000000000000001p0L, -0x1p-113L, 8.813735870195430252326093249797924452120e-1L, -6.809190618832224124185481406981900518193e-35L, 0 },
	TEST_c_c(casinh, -0x1.0000000000000000000000000001p0L, -0x1p-113L, -8.813735870195430252326093249797924452120e-1L, -6.809190618832224124185481406981900518193e-35L, 0 },
	TEST_c_c(casinh, 0x1.fp-16385L, 0x1.0000000000000000000000000001p0L, 1.962615573354718824241727964954454332780e-17L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.fp-16385L, -0x1.0000000000000000000000000001p0L, 1.962615573354718824241727964954454332780e-17L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-16385L, 0x1.0000000000000000000000000001p0L, -1.962615573354718824241727964954454332780e-17L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0x1.fp-16385L, -0x1.0000000000000000000000000001p0L, -1.962615573354718824241727964954454332780e-17L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.0000000000000000000000000001p0L, 0x1.fp-16385L, 8.813735870195430252326093249797924452120e-1L, 5.757683115456107044131264955348448400014e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -0x1.0000000000000000000000000001p0L, 0x1.fp-16385L, -8.813735870195430252326093249797924452120e-1L, 5.757683115456107044131264955348448400014e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, 0x1.0000000000000000000000000001p0L, -0x1.fp-16385L, 8.813735870195430252326093249797924452120e-1L, -5.757683115456107044131264955348448400014e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -0x1.0000000000000000000000000001p0L, -0x1.fp-16385L, -8.813735870195430252326093249797924452120e-1L, -5.757683115456107044131264955348448400014e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, 0.0L, 0x1.0000000000000000000000000001p0L, 1.962615573354718824241727964954454332780e-17L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0.0L, -0x1.0000000000000000000000000001p0L, 1.962615573354718824241727964954454332780e-17L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0.0L, 0x1.0000000000000000000000000001p0L, -1.962615573354718824241727964954454332780e-17L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, -0.0L, -0x1.0000000000000000000000000001p0L, -1.962615573354718824241727964954454332780e-17L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(casinh, 0x1.0000000000000000000000000001p0L, 0.0L, 8.813735870195430252326093249797924452120e-1L, 0.0L, 0 },
	TEST_c_c(casinh, -0x1.0000000000000000000000000001p0L, 0.0L, -8.813735870195430252326093249797924452120e-1L, 0.0L, 0 },
	TEST_c_c(casinh, 0x1.0000000000000000000000000001p0L, -0.0L, 8.813735870195430252326093249797924452120e-1L, -0.0L, 0 },
	TEST_c_c(casinh, -0x1.0000000000000000000000000001p0L, -0.0L, -8.813735870195430252326093249797924452120e-1L, -0.0L, 0 },
#endif

	TEST_c_c(casinh, 0x1p-23L, 0x0.ffffffp0L, 2.714321200917194650737217746780928423385e-4L, 1.570357140408705570520377172865353621698L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 0x1p-23L, -0x0.ffffffp0L, 2.714321200917194650737217746780928423385e-4L, -1.570357140408705570520377172865353621698L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1p-23L, 0x0.ffffffp0L, -2.714321200917194650737217746780928423385e-4L, 1.570357140408705570520377172865353621698L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1p-23L, -0x0.ffffffp0L, -2.714321200917194650737217746780928423385e-4L, -1.570357140408705570520377172865353621698L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 0x0.ffffffp0L, 0x1p-23L, 8.813735448726963984495965873956465777250e-1L, 8.429369953393598499000760938112968755257e-8L, 0 },
	TEST_c_c(casinh, -0x0.ffffffp0L, 0x1p-23L, -8.813735448726963984495965873956465777250e-1L, 8.429369953393598499000760938112968755257e-8L, 0 },
	TEST_c_c(casinh, 0x0.ffffffp0L, -0x1p-23L, 8.813735448726963984495965873956465777250e-1L, -8.429369953393598499000760938112968755257e-8L, 0 },
	TEST_c_c(casinh, -0x0.ffffffp0L, -0x1p-23L, -8.813735448726963984495965873956465777250e-1L, -8.429369953393598499000760938112968755257e-8L, 0 },
	TEST_c_c(casinh, 0x1.fp-129L, 0x0.ffffffp0L, 8.245504387859737323891843603996428534945e-36L, 1.570451059810180415643718442157112705601L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-129L, -0x0.ffffffp0L, 8.245504387859737323891843603996428534945e-36L, -1.570451059810180415643718442157112705601L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-129L, 0x0.ffffffp0L, -8.245504387859737323891843603996428534945e-36L, 1.570451059810180415643718442157112705601L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-129L, -0x0.ffffffp0L, -8.245504387859737323891843603996428534945e-36L, -1.570451059810180415643718442157112705601L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x0.ffffffp0L, 0x1.fp-129L, 8.813735448726938863015878255140556727969e-1L, 2.013062624689288675911339929224634593425e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, -0x0.ffffffp0L, 0x1.fp-129L, -8.813735448726938863015878255140556727969e-1L, 2.013062624689288675911339929224634593425e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, 0x0.ffffffp0L, -0x1.fp-129L, 8.813735448726938863015878255140556727969e-1L, -2.013062624689288675911339929224634593425e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, -0x0.ffffffp0L, -0x1.fp-129L, -8.813735448726938863015878255140556727969e-1L, -2.013062624689288675911339929224634593425e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, 0.0L, 0x0.ffffffp0L, 0.0L, 1.570451059810180415643718442157112705601L, 0 },
	TEST_c_c(casinh, 0.0L, -0x0.ffffffp0L, 0.0L, -1.570451059810180415643718442157112705601L, 0 },
	TEST_c_c(casinh, -0.0L, 0x0.ffffffp0L, -0.0L, 1.570451059810180415643718442157112705601L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0.0L, -0x0.ffffffp0L, -0.0L, -1.570451059810180415643718442157112705601L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x0.ffffffp0L, 0.0L, 8.813735448726938863015878255140556727969e-1L, 0.0L, 0 },
	TEST_c_c(casinh, -0x0.ffffffp0L, 0.0L, -8.813735448726938863015878255140556727969e-1L, 0.0L, 0 },
	TEST_c_c(casinh, 0x0.ffffffp0L, -0.0L, 8.813735448726938863015878255140556727969e-1L, -0.0L, 0 },
	TEST_c_c(casinh, -0x0.ffffffp0L, -0.0L, -8.813735448726938863015878255140556727969e-1L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-23L, 0.5L, 1.376510308240943236356088341381173571841e-7L, 5.235987755982934033165770729745469098254e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 0x1p-23L, -0.5L, 1.376510308240943236356088341381173571841e-7L, -5.235987755982934033165770729745469098254e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1p-23L, 0.5L, -1.376510308240943236356088341381173571841e-7L, 5.235987755982934033165770729745469098254e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1p-23L, -0.5L, -1.376510308240943236356088341381173571841e-7L, -5.235987755982934033165770729745469098254e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 0.5L, 0x1p-23L, 4.812118250596059896127318386463676808798e-1L, 1.066240299940009780479664302917201696886e-7L, 0 },
	TEST_c_c(casinh, -0.5L, 0x1p-23L, -4.812118250596059896127318386463676808798e-1L, 1.066240299940009780479664302917201696886e-7L, 0 },
	TEST_c_c(casinh, 0.5L, -0x1p-23L, 4.812118250596059896127318386463676808798e-1L, -1.066240299940009780479664302917201696886e-7L, 0 },
	TEST_c_c(casinh, -0.5L, -0x1p-23L, -4.812118250596059896127318386463676808798e-1L, -1.066240299940009780479664302917201696886e-7L, 0 },
	TEST_c_c(casinh, 0x1.fp-129L, 0.5L, 3.287317402534702257036015056278368274737e-39L, 5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-129L, -0.5L, 3.287317402534702257036015056278368274737e-39L, -5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-129L, 0.5L, -3.287317402534702257036015056278368274737e-39L, 5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-129L, -0.5L, -3.287317402534702257036015056278368274737e-39L, -5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0.5L, 0x1.fp-129L, 4.812118250596034474977589134243684231352e-1L, 2.546345110742945032959687790021055102355e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, -0.5L, 0x1.fp-129L, -4.812118250596034474977589134243684231352e-1L, 2.546345110742945032959687790021055102355e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, 0.5L, -0x1.fp-129L, 4.812118250596034474977589134243684231352e-1L, -2.546345110742945032959687790021055102355e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, -0.5L, -0x1.fp-129L, -4.812118250596034474977589134243684231352e-1L, -2.546345110742945032959687790021055102355e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(casinh, 0x1p-23L, 0x1p-23L, 1.192092895507818146886315028596704749235e-7L, 1.192092895507806853113684971258850504935e-7L, 0 },
	TEST_c_c(casinh, 0x1p-23L, -0x1p-23L, 1.192092895507818146886315028596704749235e-7L, -1.192092895507806853113684971258850504935e-7L, 0 },
	TEST_c_c(casinh, -0x1p-23L, 0x1p-23L, -1.192092895507818146886315028596704749235e-7L, 1.192092895507806853113684971258850504935e-7L, 0 },
	TEST_c_c(casinh, -0x1p-23L, -0x1p-23L, -1.192092895507818146886315028596704749235e-7L, -1.192092895507806853113684971258850504935e-7L, 0 },
	TEST_c_c(casinh, 0x1.fp-129L, 0x1p-23L, 2.846900380897747786805634596726756660388e-39L, 1.192092895507815323443157514352519154304e-7L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-129L, -0x1p-23L, 2.846900380897747786805634596726756660388e-39L, -1.192092895507815323443157514352519154304e-7L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-129L, 0x1p-23L, -2.846900380897747786805634596726756660388e-39L, 1.192092895507815323443157514352519154304e-7L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-129L, -0x1p-23L, -2.846900380897747786805634596726756660388e-39L, -1.192092895507815323443157514352519154304e-7L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	/* Bug 15319: underflow exception may be missing.  */
	TEST_c_c(casinh, 0x1p-23L, 0x1.fp-129L, 1.192092895507809676556842485683592032154e-7L, 2.846900380897707329917933005874691056564e-39L, UNDERFLOW_EXCEPTION_OK_FLOAT },
	TEST_c_c(casinh, -0x1p-23L, 0x1.fp-129L, -1.192092895507809676556842485683592032154e-7L, 2.846900380897707329917933005874691056564e-39L, UNDERFLOW_EXCEPTION_OK_FLOAT },
	TEST_c_c(casinh, 0x1p-23L, -0x1.fp-129L, 1.192092895507809676556842485683592032154e-7L, -2.846900380897707329917933005874691056564e-39L, UNDERFLOW_EXCEPTION_OK_FLOAT },
	TEST_c_c(casinh, -0x1p-23L, -0x1.fp-129L, -1.192092895507809676556842485683592032154e-7L, -2.846900380897707329917933005874691056564e-39L, UNDERFLOW_EXCEPTION_OK_FLOAT },
	TEST_c_c(casinh, 0.0L, 0x1p-23L, 0.0L, 1.192092895507815323443157514352519154304e-7L, 0 },
	TEST_c_c(casinh, 0.0L, -0x1p-23L, 0.0L, -1.192092895507815323443157514352519154304e-7L, 0 },
	TEST_c_c(casinh, -0.0L, 0x1p-23L, -0.0L, 1.192092895507815323443157514352519154304e-7L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0.0L, -0x1p-23L, -0.0L, -1.192092895507815323443157514352519154304e-7L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-23L, 0.0L, 1.192092895507809676556842485683592032154e-7L, 0.0L, 0 },
	TEST_c_c(casinh, -0x1p-23L, 0.0L, -1.192092895507809676556842485683592032154e-7L, 0.0L, 0 },
	TEST_c_c(casinh, 0x1p-23L, -0.0L, 1.192092895507809676556842485683592032154e-7L, -0.0L, 0 },
	TEST_c_c(casinh, -0x1p-23L, -0.0L, -1.192092895507809676556842485683592032154e-7L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-129L, 0x1.fp-129L, 2.846900380897727558361783801085126250967e-39L, 2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-129L, -0x1.fp-129L, 2.846900380897727558361783801085126250967e-39L, -2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-129L, 0x1.fp-129L, -2.846900380897727558361783801085126250967e-39L, 2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-129L, -0x1.fp-129L, -2.846900380897727558361783801085126250967e-39L, -2.846900380897727558361783801085126250967e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(casinh, 0x1p-52L, 0x0.fffffffffffff8p0L, 1.171456840272878582596796205397918831268e-8L, 1.570796307840326780080580569360500172043L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-52L, -0x0.fffffffffffff8p0L, 1.171456840272878582596796205397918831268e-8L, -1.570796307840326780080580569360500172043L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-52L, 0x0.fffffffffffff8p0L, -1.171456840272878582596796205397918831268e-8L, 1.570796307840326780080580569360500172043L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-52L, -0x0.fffffffffffff8p0L, -1.171456840272878582596796205397918831268e-8L, -1.570796307840326780080580569360500172043L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x0.fffffffffffff8p0L, 0x1p-52L, 8.813735870195429467279863907910458761820e-1L, 1.570092458683775146551022293016031474627e-16L, 0 },
	TEST_c_c(casinh, -0x0.fffffffffffff8p0L, 0x1p-52L, -8.813735870195429467279863907910458761820e-1L, 1.570092458683775146551022293016031474627e-16L, 0 },
	TEST_c_c(casinh, 0x0.fffffffffffff8p0L, -0x1p-52L, 8.813735870195429467279863907910458761820e-1L, -1.570092458683775146551022293016031474627e-16L, 0 },
	TEST_c_c(casinh, -0x0.fffffffffffff8p0L, -0x1p-52L, -8.813735870195429467279863907910458761820e-1L, -1.570092458683775146551022293016031474627e-16L, 0 },
	TEST_c_c(casinh, 0x1.fp-1025L, 0x0.fffffffffffff8p0L, 3.616396521699973256461764099945789620604e-301L, 1.570796311893735425383665303776316016594L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-1025L, -0x0.fffffffffffff8p0L, 3.616396521699973256461764099945789620604e-301L, -1.570796311893735425383665303776316016594L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, 0x0.fffffffffffff8p0L, -3.616396521699973256461764099945789620604e-301L, 1.570796311893735425383665303776316016594L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, -0x0.fffffffffffff8p0L, -3.616396521699973256461764099945789620604e-301L, -1.570796311893735425383665303776316016594L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x0.fffffffffffff8p0L, 0x1.fp-1025L, 8.813735870195429467279863907910371604180e-1L, 3.810492908885321954658152505985270452320e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -0x0.fffffffffffff8p0L, 0x1.fp-1025L, -8.813735870195429467279863907910371604180e-1L, 3.810492908885321954658152505985270452320e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, 0x0.fffffffffffff8p0L, -0x1.fp-1025L, 8.813735870195429467279863907910371604180e-1L, -3.810492908885321954658152505985270452320e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -0x0.fffffffffffff8p0L, -0x1.fp-1025L, -8.813735870195429467279863907910371604180e-1L, -3.810492908885321954658152505985270452320e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, 0.0L, 0x0.fffffffffffff8p0L, 0.0L, 1.570796311893735425383665303776316016594L, 0 },
	TEST_c_c(casinh, 0.0L, -0x0.fffffffffffff8p0L, 0.0L, -1.570796311893735425383665303776316016594L, 0 },
	TEST_c_c(casinh, -0.0L, 0x0.fffffffffffff8p0L, -0.0L, 1.570796311893735425383665303776316016594L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0.0L, -0x0.fffffffffffff8p0L, -0.0L, -1.570796311893735425383665303776316016594L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x0.fffffffffffff8p0L, 0.0L, 8.813735870195429467279863907910371604180e-1L, 0.0L, 0 },
	TEST_c_c(casinh, -0x0.fffffffffffff8p0L, 0.0L, -8.813735870195429467279863907910371604180e-1L, 0.0L, 0 },
	TEST_c_c(casinh, 0x0.fffffffffffff8p0L, -0.0L, 8.813735870195429467279863907910371604180e-1L, -0.0L, 0 },
	TEST_c_c(casinh, -0x0.fffffffffffff8p0L, -0.0L, -8.813735870195429467279863907910371604180e-1L, -0.0L, 0|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(casinh, 0x1p-52L, 0.5L, 2.563950248511418570403591756798643804971e-16L, 5.235987755982988730771072305465648369889e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-52L, -0.5L, 2.563950248511418570403591756798643804971e-16L, -5.235987755982988730771072305465648369889e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-52L, 0.5L, -2.563950248511418570403591756798643804971e-16L, 5.235987755982988730771072305465648369889e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-52L, -0.5L, -2.563950248511418570403591756798643804971e-16L, -5.235987755982988730771072305465648369889e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0.5L, 0x1p-52L, 4.812118250596034474977589134243772428682e-1L, 1.986027322597818397661816253158024383707e-16L, 0 },
	TEST_c_c(casinh, -0.5L, 0x1p-52L, -4.812118250596034474977589134243772428682e-1L, 1.986027322597818397661816253158024383707e-16L, 0 },
	TEST_c_c(casinh, 0.5L, -0x1p-52L, 4.812118250596034474977589134243772428682e-1L, -1.986027322597818397661816253158024383707e-16L, 0 },
	TEST_c_c(casinh, -0.5L, -0x1p-52L, -4.812118250596034474977589134243772428682e-1L, -1.986027322597818397661816253158024383707e-16L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(casinh, 0x1.fp-1025L, 0.5L, 6.222508863508420569166420770843207333493e-309L, 5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-1025L, -0.5L, 6.222508863508420569166420770843207333493e-309L, -5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, 0.5L, -6.222508863508420569166420770843207333493e-309L, 5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, -0.5L, -6.222508863508420569166420770843207333493e-309L, -5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0.5L, 0x1.fp-1025L, 4.812118250596034474977589134243684231352e-1L, 4.819934639999230680322935210539402497827e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -0.5L, 0x1.fp-1025L, -4.812118250596034474977589134243684231352e-1L, 4.819934639999230680322935210539402497827e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, 0.5L, -0x1.fp-1025L, 4.812118250596034474977589134243684231352e-1L, -4.819934639999230680322935210539402497827e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -0.5L, -0x1.fp-1025L, -4.812118250596034474977589134243684231352e-1L, -4.819934639999230680322935210539402497827e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
#endif
	TEST_c_c(casinh, 0x1p-52L, 0x1p-52L, 2.220446049250313080847263336181677117148e-16L, 2.220446049250313080847263336181604132852e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 0x1p-52L, -0x1p-52L, 2.220446049250313080847263336181677117148e-16L, -2.220446049250313080847263336181604132852e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1p-52L, 0x1p-52L, -2.220446049250313080847263336181677117148e-16L, 2.220446049250313080847263336181604132852e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1p-52L, -0x1p-52L, -2.220446049250313080847263336181677117148e-16L, -2.220446049250313080847263336181604132852e-16L, 0|XFAIL_NATIVE_LINUXF },
#ifndef TEST_FLOAT
	TEST_c_c(casinh, 0x1.fp-1025L, 0x1p-52L, 5.388850751072128349671657362289548938458e-309L, 2.220446049250313080847263336181658871074e-16L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-1025L, -0x1p-52L, 5.388850751072128349671657362289548938458e-309L, -2.220446049250313080847263336181658871074e-16L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, 0x1p-52L, -5.388850751072128349671657362289548938458e-309L, 2.220446049250313080847263336181658871074e-16L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, -0x1p-52L, -5.388850751072128349671657362289548938458e-309L, -2.220446049250313080847263336181658871074e-16L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	/* Bug 15319: underflow exception may be missing.  */
	TEST_c_c(casinh, 0x1p-52L, 0x1.fp-1025L, 2.220446049250313080847263336181622378926e-16L, 5.388850751072128349671657362289283247603e-309L, UNDERFLOW_EXCEPTION_OK_DOUBLE },
	TEST_c_c(casinh, -0x1p-52L, 0x1.fp-1025L, -2.220446049250313080847263336181622378926e-16L, 5.388850751072128349671657362289283247603e-309L, UNDERFLOW_EXCEPTION_OK_DOUBLE },
	TEST_c_c(casinh, 0x1p-52L, -0x1.fp-1025L, 2.220446049250313080847263336181622378926e-16L, -5.388850751072128349671657362289283247603e-309L, UNDERFLOW_EXCEPTION_OK_DOUBLE },
	TEST_c_c(casinh, -0x1p-52L, -0x1.fp-1025L, -2.220446049250313080847263336181622378926e-16L, -5.388850751072128349671657362289283247603e-309L, UNDERFLOW_EXCEPTION_OK_DOUBLE },
#endif
	TEST_c_c(casinh, 0.0L, 0x1p-52L, 0.0L, 2.220446049250313080847263336181658871074e-16L, 0 },
	TEST_c_c(casinh, 0.0L, -0x1p-52L, 0.0L, -2.220446049250313080847263336181658871074e-16L, 0 },
	TEST_c_c(casinh, -0.0L, 0x1p-52L, -0.0L, 2.220446049250313080847263336181658871074e-16L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0.0L, -0x1p-52L, -0.0L, -2.220446049250313080847263336181658871074e-16L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-52L, 0.0L, 2.220446049250313080847263336181622378926e-16L, 0.0L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1p-52L, 0.0L, -2.220446049250313080847263336181622378926e-16L, 0.0L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, 0x1p-52L, -0.0L, 2.220446049250313080847263336181622378926e-16L, -0.0L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(casinh, -0x1p-52L, -0.0L, -2.220446049250313080847263336181622378926e-16L, -0.0L, 0|XFAIL_NATIVE_LINUXF|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(casinh, 0x1.fp-1025L, 0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-1025L, -0x1.fp-1025L, 5.388850751072128349671657362289416093031e-309L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, 0x1.fp-1025L, -5.388850751072128349671657362289416093031e-309L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, -0x1.fp-1025L, -5.388850751072128349671657362289416093031e-309L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(casinh, 0x1p-63L, 0x0.ffffffffffffffffp0L, 2.588578361325995866221775673638805081337e-10L, 1.570796326376055842114524927965556247908L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x1p-63L, -0x0.ffffffffffffffffp0L, 2.588578361325995866221775673638805081337e-10L, -1.570796326376055842114524927965556247908L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1p-63L, 0x0.ffffffffffffffffp0L, -2.588578361325995866221775673638805081337e-10L, 1.570796326376055842114524927965556247908L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1p-63L, -0x0.ffffffffffffffffp0L, -2.588578361325995866221775673638805081337e-10L, -1.570796326376055842114524927965556247908L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x0.ffffffffffffffffp0L, 0x1p-63L, 8.813735870195430251942769895627079569937e-1L, 7.666467083416870407402049869781566856442e-20L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffp0L, 0x1p-63L, -8.813735870195430251942769895627079569937e-1L, 7.666467083416870407402049869781566856442e-20L, 0 },
	TEST_c_c(casinh, 0x0.ffffffffffffffffp0L, -0x1p-63L, 8.813735870195430251942769895627079569937e-1L, -7.666467083416870407402049869781566856442e-20L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffp0L, -0x1p-63L, -8.813735870195430251942769895627079569937e-1L, -7.666467083416870407402049869781566856442e-20L, 0 },
# if LDBL_MIN_EXP <= -16381
	TEST_c_c(casinh, 0x1.fp-16385L, 0x0.ffffffffffffffffp0L, 2.472906068161537187835415298076415423459e-4923L, 1.570796326465624365239962068302568888898L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x1.fp-16385L, -0x0.ffffffffffffffffp0L, 2.472906068161537187835415298076415423459e-4923L, -1.570796326465624365239962068302568888898L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, 0x0.ffffffffffffffffp0L, -2.472906068161537187835415298076415423459e-4923L, 1.570796326465624365239962068302568888898L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, -0x0.ffffffffffffffffp0L, -2.472906068161537187835415298076415423459e-4923L, -1.570796326465624365239962068302568888898L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x0.ffffffffffffffffp0L, 0x1.fp-16385L, 8.813735870195430251942769895627079569917e-1L, 5.757683115456107044287327268904964422212e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -0x0.ffffffffffffffffp0L, 0x1.fp-16385L, -8.813735870195430251942769895627079569917e-1L, 5.757683115456107044287327268904964422212e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, 0x0.ffffffffffffffffp0L, -0x1.fp-16385L, 8.813735870195430251942769895627079569917e-1L, -5.757683115456107044287327268904964422212e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -0x0.ffffffffffffffffp0L, -0x1.fp-16385L, -8.813735870195430251942769895627079569917e-1L, -5.757683115456107044287327268904964422212e-4933L, UNDERFLOW_EXCEPTION },
# endif
	TEST_c_c(casinh, 0.0L, 0x0.ffffffffffffffffp0L, 0.0L, 1.570796326465624365239962068302568888898L, 0 },
	TEST_c_c(casinh, 0.0L, -0x0.ffffffffffffffffp0L, 0.0L, -1.570796326465624365239962068302568888898L, 0 },
	TEST_c_c(casinh, -0.0L, 0x0.ffffffffffffffffp0L, -0.0L, 1.570796326465624365239962068302568888898L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0.0L, -0x0.ffffffffffffffffp0L, -0.0L, -1.570796326465624365239962068302568888898L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x0.ffffffffffffffffp0L, 0.0L, 8.813735870195430251942769895627079569917e-1L, 0.0L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffp0L, 0.0L, -8.813735870195430251942769895627079569917e-1L, 0.0L, 0 },
	TEST_c_c(casinh, 0x0.ffffffffffffffffp0L, -0.0L, 8.813735870195430251942769895627079569917e-1L, -0.0L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffp0L, -0.0L, -8.813735870195430251942769895627079569917e-1L, -0.0L, 0|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(casinh, 0x1p-63L, 0.5L, 1.251928832280966098829878787499365228659e-19L, 5.235987755982988730771072305465838140283e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-63L, -0.5L, 1.251928832280966098829878787499365228659e-19L, -5.235987755982988730771072305465838140283e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-63L, 0.5L, -1.251928832280966098829878787499365228659e-19L, 5.235987755982988730771072305465838140283e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-63L, -0.5L, -1.251928832280966098829878787499365228659e-19L, -5.235987755982988730771072305465838140283e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0.5L, 0x1p-63L, 4.812118250596034474977589134243684231373e-1L, 9.697399036122160144833087173623140436413e-20L, 0 },
	TEST_c_c(casinh, -0.5L, 0x1p-63L, -4.812118250596034474977589134243684231373e-1L, 9.697399036122160144833087173623140436413e-20L, 0 },
	TEST_c_c(casinh, 0.5L, -0x1p-63L, 4.812118250596034474977589134243684231373e-1L, -9.697399036122160144833087173623140436413e-20L, 0 },
	TEST_c_c(casinh, -0.5L, -0x1p-63L, -4.812118250596034474977589134243684231373e-1L, -9.697399036122160144833087173623140436413e-20L, 0 },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(casinh, 0x1.fp-16385L, 0.5L, 9.402257155670418112159334875199821342890e-4933L, 5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x1.fp-16385L, -0.5L, 9.402257155670418112159334875199821342890e-4933L, -5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, 0.5L, -9.402257155670418112159334875199821342890e-4933L, 5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, -0.5L, -9.402257155670418112159334875199821342890e-4933L, -5.235987755982988730771072305465838140329e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0.5L, 0x1.fp-16385L, 4.812118250596034474977589134243684231352e-1L, 7.282957076134209141226696333885150260319e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -0.5L, 0x1.fp-16385L, -4.812118250596034474977589134243684231352e-1L, 7.282957076134209141226696333885150260319e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, 0.5L, -0x1.fp-16385L, 4.812118250596034474977589134243684231352e-1L, -7.282957076134209141226696333885150260319e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -0.5L, -0x1.fp-16385L, -4.812118250596034474977589134243684231352e-1L, -7.282957076134209141226696333885150260319e-4933L, UNDERFLOW_EXCEPTION },
#endif
	TEST_c_c(casinh, 0x1p-63L, 0x1p-63L, 1.084202172485504434007452800869941711430e-19L, 1.084202172485504434007452800869941711422e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-63L, -0x1p-63L, 1.084202172485504434007452800869941711430e-19L, -1.084202172485504434007452800869941711422e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-63L, 0x1p-63L, -1.084202172485504434007452800869941711430e-19L, 1.084202172485504434007452800869941711422e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-63L, -0x1p-63L, -1.084202172485504434007452800869941711430e-19L, -1.084202172485504434007452800869941711422e-19L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(casinh, 0x1.fp-16385L, 0x1p-63L, 8.142593549724601460479922838826119584465e-4933L, 1.084202172485504434007452800869941711428e-19L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x1.fp-16385L, -0x1p-63L, 8.142593549724601460479922838826119584465e-4933L, -1.084202172485504434007452800869941711428e-19L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, 0x1p-63L, -8.142593549724601460479922838826119584465e-4933L, 1.084202172485504434007452800869941711428e-19L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, -0x1p-63L, -8.142593549724601460479922838826119584465e-4933L, -1.084202172485504434007452800869941711428e-19L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	/* Bug 15319: underflow exception may be missing.  */
	TEST_c_c(casinh, 0x1p-63L, 0x1.fp-16385L, 1.084202172485504434007452800869941711424e-19L, 8.142593549724601460479922838826119584369e-4933L, UNDERFLOW_EXCEPTION_OK },
	TEST_c_c(casinh, -0x1p-63L, 0x1.fp-16385L, -1.084202172485504434007452800869941711424e-19L, 8.142593549724601460479922838826119584369e-4933L, UNDERFLOW_EXCEPTION_OK },
	TEST_c_c(casinh, 0x1p-63L, -0x1.fp-16385L, 1.084202172485504434007452800869941711424e-19L, -8.142593549724601460479922838826119584369e-4933L, UNDERFLOW_EXCEPTION_OK },
	TEST_c_c(casinh, -0x1p-63L, -0x1.fp-16385L, -1.084202172485504434007452800869941711424e-19L, -8.142593549724601460479922838826119584369e-4933L, UNDERFLOW_EXCEPTION_OK },
#endif
	TEST_c_c(casinh, 0.0L, 0x1p-63L, 0.0L, 1.084202172485504434007452800869941711428e-19L, 0 },
	TEST_c_c(casinh, 0.0L, -0x1p-63L, 0.0L, -1.084202172485504434007452800869941711428e-19L, 0 },
	TEST_c_c(casinh, -0.0L, 0x1p-63L, -0.0L, 1.084202172485504434007452800869941711428e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0.0L, -0x1p-63L, -0.0L, -1.084202172485504434007452800869941711428e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-63L, 0.0L, 1.084202172485504434007452800869941711424e-19L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-63L, 0.0L, -1.084202172485504434007452800869941711424e-19L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-63L, -0.0L, 1.084202172485504434007452800869941711424e-19L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-63L, -0.0L, -1.084202172485504434007452800869941711424e-19L, -0.0L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(casinh, 0x1.fp-16385L, 0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x1.fp-16385L, -0x1.fp-16385L, 8.142593549724601460479922838826119584417e-4933L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, 0x1.fp-16385L, -8.142593549724601460479922838826119584417e-4933L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, -0x1.fp-16385L, -8.142593549724601460479922838826119584417e-4933L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(casinh, 0x1p-105L, 0x0.ffffffffffffffffffffffffffcp0L, 1.234330349600789959989661887846005659983e-16L, 1.570796326794896419512475791677915736035L, 0 },
	TEST_c_c(casinh, 0x1p-105L, -0x0.ffffffffffffffffffffffffffcp0L, 1.234330349600789959989661887846005659983e-16L, -1.570796326794896419512475791677915736035L, 0 },
	TEST_c_c(casinh, -0x1p-105L, 0x0.ffffffffffffffffffffffffffcp0L, -1.234330349600789959989661887846005659983e-16L, 1.570796326794896419512475791677915736035L, 0 },
	TEST_c_c(casinh, -0x1p-105L, -0x0.ffffffffffffffffffffffffffcp0L, -1.234330349600789959989661887846005659983e-16L, -1.570796326794896419512475791677915736035L, 0 },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffcp0L, 0x1p-105L, 8.813735870195430252326093249797835932642e-1L, 1.743152798421049375791483240187377443526e-32L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffcp0L, 0x1p-105L, -8.813735870195430252326093249797835932642e-1L, 1.743152798421049375791483240187377443526e-32L, 0 },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffcp0L, -0x1p-105L, 8.813735870195430252326093249797835932642e-1L, -1.743152798421049375791483240187377443526e-32L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffcp0L, -0x1p-105L, -8.813735870195430252326093249797835932642e-1L, -1.743152798421049375791483240187377443526e-32L, 0 },
	TEST_c_c(casinh, 0x1.fp-1025L, 0x0.ffffffffffffffffffffffffffcp0L, 3.432186888910770473784103112009399360940e-293L, 1.570796326794896462222075823262245502760L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casinh, 0x1.fp-1025L, -0x0.ffffffffffffffffffffffffffcp0L, 3.432186888910770473784103112009399360940e-293L, -1.570796326794896462222075823262245502760L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casinh, -0x1.fp-1025L, 0x0.ffffffffffffffffffffffffffcp0L, -3.432186888910770473784103112009399360940e-293L, 1.570796326794896462222075823262245502760L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casinh, -0x1.fp-1025L, -0x0.ffffffffffffffffffffffffffcp0L, -3.432186888910770473784103112009399360940e-293L, -1.570796326794896462222075823262245502760L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffcp0L, 0x1.fp-1025L, 8.813735870195430252326093249797835932642e-1L, 3.810492908885321743133304375216641110205e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffcp0L, 0x1.fp-1025L, -8.813735870195430252326093249797835932642e-1L, 3.810492908885321743133304375216641110205e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffcp0L, -0x1.fp-1025L, 8.813735870195430252326093249797835932642e-1L, -3.810492908885321743133304375216641110205e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffcp0L, -0x1.fp-1025L, -8.813735870195430252326093249797835932642e-1L, -3.810492908885321743133304375216641110205e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(casinh, 0.0L, 0x0.ffffffffffffffffffffffffffcp0L, 0.0L, 1.570796326794896462222075823262245502760L, 0 },
	TEST_c_c(casinh, 0.0L, -0x0.ffffffffffffffffffffffffffcp0L, 0.0L, -1.570796326794896462222075823262245502760L, 0 },
	TEST_c_c(casinh, -0.0L, 0x0.ffffffffffffffffffffffffffcp0L, -0.0L, 1.570796326794896462222075823262245502760L, 0 },
	TEST_c_c(casinh, -0.0L, -0x0.ffffffffffffffffffffffffffcp0L, -0.0L, -1.570796326794896462222075823262245502760L, 0 },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffcp0L, 0.0L, 8.813735870195430252326093249797835932642e-1L, 0.0L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffcp0L, 0.0L, -8.813735870195430252326093249797835932642e-1L, 0.0L, 0 },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffcp0L, -0.0L, 8.813735870195430252326093249797835932642e-1L, -0.0L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffcp0L, -0.0L, -8.813735870195430252326093249797835932642e-1L, -0.0L, 0 },
#endif
	TEST_c_c(casinh, 0x1p-105L, 0.5L, 2.846556599890768890940941645338546615370e-32L, 5.235987755982988730771072305465838140329e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-105L, -0.5L, 2.846556599890768890940941645338546615370e-32L, -5.235987755982988730771072305465838140329e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-105L, 0.5L, -2.846556599890768890940941645338546615370e-32L, 5.235987755982988730771072305465838140329e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-105L, -0.5L, -2.846556599890768890940941645338546615370e-32L, -5.235987755982988730771072305465838140329e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0.5L, 0x1p-105L, 4.812118250596034474977589134243684231352e-1L, 2.204933261082751447436996198490110245167e-32L, 0 },
	TEST_c_c(casinh, -0.5L, 0x1p-105L, -4.812118250596034474977589134243684231352e-1L, 2.204933261082751447436996198490110245167e-32L, 0 },
	TEST_c_c(casinh, 0.5L, -0x1p-105L, 4.812118250596034474977589134243684231352e-1L, -2.204933261082751447436996198490110245167e-32L, 0 },
	TEST_c_c(casinh, -0.5L, -0x1p-105L, -4.812118250596034474977589134243684231352e-1L, -2.204933261082751447436996198490110245167e-32L, 0 },
	TEST_c_c(casinh, 0x1p-105L, 0x1p-105L, 2.465190328815661891911651766508706967729e-32L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-105L, -0x1p-105L, 2.465190328815661891911651766508706967729e-32L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-105L, 0x1p-105L, -2.465190328815661891911651766508706967729e-32L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-105L, -0x1p-105L, -2.465190328815661891911651766508706967729e-32L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(casinh, 0x1.fp-1025L, 0x1p-105L, 5.388850751072128349671657362289416093031e-309L, 2.465190328815661891911651766508706967729e-32L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1.fp-1025L, -0x1p-105L, 5.388850751072128349671657362289416093031e-309L, -2.465190328815661891911651766508706967729e-32L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, 0x1p-105L, -5.388850751072128349671657362289416093031e-309L, 2.465190328815661891911651766508706967729e-32L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1.fp-1025L, -0x1p-105L, -5.388850751072128349671657362289416093031e-309L, -2.465190328815661891911651766508706967729e-32L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	/* Bug 15319: underflow exception may be missing.  */
	TEST_c_c(casinh, 0x1p-105L, 0x1.fp-1025L, 2.465190328815661891911651766508706967729e-32L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_OK_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-105L, 0x1.fp-1025L, -2.465190328815661891911651766508706967729e-32L, 5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_OK_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-105L, -0x1.fp-1025L, 2.465190328815661891911651766508706967729e-32L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_OK_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-105L, -0x1.fp-1025L, -2.465190328815661891911651766508706967729e-32L, -5.388850751072128349671657362289416093031e-309L, UNDERFLOW_EXCEPTION_OK_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(casinh, 0.0L, 0x1p-105L, 0.0L, 2.465190328815661891911651766508706967729e-32L, 0 },
	TEST_c_c(casinh, 0.0L, -0x1p-105L, 0.0L, -2.465190328815661891911651766508706967729e-32L, 0 },
	TEST_c_c(casinh, -0.0L, 0x1p-105L, -0.0L, 2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0.0L, -0x1p-105L, -0.0L, -2.465190328815661891911651766508706967729e-32L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-105L, 0.0L, 2.465190328815661891911651766508706967729e-32L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-105L, 0.0L, -2.465190328815661891911651766508706967729e-32L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-105L, -0.0L, 2.465190328815661891911651766508706967729e-32L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-105L, -0.0L, -2.465190328815661891911651766508706967729e-32L, -0.0L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(casinh, 0x1p-112L, 0x0.ffffffffffffffffffffffffffff8p0L, 1.091004200533850618345330909857103457781e-17L, 1.570796326794896601578502908312986698624L, 0 },
	TEST_c_c(casinh, 0x1p-112L, -0x0.ffffffffffffffffffffffffffff8p0L, 1.091004200533850618345330909857103457781e-17L, -1.570796326794896601578502908312986698624L, 0 },
	TEST_c_c(casinh, -0x1p-112L, 0x0.ffffffffffffffffffffffffffff8p0L, -1.091004200533850618345330909857103457781e-17L, 1.570796326794896601578502908312986698624L, 0 },
	TEST_c_c(casinh, -0x1p-112L, -0x0.ffffffffffffffffffffffffffff8p0L, -1.091004200533850618345330909857103457781e-17L, -1.570796326794896601578502908312986698624L, 0 },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-112L, 8.813735870195430252326093249797922409363e-1L, 1.361838123766444824837096281396380300349e-34L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-112L, -8.813735870195430252326093249797922409363e-1L, 1.361838123766444824837096281396380300349e-34L, 0 },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-112L, 8.813735870195430252326093249797922409363e-1L, -1.361838123766444824837096281396380300349e-34L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-112L, -8.813735870195430252326093249797922409363e-1L, -1.361838123766444824837096281396380300349e-34L, 0 },
	TEST_c_c(casinh, 0x1.fp-16385L, 0x0.ffffffffffffffffffffffffffff8p0L, 5.867357004219059108790368268900490184505e-4916L, 1.570796326794896605353533883825294686803L, 0 },
	TEST_c_c(casinh, 0x1.fp-16385L, -0x0.ffffffffffffffffffffffffffff8p0L, 5.867357004219059108790368268900490184505e-4916L, -1.570796326794896605353533883825294686803L, 0 },
	TEST_c_c(casinh, -0x1.fp-16385L, 0x0.ffffffffffffffffffffffffffff8p0L, -5.867357004219059108790368268900490184505e-4916L, 1.570796326794896605353533883825294686803L, 0 },
	TEST_c_c(casinh, -0x1.fp-16385L, -0x0.ffffffffffffffffffffffffffff8p0L, -5.867357004219059108790368268900490184505e-4916L, -1.570796326794896605353533883825294686803L, 0 },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1.fp-16385L, 8.813735870195430252326093249797922409363e-1L, 5.757683115456107044131264955348449231681e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1.fp-16385L, -8.813735870195430252326093249797922409363e-1L, 5.757683115456107044131264955348449231681e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1.fp-16385L, 8.813735870195430252326093249797922409363e-1L, -5.757683115456107044131264955348449231681e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1.fp-16385L, -8.813735870195430252326093249797922409363e-1L, -5.757683115456107044131264955348449231681e-4933L, UNDERFLOW_EXCEPTION },
	TEST_c_c(casinh, 0.0L, 0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, 1.570796326794896605353533883825294686803L, 0 },
	TEST_c_c(casinh, 0.0L, -0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, -1.570796326794896605353533883825294686803L, 0 },
	TEST_c_c(casinh, -0.0L, 0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, 1.570796326794896605353533883825294686803L, 0 },
	TEST_c_c(casinh, -0.0L, -0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, -1.570796326794896605353533883825294686803L, 0 },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, 8.813735870195430252326093249797922409363e-1L, 0.0L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffff8p0L, 0.0L, -8.813735870195430252326093249797922409363e-1L, 0.0L, 0 },
	TEST_c_c(casinh, 0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, 8.813735870195430252326093249797922409363e-1L, -0.0L, 0 },
	TEST_c_c(casinh, -0x0.ffffffffffffffffffffffffffff8p0L, -0.0L, -8.813735870195430252326093249797922409363e-1L, -0.0L, 0 },
#endif
	TEST_c_c(casinh, 0x1p-112L, 0.5L, 2.223872343664663196047610660420739543258e-34L, 5.235987755982988730771072305465838140329e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-112L, -0.5L, 2.223872343664663196047610660420739543258e-34L, -5.235987755982988730771072305465838140329e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-112L, 0.5L, -2.223872343664663196047610660420739543258e-34L, 5.235987755982988730771072305465838140329e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-112L, -0.5L, -2.223872343664663196047610660420739543258e-34L, -5.235987755982988730771072305465838140329e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0.5L, 0x1p-112L, 4.812118250596034474977589134243684231352e-1L, 1.722604110220899568310153280070398629037e-34L, 0 },
	TEST_c_c(casinh, -0.5L, 0x1p-112L, -4.812118250596034474977589134243684231352e-1L, 1.722604110220899568310153280070398629037e-34L, 0 },
	TEST_c_c(casinh, 0.5L, -0x1p-112L, 4.812118250596034474977589134243684231352e-1L, -1.722604110220899568310153280070398629037e-34L, 0 },
	TEST_c_c(casinh, -0.5L, -0x1p-112L, -4.812118250596034474977589134243684231352e-1L, -1.722604110220899568310153280070398629037e-34L, 0 },
	TEST_c_c(casinh, 0x1p-112L, 0x1p-112L, 1.925929944387235853055977942584927318538e-34L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-112L, -0x1p-112L, 1.925929944387235853055977942584927318538e-34L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-112L, 0x1p-112L, -1.925929944387235853055977942584927318538e-34L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-112L, -0x1p-112L, -1.925929944387235853055977942584927318538e-34L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(casinh, 0x1.fp-16385L, 0x1p-112L, 8.142593549724601460479922838826119584417e-4933L, 1.925929944387235853055977942584927318538e-34L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x1.fp-16385L, -0x1p-112L, 8.142593549724601460479922838826119584417e-4933L, -1.925929944387235853055977942584927318538e-34L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, 0x1p-112L, -8.142593549724601460479922838826119584417e-4933L, 1.925929944387235853055977942584927318538e-34L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1.fp-16385L, -0x1p-112L, -8.142593549724601460479922838826119584417e-4933L, -1.925929944387235853055977942584927318538e-34L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	/* Bug 15319: underflow exception may be missing.  */
	TEST_c_c(casinh, 0x1p-112L, 0x1.fp-16385L, 1.925929944387235853055977942584927318538e-34L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1p-112L, 0x1.fp-16385L, -1.925929944387235853055977942584927318538e-34L, 8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, 0x1p-112L, -0x1.fp-16385L, 1.925929944387235853055977942584927318538e-34L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(casinh, -0x1p-112L, -0x1.fp-16385L, -1.925929944387235853055977942584927318538e-34L, -8.142593549724601460479922838826119584417e-4933L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(casinh, 0.0L, 0x1p-112L, 0.0L, 1.925929944387235853055977942584927318538e-34L, 0 },
	TEST_c_c(casinh, 0.0L, -0x1p-112L, 0.0L, -1.925929944387235853055977942584927318538e-34L, 0 },
	TEST_c_c(casinh, -0.0L, 0x1p-112L, -0.0L, 1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0.0L, -0x1p-112L, -0.0L, -1.925929944387235853055977942584927318538e-34L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-112L, 0.0L, 1.925929944387235853055977942584927318538e-34L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-112L, 0.0L, -1.925929944387235853055977942584927318538e-34L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, 0x1p-112L, -0.0L, 1.925929944387235853055977942584927318538e-34L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(casinh, -0x1p-112L, -0.0L, -1.925929944387235853055977942584927318538e-34L, -0.0L, 0|XFAIL_NATIVE_LINUX },

	TEST_c_c(casinh, 0.75L, 1.25L, 1.03171853444778027336364058631006594L, 0.911738290968487636358489564316731207L, 0 },
	TEST_c_c(casinh, -2, -3, -1.9686379257930962917886650952454982L, -0.96465850440760279204541105949953237L, 0 },
}


static void test_catan(void)
{
	struct test *testp = lookup("catan" },

	TEST_c_c(catan, 0, 0, 0, 0, 0 },
	TEST_c_c(catan, ZERO_M, 0, ZERO_M, 0, 0 },
	TEST_c_c(catan, 0, ZERO_M, 0, ZERO_M, 0 },
	TEST_c_c(catan, ZERO_M, ZERO_M, ZERO_M, ZERO_M, 0 },

	TEST_c_c(catan, 1.0L, ZERO_P, M_PI_4l, ZERO_P, 0 },
	TEST_c_c(catan, 1.0L, ZERO_M, M_PI_4l, ZERO_M, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, ZERO_P, -M_PI_4l, ZERO_P, 0 },
	TEST_c_c(catan, -1.0L, ZERO_M, -M_PI_4l, ZERO_M, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, ZERO_P, 1.0L, ZERO_P, INF_P, DIVIDE_BY_ZERO_EXCEPTION },
	TEST_c_c(catan, ZERO_M, 1.0L, ZERO_M, INF_P, DIVIDE_BY_ZERO_EXCEPTION },
	TEST_c_c(catan, ZERO_P, -1.0L, ZERO_P, INF_M, DIVIDE_BY_ZERO_EXCEPTION },
	TEST_c_c(catan, ZERO_M, -1.0L, ZERO_M, INF_M, DIVIDE_BY_ZERO_EXCEPTION },

	TEST_c_c(catan, INF_P, INF_P, M_PI_2l, 0, 0 },
	TEST_c_c(catan, INF_P, INF_M, M_PI_2l, ZERO_M, 0 },
	TEST_c_c(catan, INF_M, INF_P, -M_PI_2l, 0, 0 },
	TEST_c_c(catan, INF_M, INF_M, -M_PI_2l, ZERO_M, 0 },


	TEST_c_c(catan, INF_P, -10.0, M_PI_2l, ZERO_M, 0 },
	TEST_c_c(catan, INF_M, -10.0, -M_PI_2l, ZERO_M, 0 },
	TEST_c_c(catan, INF_P, ZERO_M, M_PI_2l, ZERO_M, 0 },
	TEST_c_c(catan, INF_M, ZERO_M, -M_PI_2l, ZERO_M, 0 },
	TEST_c_c(catan, INF_P, 0.0, M_PI_2l, 0, 0 },
	TEST_c_c(catan, INF_M, 0.0, -M_PI_2l, 0, 0 },
	TEST_c_c(catan, INF_P, 0.1L, M_PI_2l, 0, 0 },
	TEST_c_c(catan, INF_M, 0.1L, -M_PI_2l, 0, 0 },

	TEST_c_c(catan, 0.0, INF_M, M_PI_2l, ZERO_M, 0 },
	TEST_c_c(catan, ZERO_M, INF_M, -M_PI_2l, ZERO_M, 0 },
	TEST_c_c(catan, 100.0, INF_M, M_PI_2l, ZERO_M, 0 },
	TEST_c_c(catan, -100.0, INF_M, -M_PI_2l, ZERO_M, 0 },

	TEST_c_c(catan, 0.0, INF_P, M_PI_2l, 0, 0 },
	TEST_c_c(catan, ZERO_M, INF_P, -M_PI_2l, 0, 0 },
	TEST_c_c(catan, 0.5, INF_P, M_PI_2l, 0, 0 },
	TEST_c_c(catan, -0.5, INF_P, -M_PI_2l, 0, 0 },

	TEST_c_c(catan, QNAN_P, 0.0, QNAN_P, 0, 0 },
	TEST_c_c(catan, QNAN_P, ZERO_M, QNAN_P, ZERO_M, 0 },

	TEST_c_c(catan, QNAN_P, INF_P, QNAN_P, 0, 0 },
	TEST_c_c(catan, QNAN_P, INF_M, QNAN_P, ZERO_M, 0 },

	TEST_c_c(catan, 0.0, QNAN_P, QNAN_P, QNAN_P, 0 },
	TEST_c_c(catan, ZERO_M, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(catan, INF_P, QNAN_P, M_PI_2l, 0, 0 },
	TEST_c_c(catan, INF_M, QNAN_P, -M_PI_2l, 0, 0 },

	TEST_c_c(catan, QNAN_P, 10.5, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(catan, QNAN_P, -10.5, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(catan, 0.75, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(catan, -0.75, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(catan, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(catan, 0x1p50L, 0.0L, 1.570796326794895731052901991514519103193L, 0.0L, 0 },
	TEST_c_c(catan, 0x1p50L, -0.0L, 1.570796326794895731052901991514519103193L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p50L, 0.0L, -1.570796326794895731052901991514519103193L, 0.0L, 0 },
	TEST_c_c(catan, -0x1p50L, -0.0L, -1.570796326794895731052901991514519103193L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.0L, 0x1p50L, 1.570796326794896619231321691639751442099L, 8.881784197001252323389053344728897997441e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0.0L, 0x1p50L, -1.570796326794896619231321691639751442099L, 8.881784197001252323389053344728897997441e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0.0L, -0x1p50L, 1.570796326794896619231321691639751442099L, -8.881784197001252323389053344728897997441e-16L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0.0L, -0x1p50L, -1.570796326794896619231321691639751442099L, -8.881784197001252323389053344728897997441e-16L, 0|XFAIL_NATIVE_LINUXF },
#ifndef TEST_FLOAT
	TEST_c_c(catan, 0x1p500L, 0.0L, 1.570796326794896619231321691639751442099L, 0.0L, 0 },
	TEST_c_c(catan, 0x1p500L, -0.0L, 1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p500L, 0.0L, -1.570796326794896619231321691639751442099L, 0.0L, 0 },
	TEST_c_c(catan, -0x1p500L, -0.0L, -1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.0L, 0x1p500L, 1.570796326794896619231321691639751442099L, 3.054936363499604682051979393213617699789e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.0L, 0x1p500L, -1.570796326794896619231321691639751442099L, 3.054936363499604682051979393213617699789e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.0L, -0x1p500L, 1.570796326794896619231321691639751442099L, -3.054936363499604682051979393213617699789e-151L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.0L, -0x1p500L, -1.570796326794896619231321691639751442099L, -3.054936363499604682051979393213617699789e-151L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(catan, 0x1p5000L, 0.0L, 1.570796326794896619231321691639751442099L, 0.0L, 0 },
	TEST_c_c(catan, 0x1p5000L, -0.0L, 1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1p5000L, 0.0L, -1.570796326794896619231321691639751442099L, 0.0L, 0 },
	TEST_c_c(catan, -0x1p5000L, -0.0L, -1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0.0L, 0x1p5000L, 1.570796326794896619231321691639751442099L, 7.079811261048172892385615158694057552948e-1506L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0.0L, 0x1p5000L, -1.570796326794896619231321691639751442099L, 7.079811261048172892385615158694057552948e-1506L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0.0L, -0x1p5000L, 1.570796326794896619231321691639751442099L, -7.079811261048172892385615158694057552948e-1506L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0.0L, -0x1p5000L, -1.570796326794896619231321691639751442099L, -7.079811261048172892385615158694057552948e-1506L, 0|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(catan, 0x1p63L, 0.5L, 1.570796326794896619122901474391200998698L, 5.877471754111437539843682686111228389007e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p63L, -0.5L, 1.570796326794896619122901474391200998698L, -5.877471754111437539843682686111228389007e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p63L, 0.5L, -1.570796326794896619122901474391200998698L, 5.877471754111437539843682686111228389007e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p63L, -0.5L, -1.570796326794896619122901474391200998698L, -5.877471754111437539843682686111228389007e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.5L, 0x1p63L, 1.570796326794896619231321691639751442093L, 1.084202172485504434007452800869941711427e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.5L, 0x1p63L, -1.570796326794896619231321691639751442093L, 1.084202172485504434007452800869941711427e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.5L, -0x1p63L, 1.570796326794896619231321691639751442093L, -1.084202172485504434007452800869941711427e-19L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.5L, -0x1p63L, -1.570796326794896619231321691639751442093L, -1.084202172485504434007452800869941711427e-19L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(catan, 0x1p511L, 0.5L, 1.570796326794896619231321691639751442099L, 1.112536929253600691545116358666202032110e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p511L, -0.5L, 1.570796326794896619231321691639751442099L, -1.112536929253600691545116358666202032110e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p511L, 0.5L, -1.570796326794896619231321691639751442099L, 1.112536929253600691545116358666202032110e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p511L, -0.5L, -1.570796326794896619231321691639751442099L, -1.112536929253600691545116358666202032110e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.5L, 0x1p511L, 1.570796326794896619231321691639751442099L, 1.491668146240041348658193063092586767475e-154L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.5L, 0x1p511L, -1.570796326794896619231321691639751442099L, 1.491668146240041348658193063092586767475e-154L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.5L, -0x1p511L, 1.570796326794896619231321691639751442099L, -1.491668146240041348658193063092586767475e-154L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.5L, -0x1p511L, -1.570796326794896619231321691639751442099L, -1.491668146240041348658193063092586767475e-154L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(catan, 0x1p8191L, 0.5L, 1.570796326794896619231321691639751442099L, 1.681051571556046753131338908660876301299e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1p8191L, -0.5L, 1.570796326794896619231321691639751442099L, -1.681051571556046753131338908660876301299e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1p8191L, 0.5L, -1.570796326794896619231321691639751442099L, 1.681051571556046753131338908660876301299e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1p8191L, -0.5L, -1.570796326794896619231321691639751442099L, -1.681051571556046753131338908660876301299e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0.5L, 0x1p8191L, 1.570796326794896619231321691639751442099L, 1.833603867554847165621412392048483165956e-2466L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0.5L, 0x1p8191L, -1.570796326794896619231321691639751442099L, 1.833603867554847165621412392048483165956e-2466L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0.5L, -0x1p8191L, 1.570796326794896619231321691639751442099L, -1.833603867554847165621412392048483165956e-2466L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0.5L, -0x1p8191L, -1.570796326794896619231321691639751442099L, -1.833603867554847165621412392048483165956e-2466L, 0|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(catan, 0x1p100L, 0.0L, 1.570796326794896619231321691638962581193L, 0.0L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1p100L, -0.0L, 1.570796326794896619231321691638962581193L, -0.0L, 0|XFAIL_NATIVE_LINUXF|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p100L, 0.0L, -1.570796326794896619231321691638962581193L, 0.0L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1p100L, -0.0L, -1.570796326794896619231321691638962581193L, -0.0L, 0|XFAIL_NATIVE_LINUXF|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.0L, 0x1p100L, 1.570796326794896619231321691639751442099L, 7.888609052210118054117285652827862296732e-31L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.0L, 0x1p100L, -1.570796326794896619231321691639751442099L, 7.888609052210118054117285652827862296732e-31L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.0L, -0x1p100L, 1.570796326794896619231321691639751442099L, -7.888609052210118054117285652827862296732e-31L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.0L, -0x1p100L, -1.570796326794896619231321691639751442099L, -7.888609052210118054117285652827862296732e-31L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.fp127L, 0.0L, 1.570796326794896619231321691639751442096L, 0.0L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1.fp127L, -0.0L, 1.570796326794896619231321691639751442096L, -0.0L, 0|XFAIL_NATIVE_LINUXF|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.fp127L, 0.0L, -1.570796326794896619231321691639751442096L, 0.0L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1.fp127L, -0.0L, -1.570796326794896619231321691639751442096L, -0.0L, 0|XFAIL_NATIVE_LINUXF|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.0L, 0x1.fp127L, 1.570796326794896619231321691639751442099L, 3.033533808573645181854803967025150136306e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.0L, 0x1.fp127L, -1.570796326794896619231321691639751442099L, 3.033533808573645181854803967025150136306e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.0L, -0x1.fp127L, 1.570796326794896619231321691639751442099L, -3.033533808573645181854803967025150136306e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.0L, -0x1.fp127L, -1.570796326794896619231321691639751442099L, -3.033533808573645181854803967025150136306e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.fp127L, 0x1.fp127L, 1.570796326794896619231321691639751442097L, 1.516766904286822590927401983512575068153e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.fp127L, -0x1.fp127L, 1.570796326794896619231321691639751442097L, -1.516766904286822590927401983512575068153e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.fp127L, 0x1.fp127L, -1.570796326794896619231321691639751442097L, 1.516766904286822590927401983512575068153e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.fp127L, -0x1.fp127L, -1.570796326794896619231321691639751442097L, -1.516766904286822590927401983512575068153e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(catan, 0x1p900L, 0.0L, 1.570796326794896619231321691639751442099L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p900L, -0.0L, 1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p900L, 0.0L, -1.570796326794896619231321691639751442099L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p900L, -0.0L, -1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.0L, 0x1p900L, 1.570796326794896619231321691639751442099L, 1.183052186166774710972751597518026531652e-271L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.0L, 0x1p900L, -1.570796326794896619231321691639751442099L, 1.183052186166774710972751597518026531652e-271L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.0L, -0x1p900L, 1.570796326794896619231321691639751442099L, -1.183052186166774710972751597518026531652e-271L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.0L, -0x1p900L, -1.570796326794896619231321691639751442099L, -1.183052186166774710972751597518026531652e-271L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.fp1023L, 0.0L, 1.570796326794896619231321691639751442099L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.fp1023L, -0.0L, 1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.fp1023L, 0.0L, -1.570796326794896619231321691639751442099L, 0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.fp1023L, -0.0L, -1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.0L, 0x1.fp1023L, 1.570796326794896619231321691639751442099L, 5.742126086470197117652213464083623391533e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.0L, 0x1.fp1023L, -1.570796326794896619231321691639751442099L, 5.742126086470197117652213464083623391533e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0.0L, -0x1.fp1023L, 1.570796326794896619231321691639751442099L, -5.742126086470197117652213464083623391533e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0.0L, -0x1.fp1023L, -1.570796326794896619231321691639751442099L, -5.742126086470197117652213464083623391533e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.fp1023L, 0x1.fp1023L, 1.570796326794896619231321691639751442099L, 2.871063043235098558826106732041811695767e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.fp1023L, -0x1.fp1023L, 1.570796326794896619231321691639751442099L, -2.871063043235098558826106732041811695767e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.fp1023L, 0x1.fp1023L, -1.570796326794896619231321691639751442099L, 2.871063043235098558826106732041811695767e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.fp1023L, -0x1.fp1023L, -1.570796326794896619231321691639751442099L, -2.871063043235098558826106732041811695767e-309L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(catan, 0x1p10000L, 0.0L, 1.570796326794896619231321691639751442099L, 0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1p10000L, -0.0L, 1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1p10000L, 0.0L, -1.570796326794896619231321691639751442099L, 0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1p10000L, -0.0L, -1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0.0L, 0x1p10000L, 1.570796326794896619231321691639751442099L, 5.012372749206452009297555933742977749322e-3011L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0.0L, 0x1p10000L, -1.570796326794896619231321691639751442099L, 5.012372749206452009297555933742977749322e-3011L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0.0L, -0x1p10000L, 1.570796326794896619231321691639751442099L, -5.012372749206452009297555933742977749322e-3011L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0.0L, -0x1p10000L, -1.570796326794896619231321691639751442099L, -5.012372749206452009297555933742977749322e-3011L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1.fp16383L, 0.0L, 1.570796326794896619231321691639751442099L, 0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1.fp16383L, -0.0L, 1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.fp16383L, 0.0L, -1.570796326794896619231321691639751442099L, 0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.fp16383L, -0.0L, -1.570796326794896619231321691639751442099L, -0.0L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0.0L, 0x1.fp16383L, 1.570796326794896619231321691639751442099L, 8.676395208031209048419813722120651877672e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0.0L, 0x1.fp16383L, -1.570796326794896619231321691639751442099L, 8.676395208031209048419813722120651877672e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0.0L, -0x1.fp16383L, 1.570796326794896619231321691639751442099L, -8.676395208031209048419813722120651877672e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0.0L, -0x1.fp16383L, -1.570796326794896619231321691639751442099L, -8.676395208031209048419813722120651877672e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1.fp16383L, 0x1.fp16383L, 1.570796326794896619231321691639751442099L, 4.338197604015604524209906861060325938836e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1.fp16383L, -0x1.fp16383L, 1.570796326794896619231321691639751442099L, -4.338197604015604524209906861060325938836e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.fp16383L, 0x1.fp16383L, -1.570796326794896619231321691639751442099L, 4.338197604015604524209906861060325938836e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.fp16383L, -0x1.fp16383L, -1.570796326794896619231321691639751442099L, -4.338197604015604524209906861060325938836e-4933L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(catan, 0x1p-13L, 1.0L, 7.854286809755354140031716771044626356262e-1L, 4.852030264850939738801379894163661227127L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1p-13L, -1.0L, 7.854286809755354140031716771044626356262e-1L, -4.852030264850939738801379894163661227127L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1p-13L, 1.0L, -7.854286809755354140031716771044626356262e-1L, 4.852030264850939738801379894163661227127L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1p-13L, -1.0L, -7.854286809755354140031716771044626356262e-1L, -4.852030264850939738801379894163661227127L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 1.0L, 0x1p-13L, 7.853981671227386080775748393881580082970e-1L, 6.103515609841754902688560615027452023669e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -1.0L, 0x1p-13L, -7.853981671227386080775748393881580082970e-1L, 6.103515609841754902688560615027452023669e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 1.0L, -0x1p-13L, 7.853981671227386080775748393881580082970e-1L, -6.103515609841754902688560615027452023669e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -1.0L, -0x1p-13L, -7.853981671227386080775748393881580082970e-1L, -6.103515609841754902688560615027452023669e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1p-27L, 1.0L, 7.853981652600934588466178684534110069553e-1L, 9.704060527839234335310696652368086117807L, 0 },
	TEST_c_c(catan, 0x1p-27L, -1.0L, 7.853981652600934588466178684534110069553e-1L, -9.704060527839234335310696652368086117807L, 0 },
	TEST_c_c(catan, -0x1p-27L, 1.0L, -7.853981652600934588466178684534110069553e-1L, 9.704060527839234335310696652368086117807L, 0 },
	TEST_c_c(catan, -0x1p-27L, -1.0L, -7.853981652600934588466178684534110069553e-1L, -9.704060527839234335310696652368086117807L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 1.0L, 0x1p-27L, 7.853981633974483234934486536343324763447e-1L, 3.725290298461914028034141143623846306386e-9L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -1.0L, 0x1p-27L, -7.853981633974483234934486536343324763447e-1L, 3.725290298461914028034141143623846306386e-9L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 1.0L, -0x1p-27L, 7.853981633974483234934486536343324763447e-1L, -3.725290298461914028034141143623846306386e-9L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -1.0L, -0x1p-27L, -7.853981633974483234934486536343324763447e-1L, -3.725290298461914028034141143623846306386e-9L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1p-33L, 1.0L, 7.853981634265521400723945494331241018449e-1L, 1.178350206951907026009379309773625595762e1L, 0 },
	TEST_c_c(catan, 0x1p-33L, -1.0L, 7.853981634265521400723945494331241018449e-1L, -1.178350206951907026009379309773625595762e1L, 0 },
	TEST_c_c(catan, -0x1p-33L, 1.0L, -7.853981634265521400723945494331241018449e-1L, 1.178350206951907026009379309773625595762e1L, 0 },
	TEST_c_c(catan, -0x1p-33L, -1.0L, -7.853981634265521400723945494331241018449e-1L, -1.178350206951907026009379309773625595762e1L, 0 },
	TEST_c_c(catan, 1.0L, 0x1p-33L, 7.853981633974483096190489776088929224056e-1L, 5.820766091346740722643102318246316469910e-11L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -1.0L, 0x1p-33L, -7.853981633974483096190489776088929224056e-1L, 5.820766091346740722643102318246316469910e-11L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 1.0L, -0x1p-33L, 7.853981633974483096190489776088929224056e-1L, -5.820766091346740722643102318246316469910e-11L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -1.0L, -0x1p-33L, -7.853981633974483096190489776088929224056e-1L, -5.820766091346740722643102318246316469910e-11L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1p-54L, 1.0L, 7.853981633974483234934486536343324763447e-1L, 1.906154746539849600897388334009985581467e1L, 0 },
	TEST_c_c(catan, 0x1p-54L, -1.0L, 7.853981633974483234934486536343324763447e-1L, -1.906154746539849600897388334009985581467e1L, 0 },
	TEST_c_c(catan, -0x1p-54L, 1.0L, -7.853981633974483234934486536343324763447e-1L, 1.906154746539849600897388334009985581467e1L, 0 },
	TEST_c_c(catan, -0x1p-54L, -1.0L, -7.853981633974483234934486536343324763447e-1L, -1.906154746539849600897388334009985581467e1L, 0 },
	TEST_c_c(catan, 1.0L, 0x1p-54L, 7.853981633974483096156608458198764914213e-1L, 2.775557561562891351059079170227049355775e-17L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, 0x1p-54L, -7.853981633974483096156608458198764914213e-1L, 2.775557561562891351059079170227049355775e-17L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 1.0L, -0x1p-54L, 7.853981633974483096156608458198764914213e-1L, -2.775557561562891351059079170227049355775e-17L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, -0x1p-54L, -7.853981633974483096156608458198764914213e-1L, -2.775557561562891351059079170227049355775e-17L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-57L, 1.0L, 7.853981633974483113503843217966828154612e-1L, 2.010126823623841397309973152228712047720e1L, 0 },
	TEST_c_c(catan, 0x1p-57L, -1.0L, 7.853981633974483113503843217966828154612e-1L, -2.010126823623841397309973152228712047720e1L, 0 },
	TEST_c_c(catan, -0x1p-57L, 1.0L, -7.853981633974483113503843217966828154612e-1L, 2.010126823623841397309973152228712047720e1L, 0 },
	TEST_c_c(catan, -0x1p-57L, -1.0L, -7.853981633974483113503843217966828154612e-1L, -2.010126823623841397309973152228712047720e1L, 0 },
	TEST_c_c(catan, 1.0L, 0x1p-57L, 7.853981633974483096156608458198757330864e-1L, 3.469446951953614188823848962783813448721e-18L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, 0x1p-57L, -7.853981633974483096156608458198757330864e-1L, 3.469446951953614188823848962783813448721e-18L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 1.0L, -0x1p-57L, 7.853981633974483096156608458198757330864e-1L, -3.469446951953614188823848962783813448721e-18L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, -0x1p-57L, -7.853981633974483096156608458198757330864e-1L, -3.469446951953614188823848962783813448721e-18L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-13L, 0x1.000002p0L, 7.859169620684960844300240092596908675974e-1L, 4.852030056234795712498957387213592193975L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1p-13L, -0x1.000002p0L, 7.859169620684960844300240092596908675974e-1L, -4.852030056234795712498957387213592193975L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1p-13L, 0x1.000002p0L, -7.859169620684960844300240092596908675974e-1L, 4.852030056234795712498957387213592193975L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1p-13L, -0x1.000002p0L, -7.859169620684960844300240092596908675974e-1L, -4.852030056234795712498957387213592193975L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1.000002p0L, 0x1p-13L, 7.853982267273793866654490522673596014524e-1L, 6.103514882246036852433556327261700380577e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1.000002p0L, 0x1p-13L, -7.853982267273793866654490522673596014524e-1L, 6.103514882246036852433556327261700380577e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1.000002p0L, -0x1p-13L, 7.853982267273793866654490522673596014524e-1L, -6.103514882246036852433556327261700380577e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1.000002p0L, -0x1p-13L, -7.853982267273793866654490522673596014524e-1L, -6.103514882246036852433556327261700380577e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1p-13L, 0x0.ffffffp0L, 7.851845403708474595909269086711426246675e-1L, 4.852030190345140708455871037447717761868L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1p-13L, -0x0.ffffffp0L, 7.851845403708474595909269086711426246675e-1L, -4.852030190345140708455871037447717761868L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1p-13L, 0x0.ffffffp0L, -7.851845403708474595909269086711426246675e-1L, 4.852030190345140708455871037447717761868L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1p-13L, -0x0.ffffffp0L, -7.851845403708474595909269086711426246675e-1L, -4.852030190345140708455871037447717761868L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x0.ffffffp0L, 0x1p-13L, 7.853981373204155542484315721351697277336e-1L, 6.103515973639646453881721999956617260502e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x0.ffffffp0L, 0x1p-13L, -7.853981373204155542484315721351697277336e-1L, 6.103515973639646453881721999956617260502e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x0.ffffffp0L, -0x1p-13L, 7.853981373204155542484315721351697277336e-1L, -6.103515973639646453881721999956617260502e-5L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x0.ffffffp0L, -0x1p-13L, -7.853981373204155542484315721351697277336e-1L, -6.103515973639646453881721999956617260502e-5L, 0|XFAIL_NATIVE_LINUXF },
#ifndef TEST_FLOAT
	TEST_c_c(catan, 0x1p-27L, 0x1.0000000000001p0L, 7.853981801612546526942695000283242525531e-1L, 9.704060527839234168777242958594699810015L, 0 },
	TEST_c_c(catan, 0x1p-27L, -0x1.0000000000001p0L, 7.853981801612546526942695000283242525531e-1L, -9.704060527839234168777242958594699810015L, 0 },
	TEST_c_c(catan, -0x1p-27L, 0x1.0000000000001p0L, -7.853981801612546526942695000283242525531e-1L, 9.704060527839234168777242958594699810015L, 0 },
	TEST_c_c(catan, -0x1p-27L, -0x1.0000000000001p0L, -7.853981801612546526942695000283242525531e-1L, -9.704060527839234168777242958594699810015L, 0 },
	TEST_c_c(catan, 0x1.0000000000001p0L, 0x1p-27L, 7.853981633974484345157511161499711112683e-1L, 3.725290298461913200853528590596263270474e-9L, 0 },
	TEST_c_c(catan, -0x1.0000000000001p0L, 0x1p-27L, -7.853981633974484345157511161499711112683e-1L, 3.725290298461913200853528590596263270474e-9L, 0 },
	TEST_c_c(catan, 0x1.0000000000001p0L, -0x1p-27L, 7.853981633974484345157511161499711112683e-1L, -3.725290298461913200853528590596263270474e-9L, 0 },
	TEST_c_c(catan, -0x1.0000000000001p0L, -0x1p-27L, -7.853981633974484345157511161499711112683e-1L, -3.725290298461913200853528590596263270474e-9L, 0 },
	TEST_c_c(catan, 0x1p-27L, 0x0.fffffffffffff8p0L, 7.853981578095128619227903983047292781021e-1L, 9.704060527839234252043969805481351363824L, 0 },
	TEST_c_c(catan, 0x1p-27L, -0x0.fffffffffffff8p0L, 7.853981578095128619227903983047292781021e-1L, -9.704060527839234252043969805481351363824L, 0 },
	TEST_c_c(catan, -0x1p-27L, 0x0.fffffffffffff8p0L, -7.853981578095128619227903983047292781021e-1L, 9.704060527839234252043969805481351363824L, 0 },
	TEST_c_c(catan, -0x1p-27L, -0x0.fffffffffffff8p0L, -7.853981578095128619227903983047292781021e-1L, -9.704060527839234252043969805481351363824L, 0 },
	TEST_c_c(catan, 0x0.fffffffffffff8p0L, 0x1p-27L, 7.853981633974482679822974223765039144191e-1L, 3.725290298461914441624447420137706700965e-9L, 0 },
	TEST_c_c(catan, -0x0.fffffffffffff8p0L, 0x1p-27L, -7.853981633974482679822974223765039144191e-1L, 3.725290298461914441624447420137706700965e-9L, 0 },
	TEST_c_c(catan, 0x0.fffffffffffff8p0L, -0x1p-27L, 7.853981633974482679822974223765039144191e-1L, -3.725290298461914441624447420137706700965e-9L, 0 },
	TEST_c_c(catan, -0x0.fffffffffffff8p0L, -0x1p-27L, -7.853981633974482679822974223765039144191e-1L, -3.725290298461914441624447420137706700965e-9L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(catan, 0x1p-33L, 0x1.0000000000000002p0L, 7.853981638922134273801338071094141188767e-1L, 1.178350206951907025990405771755129268176e1L, 0 },
	TEST_c_c(catan, 0x1p-33L, -0x1.0000000000000002p0L, 7.853981638922134273801338071094141188767e-1L, -1.178350206951907025990405771755129268176e1L, 0 },
	TEST_c_c(catan, -0x1p-33L, 0x1.0000000000000002p0L, -7.853981638922134273801338071094141188767e-1L, 1.178350206951907025990405771755129268176e1L, 0 },
	TEST_c_c(catan, -0x1p-33L, -0x1.0000000000000002p0L, -7.853981638922134273801338071094141188767e-1L, -1.178350206951907025990405771755129268176e1L, 0 },
	TEST_c_c(catan, 0x1.0000000000000002p0L, 0x1p-33L, 7.853981633974483096732590862331681441026e-1L, 5.820766091346740722012013594069507025615e-11L, 0 },
	TEST_c_c(catan, -0x1.0000000000000002p0L, 0x1p-33L, -7.853981633974483096732590862331681441026e-1L, 5.820766091346740722012013594069507025615e-11L, 0 },
	TEST_c_c(catan, 0x1.0000000000000002p0L, -0x1p-33L, 7.853981633974483096732590862331681441026e-1L, -5.820766091346740722012013594069507025615e-11L, 0 },
	TEST_c_c(catan, -0x1.0000000000000002p0L, -0x1p-33L, -7.853981633974483096732590862331681441026e-1L, -5.820766091346740722012013594069507025615e-11L, 0 },
	TEST_c_c(catan, 0x1p-33L, 0x0.ffffffffffffffffp0L, 7.853981631937214964185249205444919953948e-1L, 1.178350206951907026002603046195591193050e1L, 0 },
	TEST_c_c(catan, 0x1p-33L, -0x0.ffffffffffffffffp0L, 7.853981631937214964185249205444919953948e-1L, -1.178350206951907026002603046195591193050e1L, 0 },
	TEST_c_c(catan, -0x1p-33L, 0x0.ffffffffffffffffp0L, -7.853981631937214964185249205444919953948e-1L, 1.178350206951907026002603046195591193050e1L, 0 },
	TEST_c_c(catan, -0x1p-33L, -0x0.ffffffffffffffffp0L, -7.853981631937214964185249205444919953948e-1L, -1.178350206951907026002603046195591193050e1L, 0 },
	TEST_c_c(catan, 0x0.ffffffffffffffffp0L, 0x1p-33L, 7.853981633974483095919439232967553115548e-1L, 5.820766091346740722958646680334721192083e-11L, 0 },
	TEST_c_c(catan, -0x0.ffffffffffffffffp0L, 0x1p-33L, -7.853981633974483095919439232967553115548e-1L, 5.820766091346740722958646680334721192083e-11L, 0 },
	TEST_c_c(catan, 0x0.ffffffffffffffffp0L, -0x1p-33L, 7.853981633974483095919439232967553115548e-1L, -5.820766091346740722958646680334721192083e-11L, 0 },
	TEST_c_c(catan, -0x0.ffffffffffffffffp0L, -0x1p-33L, -7.853981633974483095919439232967553115548e-1L, -5.820766091346740722958646680334721192083e-11L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(catan, 0x1p-54L, 0x1.000000000000000000000000008p0L, 7.853981633974485455380535786656405610710e-1L, 1.906154746539849600897388334009981267384e1L, 0 },
	TEST_c_c(catan, 0x1p-54L, -0x1.000000000000000000000000008p0L, 7.853981633974485455380535786656405610710e-1L, -1.906154746539849600897388334009981267384e1L, 0 },
	TEST_c_c(catan, -0x1p-54L, 0x1.000000000000000000000000008p0L, -7.853981633974485455380535786656405610710e-1L, 1.906154746539849600897388334009981267384e1L, 0 },
	TEST_c_c(catan, -0x1p-54L, -0x1.000000000000000000000000008p0L, -7.853981633974485455380535786656405610710e-1L, -1.906154746539849600897388334009981267384e1L, 0 },
	TEST_c_c(catan, 0x1.000000000000000000000000008p0L, 0x1p-54L, 7.853981633974483096156608458198888173729e-1L, 2.775557561562891351059079170226980932999e-17L, 0 },
	TEST_c_c(catan, -0x1.000000000000000000000000008p0L, 0x1p-54L, -7.853981633974483096156608458198888173729e-1L, 2.775557561562891351059079170226980932999e-17L, 0 },
	TEST_c_c(catan, 0x1.000000000000000000000000008p0L, -0x1p-54L, 7.853981633974483096156608458198888173729e-1L, -2.775557561562891351059079170226980932999e-17L, 0 },
	TEST_c_c(catan, -0x1.000000000000000000000000008p0L, -0x1p-54L, -7.853981633974483096156608458198888173729e-1L, -2.775557561562891351059079170226980932999e-17L, 0 },
	TEST_c_c(catan, 0x1p-54L, 0x0.ffffffffffffffffffffffffffcp0L, 7.853981633974482124711461911186784339815e-1L, 1.906154746539849600897388334009984040723e1L, 0 },
	TEST_c_c(catan, 0x1p-54L, -0x0.ffffffffffffffffffffffffffcp0L, 7.853981633974482124711461911186784339815e-1L, -1.906154746539849600897388334009984040723e1L, 0 },
	TEST_c_c(catan, -0x1p-54L, 0x0.ffffffffffffffffffffffffffcp0L, -7.853981633974482124711461911186784339815e-1L, 1.906154746539849600897388334009984040723e1L, 0 },
	TEST_c_c(catan, -0x1p-54L, -0x0.ffffffffffffffffffffffffffcp0L, -7.853981633974482124711461911186784339815e-1L, -1.906154746539849600897388334009984040723e1L, 0 },
	TEST_c_c(catan, 0x0.ffffffffffffffffffffffffffcp0L, 0x1p-54L, 7.853981633974483096156608458198703284454e-1L, 2.775557561562891351059079170227083567164e-17L, 0 },
	TEST_c_c(catan, -0x0.ffffffffffffffffffffffffffcp0L, 0x1p-54L, -7.853981633974483096156608458198703284454e-1L, 2.775557561562891351059079170227083567164e-17L, 0 },
	TEST_c_c(catan, 0x0.ffffffffffffffffffffffffffcp0L, -0x1p-54L, 7.853981633974483096156608458198703284454e-1L, -2.775557561562891351059079170227083567164e-17L, 0 },
	TEST_c_c(catan, -0x0.ffffffffffffffffffffffffffcp0L, -0x1p-54L, -7.853981633974483096156608458198703284454e-1L, -2.775557561562891351059079170227083567164e-17L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(catan, 0x1p-57L, 0x1.0000000000000000000000000001p0L, 7.853981633974483252281721296111395707566e-1L, 2.010126823623841397309973152228712033275e1L, 0 },
	TEST_c_c(catan, 0x1p-57L, -0x1.0000000000000000000000000001p0L, 7.853981633974483252281721296111395707566e-1L, -2.010126823623841397309973152228712033275e1L, 0 },
	TEST_c_c(catan, -0x1p-57L, 0x1.0000000000000000000000000001p0L, -7.853981633974483252281721296111395707566e-1L, 2.010126823623841397309973152228712033275e1L, 0 },
	TEST_c_c(catan, -0x1p-57L, -0x1.0000000000000000000000000001p0L, -7.853981633974483252281721296111395707566e-1L, -2.010126823623841397309973152228712033275e1L, 0 },
	TEST_c_c(catan, 0x1.0000000000000000000000000001p0L, 0x1p-57L, 7.853981633974483096156608458198758293829e-1L, 3.469446951953614188823848962783812780530e-18L, 0 },
	TEST_c_c(catan, -0x1.0000000000000000000000000001p0L, 0x1p-57L, -7.853981633974483096156608458198758293829e-1L, 3.469446951953614188823848962783812780530e-18L, 0 },
	TEST_c_c(catan, 0x1.0000000000000000000000000001p0L, -0x1p-57L, 7.853981633974483096156608458198758293829e-1L, -3.469446951953614188823848962783812780530e-18L, 0 },
	TEST_c_c(catan, -0x1.0000000000000000000000000001p0L, -0x1p-57L, -7.853981633974483096156608458198758293829e-1L, -3.469446951953614188823848962783812780530e-18L, 0 },
	TEST_c_c(catan, 0x1p-57L, 0x0.ffffffffffffffffffffffffffff8p0L, 7.853981633974483044114904178894544378135e-1L, 2.010126823623841397309973152228712040498e1L, 0 },
	TEST_c_c(catan, 0x1p-57L, -0x0.ffffffffffffffffffffffffffff8p0L, 7.853981633974483044114904178894544378135e-1L, -2.010126823623841397309973152228712040498e1L, 0 },
	TEST_c_c(catan, -0x1p-57L, 0x0.ffffffffffffffffffffffffffff8p0L, -7.853981633974483044114904178894544378135e-1L, 2.010126823623841397309973152228712040498e1L, 0 },
	TEST_c_c(catan, -0x1p-57L, -0x0.ffffffffffffffffffffffffffff8p0L, -7.853981633974483044114904178894544378135e-1L, -2.010126823623841397309973152228712040498e1L, 0 },
	TEST_c_c(catan, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-57L, 7.853981633974483096156608458198756849381e-1L, 3.469446951953614188823848962783813782817e-18L, 0 },
	TEST_c_c(catan, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-57L, -7.853981633974483096156608458198756849381e-1L, 3.469446951953614188823848962783813782817e-18L, 0 },
	TEST_c_c(catan, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-57L, 7.853981633974483096156608458198756849381e-1L, -3.469446951953614188823848962783813782817e-18L, 0 },
	TEST_c_c(catan, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-57L, -7.853981633974483096156608458198756849381e-1L, -3.469446951953614188823848962783813782817e-18L, 0 },
#endif
	TEST_c_c(catan, 1.0L, 0x1p-64L, 7.853981633974483096156608458198757210500e-1L, 2.710505431213761085018632002174854278563e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 1.0L, -0x1p-64L, 7.853981633974483096156608458198757210500e-1L, -2.710505431213761085018632002174854278563e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, 0x1p-64L, -7.853981633974483096156608458198757210500e-1L, 2.710505431213761085018632002174854278563e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, -0x1p-64L, -7.853981633974483096156608458198757210500e-1L, -2.710505431213761085018632002174854278563e-20L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-64L, 1.0L, 7.853981633974483096292133729759445264744e-1L, 2.252728336819822255606004394739073846245e1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1p-64L, 1.0L, -7.853981633974483096292133729759445264744e-1L, 2.252728336819822255606004394739073846245e1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1p-64L, -1.0L, 7.853981633974483096292133729759445264744e-1L, -2.252728336819822255606004394739073846245e1L, 0 },
	TEST_c_c(catan, -0x1p-64L, -1.0L, -7.853981633974483096292133729759445264744e-1L, -2.252728336819822255606004394739073846245e1L, 0 },
	TEST_c_c(catan, 1.0L, 0x1.3p-73L, 7.853981633974483096156608458198757210493e-1L, 6.286572655403010329022706059731717687100e-23L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 1.0L, -0x1.3p-73L, 7.853981633974483096156608458198757210493e-1L, -6.286572655403010329022706059731717687100e-23L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, 0x1.3p-73L, -7.853981633974483096156608458198757210493e-1L, 6.286572655403010329022706059731717687100e-23L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, -0x1.3p-73L, -7.853981633974483096156608458198757210493e-1L, -6.286572655403010329022706059731717687100e-23L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.3p-73L, 1.0L, 7.853981633974483096156922786831527361009e-1L, 2.556052055225464683726753902092495938633e1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1.3p-73L, 1.0L, -7.853981633974483096156922786831527361009e-1L, 2.556052055225464683726753902092495938633e1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1.3p-73L, -1.0L, 7.853981633974483096156922786831527361009e-1L, -2.556052055225464683726753902092495938633e1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1.3p-73L, -1.0L, -7.853981633974483096156922786831527361009e-1L, -2.556052055225464683726753902092495938633e1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 1.0L, 0x1p-124L, 7.853981633974483096156608458198757210493e-1L, 2.350988701644575015937473074444491355637e-38L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 1.0L, -0x1p-124L, 7.853981633974483096156608458198757210493e-1L, -2.350988701644575015937473074444491355637e-38L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, 0x1p-124L, -7.853981633974483096156608458198757210493e-1L, 2.350988701644575015937473074444491355637e-38L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, -0x1p-124L, -7.853981633974483096156608458198757210493e-1L, -2.350988701644575015937473074444491355637e-38L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-124L, 1.0L, 7.853981633974483096156608458198757210610e-1L, 4.332169878499658183857700759113603550472e1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1p-124L, 1.0L, -7.853981633974483096156608458198757210610e-1L, 4.332169878499658183857700759113603550472e1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1p-124L, -1.0L, 7.853981633974483096156608458198757210610e-1L, -4.332169878499658183857700759113603550472e1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, -0x1p-124L, -1.0L, -7.853981633974483096156608458198757210610e-1L, -4.332169878499658183857700759113603550472e1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catan, 0x1.000002p0L, 0x1p-126L, 7.853982230020895322927482174768220815493e-1L, 5.877471053462247139383742414506671233336e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.000002p0L, -0x1p-126L, 7.853982230020895322927482174768220815493e-1L, -5.877471053462247139383742414506671233336e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.000002p0L, 0x1p-126L, -7.853982230020895322927482174768220815493e-1L, 5.877471053462247139383742414506671233336e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.000002p0L, -0x1p-126L, -7.853982230020895322927482174768220815493e-1L, -5.877471053462247139383742414506671233336e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-126L, 0x1.000002p0L, 1.570796326794896619231321691639702138295L, 8.317766196521665212523713550410777685446L, 0 },
	TEST_c_c(catan, -0x1p-126L, 0x1.000002p0L, -1.570796326794896619231321691639702138295L, 8.317766196521665212523713550410777685446L, 0 },
	TEST_c_c(catan, 0x1p-126L, -0x1.000002p0L, 1.570796326794896619231321691639702138295L, -8.317766196521665212523713550410777685446L, 0 },
	TEST_c_c(catan, -0x1p-126L, -0x1.000002p0L, -1.570796326794896619231321691639702138295L, -8.317766196521665212523713550410777685446L, 0 },
	TEST_c_c(catan, 0x0.ffffffp0L, 0x1p-126L, 7.853981335951250337419109991749089175724e-1L, 5.877472104436064061535099214664320916184e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x0.ffffffp0L, -0x1p-126L, 7.853981335951250337419109991749089175724e-1L, -5.877472104436064061535099214664320916184e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x0.ffffffp0L, 0x1p-126L, -7.853981335951250337419109991749089175724e-1L, 5.877472104436064061535099214664320916184e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x0.ffffffp0L, -0x1p-126L, -7.853981335951250337419109991749089175724e-1L, -5.877472104436064061535099214664320916184e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-126L, 0x0.ffffffp0L, 9.860761609136244031334147100512154160097e-32L, 8.664339742098154951823135931565866792454L, 0 },
	TEST_c_c(catan, -0x1p-126L, 0x0.ffffffp0L, -9.860761609136244031334147100512154160097e-32L, 8.664339742098154951823135931565866792454L, 0 },
	TEST_c_c(catan, 0x1p-126L, -0x0.ffffffp0L, 9.860761609136244031334147100512154160097e-32L, -8.664339742098154951823135931565866792454L, 0 },
	TEST_c_c(catan, -0x1p-126L, -0x0.ffffffp0L, -9.860761609136244031334147100512154160097e-32L, -8.664339742098154951823135931565866792454L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(catan, 1.0L, 0x1p-512L, 7.853981633974483096156608458198757210493e-1L, 3.729170365600103371645482657731466918688e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 1.0L, -0x1p-512L, 7.853981633974483096156608458198757210493e-1L, -3.729170365600103371645482657731466918688e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, 0x1p-512L, -7.853981633974483096156608458198757210493e-1L, 3.729170365600103371645482657731466918688e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, -0x1p-512L, -7.853981633974483096156608458198757210493e-1L, -3.729170365600103371645482657731466918688e-155L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-512L, 1.0L, 7.853981633974483096156608458198757210493e-1L, 1.777922518136259718655200391540222897114e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p-512L, 1.0L, -7.853981633974483096156608458198757210493e-1L, 1.777922518136259718655200391540222897114e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-512L, -1.0L, 7.853981633974483096156608458198757210493e-1L, -1.777922518136259718655200391540222897114e2L, 0 },
	TEST_c_c(catan, -0x1p-512L, -1.0L, -7.853981633974483096156608458198757210493e-1L, -1.777922518136259718655200391540222897114e2L, 0 },
	TEST_c_c(catan, 1.0L, 0x1.3p-536L, 7.853981633974483096156608458198757210493e-1L, 2.639526015013529511588222179446290115084e-162L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 1.0L, -0x1.3p-536L, 7.853981633974483096156608458198757210493e-1L, -2.639526015013529511588222179446290115084e-162L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, 0x1.3p-536L, -7.853981633974483096156608458198757210493e-1L, 2.639526015013529511588222179446290115084e-162L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, -0x1.3p-536L, -7.853981633974483096156608458198757210493e-1L, -2.639526015013529511588222179446290115084e-162L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.3p-536L, 1.0L, 7.853981633974483096156608458198757210493e-1L, 1.860240928518819859673567751384928348958e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.3p-536L, 1.0L, -7.853981633974483096156608458198757210493e-1L, 1.860240928518819859673567751384928348958e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.3p-536L, -1.0L, 7.853981633974483096156608458198757210493e-1L, -1.860240928518819859673567751384928348958e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.3p-536L, -1.0L, -7.853981633974483096156608458198757210493e-1L, -1.860240928518819859673567751384928348958e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 1.0L, 0x1p-1020L, 7.853981633974483096156608458198757210493e-1L, 4.450147717014402766180465434664808128438e-308L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 1.0L, -0x1p-1020L, 7.853981633974483096156608458198757210493e-1L, -4.450147717014402766180465434664808128438e-308L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, 0x1p-1020L, -7.853981633974483096156608458198757210493e-1L, 4.450147717014402766180465434664808128438e-308L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -1.0L, -0x1p-1020L, -7.853981633974483096156608458198757210493e-1L, -4.450147717014402766180465434664808128438e-308L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-1020L, 1.0L, 7.853981633974483096156608458198757210493e-1L, 3.538516356758520804574969980043991380025e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p-1020L, 1.0L, -7.853981633974483096156608458198757210493e-1L, 3.538516356758520804574969980043991380025e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-1020L, -1.0L, 7.853981633974483096156608458198757210493e-1L, -3.538516356758520804574969980043991380025e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1p-1020L, -1.0L, -7.853981633974483096156608458198757210493e-1L, -3.538516356758520804574969980043991380025e2L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.0000000000001p0L, 0x1p-1022L, 7.853981633974484206379633083355174374608e-1L, 1.112536929253600444512293438042957369978e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1.0000000000001p0L, -0x1p-1022L, 7.853981633974484206379633083355174374608e-1L, -1.112536929253600444512293438042957369978e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.0000000000001p0L, 0x1p-1022L, -7.853981633974484206379633083355174374608e-1L, 1.112536929253600444512293438042957369978e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x1.0000000000001p0L, -0x1p-1022L, -7.853981633974484206379633083355174374608e-1L, -1.112536929253600444512293438042957369978e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-1022L, 0x1.0000000000001p0L, 1.570796326794896619231321691639751442099L, 1.836840028483855075506780244989950299369e1L, 0 },
	TEST_c_c(catan, -0x1p-1022L, 0x1.0000000000001p0L, -1.570796326794896619231321691639751442099L, 1.836840028483855075506780244989950299369e1L, 0 },
	TEST_c_c(catan, 0x1p-1022L, -0x1.0000000000001p0L, 1.570796326794896619231321691639751442099L, -1.836840028483855075506780244989950299369e1L, 0 },
	TEST_c_c(catan, -0x1p-1022L, -0x1.0000000000001p0L, -1.570796326794896619231321691639751442099L, -1.836840028483855075506780244989950299369e1L, 0 },
	TEST_c_c(catan, 0x0.fffffffffffff8p0L, 0x1p-1022L, 7.853981633974482541045096145620456183798e-1L, 1.112536929253600815061527818977844932790e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x0.fffffffffffff8p0L, -0x1p-1022L, 7.853981633974482541045096145620456183798e-1L, -1.112536929253600815061527818977844932790e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x0.fffffffffffff8p0L, 0x1p-1022L, -7.853981633974482541045096145620456183798e-1L, 1.112536929253600815061527818977844932790e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, -0x0.fffffffffffff8p0L, -0x1p-1022L, -7.853981633974482541045096145620456183798e-1L, -1.112536929253600815061527818977844932790e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catan, 0x1p-1022L, 0x0.fffffffffffff8p0L, 1.002084180004486444624900488355118689113e-292L, 1.871497387511852332650969166374185305708e1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(catan, -0x1p-1022L, 0x0.fffffffffffff8p0L, -1.002084180004486444624900488355118689113e-292L, 1.871497387511852332650969166374185305708e1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(catan, 0x1p-1022L, -0x0.fffffffffffff8p0L, 1.002084180004486444624900488355118689113e-292L, -1.871497387511852332650969166374185305708e1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(catan, -0x1p-1022L, -0x0.fffffffffffff8p0L, -1.002084180004486444624900488355118689113e-292L, -1.871497387511852332650969166374185305708e1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(catan, 0x1.0000000000000002p0L, 0x1p-1022L, 7.853981633974483096698709544441509427467e-1L, 1.112536929253600691424494863099491450042e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1.0000000000000002p0L, -0x1p-1022L, 7.853981633974483096698709544441509427467e-1L, -1.112536929253600691424494863099491450042e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.0000000000000002p0L, 0x1p-1022L, -7.853981633974483096698709544441509427467e-1L, 1.112536929253600691424494863099491450042e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.0000000000000002p0L, -0x1p-1022L, -7.853981633974483096698709544441509427467e-1L, -1.112536929253600691424494863099491450042e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1p-1022L, 0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, 2.218070977791824990137853294097378778927e1L, 0 },
	TEST_c_c(catan, -0x1p-1022L, 0x1.0000000000000002p0L, -1.570796326794896619231321691639751442099L, 2.218070977791824990137853294097378778927e1L, 0 },
	TEST_c_c(catan, 0x1p-1022L, -0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, -2.218070977791824990137853294097378778927e1L, 0 },
	TEST_c_c(catan, -0x1p-1022L, -0x1.0000000000000002p0L, -1.570796326794896619231321691639751442099L, -2.218070977791824990137853294097378778927e1L, 0 },
	TEST_c_c(catan, 0x0.ffffffffffffffffp0L, 0x1p-1022L, 7.853981633974483095885557915077381101984e-1L, 1.112536929253600691605427106449557323148e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x0.ffffffffffffffffp0L, -0x1p-1022L, 7.853981633974483095885557915077381101984e-1L, -1.112536929253600691605427106449557323148e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x0.ffffffffffffffffp0L, 0x1p-1022L, -7.853981633974483095885557915077381101984e-1L, 1.112536929253600691605427106449557323148e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x0.ffffffffffffffffp0L, -0x1p-1022L, -7.853981633974483095885557915077381101984e-1L, -1.112536929253600691605427106449557323148e-308L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1p-1022L, 0x0.ffffffffffffffffp0L, 2.052268400649188124723641491045245971623e-289L, 2.252728336819822255604649142023466965703e1L, 0 },
	TEST_c_c(catan, -0x1p-1022L, 0x0.ffffffffffffffffp0L, -2.052268400649188124723641491045245971623e-289L, 2.252728336819822255604649142023466965703e1L, 0 },
	TEST_c_c(catan, 0x1p-1022L, -0x0.ffffffffffffffffp0L, 2.052268400649188124723641491045245971623e-289L, -2.252728336819822255604649142023466965703e1L, 0 },
	TEST_c_c(catan, -0x1p-1022L, -0x0.ffffffffffffffffp0L, -2.052268400649188124723641491045245971623e-289L, -2.252728336819822255604649142023466965703e1L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(catan, 0x1.000000000000000000000000008p0L, 0x1p-1022L, 7.853981633974483096156608458198880470009e-1L, 1.112536929253600691545116358666174605957e-308L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catan, 0x1.000000000000000000000000008p0L, -0x1p-1022L, 7.853981633974483096156608458198880470009e-1L, -1.112536929253600691545116358666174605957e-308L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catan, -0x1.000000000000000000000000008p0L, 0x1p-1022L, -7.853981633974483096156608458198880470009e-1L, 1.112536929253600691545116358666174605957e-308L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catan, -0x1.000000000000000000000000008p0L, -0x1p-1022L, -7.853981633974483096156608458198880470009e-1L, -1.112536929253600691545116358666174605957e-308L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catan, 0x1p-1022L, 0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639751442099L, 3.673680056967710139911330243728336427098e1L, 0 },
	TEST_c_c(catan, -0x1p-1022L, 0x1.000000000000000000000000008p0L, -1.570796326794896619231321691639751442099L, 3.673680056967710139911330243728336427098e1L, 0 },
	TEST_c_c(catan, 0x1p-1022L, -0x1.000000000000000000000000008p0L, 1.570796326794896619231321691639751442099L, -3.673680056967710139911330243728336427098e1L, 0 },
	TEST_c_c(catan, -0x1p-1022L, -0x1.000000000000000000000000008p0L, -1.570796326794896619231321691639751442099L, -3.673680056967710139911330243728336427098e1L, 0 },
	TEST_c_c(catan, 0x0.ffffffffffffffffffffffffffcp0L, 0x1p-1022L, 7.853981633974483096156608458198695580735e-1L, 1.112536929253600691545116358666215745186e-308L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catan, 0x0.ffffffffffffffffffffffffffcp0L, -0x1p-1022L, 7.853981633974483096156608458198695580735e-1L, -1.112536929253600691545116358666215745186e-308L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catan, -0x0.ffffffffffffffffffffffffffcp0L, 0x1p-1022L, -7.853981633974483096156608458198695580735e-1L, 1.112536929253600691545116358666215745186e-308L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catan, -0x0.ffffffffffffffffffffffffffcp0L, -0x1p-1022L, -7.853981633974483096156608458198695580735e-1L, -1.112536929253600691545116358666215745186e-308L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catan, 0x1p-1022L, 0x0.ffffffffffffffffffffffffffcp0L, 9.025971879324147880346310405868788320726e-277L, 3.708337415995707405382191849801244331055e1L, 0 },
	TEST_c_c(catan, -0x1p-1022L, 0x0.ffffffffffffffffffffffffffcp0L, -9.025971879324147880346310405868788320726e-277L, 3.708337415995707405382191849801244331055e1L, 0 },
	TEST_c_c(catan, 0x1p-1022L, -0x0.ffffffffffffffffffffffffffcp0L, 9.025971879324147880346310405868788320726e-277L, -3.708337415995707405382191849801244331055e1L, 0 },
	TEST_c_c(catan, -0x1p-1022L, -0x0.ffffffffffffffffffffffffffcp0L, -9.025971879324147880346310405868788320726e-277L, -3.708337415995707405382191849801244331055e1L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(catan, 1.0L, 0x1p-8192L, 7.853981633974483096156608458198757210493e-1L, 4.584009668887117914053530980121207914891e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 1.0L, -0x1p-8192L, 7.853981633974483096156608458198757210493e-1L, -4.584009668887117914053530980121207914891e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -1.0L, 0x1p-8192L, -7.853981633974483096156608458198757210493e-1L, 4.584009668887117914053530980121207914891e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -1.0L, -0x1p-8192L, -7.853981633974483096156608458198757210493e-1L, -4.584009668887117914053530980121207914891e-2467L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1p-8192L, 1.0L, 7.853981633974483096156608458198757210493e-1L, 2.839477425163815960027691385553420311121e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1p-8192L, 1.0L, -7.853981633974483096156608458198757210493e-1L, 2.839477425163815960027691385553420311121e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1p-8192L, -1.0L, 7.853981633974483096156608458198757210493e-1L, -2.839477425163815960027691385553420311121e3L, 0 },
	TEST_c_c(catan, -0x1p-8192L, -1.0L, -7.853981633974483096156608458198757210493e-1L, -2.839477425163815960027691385553420311121e3L, 0 },
	TEST_c_c(catan, 1.0L, 0x1.3p-8221L, 7.853981633974483096156608458198757210493e-1L, 1.013933025636421986471594877335044443409e-2475L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 1.0L, -0x1.3p-8221L, 7.853981633974483096156608458198757210493e-1L, -1.013933025636421986471594877335044443409e-2475L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -1.0L, 0x1.3p-8221L, -7.853981633974483096156608458198757210493e-1L, 1.013933025636421986471594877335044443409e-2475L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -1.0L, -0x1.3p-8221L, -7.853981633974483096156608458198757210493e-1L, -1.013933025636421986471594877335044443409e-2475L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1.3p-8221L, 1.0L, 7.853981633974483096156608458198757210493e-1L, 2.849442134153471837403071201841536297726e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.3p-8221L, 1.0L, -7.853981633974483096156608458198757210493e-1L, 2.849442134153471837403071201841536297726e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1.3p-8221L, -1.0L, 7.853981633974483096156608458198757210493e-1L, -2.849442134153471837403071201841536297726e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.3p-8221L, -1.0L, -7.853981633974483096156608458198757210493e-1L, -2.849442134153471837403071201841536297726e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 1.0L, 0x1.3p-8246L, 7.853981633974483096156608458198757210493e-1L, 3.021755890954798419688924781486524472858e-2483L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 1.0L, -0x1.3p-8246L, 7.853981633974483096156608458198757210493e-1L, -3.021755890954798419688924781486524472858e-2483L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -1.0L, 0x1.3p-8246L, -7.853981633974483096156608458198757210493e-1L, 3.021755890954798419688924781486524472858e-2483L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -1.0L, -0x1.3p-8246L, -7.853981633974483096156608458198757210493e-1L, -3.021755890954798419688924781486524472858e-2483L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1.3p-8246L, 1.0L, 7.853981633974483096156608458198757210493e-1L, 2.858106473910471153770786603359763504827e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.3p-8246L, 1.0L, -7.853981633974483096156608458198757210493e-1L, 2.858106473910471153770786603359763504827e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1.3p-8246L, -1.0L, 7.853981633974483096156608458198757210493e-1L, -2.858106473910471153770786603359763504827e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.3p-8246L, -1.0L, -7.853981633974483096156608458198757210493e-1L, -2.858106473910471153770786603359763504827e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 1.0L, 0x1p-16380L, 7.853981633974483096156608458198757210493e-1L, 6.724206286224187012525355634643505205196e-4932L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 1.0L, -0x1p-16380L, 7.853981633974483096156608458198757210493e-1L, -6.724206286224187012525355634643505205196e-4932L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -1.0L, 0x1p-16380L, -7.853981633974483096156608458198757210493e-1L, 6.724206286224187012525355634643505205196e-4932L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -1.0L, -0x1p-16380L, -7.853981633974483096156608458198757210493e-1L, -6.724206286224187012525355634643505205196e-4932L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1p-16380L, 1.0L, 7.853981633974483096156608458198757210493e-1L, 5.677221982376232056781839690803195180822e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1p-16380L, 1.0L, -7.853981633974483096156608458198757210493e-1L, 5.677221982376232056781839690803195180822e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1p-16380L, -1.0L, 7.853981633974483096156608458198757210493e-1L, -5.677221982376232056781839690803195180822e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1p-16380L, -1.0L, -7.853981633974483096156608458198757210493e-1L, -5.677221982376232056781839690803195180822e3L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1.0000000000000002p0L, 0x1p-16382L, 7.853981633974483096698709544441509427467e-1L, 1.681051571556046752949078932066752571182e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1.0000000000000002p0L, -0x1p-16382L, 7.853981633974483096698709544441509427467e-1L, -1.681051571556046752949078932066752571182e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.0000000000000002p0L, 0x1p-16382L, -7.853981633974483096698709544441509427467e-1L, 1.681051571556046752949078932066752571182e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x1.0000000000000002p0L, -0x1p-16382L, -7.853981633974483096698709544441509427467e-1L, -1.681051571556046752949078932066752571182e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1p-16382L, 0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, 2.218070977791824990137853294097378778927e1L, 0 },
	TEST_c_c(catan, -0x1p-16382L, 0x1.0000000000000002p0L, -1.570796326794896619231321691639751442099L, 2.218070977791824990137853294097378778927e1L, 0 },
	TEST_c_c(catan, 0x1p-16382L, -0x1.0000000000000002p0L, 1.570796326794896619231321691639751442099L, -2.218070977791824990137853294097378778927e1L, 0 },
	TEST_c_c(catan, -0x1p-16382L, -0x1.0000000000000002p0L, -1.570796326794896619231321691639751442099L, -2.218070977791824990137853294097378778927e1L, 0 },
	TEST_c_c(catan, 0x0.ffffffffffffffffp0L, 0x1p-16382L, 7.853981633974483095885557915077381101984e-1L, 1.681051571556046753222468896957938166365e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x0.ffffffffffffffffp0L, -0x1p-16382L, 7.853981633974483095885557915077381101984e-1L, -1.681051571556046753222468896957938166365e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x0.ffffffffffffffffp0L, 0x1p-16382L, -7.853981633974483095885557915077381101984e-1L, 1.681051571556046753222468896957938166365e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, -0x0.ffffffffffffffffp0L, -0x1p-16382L, -7.853981633974483095885557915077381101984e-1L, -1.681051571556046753222468896957938166365e-4932L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catan, 0x1p-16382L, 0x0.ffffffffffffffffp0L, 3.100992811520163369065387859792822623745e-4913L, 2.252728336819822255604649142023466965703e1L, 0 },
	TEST_c_c(catan, -0x1p-16382L, 0x0.ffffffffffffffffp0L, -3.100992811520163369065387859792822623745e-4913L, 2.252728336819822255604649142023466965703e1L, 0 },
	TEST_c_c(catan, 0x1p-16382L, -0x0.ffffffffffffffffp0L, 3.100992811520163369065387859792822623745e-4913L, -2.252728336819822255604649142023466965703e1L, 0 },
	TEST_c_c(catan, -0x1p-16382L, -0x0.ffffffffffffffffp0L, -3.100992811520163369065387859792822623745e-4913L, -2.252728336819822255604649142023466965703e1L, 0 },
# if LDBL_MANT_DIG >= 113
	TEST_c_c(catan, 0x1.0000000000000000000000000001p0L, 0x1p-16382L, 7.853981633974483096156608458198758173458e-1L, 1.681051571556046753131338908660875977540e-4932L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catan, 0x1.0000000000000000000000000001p0L, -0x1p-16382L, 7.853981633974483096156608458198758173458e-1L, -1.681051571556046753131338908660875977540e-4932L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catan, -0x1.0000000000000000000000000001p0L, 0x1p-16382L, -7.853981633974483096156608458198758173458e-1L, 1.681051571556046753131338908660875977540e-4932L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catan, -0x1.0000000000000000000000000001p0L, -0x1p-16382L, -7.853981633974483096156608458198758173458e-1L, -1.681051571556046753131338908660875977540e-4932L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catan, 0x1p-16382L, 0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751442099L, 3.916281570163690998207361486238697614441e1L, 0 },
	TEST_c_c(catan, -0x1p-16382L, 0x1.0000000000000000000000000001p0L, -1.570796326794896619231321691639751442099L, 3.916281570163690998207361486238697614441e1L, 0 },
	TEST_c_c(catan, 0x1p-16382L, -0x1.0000000000000000000000000001p0L, 1.570796326794896619231321691639751442099L, -3.916281570163690998207361486238697614441e1L, 0 },
	TEST_c_c(catan, -0x1p-16382L, -0x1.0000000000000000000000000001p0L, -1.570796326794896619231321691639751442099L, -3.916281570163690998207361486238697614441e1L, 0 },
	TEST_c_c(catan, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-16382L, 7.853981633974483096156608458198756729010e-1L, 1.681051571556046753131338908660876463178e-4932L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catan, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-16382L, 7.853981633974483096156608458198756729010e-1L, -1.681051571556046753131338908660876463178e-4932L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catan, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-16382L, -7.853981633974483096156608458198756729010e-1L, 1.681051571556046753131338908660876463178e-4932L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catan, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-16382L, -7.853981633974483096156608458198756729010e-1L, -1.681051571556046753131338908660876463178e-4932L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catan, 0x1p-16382L, 0x0.ffffffffffffffffffffffffffff8p0L, 1.745703758805099310527547423749501866998e-4898L, 3.950938929191688263678223092311606435623e1L, 0 },
	TEST_c_c(catan, -0x1p-16382L, 0x0.ffffffffffffffffffffffffffff8p0L, -1.745703758805099310527547423749501866998e-4898L, 3.950938929191688263678223092311606435623e1L, 0 },
	TEST_c_c(catan, 0x1p-16382L, -0x0.ffffffffffffffffffffffffffff8p0L, 1.745703758805099310527547423749501866998e-4898L, -3.950938929191688263678223092311606435623e1L, 0 },
	TEST_c_c(catan, -0x1p-16382L, -0x0.ffffffffffffffffffffffffffff8p0L, -1.745703758805099310527547423749501866998e-4898L, -3.950938929191688263678223092311606435623e1L, 0 },
# endif
#endif

	TEST_c_c(catan, 0.75L, 1.25L, 1.10714871779409050301706546017853704L, 0.549306144334054845697622618461262852L, 0 },
	TEST_c_c(catan, -2, -3, -1.4099210495965755225306193844604208L, -0.22907268296853876629588180294200276L, 0 },
}


static void test_catanh(void)
{
	struct test *testp = lookup("catanh" },

	TEST_c_c(catan, 1.0e00, 2.0e00, 1.338972523808479309e00, 4.023594781756401062e-01, 0 },
	TEST_c_c(catan, 0.0e00, 2.0e00, 1.570796325802803039e00, 5.493061468005180358e-01, 0 },
	TEST_c_c(catan, 1.0e00, 0.0e00, 7.853981629014015197e-01, 0.0e00, 0 },
	TEST_c_c(catan, -2.0e00, 2.0e00, -1.311223268508911132e00, 2.388778626918792724e-01, 0 },
	TEST_c_c(catan, -1.0e00, -2.0e00, -1.338972523808479309e00, -4.023594819009304046e-01, 0 },
	TEST_c_c(catan, 1.0e00, -2.0e00, 1.338972523808479309e00, -4.023594819009304046e-01, 0 },

	TEST_c_c(catanh, 0, 0, 0.0, 0.0, 0 },
	TEST_c_c(catanh, ZERO_M, 0, ZERO_M, 0.0, 0 },
	TEST_c_c(catanh, 0, ZERO_M, 0.0, ZERO_M, 0 },
	TEST_c_c(catanh, ZERO_M, ZERO_M, ZERO_M, ZERO_M, 0 },

	TEST_c_c(catanh, 1.0L, ZERO_P, INF_P, ZERO_P, DIVIDE_BY_ZERO_EXCEPTION },
	TEST_c_c(catanh, 1.0L, ZERO_M, INF_P, ZERO_M, DIVIDE_BY_ZERO_EXCEPTION },
	TEST_c_c(catanh, -1.0L, ZERO_P, INF_M, ZERO_P, DIVIDE_BY_ZERO_EXCEPTION },
	TEST_c_c(catanh, -1.0L, ZERO_M, INF_M, ZERO_M, DIVIDE_BY_ZERO_EXCEPTION },
	TEST_c_c(catanh, ZERO_P, 1.0L, ZERO_P, M_PI_4l, 0 },
	TEST_c_c(catanh, ZERO_M, 1.0L, ZERO_M, M_PI_4l, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, ZERO_P, -1.0L, ZERO_P, -M_PI_4l, 0 },
	TEST_c_c(catanh, ZERO_M, -1.0L, ZERO_M, -M_PI_4l, 0|XFAIL_NATIVE_LINUX },

	TEST_c_c(catanh, INF_P, INF_P, 0.0, M_PI_2l, 0 },
	TEST_c_c(catanh, INF_P, INF_M, 0.0, -M_PI_2l, 0 },
	TEST_c_c(catanh, INF_M, INF_P, ZERO_M, M_PI_2l, 0 },
	TEST_c_c(catanh, INF_M, INF_M, ZERO_M, -M_PI_2l, 0 },

	TEST_c_c(catanh, -10.0, INF_P, ZERO_M, M_PI_2l, 0 },
	TEST_c_c(catanh, -10.0, INF_M, ZERO_M, -M_PI_2l, 0 },
	TEST_c_c(catanh, ZERO_M, INF_P, ZERO_M, M_PI_2l, 0 },
	TEST_c_c(catanh, ZERO_M, INF_M, ZERO_M, -M_PI_2l, 0 },
	TEST_c_c(catanh, 0, INF_P, 0.0, M_PI_2l, 0 },
	TEST_c_c(catanh, 0, INF_M, 0.0, -M_PI_2l, 0 },
	TEST_c_c(catanh, 0.1L, INF_P, 0.0, M_PI_2l, 0 },
	TEST_c_c(catanh, 0.1L, INF_M, 0.0, -M_PI_2l, 0 },

	TEST_c_c(catanh, INF_M, 0, ZERO_M, M_PI_2l, 0 },
	TEST_c_c(catanh, INF_M, ZERO_M, ZERO_M, -M_PI_2l, 0 },
	TEST_c_c(catanh, INF_M, 100, ZERO_M, M_PI_2l, 0 },
	TEST_c_c(catanh, INF_M, -100, ZERO_M, -M_PI_2l, 0 },

	TEST_c_c(catanh, INF_P, 0, 0.0, M_PI_2l, 0 },
	TEST_c_c(catanh, INF_P, ZERO_M, 0.0, -M_PI_2l, 0 },
	TEST_c_c(catanh, INF_P, 0.5, 0.0, M_PI_2l, 0 },
	TEST_c_c(catanh, INF_P, -0.5, 0.0, -M_PI_2l, 0 },

	TEST_c_c(catanh, 0, QNAN_P, 0.0, QNAN_P, 0 },
	TEST_c_c(catanh, ZERO_M, QNAN_P, ZERO_M, QNAN_P, 0 },

	TEST_c_c(catanh, INF_P, QNAN_P, 0.0, QNAN_P, 0 },
	TEST_c_c(catanh, INF_M, QNAN_P, ZERO_M, QNAN_P, 0 },

	TEST_c_c(catanh, QNAN_P, 0, QNAN_P, QNAN_P, 0 },
	TEST_c_c(catanh, QNAN_P, ZERO_M, QNAN_P, QNAN_P, 0 },

	TEST_c_c(catanh, QNAN_P, INF_P, 0.0, M_PI_2l, 0 },
	TEST_c_c(catanh, QNAN_P, INF_M, 0.0, -M_PI_2l, 0 },

	TEST_c_c(catanh, 10.5, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(catanh, -10.5, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(catanh, QNAN_P, 0.75, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(catanh, QNAN_P, -0.75, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(catanh, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(catanh, 0x1p50L, 0.0L, 8.881784197001252323389053344728897997441e-16L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p50L, -0.0L, 8.881784197001252323389053344728897997441e-16L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p50L, 0.0L, -8.881784197001252323389053344728897997441e-16L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p50L, -0.0L, -8.881784197001252323389053344728897997441e-16L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, 0x1p50L, 0.0L, 1.570796326794895731052901991514519103193L, 0 },
	TEST_c_c(catanh, -0.0L, 0x1p50L, -0.0L, 1.570796326794895731052901991514519103193L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, -0x1p50L, 0.0L, -1.570796326794895731052901991514519103193L, 0 },
	TEST_c_c(catanh, -0.0L, -0x1p50L, -0.0L, -1.570796326794895731052901991514519103193L, 0|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(catanh, 0x1p500L, 0.0L, 3.054936363499604682051979393213617699789e-151L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p500L, -0.0L, 3.054936363499604682051979393213617699789e-151L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p500L, 0.0L, -3.054936363499604682051979393213617699789e-151L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p500L, -0.0L, -3.054936363499604682051979393213617699789e-151L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, 0x1p500L, 0.0L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0.0L, 0x1p500L, -0.0L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, -0x1p500L, 0.0L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0.0L, -0x1p500L, -0.0L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(catanh, 0x1p5000L, 0.0L, 7.079811261048172892385615158694057552948e-1506L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1p5000L, -0.0L, 7.079811261048172892385615158694057552948e-1506L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p5000L, 0.0L, -7.079811261048172892385615158694057552948e-1506L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p5000L, -0.0L, -7.079811261048172892385615158694057552948e-1506L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0.0L, 0x1p5000L, 0.0L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0.0L, 0x1p5000L, -0.0L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0.0L, -0x1p5000L, 0.0L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0.0L, -0x1p5000L, -0.0L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(catanh, 0x1p63L, 0.5L, 1.084202172485504434007452800869941711427e-19L, 1.570796326794896619231321691639751442093L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p63L, -0.5L, 1.084202172485504434007452800869941711427e-19L, -1.570796326794896619231321691639751442093L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p63L, 0.5L, -1.084202172485504434007452800869941711427e-19L, 1.570796326794896619231321691639751442093L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p63L, -0.5L, -1.084202172485504434007452800869941711427e-19L, -1.570796326794896619231321691639751442093L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.5L, 0x1p63L, 5.877471754111437539843682686111228389007e-39L, 1.570796326794896619122901474391200998698L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0.5L, 0x1p63L, -5.877471754111437539843682686111228389007e-39L, 1.570796326794896619122901474391200998698L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.5L, -0x1p63L, 5.877471754111437539843682686111228389007e-39L, -1.570796326794896619122901474391200998698L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0.5L, -0x1p63L, -5.877471754111437539843682686111228389007e-39L, -1.570796326794896619122901474391200998698L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(catanh, 0x1p511L, 0.5L, 1.491668146240041348658193063092586767475e-154L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p511L, -0.5L, 1.491668146240041348658193063092586767475e-154L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p511L, 0.5L, -1.491668146240041348658193063092586767475e-154L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p511L, -0.5L, -1.491668146240041348658193063092586767475e-154L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.5L, 0x1p511L, 1.112536929253600691545116358666202032110e-308L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0.5L, 0x1p511L, -1.112536929253600691545116358666202032110e-308L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.5L, -0x1p511L, 1.112536929253600691545116358666202032110e-308L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0.5L, -0x1p511L, -1.112536929253600691545116358666202032110e-308L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(catanh, 0x1p8191L, 0.5L, 1.833603867554847165621412392048483165956e-2466L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1p8191L, -0.5L, 1.833603867554847165621412392048483165956e-2466L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p8191L, 0.5L, -1.833603867554847165621412392048483165956e-2466L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p8191L, -0.5L, -1.833603867554847165621412392048483165956e-2466L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0.5L, 0x1p8191L, 1.681051571556046753131338908660876301299e-4932L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0.5L, 0x1p8191L, -1.681051571556046753131338908660876301299e-4932L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0.5L, -0x1p8191L, 1.681051571556046753131338908660876301299e-4932L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0.5L, -0x1p8191L, -1.681051571556046753131338908660876301299e-4932L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(catanh, 0x1p100L, 0.0L, 7.888609052210118054117285652827862296732e-31L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p100L, -0.0L, 7.888609052210118054117285652827862296732e-31L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p100L, 0.0L, -7.888609052210118054117285652827862296732e-31L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p100L, -0.0L, -7.888609052210118054117285652827862296732e-31L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, 0x1p100L, 0.0L, 1.570796326794896619231321691638962581193L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -0.0L, 0x1p100L, -0.0L, 1.570796326794896619231321691638962581193L, 0|XFAIL_NATIVE_LINUXF|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, -0x1p100L, 0.0L, -1.570796326794896619231321691638962581193L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -0.0L, -0x1p100L, -0.0L, -1.570796326794896619231321691638962581193L, 0|XFAIL_NATIVE_LINUXF|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.fp127L, 0.0L, 3.033533808573645181854803967025150136306e-39L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.fp127L, -0.0L, 3.033533808573645181854803967025150136306e-39L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.fp127L, 0.0L, -3.033533808573645181854803967025150136306e-39L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.fp127L, -0.0L, -3.033533808573645181854803967025150136306e-39L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, 0x1.fp127L, 0.0L, 1.570796326794896619231321691639751442096L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -0.0L, 0x1.fp127L, -0.0L, 1.570796326794896619231321691639751442096L, 0|XFAIL_NATIVE_LINUXF|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, -0x1.fp127L, 0.0L, -1.570796326794896619231321691639751442096L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -0.0L, -0x1.fp127L, -0.0L, -1.570796326794896619231321691639751442096L, 0|XFAIL_NATIVE_LINUXF|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.fp127L, 0x1.fp127L, 1.516766904286822590927401983512575068153e-39L, 1.570796326794896619231321691639751442097L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.fp127L, -0x1.fp127L, 1.516766904286822590927401983512575068153e-39L, -1.570796326794896619231321691639751442097L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.fp127L, 0x1.fp127L, -1.516766904286822590927401983512575068153e-39L, 1.570796326794896619231321691639751442097L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.fp127L, -0x1.fp127L, -1.516766904286822590927401983512575068153e-39L, -1.570796326794896619231321691639751442097L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(catanh, 0x1p900L, 0.0L, 1.183052186166774710972751597518026531652e-271L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p900L, -0.0L, 1.183052186166774710972751597518026531652e-271L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p900L, 0.0L, -1.183052186166774710972751597518026531652e-271L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p900L, -0.0L, -1.183052186166774710972751597518026531652e-271L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, 0x1p900L, 0.0L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0.0L, 0x1p900L, -0.0L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, -0x1p900L, 0.0L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0.0L, -0x1p900L, -0.0L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.fp1023L, 0.0L, 5.742126086470197117652213464083623391533e-309L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.fp1023L, -0.0L, 5.742126086470197117652213464083623391533e-309L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.fp1023L, 0.0L, -5.742126086470197117652213464083623391533e-309L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.fp1023L, -0.0L, -5.742126086470197117652213464083623391533e-309L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, 0x1.fp1023L, 0.0L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0.0L, 0x1.fp1023L, -0.0L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0.0L, -0x1.fp1023L, 0.0L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0.0L, -0x1.fp1023L, -0.0L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.fp1023L, 0x1.fp1023L, 2.871063043235098558826106732041811695767e-309L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.fp1023L, -0x1.fp1023L, 2.871063043235098558826106732041811695767e-309L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.fp1023L, 0x1.fp1023L, -2.871063043235098558826106732041811695767e-309L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.fp1023L, -0x1.fp1023L, -2.871063043235098558826106732041811695767e-309L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(catanh, 0x1p10000L, 0.0L, 5.012372749206452009297555933742977749322e-3011L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1p10000L, -0.0L, 5.012372749206452009297555933742977749322e-3011L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p10000L, 0.0L, -5.012372749206452009297555933742977749322e-3011L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p10000L, -0.0L, -5.012372749206452009297555933742977749322e-3011L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0.0L, 0x1p10000L, 0.0L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0.0L, 0x1p10000L, -0.0L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0.0L, -0x1p10000L, 0.0L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0.0L, -0x1p10000L, -0.0L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1.fp16383L, 0.0L, 8.676395208031209048419813722120651877672e-4933L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1.fp16383L, -0.0L, 8.676395208031209048419813722120651877672e-4933L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1.fp16383L, 0.0L, -8.676395208031209048419813722120651877672e-4933L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1.fp16383L, -0.0L, -8.676395208031209048419813722120651877672e-4933L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0.0L, 0x1.fp16383L, 0.0L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0.0L, 0x1.fp16383L, -0.0L, 1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0.0L, -0x1.fp16383L, 0.0L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0.0L, -0x1.fp16383L, -0.0L, -1.570796326794896619231321691639751442099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1.fp16383L, 0x1.fp16383L, 4.338197604015604524209906861060325938836e-4933L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1.fp16383L, -0x1.fp16383L, 4.338197604015604524209906861060325938836e-4933L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1.fp16383L, 0x1.fp16383L, -4.338197604015604524209906861060325938836e-4933L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1.fp16383L, -0x1.fp16383L, -4.338197604015604524209906861060325938836e-4933L, -1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif
	TEST_c_c(catanh, 0x1p-13L, 1.0L, 6.103515609841754902688560615027452023669e-5L, 7.853981671227386080775748393881580082970e-1L, 0 },
	TEST_c_c(catanh, 0x1p-13L, -1.0L, 6.103515609841754902688560615027452023669e-5L, -7.853981671227386080775748393881580082970e-1L, 0 },
	TEST_c_c(catanh, -0x1p-13L, 1.0L, -6.103515609841754902688560615027452023669e-5L, 7.853981671227386080775748393881580082970e-1L, 0 },
	TEST_c_c(catanh, -0x1p-13L, -1.0L, -6.103515609841754902688560615027452023669e-5L, -7.853981671227386080775748393881580082970e-1L, 0 },
	TEST_c_c(catanh, 1.0L, 0x1p-13L, 4.852030264850939738801379894163661227127L, 7.854286809755354140031716771044626356262e-1L, 0 },
	TEST_c_c(catanh, -1.0L, 0x1p-13L, -4.852030264850939738801379894163661227127L, 7.854286809755354140031716771044626356262e-1L, 0 },
	TEST_c_c(catanh, 1.0L, -0x1p-13L, 4.852030264850939738801379894163661227127L, -7.854286809755354140031716771044626356262e-1L, 0 },
	TEST_c_c(catanh, -1.0L, -0x1p-13L, -4.852030264850939738801379894163661227127L, -7.854286809755354140031716771044626356262e-1L, 0 },
	TEST_c_c(catanh, 0x1p-27L, 1.0L, 3.725290298461914028034141143623846306386e-9L, 7.853981633974483234934486536343324763447e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, 0x1p-27L, -1.0L, 3.725290298461914028034141143623846306386e-9L, -7.853981633974483234934486536343324763447e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -0x1p-27L, 1.0L, -3.725290298461914028034141143623846306386e-9L, 7.853981633974483234934486536343324763447e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -0x1p-27L, -1.0L, -3.725290298461914028034141143623846306386e-9L, -7.853981633974483234934486536343324763447e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, 1.0L, 0x1p-27L, 9.704060527839234335310696652368086117807L, 7.853981652600934588466178684534110069553e-1L, 0 },
	TEST_c_c(catanh, -1.0L, 0x1p-27L, -9.704060527839234335310696652368086117807L, 7.853981652600934588466178684534110069553e-1L, 0 },
	TEST_c_c(catanh, 1.0L, -0x1p-27L, 9.704060527839234335310696652368086117807L, -7.853981652600934588466178684534110069553e-1L, 0 },
	TEST_c_c(catanh, -1.0L, -0x1p-27L, -9.704060527839234335310696652368086117807L, -7.853981652600934588466178684534110069553e-1L, 0 },
	TEST_c_c(catanh, 0x1p-33L, 1.0L, 5.820766091346740722643102318246316469910e-11L, 7.853981633974483096190489776088929224056e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, 0x1p-33L, -1.0L, 5.820766091346740722643102318246316469910e-11L, -7.853981633974483096190489776088929224056e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -0x1p-33L, 1.0L, -5.820766091346740722643102318246316469910e-11L, 7.853981633974483096190489776088929224056e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -0x1p-33L, -1.0L, -5.820766091346740722643102318246316469910e-11L, -7.853981633974483096190489776088929224056e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, 1.0L, 0x1p-33L, 1.178350206951907026009379309773625595762e1L, 7.853981634265521400723945494331241018449e-1L, 0 },
	TEST_c_c(catanh, -1.0L, 0x1p-33L, -1.178350206951907026009379309773625595762e1L, 7.853981634265521400723945494331241018449e-1L, 0 },
	TEST_c_c(catanh, 1.0L, -0x1p-33L, 1.178350206951907026009379309773625595762e1L, -7.853981634265521400723945494331241018449e-1L, 0 },
	TEST_c_c(catanh, -1.0L, -0x1p-33L, -1.178350206951907026009379309773625595762e1L, -7.853981634265521400723945494331241018449e-1L, 0 },
	TEST_c_c(catanh, 0x1p-54L, 1.0L, 2.775557561562891351059079170227049355775e-17L, 7.853981633974483096156608458198764914213e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p-54L, -1.0L, 2.775557561562891351059079170227049355775e-17L, -7.853981633974483096156608458198764914213e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-54L, 1.0L, -2.775557561562891351059079170227049355775e-17L, 7.853981633974483096156608458198764914213e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-54L, -1.0L, -2.775557561562891351059079170227049355775e-17L, -7.853981633974483096156608458198764914213e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 1.0L, 0x1p-54L, 1.906154746539849600897388334009985581467e1L, 7.853981633974483234934486536343324763447e-1L, 0 },
	TEST_c_c(catanh, -1.0L, 0x1p-54L, -1.906154746539849600897388334009985581467e1L, 7.853981633974483234934486536343324763447e-1L, 0 },
	TEST_c_c(catanh, 1.0L, -0x1p-54L, 1.906154746539849600897388334009985581467e1L, -7.853981633974483234934486536343324763447e-1L, 0 },
	TEST_c_c(catanh, -1.0L, -0x1p-54L, -1.906154746539849600897388334009985581467e1L, -7.853981633974483234934486536343324763447e-1L, 0 },
	TEST_c_c(catanh, 0x1p-57L, 1.0L, 3.469446951953614188823848962783813448721e-18L, 7.853981633974483096156608458198757330864e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p-57L, -1.0L, 3.469446951953614188823848962783813448721e-18L, -7.853981633974483096156608458198757330864e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-57L, 1.0L, -3.469446951953614188823848962783813448721e-18L, 7.853981633974483096156608458198757330864e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-57L, -1.0L, -3.469446951953614188823848962783813448721e-18L, -7.853981633974483096156608458198757330864e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 1.0L, 0x1p-57L, 2.010126823623841397309973152228712047720e1L, 7.853981633974483113503843217966828154612e-1L, 0 },
	TEST_c_c(catanh, -1.0L, 0x1p-57L, -2.010126823623841397309973152228712047720e1L, 7.853981633974483113503843217966828154612e-1L, 0 },
	TEST_c_c(catanh, 1.0L, -0x1p-57L, 2.010126823623841397309973152228712047720e1L, -7.853981633974483113503843217966828154612e-1L, 0 },
	TEST_c_c(catanh, -1.0L, -0x1p-57L, -2.010126823623841397309973152228712047720e1L, -7.853981633974483113503843217966828154612e-1L, 0 },
	TEST_c_c(catanh, 0x1p-13L, 0x1.000002p0L, 6.103514882246036852433556327261700380577e-5L, 7.853982267273793866654490522673596014524e-1L, 0 },
	TEST_c_c(catanh, 0x1p-13L, -0x1.000002p0L, 6.103514882246036852433556327261700380577e-5L, -7.853982267273793866654490522673596014524e-1L, 0 },
	TEST_c_c(catanh, -0x1p-13L, 0x1.000002p0L, -6.103514882246036852433556327261700380577e-5L, 7.853982267273793866654490522673596014524e-1L, 0 },
	TEST_c_c(catanh, -0x1p-13L, -0x1.000002p0L, -6.103514882246036852433556327261700380577e-5L, -7.853982267273793866654490522673596014524e-1L, 0 },
	TEST_c_c(catanh, 0x1.000002p0L, 0x1p-13L, 4.852030056234795712498957387213592193975L, 7.859169620684960844300240092596908675974e-1L, 0 },
	TEST_c_c(catanh, -0x1.000002p0L, 0x1p-13L, -4.852030056234795712498957387213592193975L, 7.859169620684960844300240092596908675974e-1L, 0 },
	TEST_c_c(catanh, 0x1.000002p0L, -0x1p-13L, 4.852030056234795712498957387213592193975L, -7.859169620684960844300240092596908675974e-1L, 0 },
	TEST_c_c(catanh, -0x1.000002p0L, -0x1p-13L, -4.852030056234795712498957387213592193975L, -7.859169620684960844300240092596908675974e-1L, 0 },
	TEST_c_c(catanh, 0x1p-13L, 0x0.ffffffp0L, 6.103515973639646453881721999956617260502e-5L, 7.853981373204155542484315721351697277336e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, 0x1p-13L, -0x0.ffffffp0L, 6.103515973639646453881721999956617260502e-5L, -7.853981373204155542484315721351697277336e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -0x1p-13L, 0x0.ffffffp0L, -6.103515973639646453881721999956617260502e-5L, 7.853981373204155542484315721351697277336e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -0x1p-13L, -0x0.ffffffp0L, -6.103515973639646453881721999956617260502e-5L, -7.853981373204155542484315721351697277336e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, 0x0.ffffffp0L, 0x1p-13L, 4.852030190345140708455871037447717761868L, 7.851845403708474595909269086711426246675e-1L, 0 },
	TEST_c_c(catanh, -0x0.ffffffp0L, 0x1p-13L, -4.852030190345140708455871037447717761868L, 7.851845403708474595909269086711426246675e-1L, 0 },
	TEST_c_c(catanh, 0x0.ffffffp0L, -0x1p-13L, 4.852030190345140708455871037447717761868L, -7.851845403708474595909269086711426246675e-1L, 0 },
	TEST_c_c(catanh, -0x0.ffffffp0L, -0x1p-13L, -4.852030190345140708455871037447717761868L, -7.851845403708474595909269086711426246675e-1L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(catanh, 0x1p-27L, 0x1.0000000000001p0L, 3.725290298461913200853528590596263270474e-9L, 7.853981633974484345157511161499711112683e-1L, 0 },
	TEST_c_c(catanh, 0x1p-27L, -0x1.0000000000001p0L, 3.725290298461913200853528590596263270474e-9L, -7.853981633974484345157511161499711112683e-1L, 0 },
	TEST_c_c(catanh, -0x1p-27L, 0x1.0000000000001p0L, -3.725290298461913200853528590596263270474e-9L, 7.853981633974484345157511161499711112683e-1L, 0 },
	TEST_c_c(catanh, -0x1p-27L, -0x1.0000000000001p0L, -3.725290298461913200853528590596263270474e-9L, -7.853981633974484345157511161499711112683e-1L, 0 },
	TEST_c_c(catanh, 0x1.0000000000001p0L, 0x1p-27L, 9.704060527839234168777242958594699810015L, 7.853981801612546526942695000283242525531e-1L, 0 },
	TEST_c_c(catanh, -0x1.0000000000001p0L, 0x1p-27L, -9.704060527839234168777242958594699810015L, 7.853981801612546526942695000283242525531e-1L, 0 },
	TEST_c_c(catanh, 0x1.0000000000001p0L, -0x1p-27L, 9.704060527839234168777242958594699810015L, -7.853981801612546526942695000283242525531e-1L, 0 },
	TEST_c_c(catanh, -0x1.0000000000001p0L, -0x1p-27L, -9.704060527839234168777242958594699810015L, -7.853981801612546526942695000283242525531e-1L, 0 },
	TEST_c_c(catanh, 0x1p-27L, 0x0.fffffffffffff8p0L, 3.725290298461914441624447420137706700965e-9L, 7.853981633974482679822974223765039144191e-1L, 0 },
	TEST_c_c(catanh, 0x1p-27L, -0x0.fffffffffffff8p0L, 3.725290298461914441624447420137706700965e-9L, -7.853981633974482679822974223765039144191e-1L, 0 },
	TEST_c_c(catanh, -0x1p-27L, 0x0.fffffffffffff8p0L, -3.725290298461914441624447420137706700965e-9L, 7.853981633974482679822974223765039144191e-1L, 0 },
	TEST_c_c(catanh, -0x1p-27L, -0x0.fffffffffffff8p0L, -3.725290298461914441624447420137706700965e-9L, -7.853981633974482679822974223765039144191e-1L, 0 },
	TEST_c_c(catanh, 0x0.fffffffffffff8p0L, 0x1p-27L, 9.704060527839234252043969805481351363824L, 7.853981578095128619227903983047292781021e-1L, 0 },
	TEST_c_c(catanh, -0x0.fffffffffffff8p0L, 0x1p-27L, -9.704060527839234252043969805481351363824L, 7.853981578095128619227903983047292781021e-1L, 0 },
	TEST_c_c(catanh, 0x0.fffffffffffff8p0L, -0x1p-27L, 9.704060527839234252043969805481351363824L, -7.853981578095128619227903983047292781021e-1L, 0 },
	TEST_c_c(catanh, -0x0.fffffffffffff8p0L, -0x1p-27L, -9.704060527839234252043969805481351363824L, -7.853981578095128619227903983047292781021e-1L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(catanh, 0x1p-33L, 0x1.0000000000000002p0L, 5.820766091346740722012013594069507025615e-11L, 7.853981633974483096732590862331681441026e-1L, 0 },
	TEST_c_c(catanh, 0x1p-33L, -0x1.0000000000000002p0L, 5.820766091346740722012013594069507025615e-11L, -7.853981633974483096732590862331681441026e-1L, 0 },
	TEST_c_c(catanh, -0x1p-33L, 0x1.0000000000000002p0L, -5.820766091346740722012013594069507025615e-11L, 7.853981633974483096732590862331681441026e-1L, 0 },
	TEST_c_c(catanh, -0x1p-33L, -0x1.0000000000000002p0L, -5.820766091346740722012013594069507025615e-11L, -7.853981633974483096732590862331681441026e-1L, 0 },
	TEST_c_c(catanh, 0x1.0000000000000002p0L, 0x1p-33L, 1.178350206951907025990405771755129268176e1L, 7.853981638922134273801338071094141188767e-1L, 0 },
	TEST_c_c(catanh, -0x1.0000000000000002p0L, 0x1p-33L, -1.178350206951907025990405771755129268176e1L, 7.853981638922134273801338071094141188767e-1L, 0 },
	TEST_c_c(catanh, 0x1.0000000000000002p0L, -0x1p-33L, 1.178350206951907025990405771755129268176e1L, -7.853981638922134273801338071094141188767e-1L, 0 },
	TEST_c_c(catanh, -0x1.0000000000000002p0L, -0x1p-33L, -1.178350206951907025990405771755129268176e1L, -7.853981638922134273801338071094141188767e-1L, 0 },
	TEST_c_c(catanh, 0x1p-33L, 0x0.ffffffffffffffffp0L, 5.820766091346740722958646680334721192083e-11L, 7.853981633974483095919439232967553115548e-1L, 0 },
	TEST_c_c(catanh, 0x1p-33L, -0x0.ffffffffffffffffp0L, 5.820766091346740722958646680334721192083e-11L, -7.853981633974483095919439232967553115548e-1L, 0 },
	TEST_c_c(catanh, -0x1p-33L, 0x0.ffffffffffffffffp0L, -5.820766091346740722958646680334721192083e-11L, 7.853981633974483095919439232967553115548e-1L, 0 },
	TEST_c_c(catanh, -0x1p-33L, -0x0.ffffffffffffffffp0L, -5.820766091346740722958646680334721192083e-11L, -7.853981633974483095919439232967553115548e-1L, 0 },
	TEST_c_c(catanh, 0x0.ffffffffffffffffp0L, 0x1p-33L, 1.178350206951907026002603046195591193050e1L, 7.853981631937214964185249205444919953948e-1L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffp0L, 0x1p-33L, -1.178350206951907026002603046195591193050e1L, 7.853981631937214964185249205444919953948e-1L, 0 },
	TEST_c_c(catanh, 0x0.ffffffffffffffffp0L, -0x1p-33L, 1.178350206951907026002603046195591193050e1L, -7.853981631937214964185249205444919953948e-1L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffp0L, -0x1p-33L, -1.178350206951907026002603046195591193050e1L, -7.853981631937214964185249205444919953948e-1L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(catanh, 0x1p-54L, 0x1.000000000000000000000000008p0L, 2.775557561562891351059079170226980932999e-17L, 7.853981633974483096156608458198888173729e-1L, 0 },
	TEST_c_c(catanh, 0x1p-54L, -0x1.000000000000000000000000008p0L, 2.775557561562891351059079170226980932999e-17L, -7.853981633974483096156608458198888173729e-1L, 0 },
	TEST_c_c(catanh, -0x1p-54L, 0x1.000000000000000000000000008p0L, -2.775557561562891351059079170226980932999e-17L, 7.853981633974483096156608458198888173729e-1L, 0 },
	TEST_c_c(catanh, -0x1p-54L, -0x1.000000000000000000000000008p0L, -2.775557561562891351059079170226980932999e-17L, -7.853981633974483096156608458198888173729e-1L, 0 },
	TEST_c_c(catanh, 0x1.000000000000000000000000008p0L, 0x1p-54L, 1.906154746539849600897388334009981267384e1L, 7.853981633974485455380535786656405610710e-1L, 0 },
	TEST_c_c(catanh, -0x1.000000000000000000000000008p0L, 0x1p-54L, -1.906154746539849600897388334009981267384e1L, 7.853981633974485455380535786656405610710e-1L, 0 },
	TEST_c_c(catanh, 0x1.000000000000000000000000008p0L, -0x1p-54L, 1.906154746539849600897388334009981267384e1L, -7.853981633974485455380535786656405610710e-1L, 0 },
	TEST_c_c(catanh, -0x1.000000000000000000000000008p0L, -0x1p-54L, -1.906154746539849600897388334009981267384e1L, -7.853981633974485455380535786656405610710e-1L, 0 },
	TEST_c_c(catanh, 0x1p-54L, 0x0.ffffffffffffffffffffffffffcp0L, 2.775557561562891351059079170227083567164e-17L, 7.853981633974483096156608458198703284454e-1L, 0 },
	TEST_c_c(catanh, 0x1p-54L, -0x0.ffffffffffffffffffffffffffcp0L, 2.775557561562891351059079170227083567164e-17L, -7.853981633974483096156608458198703284454e-1L, 0 },
	TEST_c_c(catanh, -0x1p-54L, 0x0.ffffffffffffffffffffffffffcp0L, -2.775557561562891351059079170227083567164e-17L, 7.853981633974483096156608458198703284454e-1L, 0 },
	TEST_c_c(catanh, -0x1p-54L, -0x0.ffffffffffffffffffffffffffcp0L, -2.775557561562891351059079170227083567164e-17L, -7.853981633974483096156608458198703284454e-1L, 0 },
	TEST_c_c(catanh, 0x0.ffffffffffffffffffffffffffcp0L, 0x1p-54L, 1.906154746539849600897388334009984040723e1L, 7.853981633974482124711461911186784339815e-1L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffffffffffffcp0L, 0x1p-54L, -1.906154746539849600897388334009984040723e1L, 7.853981633974482124711461911186784339815e-1L, 0 },
	TEST_c_c(catanh, 0x0.ffffffffffffffffffffffffffcp0L, -0x1p-54L, 1.906154746539849600897388334009984040723e1L, -7.853981633974482124711461911186784339815e-1L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffffffffffffcp0L, -0x1p-54L, -1.906154746539849600897388334009984040723e1L, -7.853981633974482124711461911186784339815e-1L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(catanh, 0x1p-57L, 0x1.0000000000000000000000000001p0L, 3.469446951953614188823848962783812780530e-18L, 7.853981633974483096156608458198758293829e-1L, 0 },
	TEST_c_c(catanh, 0x1p-57L, -0x1.0000000000000000000000000001p0L, 3.469446951953614188823848962783812780530e-18L, -7.853981633974483096156608458198758293829e-1L, 0 },
	TEST_c_c(catanh, -0x1p-57L, 0x1.0000000000000000000000000001p0L, -3.469446951953614188823848962783812780530e-18L, 7.853981633974483096156608458198758293829e-1L, 0 },
	TEST_c_c(catanh, -0x1p-57L, -0x1.0000000000000000000000000001p0L, -3.469446951953614188823848962783812780530e-18L, -7.853981633974483096156608458198758293829e-1L, 0 },
	TEST_c_c(catanh, 0x1.0000000000000000000000000001p0L, 0x1p-57L, 2.010126823623841397309973152228712033275e1L, 7.853981633974483252281721296111395707566e-1L, 0 },
	TEST_c_c(catanh, -0x1.0000000000000000000000000001p0L, 0x1p-57L, -2.010126823623841397309973152228712033275e1L, 7.853981633974483252281721296111395707566e-1L, 0 },
	TEST_c_c(catanh, 0x1.0000000000000000000000000001p0L, -0x1p-57L, 2.010126823623841397309973152228712033275e1L, -7.853981633974483252281721296111395707566e-1L, 0 },
	TEST_c_c(catanh, -0x1.0000000000000000000000000001p0L, -0x1p-57L, -2.010126823623841397309973152228712033275e1L, -7.853981633974483252281721296111395707566e-1L, 0 },
	TEST_c_c(catanh, 0x1p-57L, 0x0.ffffffffffffffffffffffffffff8p0L, 3.469446951953614188823848962783813782817e-18L, 7.853981633974483096156608458198756849381e-1L, 0 },
	TEST_c_c(catanh, 0x1p-57L, -0x0.ffffffffffffffffffffffffffff8p0L, 3.469446951953614188823848962783813782817e-18L, -7.853981633974483096156608458198756849381e-1L, 0 },
	TEST_c_c(catanh, -0x1p-57L, 0x0.ffffffffffffffffffffffffffff8p0L, -3.469446951953614188823848962783813782817e-18L, 7.853981633974483096156608458198756849381e-1L, 0 },
	TEST_c_c(catanh, -0x1p-57L, -0x0.ffffffffffffffffffffffffffff8p0L, -3.469446951953614188823848962783813782817e-18L, -7.853981633974483096156608458198756849381e-1L, 0 },
	TEST_c_c(catanh, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-57L, 2.010126823623841397309973152228712040498e1L, 7.853981633974483044114904178894544378135e-1L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-57L, -2.010126823623841397309973152228712040498e1L, 7.853981633974483044114904178894544378135e-1L, 0 },
	TEST_c_c(catanh, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-57L, 2.010126823623841397309973152228712040498e1L, -7.853981633974483044114904178894544378135e-1L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-57L, -2.010126823623841397309973152228712040498e1L, -7.853981633974483044114904178894544378135e-1L, 0 },
#endif
	TEST_c_c(catanh, 1.0L, 0x1p-64L, 2.252728336819822255606004394739073846245e1L, 7.853981633974483096292133729759445264744e-1L, 0 },
	TEST_c_c(catanh, 1.0L, -0x1p-64L, 2.252728336819822255606004394739073846245e1L, -7.853981633974483096292133729759445264744e-1L, 0 },
	TEST_c_c(catanh, -1.0L, 0x1p-64L, -2.252728336819822255606004394739073846245e1L, 7.853981633974483096292133729759445264744e-1L, 0 },
	TEST_c_c(catanh, -1.0L, -0x1p-64L, -2.252728336819822255606004394739073846245e1L, -7.853981633974483096292133729759445264744e-1L, 0 },
	TEST_c_c(catanh, 0x1p-64L, 1.0L, 2.710505431213761085018632002174854278563e-20L, 7.853981633974483096156608458198757210500e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-64L, 1.0L, -2.710505431213761085018632002174854278563e-20L, 7.853981633974483096156608458198757210500e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p-64L, -1.0L, 2.710505431213761085018632002174854278563e-20L, -7.853981633974483096156608458198757210500e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-64L, -1.0L, -2.710505431213761085018632002174854278563e-20L, -7.853981633974483096156608458198757210500e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 1.0L, 0x1.3p-73L, 2.556052055225464683726753902092495938633e1L, 7.853981633974483096156922786831527361009e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, 1.0L, -0x1.3p-73L, 2.556052055225464683726753902092495938633e1L, -7.853981633974483096156922786831527361009e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -1.0L, 0x1.3p-73L, -2.556052055225464683726753902092495938633e1L, 7.853981633974483096156922786831527361009e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -1.0L, -0x1.3p-73L, -2.556052055225464683726753902092495938633e1L, -7.853981633974483096156922786831527361009e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, 0x1.3p-73L, 1.0L, 6.286572655403010329022706059731717687100e-23L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.3p-73L, 1.0L, -6.286572655403010329022706059731717687100e-23L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.3p-73L, -1.0L, 6.286572655403010329022706059731717687100e-23L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.3p-73L, -1.0L, -6.286572655403010329022706059731717687100e-23L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 1.0L, 0x1p-124L, 4.332169878499658183857700759113603550472e1L, 7.853981633974483096156608458198757210610e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, 1.0L, -0x1p-124L, 4.332169878499658183857700759113603550472e1L, -7.853981633974483096156608458198757210610e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -1.0L, 0x1p-124L, -4.332169878499658183857700759113603550472e1L, 7.853981633974483096156608458198757210610e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, -1.0L, -0x1p-124L, -4.332169878499658183857700759113603550472e1L, -7.853981633974483096156608458198757210610e-1L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(catanh, 0x1p-124L, 1.0L, 2.350988701644575015937473074444491355637e-38L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-124L, 1.0L, -2.350988701644575015937473074444491355637e-38L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p-124L, -1.0L, 2.350988701644575015937473074444491355637e-38L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-124L, -1.0L, -2.350988701644575015937473074444491355637e-38L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.000002p0L, 0x1p-126L, 8.317766196521665212523713550410777685446L, 1.570796326794896619231321691639702138295L, 0 },
	TEST_c_c(catanh, 0x1.000002p0L, -0x1p-126L, 8.317766196521665212523713550410777685446L, -1.570796326794896619231321691639702138295L, 0 },
	TEST_c_c(catanh, -0x1.000002p0L, 0x1p-126L, -8.317766196521665212523713550410777685446L, 1.570796326794896619231321691639702138295L, 0 },
	TEST_c_c(catanh, -0x1.000002p0L, -0x1p-126L, -8.317766196521665212523713550410777685446L, -1.570796326794896619231321691639702138295L, 0 },
	TEST_c_c(catanh, 0x1p-126L, 0x1.000002p0L, 5.877471053462247139383742414506671233336e-39L, 7.853982230020895322927482174768220815493e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-126L, 0x1.000002p0L, -5.877471053462247139383742414506671233336e-39L, 7.853982230020895322927482174768220815493e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p-126L, -0x1.000002p0L, 5.877471053462247139383742414506671233336e-39L, -7.853982230020895322927482174768220815493e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-126L, -0x1.000002p0L, -5.877471053462247139383742414506671233336e-39L, -7.853982230020895322927482174768220815493e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x0.ffffffp0L, 0x1p-126L, 8.664339742098154951823135931565866792454L, 9.860761609136244031334147100512154160097e-32L, 0 },
	TEST_c_c(catanh, 0x0.ffffffp0L, -0x1p-126L, 8.664339742098154951823135931565866792454L, -9.860761609136244031334147100512154160097e-32L, 0 },
	TEST_c_c(catanh, -0x0.ffffffp0L, 0x1p-126L, -8.664339742098154951823135931565866792454L, 9.860761609136244031334147100512154160097e-32L, 0 },
	TEST_c_c(catanh, -0x0.ffffffp0L, -0x1p-126L, -8.664339742098154951823135931565866792454L, -9.860761609136244031334147100512154160097e-32L, 0 },
	TEST_c_c(catanh, 0x1p-126L, 0x0.ffffffp0L, 5.877472104436064061535099214664320916184e-39L, 7.853981335951250337419109991749089175724e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-126L, 0x0.ffffffp0L, -5.877472104436064061535099214664320916184e-39L, 7.853981335951250337419109991749089175724e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p-126L, -0x0.ffffffp0L, 5.877472104436064061535099214664320916184e-39L, -7.853981335951250337419109991749089175724e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-126L, -0x0.ffffffp0L, -5.877472104436064061535099214664320916184e-39L, -7.853981335951250337419109991749089175724e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(catanh, 1.0L, 0x1p-512L, 1.777922518136259718655200391540222897114e2L, 7.853981633974483096156608458198757210493e-1L, 0 },
	TEST_c_c(catanh, 1.0L, -0x1p-512L, 1.777922518136259718655200391540222897114e2L, -7.853981633974483096156608458198757210493e-1L, 0 },
	TEST_c_c(catanh, -1.0L, 0x1p-512L, -1.777922518136259718655200391540222897114e2L, 7.853981633974483096156608458198757210493e-1L, 0 },
	TEST_c_c(catanh, -1.0L, -0x1p-512L, -1.777922518136259718655200391540222897114e2L, -7.853981633974483096156608458198757210493e-1L, 0 },
	TEST_c_c(catanh, 0x1p-512L, 1.0L, 3.729170365600103371645482657731466918688e-155L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-512L, 1.0L, -3.729170365600103371645482657731466918688e-155L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p-512L, -1.0L, 3.729170365600103371645482657731466918688e-155L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-512L, -1.0L, -3.729170365600103371645482657731466918688e-155L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 1.0L, 0x1.3p-536L, 1.860240928518819859673567751384928348958e2L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 1.0L, -0x1.3p-536L, 1.860240928518819859673567751384928348958e2L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -1.0L, 0x1.3p-536L, -1.860240928518819859673567751384928348958e2L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -1.0L, -0x1.3p-536L, -1.860240928518819859673567751384928348958e2L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.3p-536L, 1.0L, 2.639526015013529511588222179446290115084e-162L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.3p-536L, 1.0L, -2.639526015013529511588222179446290115084e-162L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.3p-536L, -1.0L, 2.639526015013529511588222179446290115084e-162L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1.3p-536L, -1.0L, -2.639526015013529511588222179446290115084e-162L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 1.0L, 0x1p-1020L, 3.538516356758520804574969980043991380025e2L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 1.0L, -0x1p-1020L, 3.538516356758520804574969980043991380025e2L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -1.0L, 0x1p-1020L, -3.538516356758520804574969980043991380025e2L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -1.0L, -0x1p-1020L, -3.538516356758520804574969980043991380025e2L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p-1020L, 1.0L, 4.450147717014402766180465434664808128438e-308L, 7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-1020L, 1.0L, -4.450147717014402766180465434664808128438e-308L, 7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p-1020L, -1.0L, 4.450147717014402766180465434664808128438e-308L, -7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-1020L, -1.0L, -4.450147717014402766180465434664808128438e-308L, -7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1.0000000000001p0L, 0x1p-1022L, 1.836840028483855075506780244989950299369e1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, 0x1.0000000000001p0L, -0x1p-1022L, 1.836840028483855075506780244989950299369e1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0x1.0000000000001p0L, 0x1p-1022L, -1.836840028483855075506780244989950299369e1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0x1.0000000000001p0L, -0x1p-1022L, -1.836840028483855075506780244989950299369e1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, 0x1p-1022L, 0x1.0000000000001p0L, 1.112536929253600444512293438042957369978e-308L, 7.853981633974484206379633083355174374608e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-1022L, 0x1.0000000000001p0L, -1.112536929253600444512293438042957369978e-308L, 7.853981633974484206379633083355174374608e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p-1022L, -0x1.0000000000001p0L, 1.112536929253600444512293438042957369978e-308L, -7.853981633974484206379633083355174374608e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-1022L, -0x1.0000000000001p0L, -1.112536929253600444512293438042957369978e-308L, -7.853981633974484206379633083355174374608e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x0.fffffffffffff8p0L, 0x1p-1022L, 1.871497387511852332650969166374185305708e1L, 1.002084180004486444624900488355118689113e-292L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(catanh, 0x0.fffffffffffff8p0L, -0x1p-1022L, 1.871497387511852332650969166374185305708e1L, -1.002084180004486444624900488355118689113e-292L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(catanh, -0x0.fffffffffffff8p0L, 0x1p-1022L, -1.871497387511852332650969166374185305708e1L, 1.002084180004486444624900488355118689113e-292L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(catanh, -0x0.fffffffffffff8p0L, -0x1p-1022L, -1.871497387511852332650969166374185305708e1L, -1.002084180004486444624900488355118689113e-292L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
	TEST_c_c(catanh, 0x1p-1022L, 0x0.fffffffffffff8p0L, 1.112536929253600815061527818977844932790e-308L, 7.853981633974482541045096145620456183798e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-1022L, 0x0.fffffffffffff8p0L, -1.112536929253600815061527818977844932790e-308L, 7.853981633974482541045096145620456183798e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, 0x1p-1022L, -0x0.fffffffffffff8p0L, 1.112536929253600815061527818977844932790e-308L, -7.853981633974482541045096145620456183798e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(catanh, -0x1p-1022L, -0x0.fffffffffffff8p0L, -1.112536929253600815061527818977844932790e-308L, -7.853981633974482541045096145620456183798e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(catanh, 0x1.0000000000000002p0L, 0x1p-1022L, 2.218070977791824990137853294097378778927e1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, 0x1.0000000000000002p0L, -0x1p-1022L, 2.218070977791824990137853294097378778927e1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0x1.0000000000000002p0L, 0x1p-1022L, -2.218070977791824990137853294097378778927e1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0x1.0000000000000002p0L, -0x1p-1022L, -2.218070977791824990137853294097378778927e1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, 0x1p-1022L, 0x1.0000000000000002p0L, 1.112536929253600691424494863099491450042e-308L, 7.853981633974483096698709544441509427467e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-1022L, 0x1.0000000000000002p0L, -1.112536929253600691424494863099491450042e-308L, 7.853981633974483096698709544441509427467e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1p-1022L, -0x1.0000000000000002p0L, 1.112536929253600691424494863099491450042e-308L, -7.853981633974483096698709544441509427467e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-1022L, -0x1.0000000000000002p0L, -1.112536929253600691424494863099491450042e-308L, -7.853981633974483096698709544441509427467e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x0.ffffffffffffffffp0L, 0x1p-1022L, 2.252728336819822255604649142023466965703e1L, 2.052268400649188124723641491045245971623e-289L, 0 },
	TEST_c_c(catanh, 0x0.ffffffffffffffffp0L, -0x1p-1022L, 2.252728336819822255604649142023466965703e1L, -2.052268400649188124723641491045245971623e-289L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffp0L, 0x1p-1022L, -2.252728336819822255604649142023466965703e1L, 2.052268400649188124723641491045245971623e-289L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffp0L, -0x1p-1022L, -2.252728336819822255604649142023466965703e1L, -2.052268400649188124723641491045245971623e-289L, 0 },
	TEST_c_c(catanh, 0x1p-1022L, 0x0.ffffffffffffffffp0L, 1.112536929253600691605427106449557323148e-308L, 7.853981633974483095885557915077381101984e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-1022L, 0x0.ffffffffffffffffp0L, -1.112536929253600691605427106449557323148e-308L, 7.853981633974483095885557915077381101984e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1p-1022L, -0x0.ffffffffffffffffp0L, 1.112536929253600691605427106449557323148e-308L, -7.853981633974483095885557915077381101984e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-1022L, -0x0.ffffffffffffffffp0L, -1.112536929253600691605427106449557323148e-308L, -7.853981633974483095885557915077381101984e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUXL },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(catanh, 0x1.000000000000000000000000008p0L, 0x1p-1022L, 3.673680056967710139911330243728336427098e1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, 0x1.000000000000000000000000008p0L, -0x1p-1022L, 3.673680056967710139911330243728336427098e1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0x1.000000000000000000000000008p0L, 0x1p-1022L, -3.673680056967710139911330243728336427098e1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0x1.000000000000000000000000008p0L, -0x1p-1022L, -3.673680056967710139911330243728336427098e1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, 0x1p-1022L, 0x1.000000000000000000000000008p0L, 1.112536929253600691545116358666174605957e-308L, 7.853981633974483096156608458198880470009e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catanh, -0x1p-1022L, 0x1.000000000000000000000000008p0L, -1.112536929253600691545116358666174605957e-308L, 7.853981633974483096156608458198880470009e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catanh, 0x1p-1022L, -0x1.000000000000000000000000008p0L, 1.112536929253600691545116358666174605957e-308L, -7.853981633974483096156608458198880470009e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catanh, -0x1p-1022L, -0x1.000000000000000000000000008p0L, -1.112536929253600691545116358666174605957e-308L, -7.853981633974483096156608458198880470009e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catanh, 0x0.ffffffffffffffffffffffffffcp0L, 0x1p-1022L, 3.708337415995707405382191849801244331055e1L, 9.025971879324147880346310405868788320726e-277L, 0 },
	TEST_c_c(catanh, 0x0.ffffffffffffffffffffffffffcp0L, -0x1p-1022L, 3.708337415995707405382191849801244331055e1L, -9.025971879324147880346310405868788320726e-277L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffffffffffffcp0L, 0x1p-1022L, -3.708337415995707405382191849801244331055e1L, 9.025971879324147880346310405868788320726e-277L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffffffffffffcp0L, -0x1p-1022L, -3.708337415995707405382191849801244331055e1L, -9.025971879324147880346310405868788320726e-277L, 0 },
	TEST_c_c(catanh, 0x1p-1022L, 0x0.ffffffffffffffffffffffffffcp0L, 1.112536929253600691545116358666215745186e-308L, 7.853981633974483096156608458198695580735e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catanh, -0x1p-1022L, 0x0.ffffffffffffffffffffffffffcp0L, -1.112536929253600691545116358666215745186e-308L, 7.853981633974483096156608458198695580735e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catanh, 0x1p-1022L, -0x0.ffffffffffffffffffffffffffcp0L, 1.112536929253600691545116358666215745186e-308L, -7.853981633974483096156608458198695580735e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(catanh, -0x1p-1022L, -0x0.ffffffffffffffffffffffffffcp0L, -1.112536929253600691545116358666215745186e-308L, -7.853981633974483096156608458198695580735e-1L, UNDERFLOW_EXCEPTION_DOUBLE },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(catanh, 1.0L, 0x1p-8192L, 2.839477425163815960027691385553420311121e3L, 7.853981633974483096156608458198757210493e-1L, 0 },
	TEST_c_c(catanh, 1.0L, -0x1p-8192L, 2.839477425163815960027691385553420311121e3L, -7.853981633974483096156608458198757210493e-1L, 0 },
	TEST_c_c(catanh, -1.0L, 0x1p-8192L, -2.839477425163815960027691385553420311121e3L, 7.853981633974483096156608458198757210493e-1L, 0 },
	TEST_c_c(catanh, -1.0L, -0x1p-8192L, -2.839477425163815960027691385553420311121e3L, -7.853981633974483096156608458198757210493e-1L, 0 },
	TEST_c_c(catanh, 0x1p-8192L, 1.0L, 4.584009668887117914053530980121207914891e-2467L, 7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-8192L, 1.0L, -4.584009668887117914053530980121207914891e-2467L, 7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1p-8192L, -1.0L, 4.584009668887117914053530980121207914891e-2467L, -7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-8192L, -1.0L, -4.584009668887117914053530980121207914891e-2467L, -7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 1.0L, 0x1.3p-8221L, 2.849442134153471837403071201841536297726e3L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 1.0L, -0x1.3p-8221L, 2.849442134153471837403071201841536297726e3L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -1.0L, 0x1.3p-8221L, -2.849442134153471837403071201841536297726e3L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -1.0L, -0x1.3p-8221L, -2.849442134153471837403071201841536297726e3L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1.3p-8221L, 1.0L, 1.013933025636421986471594877335044443409e-2475L, 7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1.3p-8221L, 1.0L, -1.013933025636421986471594877335044443409e-2475L, 7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1.3p-8221L, -1.0L, 1.013933025636421986471594877335044443409e-2475L, -7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1.3p-8221L, -1.0L, -1.013933025636421986471594877335044443409e-2475L, -7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 1.0L, 0x1.3p-8246L, 2.858106473910471153770786603359763504827e3L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 1.0L, -0x1.3p-8246L, 2.858106473910471153770786603359763504827e3L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -1.0L, 0x1.3p-8246L, -2.858106473910471153770786603359763504827e3L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -1.0L, -0x1.3p-8246L, -2.858106473910471153770786603359763504827e3L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1.3p-8246L, 1.0L, 3.021755890954798419688924781486524472858e-2483L, 7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1.3p-8246L, 1.0L, -3.021755890954798419688924781486524472858e-2483L, 7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1.3p-8246L, -1.0L, 3.021755890954798419688924781486524472858e-2483L, -7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1.3p-8246L, -1.0L, -3.021755890954798419688924781486524472858e-2483L, -7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 1.0L, 0x1p-16380L, 5.677221982376232056781839690803195180822e3L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 1.0L, -0x1p-16380L, 5.677221982376232056781839690803195180822e3L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -1.0L, 0x1p-16380L, -5.677221982376232056781839690803195180822e3L, 7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -1.0L, -0x1p-16380L, -5.677221982376232056781839690803195180822e3L, -7.853981633974483096156608458198757210493e-1L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1p-16380L, 1.0L, 6.724206286224187012525355634643505205196e-4932L, 7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-16380L, 1.0L, -6.724206286224187012525355634643505205196e-4932L, 7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1p-16380L, -1.0L, 6.724206286224187012525355634643505205196e-4932L, -7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-16380L, -1.0L, -6.724206286224187012525355634643505205196e-4932L, -7.853981633974483096156608458198757210493e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1.0000000000000002p0L, 0x1p-16382L, 2.218070977791824990137853294097378778927e1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, 0x1.0000000000000002p0L, -0x1p-16382L, 2.218070977791824990137853294097378778927e1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0x1.0000000000000002p0L, 0x1p-16382L, -2.218070977791824990137853294097378778927e1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0x1.0000000000000002p0L, -0x1p-16382L, -2.218070977791824990137853294097378778927e1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, 0x1p-16382L, 0x1.0000000000000002p0L, 1.681051571556046752949078932066752571182e-4932L, 7.853981633974483096698709544441509427467e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-16382L, 0x1.0000000000000002p0L, -1.681051571556046752949078932066752571182e-4932L, 7.853981633974483096698709544441509427467e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1p-16382L, -0x1.0000000000000002p0L, 1.681051571556046752949078932066752571182e-4932L, -7.853981633974483096698709544441509427467e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-16382L, -0x1.0000000000000002p0L, -1.681051571556046752949078932066752571182e-4932L, -7.853981633974483096698709544441509427467e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x0.ffffffffffffffffp0L, 0x1p-16382L, 2.252728336819822255604649142023466965703e1L, 3.100992811520163369065387859792822623745e-4913L, 0 },
	TEST_c_c(catanh, 0x0.ffffffffffffffffp0L, -0x1p-16382L, 2.252728336819822255604649142023466965703e1L, -3.100992811520163369065387859792822623745e-4913L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffp0L, 0x1p-16382L, -2.252728336819822255604649142023466965703e1L, 3.100992811520163369065387859792822623745e-4913L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffp0L, -0x1p-16382L, -2.252728336819822255604649142023466965703e1L, -3.100992811520163369065387859792822623745e-4913L, 0 },
	TEST_c_c(catanh, 0x1p-16382L, 0x0.ffffffffffffffffp0L, 1.681051571556046753222468896957938166365e-4932L, 7.853981633974483095885557915077381101984e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-16382L, 0x0.ffffffffffffffffp0L, -1.681051571556046753222468896957938166365e-4932L, 7.853981633974483095885557915077381101984e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, 0x1p-16382L, -0x0.ffffffffffffffffp0L, 1.681051571556046753222468896957938166365e-4932L, -7.853981633974483095885557915077381101984e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(catanh, -0x1p-16382L, -0x0.ffffffffffffffffp0L, -1.681051571556046753222468896957938166365e-4932L, -7.853981633974483095885557915077381101984e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
# if LDBL_MANT_DIG >= 113
	TEST_c_c(catanh, 0x1.0000000000000000000000000001p0L, 0x1p-16382L, 3.916281570163690998207361486238697614441e1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, 0x1.0000000000000000000000000001p0L, -0x1p-16382L, 3.916281570163690998207361486238697614441e1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0x1.0000000000000000000000000001p0L, 0x1p-16382L, -3.916281570163690998207361486238697614441e1L, 1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, -0x1.0000000000000000000000000001p0L, -0x1p-16382L, -3.916281570163690998207361486238697614441e1L, -1.570796326794896619231321691639751442099L, 0 },
	TEST_c_c(catanh, 0x1p-16382L, 0x1.0000000000000000000000000001p0L, 1.681051571556046753131338908660875977540e-4932L, 7.853981633974483096156608458198758173458e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catanh, -0x1p-16382L, 0x1.0000000000000000000000000001p0L, -1.681051571556046753131338908660875977540e-4932L, 7.853981633974483096156608458198758173458e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catanh, 0x1p-16382L, -0x1.0000000000000000000000000001p0L, 1.681051571556046753131338908660875977540e-4932L, -7.853981633974483096156608458198758173458e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catanh, -0x1p-16382L, -0x1.0000000000000000000000000001p0L, -1.681051571556046753131338908660875977540e-4932L, -7.853981633974483096156608458198758173458e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catanh, 0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-16382L, 3.950938929191688263678223092311606435623e1L, 1.745703758805099310527547423749501866998e-4898L, 0 },
	TEST_c_c(catanh, 0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-16382L, 3.950938929191688263678223092311606435623e1L, -1.745703758805099310527547423749501866998e-4898L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffffffffffffff8p0L, 0x1p-16382L, -3.950938929191688263678223092311606435623e1L, 1.745703758805099310527547423749501866998e-4898L, 0 },
	TEST_c_c(catanh, -0x0.ffffffffffffffffffffffffffff8p0L, -0x1p-16382L, -3.950938929191688263678223092311606435623e1L, -1.745703758805099310527547423749501866998e-4898L, 0 },
	TEST_c_c(catanh, 0x1p-16382L, 0x0.ffffffffffffffffffffffffffff8p0L, 1.681051571556046753131338908660876463178e-4932L, 7.853981633974483096156608458198756729010e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catanh, -0x1p-16382L, 0x0.ffffffffffffffffffffffffffff8p0L, -1.681051571556046753131338908660876463178e-4932L, 7.853981633974483096156608458198756729010e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catanh, 0x1p-16382L, -0x0.ffffffffffffffffffffffffffff8p0L, 1.681051571556046753131338908660876463178e-4932L, -7.853981633974483096156608458198756729010e-1L, UNDERFLOW_EXCEPTION },
	TEST_c_c(catanh, -0x1p-16382L, -0x0.ffffffffffffffffffffffffffff8p0L, -1.681051571556046753131338908660876463178e-4932L, -7.853981633974483096156608458198756729010e-1L, UNDERFLOW_EXCEPTION },
# endif
#endif

	TEST_c_c(catanh, 0.75L, 1.25L, 0.261492138795671927078652057366532140L, 0.996825126463918666098902241310446708L, 0 },
	TEST_c_c(catanh, -2, -3, -0.14694666622552975204743278515471595L, -1.3389725222944935611241935759091443L, 0 },
}


static void test_ccos(void)
{
	struct test *testp = lookup("ccos" },

	TEST_c_c(ccos, 1.0e00, 2.0e00, 2.032723009586334228e00, -3.051897794008255004e00, 0 },
	TEST_c_c(ccos, 0.0e00, 2.0e00, 3.76219569108363138810e+00, -0.0e00, 0 },
	TEST_c_c(ccos, 1.0e00, 0.0e00, 5.40302305868139765010e-01, -0.0e00, 0 },
	TEST_c_c(ccos, -2.0e00, 2.0e00, -1.565625846385955810e00, 3.297894805669784545e00, 0 },
	TEST_c_c(ccos, -1.0e00, -2.0e00, 2.032723009586334228e00, -3.051897794008255004e00, 0 },
	TEST_c_c(ccos, 1.0e00, -2.0e00, 2.032723009586334228e00, 3.051897794008255004e00, 0 },

	TEST_c_c(ccos, 0.0, 0.0, 1.0, ZERO_M, 0 },
	TEST_c_c(ccos, ZERO_M, 0.0, 1.0, 0.0, 0 },
	TEST_c_c(ccos, 0.0, ZERO_M, 1.0, 0.0, 0 },
	TEST_c_c(ccos, ZERO_M, ZERO_M, 1.0, ZERO_M, 0 },

	TEST_c_c(ccos, INF_P, 0.0, QNAN_P, 0.0, INVALID_EXCEPTION },
	TEST_c_c(ccos, INF_P, ZERO_M, QNAN_P, 0.0, INVALID_EXCEPTION },
	TEST_c_c(ccos, INF_M, 0.0, QNAN_P, 0.0, INVALID_EXCEPTION },
	TEST_c_c(ccos, INF_M, ZERO_M, QNAN_P, 0.0, INVALID_EXCEPTION },

	TEST_c_c(ccos, 0.0, INF_P, INF_P, ZERO_M, 0 },
	TEST_c_c(ccos, 0.0, INF_M, INF_P, 0.0, 0 },
	TEST_c_c(ccos, ZERO_M, INF_P, INF_P, 0.0, 0 },
	TEST_c_c(ccos, ZERO_M, INF_M, INF_P, ZERO_M, 0 },

	TEST_c_c(ccos, INF_P, INF_P, INF_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccos, INF_M, INF_P, INF_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccos, INF_P, INF_M, INF_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccos, INF_M, INF_M, INF_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(ccos, 4.625, INF_P, INF_M, INF_P, 0 },
	TEST_c_c(ccos, 4.625, INF_M, INF_M, INF_M, 0 },
	TEST_c_c(ccos, -4.625, INF_P, INF_M, INF_M, 0 },
	TEST_c_c(ccos, -4.625, INF_M, INF_M, INF_P, 0 },

	TEST_c_c(ccos, INF_P, 6.75, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccos, INF_P, -6.75, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccos, INF_M, 6.75, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccos, INF_M, -6.75, QNAN_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(ccos, QNAN_P, 0.0, QNAN_P, 0.0, 0 },
	TEST_c_c(ccos, QNAN_P, ZERO_M, QNAN_P, 0.0, 0 },

	TEST_c_c(ccos, QNAN_P, INF_P, INF_P, QNAN_P, 0 },
	TEST_c_c(ccos, QNAN_P, INF_M, INF_P, QNAN_P, 0 },

	TEST_c_c(ccos, QNAN_P, 9.0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ccos, QNAN_P, -9.0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(ccos, 0.0, QNAN_P, QNAN_P, 0.0, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(ccos, ZERO_M, QNAN_P, QNAN_P, 0.0, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(ccos, 10.0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ccos, -10.0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(ccos, INF_P, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ccos, INF_M, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(ccos, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(ccos, 0.75L, 1.25L, 1.38173873063425888530729933139078645L, -1.09193013555397466170919531722024128L, 0 },
	TEST_c_c(ccos, -2, -3, -4.18962569096880723013255501961597373L, -9.10922789375533659797919726277886212L, 0 },

	TEST_c_c(ccos, 0.75, 89.5, 2.708024460708609732016532185663087200560e38L, -2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(ccos, 0.75, -89.5, 2.708024460708609732016532185663087200560e38L, 2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(ccos, -0.75, 89.5, 2.708024460708609732016532185663087200560e38L, 2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(ccos, -0.75, -89.5, 2.708024460708609732016532185663087200560e38L, -2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },

#ifndef TEST_FLOAT
	TEST_c_c(ccos, 0.75, 710.5, 1.347490911916428129246890157395342279438e308L, -1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(ccos, 0.75, -710.5, 1.347490911916428129246890157395342279438e308L, 1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(ccos, -0.75, 710.5, 1.347490911916428129246890157395342279438e308L, 1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(ccos, -0.75, -710.5, 1.347490911916428129246890157395342279438e308L, -1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(ccos, 0.75, 11357.25, 9.001213196851067077465606717616495588201e4931L, -8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(ccos, 0.75, -11357.25, 9.001213196851067077465606717616495588201e4931L, 8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(ccos, -0.75, 11357.25, 9.001213196851067077465606717616495588201e4931L, 8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(ccos, -0.75, -11357.25, 9.001213196851067077465606717616495588201e4931L, -8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
#endif

#ifdef TEST_FLOAT
	TEST_c_c(ccos, 0x1p-149, 180, INF_P, -1.043535896672617552965983803453927655332e33L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXF },
#endif

#if defined TEST_DOUBLE || (defined TEST_LDOUBLE && LDBL_MAX_EXP == 1024)
	TEST_c_c(ccos, 0x1p-1074, 1440, INF_P, -5.981479269486130556466515778180916082415e301L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(ccos, 0x1p-16434L, 22730, INF_P, -1.217853148905605987081057582351152052687e4924L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(ccos, SUBNORM_P * 0x1p120, 0x1p-120, 1.0, SUBNORM_M, UNDERFLOW_EXCEPTION },
	TEST_c_c(ccos, 0x1p-120, SUBNORM_P * 0x1p120, 1.0, SUBNORM_M, UNDERFLOW_EXCEPTION },
}


static void test_ccosh(void)
{
	struct test *testp = lookup("ccosh" },

	TEST_c_c(ccosh, 1.0e00, 2.0e0, -6.421481221914291381e-01, 1.068607419729232788e00, 0 },
	TEST_c_c(ccosh, 0.0e00, 2.0e00, -4.161468371748924255e-01, 0.0e00, 0 },
	TEST_c_c(ccosh, 1.0e00, 0.0e00, 1.543080642819404602e00, 0.0e00, 0 },
	TEST_c_c(ccosh, -2.0e00, 2.0e00, -1.565625831484794616e00, -3.297894805669784545e00, 0 },
	TEST_c_c(ccosh, -1.0e00, -2.0e00, -6.421481221914291381e-01, 1.068607419729232788e00, 0 },
	TEST_c_c(ccosh, 1.0e00, -2.0e00, -6.421481221914291381e-01, -1.068607419729232788e00, 0 },

	TEST_c_c(ccosh, 0.0, 0.0, 1.0, 0.0, 0 },
	TEST_c_c(ccosh, ZERO_M, 0.0, 1.0, ZERO_M, 0 },
	TEST_c_c(ccosh, 0.0, ZERO_M, 1.0, ZERO_M, 0 },
	TEST_c_c(ccosh, ZERO_M, ZERO_M, 1.0, 0.0, 0 },

	TEST_c_c(ccosh, 0.0, INF_P, QNAN_P, 0.0, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(ccosh, ZERO_M, INF_P, QNAN_P, 0.0, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(ccosh, 0.0, INF_M, QNAN_P, 0.0, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(ccosh, ZERO_M, INF_M, QNAN_P, 0.0, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },

	TEST_c_c(ccosh, INF_P, 0.0, INF_P, 0.0, 0 },
	TEST_c_c(ccosh, INF_M, 0.0, INF_P, ZERO_M, 0 },
	TEST_c_c(ccosh, INF_P, ZERO_M, INF_P, ZERO_M, 0 },
	TEST_c_c(ccosh, INF_M, ZERO_M, INF_P, 0.0, 0 },

	TEST_c_c(ccosh, INF_P, INF_P, INF_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccosh, INF_M, INF_P, INF_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccosh, INF_P, INF_M, INF_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccosh, INF_M, INF_M, INF_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(ccosh, INF_P, 4.625, INF_M, INF_M, 0 },
	TEST_c_c(ccosh, INF_M, 4.625, INF_M, INF_P, 0 },
	TEST_c_c(ccosh, INF_P, -4.625, INF_M, INF_P, 0 },
	TEST_c_c(ccosh, INF_M, -4.625, INF_M, INF_M, 0 },

	TEST_c_c(ccosh, 6.75, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccosh, -6.75, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccosh, 6.75, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ccosh, -6.75, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(ccosh, 0.0, QNAN_P, QNAN_P, 0.0, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(ccosh, ZERO_M, QNAN_P, QNAN_P, 0.0, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(ccosh, INF_P, QNAN_P, INF_P, QNAN_P, 0 },
	TEST_c_c(ccosh, INF_M, QNAN_P, INF_P, QNAN_P, 0 },

	TEST_c_c(ccosh, 9.0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ccosh, -9.0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(ccosh, QNAN_P, 0.0, QNAN_P, 0.0, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(ccosh, QNAN_P, ZERO_M, QNAN_P, 0.0, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(ccosh, QNAN_P, 10.0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ccosh, QNAN_P, -10.0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(ccosh, QNAN_P, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ccosh, QNAN_P, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(ccosh, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(ccosh, 0.75L, 1.25L, 0.408242591877968807788852146397499084L, 0.780365930845853240391326216300863152L, 0 },

	TEST_c_c(ccosh, -2, -3, -3.72454550491532256547397070325597253L, 0.511822569987384608834463849801875634L, 0 },

	TEST_c_c(ccosh, 89.5, 0.75, 2.708024460708609732016532185663087200560e38L, 2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(ccosh, -89.5, 0.75, 2.708024460708609732016532185663087200560e38L, -2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(ccosh, 89.5, -0.75, 2.708024460708609732016532185663087200560e38L, -2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(ccosh, -89.5, -0.75, 2.708024460708609732016532185663087200560e38L, 2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },

#ifndef TEST_FLOAT
	TEST_c_c(ccosh, 710.5, 0.75, 1.347490911916428129246890157395342279438e308L, 1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(ccosh, -710.5, 0.75, 1.347490911916428129246890157395342279438e308L, -1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(ccosh, 710.5, -0.75, 1.347490911916428129246890157395342279438e308L, -1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(ccosh, -710.5, -0.75, 1.347490911916428129246890157395342279438e308L, 1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(ccosh, 11357.25, 0.75, 9.001213196851067077465606717616495588201e4931L, 8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(ccosh, -11357.25, 0.75, 9.001213196851067077465606717616495588201e4931L, -8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(ccosh, 11357.25, -0.75, 9.001213196851067077465606717616495588201e4931L, -8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(ccosh, -11357.25, -0.75, 9.001213196851067077465606717616495588201e4931L, 8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
#endif

#ifdef TEST_FLOAT
	TEST_c_c(ccosh, 180, 0x1p-149, INF_P, 1.043535896672617552965983803453927655332e33L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXF },
#endif

#if defined TEST_DOUBLE || (defined TEST_LDOUBLE && LDBL_MAX_EXP == 1024)
	TEST_c_c(ccosh, 1440, 0x1p-1074, INF_P, 5.981479269486130556466515778180916082415e301L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(ccosh, 22730, 0x1p-16434L, INF_P, 1.217853148905605987081057582351152052687e4924L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(ccosh, SUBNORM_P * 0x1p120, 0x1p-120, 1.0, SUBNORM_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(ccosh, 0x1p-120, SUBNORM_P * 0x1p120, 1.0, SUBNORM_P, UNDERFLOW_EXCEPTION },
}

static void test_cexp(void)
{
	struct test *testp = lookup("cexp" },

	TEST_c_c(cexp, 1.0e00, 2.0e00, -1.131204381585121154e00, 2.471726655960083007e00, 0 },
	TEST_c_c(cexp, 0.0e00, 2.0e00, -4.161468371748924255e-01, 9.092974215745925903e-01, 0 },
	TEST_c_c(cexp, 1.0e00, 0.0e00, 2.718281835317611694e00, 0.0e00, 0 },
	TEST_c_c(cexp, -2.0e00, 2.0e00, -5.631934991106390953e-02, 1.230600243434309959e-01, 0 },
	TEST_c_c(cexp, -1.0e00, -2.0e00, -1.530918665230274200e-01, -3.345118276774883270e-01, 0 },
	TEST_c_c(cexp, 1.0e00, -2.0e00, -1.131204381585121154e00, -2.471726655960083007e00, 0 },

	TEST_c_c(cexp, ZERO_P, ZERO_P, 1, 0.0, 0 },
	TEST_c_c(cexp, ZERO_M, ZERO_P, 1, 0.0, 0 },
	TEST_c_c(cexp, ZERO_P, ZERO_M, 1, ZERO_M, 0 },
	TEST_c_c(cexp, ZERO_M, ZERO_M, 1, ZERO_M, 0 },

	TEST_c_c(cexp, INF_P, ZERO_P, INF_P, 0.0, 0 },
	TEST_c_c(cexp, INF_P, ZERO_M, INF_P, ZERO_M, 0 },

	TEST_c_c(cexp, INF_M, ZERO_P, 0.0, 0.0, 0 },
	TEST_c_c(cexp, INF_M, ZERO_M, 0.0, ZERO_M, 0 },

	TEST_c_c(cexp, 0.0, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(cexp, ZERO_M, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(cexp, 0.0, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(cexp, ZERO_M, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(cexp, 100.0, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(cexp, -100.0, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(cexp, 100.0, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(cexp, -100.0, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(cexp, INF_M, 2.0, ZERO_M, 0.0, 0 },
	TEST_c_c(cexp, INF_M, 4.0, ZERO_M, ZERO_M, 0 },
	TEST_c_c(cexp, INF_P, 2.0, INF_M, INF_P, 0 },
	TEST_c_c(cexp, INF_P, 4.0, INF_M, INF_M, 0 },

	TEST_c_c(cexp, INF_P, INF_P, INF_P, QNAN_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(cexp, INF_P, INF_M, INF_P, QNAN_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },

	TEST_c_c(cexp, INF_M, INF_P, 0.0, 0.0, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(cexp, INF_M, INF_M, 0.0, ZERO_M, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(cexp, INF_M, QNAN_P, 0, 0, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(cexp, INF_P, QNAN_P, INF_P, QNAN_P, 0 },

	TEST_c_c(cexp, QNAN_P, 0.0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(cexp, QNAN_P, 1.0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(cexp, QNAN_P, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(cexp, 0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(cexp, 1, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(cexp, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(cexp, 0.75L, 1.25L, 0.667537446429131586942201977015932112L, 2.00900045494094876258347228145863909L, 0 },
	TEST_c_c(cexp, -2.0, -3.0, -0.13398091492954261346140525546115575L, -0.019098516261135196432576240858800925L, 0 },

	TEST_c_c(cexp, 0, 0x1p65, 0.99888622066058013610642172179340364209972L, -0.047183876212354673805106149805700013943218L, 0|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
	TEST_c_c(cexp, 0, -0x1p65, 0.99888622066058013610642172179340364209972L, 0.047183876212354673805106149805700013943218L, 0|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
	TEST_c_c(cexp, 50, 0x1p127, 4.053997150228616856622417636046265337193e21L, 3.232070315463388524466674772633810238819e21L, 0|XFAIL_NATIVE_LINUXL|XFAIL_FPU },

#ifndef TEST_FLOAT
	TEST_c_c(cexp, 0, 1e22, 0.5232147853951389454975944733847094921409L, -0.8522008497671888017727058937530293682618L, 0|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
	TEST_c_c(cexp, 0, 0x1p1023, -0.826369834614147994500785680811743734805L, 0.5631277798508840134529434079444683477104L, 0|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
	TEST_c_c(cexp, 500, 0x1p1023, -1.159886268932754433233243794561351783426e217L, 7.904017694554466595359379965081774849708e216L, 0|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(cexp, 0, 0x1p16383L, 0.9210843909921906206874509522505756251609L, 0.3893629985894208126948115852610595405563L, 0|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
	TEST_c_c(cexp, -10000, 0x1p16383L, 1.045876464564882298442774542991176546722e-4343L, 4.421154026488516836023811173959413420548e-4344L, 0|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif

	TEST_c_c(cexp, 88.75, 0.75, 2.558360358486542817001900410314204322891e38L, 2.383359453227311447654736314679677655100e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(cexp, -95, 0.75, 4.039714446238306526889476684000081624047e-42L, 3.763383677300535390271646960780570275931e-42L, UNDERFLOW_EXCEPTION_FLOAT },

#ifndef TEST_FLOAT
	TEST_c_c(cexp, 709.8125, 0.75, 1.355121963080879535248452862759108365762e308L, 1.262426823598609432507811340856186873507e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(cexp, -720, 0.75, 1.486960657116368433685753325516638551722e-313L, 1.385247284245720590980701226843815229385e-313L, UNDERFLOW_EXCEPTION_DOUBLE },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(cexp, 11356.5625, 0.75, 9.052188470850960144814815984311663764287e4931L, 8.432986734191301036267148978260970230200e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(cexp, -11370, 0.75, 8.631121063182211587489310508568170739592e-4939L, 8.040721827809267291427062346918413482824e-4939L, UNDERFLOW_EXCEPTION|XFAIL_TODO },
#endif

#ifdef TEST_FLOAT
	TEST_c_c(cexp, 180, 0x1p-149, INF_P, 2.087071793345235105931967606907855310664e33L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXF },
#endif

#if defined TEST_DOUBLE || (defined TEST_LDOUBLE && LDBL_MAX_EXP == 1024)
	TEST_c_c(cexp, 1440, 0x1p-1074, INF_P, 1.196295853897226111293303155636183216483e302L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(cexp, 22730, 0x1p-16434L, INF_P, 2.435706297811211974162115164702304105374e4924L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(cexp, 1e6, 0, INF_P, 0, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(cexp, 1e6, MIN_P, INF_P, INF_P, OVERFLOW_EXCEPTION },
	TEST_c_c(cexp, 1e6, MIN_M, INF_P, INF_M, OVERFLOW_EXCEPTION },

	TEST_c_c(cexp, MIN_P, SUBNORM_P, 1.0, SUBNORM_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(cexp, MIN_P, SUBNORM_M, 1.0, SUBNORM_M, UNDERFLOW_EXCEPTION },
}


static void test_clog(void)
{
	struct test *testp = lookup("clog" },

	TEST_c_c(clog, 1.0e00, 2.0e00, 8.047189563512802124e-01, 1.107148721814155578e00, 0 },
	TEST_c_c(clog, 0.0e00, 2.0e00, 6.931471824645996093e-01, 1.570796325802803039e00, 0 },
	TEST_c_c(clog, 1.0e00, 0.0e00, 0.0e00, 0.0e00, 0 },
	TEST_c_c(clog, -2.0e00, 2.0e00, 1.039720773696899414e00, 2.356194496154785156e00, 0 },
	TEST_c_c(clog, -1.0e00, -2.0e00, 8.047189563512802124e-01, -2.034443944692611694e00, 0 },
	TEST_c_c(clog, 1.0e00, -2.0e00, 8.047189563512802124e-01, -1.107148721814155578e00, 0 },

	TEST_c_c(clog, ZERO_M, 0, INF_M, M_PIl, DIVIDE_BY_ZERO_EXCEPTION },
	TEST_c_c(clog, ZERO_M, ZERO_M, INF_M, -M_PIl, DIVIDE_BY_ZERO_EXCEPTION },

	TEST_c_c(clog, 0, 0, INF_M, 0.0, DIVIDE_BY_ZERO_EXCEPTION },
	TEST_c_c(clog, 0, ZERO_M, INF_M, ZERO_M, DIVIDE_BY_ZERO_EXCEPTION },

	TEST_c_c(clog, INF_M, INF_P, INF_P, M_PI_34l, 0 },
	TEST_c_c(clog, INF_M, INF_M, INF_P, -M_PI_34l, 0 },

	TEST_c_c(clog, INF_P, INF_P, INF_P, M_PI_4l, 0 },
	TEST_c_c(clog, INF_P, INF_M, INF_P, -M_PI_4l, 0 },

	TEST_c_c(clog, 0, INF_P, INF_P, M_PI_2l, 0 },
	TEST_c_c(clog, 3, INF_P, INF_P, M_PI_2l, 0 },
	TEST_c_c(clog, ZERO_M, INF_P, INF_P, M_PI_2l, 0 },
	TEST_c_c(clog, -3, INF_P, INF_P, M_PI_2l, 0 },
	TEST_c_c(clog, 0, INF_M, INF_P, -M_PI_2l, 0 },
	TEST_c_c(clog, 3, INF_M, INF_P, -M_PI_2l, 0 },
	TEST_c_c(clog, ZERO_M, INF_M, INF_P, -M_PI_2l, 0 },
	TEST_c_c(clog, -3, INF_M, INF_P, -M_PI_2l, 0 },

	TEST_c_c(clog, INF_M, 0, INF_P, M_PIl, 0 },
	TEST_c_c(clog, INF_M, 1, INF_P, M_PIl, 0 },
	TEST_c_c(clog, INF_M, ZERO_M, INF_P, -M_PIl, 0 },
	TEST_c_c(clog, INF_M, -1, INF_P, -M_PIl, 0 },

	TEST_c_c(clog, INF_P, 0, INF_P, 0.0, 0 },
	TEST_c_c(clog, INF_P, 1, INF_P, 0.0, 0 },
	TEST_c_c(clog, INF_P, ZERO_M, INF_P, ZERO_M, 0 },
	TEST_c_c(clog, INF_P, -1, INF_P, ZERO_M, 0 },

	TEST_c_c(clog, INF_P, QNAN_P, INF_P, QNAN_P, 0 },
	TEST_c_c(clog, INF_M, QNAN_P, INF_P, QNAN_P, 0 },

	TEST_c_c(clog, QNAN_P, INF_P, INF_P, QNAN_P, 0 },
	TEST_c_c(clog, QNAN_P, INF_M, INF_P, QNAN_P, 0 },

	TEST_c_c(clog, 0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog, 3, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog, ZERO_M, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog, -3, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(clog, QNAN_P, 0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog, QNAN_P, 5, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog, QNAN_P, ZERO_M, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog, QNAN_P, -5, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(clog, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(clog, 0.75L, 1.25L, 0.376885901188190075998919126749298416L, 1.03037682652431246378774332703115153L, 0 },
	TEST_c_c(clog, -2, -3, 1.2824746787307683680267437207826593L, -2.1587989303424641704769327722648368L, 0 },

	TEST_c_c(clog, 0x1.fffffep+127L, 0x1.fffffep+127L, 89.06941264234832570836679262104313101776L, M_PI_4l, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, 0x1.fffffep+127L, 1.0L, 88.72283905206835305365817656031404273372L, 2.938736052218037251011746307725933020145e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(clog, 0x1p-149L, 0x1p-149L, -102.9323563131518784484589700365392203592L, M_PI_4l, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, 0x1p-147L, 0x1p-147L, -101.5460619520319878296245057936228672231L, M_PI_4l, 0|XFAIL_NATIVE_LINUXF },

#ifndef TEST_FLOAT
	TEST_c_c(clog, 0x1.fffffffffffffp+1023L, 0x1.fffffffffffffp+1023L, 710.1292864836639693869320059713862337880L, M_PI_4l, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1.fffffffffffffp+1023L, 0x1p+1023L, 709.8942846690411016323109979483151967689L, 0.4636476090008061606231772164674799632783L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1p-1074L, 0x1p-1074L, -744.0934983311012896593986823853525458290L, M_PI_4l, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1p-1073L, 0x1p-1073L, -743.4003511505413443499814502638943692610L, M_PI_4l, 0|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(clog, 0x1.fp+16383L, 0x1.fp+16383L, 11356.83823118610934184548269774874545400L, M_PI_4l, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog, 0x1.fp+16383L, 0x1p+16383L, 11356.60974243783798653123798337822335902L, 0.4764674194737066993385333770295162295856L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog, 0x1p-16440L, 0x1p-16441L, -11395.22807662984378194141292922726786191L, 0.4636476090008061162142562314612144020285L, 0|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(clog, 0x1p-149L, 0x1.fp+127L, 88.69109041335841930424871526389807508374L, M_PI_2l, 0 },
	TEST_c_c(clog, -0x1p-149L, 0x1.fp+127L, 88.69109041335841930424871526389807508374L, M_PI_2l, 0 },
	TEST_c_c(clog, 0x1p-149L, -0x1.fp+127L, 88.69109041335841930424871526389807508374L, -M_PI_2l, 0 },
	TEST_c_c(clog, -0x1p-149L, -0x1.fp+127L, 88.69109041335841930424871526389807508374L, -M_PI_2l, 0 },
	TEST_c_c(clog, -0x1.fp+127L, 0x1p-149L, 88.69109041335841930424871526389807508374L, M_PIl, 0 },
	TEST_c_c(clog, -0x1.fp+127L, -0x1p-149L, 88.69109041335841930424871526389807508374L, -M_PIl, 0 },
#ifdef TEST_FLOAT
	TEST_c_c(clog, 0x1.fp+127L, 0x1p-149L, 88.69109041335841930424871526389807508374L, ZERO_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(clog, 0x1.fp+127L, -0x1p-149L, 88.69109041335841930424871526389807508374L, ZERO_M, UNDERFLOW_EXCEPTION },
#endif

#ifndef TEST_FLOAT
	TEST_c_c(clog, 0x1p-1074L, 0x1.fp+1023L, 709.7509641950694165420886960904242800794L, M_PI_2l, 0 },
	TEST_c_c(clog, -0x1p-1074L, 0x1.fp+1023L, 709.7509641950694165420886960904242800794L, M_PI_2l, 0 },
	TEST_c_c(clog, 0x1p-1074L, -0x1.fp+1023L, 709.7509641950694165420886960904242800794L, -M_PI_2l, 0 },
	TEST_c_c(clog, -0x1p-1074L, -0x1.fp+1023L, 709.7509641950694165420886960904242800794L, -M_PI_2l, 0 },
	TEST_c_c(clog, -0x1.fp+1023L, 0x1p-1074L, 709.7509641950694165420886960904242800794L, M_PIl, 0 },
	TEST_c_c(clog, -0x1.fp+1023L, -0x1p-1074L, 709.7509641950694165420886960904242800794L, -M_PIl, 0 },
#endif
#if defined TEST_DOUBLE || (defined TEST_LDOUBLE && LDBL_MAX_EXP == 1024)
	TEST_c_c(clog, 0x1.fp+1023L, 0x1p-1074L, 709.7509641950694165420886960904242800794L, ZERO_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(clog, 0x1.fp+1023L, -0x1p-1074L, 709.7509641950694165420886960904242800794L, ZERO_M, UNDERFLOW_EXCEPTION },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(clog, 0x1p-16445L, 0x1.fp+16383L, 11356.49165759582936919077408168801636572L, M_PI_2l, 0 },
	TEST_c_c(clog, -0x1p-16445L, 0x1.fp+16383L, 11356.49165759582936919077408168801636572L, M_PI_2l, 0 },
	TEST_c_c(clog, 0x1p-16445L, -0x1.fp+16383L, 11356.49165759582936919077408168801636572L, -M_PI_2l, 0 },
	TEST_c_c(clog, -0x1p-16445L, -0x1.fp+16383L, 11356.49165759582936919077408168801636572L, -M_PI_2l, 0 },
	TEST_c_c(clog, -0x1.fp+16383L, 0x1p-16445L, 11356.49165759582936919077408168801636572L, M_PIl, 0 },
	TEST_c_c(clog, -0x1.fp+16383L, -0x1p-16445L, 11356.49165759582936919077408168801636572L, -M_PIl, 0 },
	TEST_c_c(clog, 0x1.fp+16383L, 0x1p-16445L, 11356.49165759582936919077408168801636572L, ZERO_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(clog, 0x1.fp+16383L, -0x1p-16445L, 11356.49165759582936919077408168801636572L, ZERO_M, UNDERFLOW_EXCEPTION },
# if LDBL_MANT_DIG >= 113
	TEST_c_c(clog, 0x1p-16494L, 0x1.fp+16383L, 11356.49165759582936919077408168801636572L, M_PI_2l, 0 },
	TEST_c_c(clog, -0x1p-16494L, 0x1.fp+16383L, 11356.49165759582936919077408168801636572L, M_PI_2l, 0 },
	TEST_c_c(clog, 0x1p-16494L, -0x1.fp+16383L, 11356.49165759582936919077408168801636572L, -M_PI_2l, 0 },
	TEST_c_c(clog, -0x1p-16494L, -0x1.fp+16383L, 11356.49165759582936919077408168801636572L, -M_PI_2l, 0 },
	TEST_c_c(clog, -0x1.fp+16383L, 0x1p-16494L, 11356.49165759582936919077408168801636572L, M_PIl, 0 },
	TEST_c_c(clog, -0x1.fp+16383L, -0x1p-16494L, 11356.49165759582936919077408168801636572L, -M_PIl, 0 },
	TEST_c_c(clog, 0x1.fp+16383L, 0x1p-16494L, 11356.49165759582936919077408168801636572L, ZERO_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(clog, 0x1.fp+16383L, -0x1p-16494L, 11356.49165759582936919077408168801636572L, ZERO_M, UNDERFLOW_EXCEPTION },
# endif
#endif

	TEST_c_c(clog, 1.0L, 0x1.234566p-10L, 6.172834701221959432440126967147726538097e-7L, 1.111110564353742042376451655136933182201e-3L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, -1.0L, 0x1.234566p-20L, 5.886877547844618300918562490463748605537e-13L, 3.141591568520436206990380699322226378452L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1.234566p-30L, 1.0L, 5.614163921211322622623353961365728040115e-19L, 1.570796325735258575254858696548386439740L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, -0x1.234566p-40L, -1.0L, 5.354083939753840089583620652120903838944e-25L, -1.570796326795931422008642456283782656359L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1.234566p-50L, 1.0L, 5.106052341226425256332038420428899201070e-31L, 1.570796326794895608681734464330528755366L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1.234566p-60L, 1.0L, 4.869510976053643471080816669875627875933e-37L, 1.570796326794896618244456860363082279319L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1p-62L, 1.0L, 2.350988701644575015937473074444491355582e-38L, 1.570796326794896619014481257142650555297L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1p-63L, 1.0L, 5.877471754111437539843682686111228389059e-39L, 1.570796326794896619122901474391200998698L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1p-64L, 1.0L, 1.469367938527859384960920671527807097271e-39L, 1.570796326794896619177111583015476220398L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(clog, 0x1p-510L, 1.0L, 4.450147717014402766180465434664808128438e-308L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1p-511L, 1.0L, 1.112536929253600691545116358666202032110e-308L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1p-512L, 1.0L, 2.781342323134001728862790896665505080274e-309L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(clog, 0x1p-8190L, 1.0L, 6.724206286224187012525355634643505205196e-4932L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog, 0x1p-8191L, 1.0L, 1.681051571556046753131338908660876301299e-4932L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog, 0x1p-8192L, 1.0L, 4.202628928890116882828347271652190753248e-4933L, 1.570796326794896619231321691639751442099L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(clog, 0x1.000566p0L, 0x1.234p-10L, 8.298731898331237038231468223024422855654e-5L, 1.110938609507128729312743251313024793990e-3L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, 0x1.000566p0L, 0x1.234p-100L, 8.237022655933121125560939513260027133767e-5L, 8.974094312218060110948251664314290484113e-31L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(clog, -0x1.0000000123456p0L, 0x1.2345678p-30L, 2.649094282537168795982991778475646793277e-10L, 3.141592652530155111500161671113150737892L, 0 },
	TEST_c_c(clog, -0x1.0000000123456p0L, 0x1.2345678p-1000L, 2.649094276923003995420209214900915462737e-10L, 3.141592653589793238462643383279502884197L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(clog, 0x1.00000000000000123456789abcp0L, 0x1.23456789p-60L, 9.868649107778739757272772275265050767867e-19L, 9.868649106423871142816660980898339912137e-19L, 0 },
	TEST_c_c(clog, 0x1.00000000000000123456789abcp0L, 0x1.23456789p-1000L, 9.868649107778739752403260515979017248596e-19L, 1.061846605795612822522063052130030717368e-301L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
#endif

	TEST_c_c(clog, 0x0.ffffffp0L, 0x0.ffffffp-100L, -5.960464655174753498633255797994360530379e-8L, 7.888609052210118054117285652827862296732e-31L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(clog, 0x0.fffffffffffff8p0L, 0x0.fffffffffffff8p-1000L, -1.110223024625156602053389888482372171810e-16L, 9.332636185032188789900895447238171696171e-302L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
#endif
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(clog, 0x0.ffffffffffffffffp0L, 0x0.ffffffffffffffffp-15000L, -5.421010862427522170184200798202494495630e-20L, 3.548665303440282824232502561095699343814e-4516L, 0 },
#endif

	TEST_c_c(clog, 0x1a6p-10L, 0x3a5p-10L, -1.4305135209763571252847059962654228661815e-06L, 1.1460277178115757370775644871674016684074L, 0 },
	TEST_c_c(clog, 0xf2p-10L, 0x3e3p-10L, 6.1988446308070710970664736815277450078106e-06L, 1.3322126499153926210226335249558203898460L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, 0x4d4ep-15L, 0x6605p-15L, -1.6298145321400412054744424587143483169412e-08L, 0.9223574537155056772124552172295398141249L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, 0x2818p-15L, 0x798fp-15L, 1.5366822245016167178749091974664853785194e-08L, 1.2522014929038946066987318471922169174157L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, 0x9b57bp-20L, 0xcb7b4p-20L, -3.9563019528687610863490232935890272740908e-11L, 0.9187593477446338910857133065497364950682L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, 0x2731p-20L, 0xfffd0p-20L, 4.4110493034041283943115971658295280288115e-11L, 1.5612279663766352262688735061954290528838L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, 0x2ede88p-23L, 0x771c3fp-23L, -4.4764192352906350039050902870893173560494e-13L, 1.1959106857549200806818600493552847793381L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, 0x11682p-23L, 0x7ffed1p-23L, 1.1723955140027907954461000991619077811832e-12L, 1.5622968405332756349813737986164832897108L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, 0xa1f2c1p-24L, 0xc643aep-24L, -1.0480505352462576151523512837107080269981e-13L, 0.8858771987699967480545613322309315260313L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog, 0x659feap-24L, 0xeaf6f9p-24L, 3.7303493627403868207597214252239749960738e-14L, 1.1625816408046866464773042283673653469061L, 0|XFAIL_NATIVE_LINUXF },
#ifndef TEST_FLOAT
	/* disabled because real result is close to zero, but not exactly determinable */
#ifndef TEST_DOUBLE
	TEST_c_c(clog, 0x4447d7175p-35L, 0x6c445e00ap-35L, -1.4823076576950255933915367361099865652625e-20L, 1.0081311552703893116404606212158840190615L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x2dd46725bp-35L, 0x7783a1284p-35L, 4.4469229730850767799109418892826021157328e-20L, 1.2046235979300843056806465045930070146351L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x55cb6d0c83af5p-55L, 0x7fe33c0c7c4e90p-55L, -5.2000108498455368032511404449795741611813e-32L, 1.5288921536982513453421343495466824420259L, 0|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(clog, 0x164c74eea876p-45L, 0x16f393482f77p-45L, -3.0292258760486853327810377824479932031744e-26L, 0.7998237934177411746093524982030330293980L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0xfe961079616p-45L, 0x1bc37e09e6d1p-45L, 5.3718272201930019901317065495843842735179e-26L, 1.0503831592447830576186444373011142397404L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0xa4722f19346cp-51L, 0x7f9631c5e7f07p-51L, -6.2122796286154679676173624516405339768606e-30L, 1.4904138780720095276446375492434049214172L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x10673dd0f2481p-51L, 0x7ef1d17cefbd2p-51L, 3.2047474274603604594851472963586149973093e-29L, 1.4422922682185099608731642353544207976604L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x8ecbf810c4ae6p-52L, 0xd479468b09a37p-52L, -9.7375017988218644730510244778042114638107e-30L, 0.9790637929494922564724108399524154766631L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x5b06b680ea2ccp-52L, 0xef452b965da9fp-52L, 8.3076914081087805757422664530653247447136e-30L, 1.2072712126771536614482822173033535043206L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x659b70ab7971bp-53L, 0x1f5d111e08abecp-53L, -2.5083311595699359750201056724289010648701e-30L, 1.3710185432462268491534742969536240564640L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x15cfbd1990d1ffp-53L, 0x176a3973e09a9ap-53L, 1.0168910106364605304135563536838075568606e-30L, 0.8208373755522359859870890246475340086663L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog, 0x1367a310575591p-54L, 0x3cfcc0a0541f60p-54L, 5.0844550531823026520677817684239496041087e-32L, 1.2627468605458094918919206628466016525397L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(clog, 0x298c62cb546588a7p-63L, 0x7911b1dfcc4ecdaep-63L, -1.1931267660846218205882675852805793644095e-36L, 1.2402109774337032400594953899784058127412L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog, 0x4d9c37e2b5cb4533p-63L, 0x65c98be2385a042ep-63L, 6.4064442119814669184296141278612389400075e-37L, 0.9193591364645830864185131402313014890145L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog, 0x602fd5037c4792efp-64L, 0xed3e2086dcca80b8p-64L, -2.3362950222592964220878638677292132852104e-37L, 1.1856121127236268105413184264288408265852L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog, 0x6b10b4f3520217b6p-64L, 0xe8893cbb449253a1p-64L, 2.4244570985709679851855191080208817099132e-37L, 1.1393074519572050614551047548718495655972L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog, 0x81b7efa81fc35ad1p-65L, 0x1ef4b835f1c79d812p-65L, -9.9182335850630508484862145328126979066934e-39L, 1.3146479888794807046338799047003947008804L, 0|XFAIL_NATIVE_LINUXL },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(clog, 0x3f96469050f650869c2p-75L, 0x6f16b2c9c8b05988335p-75L, -1.0509738482436128031927971874674370984602e-45L, 1.0509191467640012308402149909370784281448L, 0 },
	TEST_c_c(clog, 0x3157fc1d73233e580c8p-75L, 0x761b52ccd435d7c7f5fp-75L, 1.3487497719126364307640897239165442763573e-43L, 1.1750493008528425228929764149024375035382L, 0 },
	TEST_c_c(clog, 0x155f8afc4c48685bf63610p-85L, 0x17d0cf2652cdbeb1294e19p-85L, -4.7775669192897997174762089350332738583822e-50L, 0.8393953487996880419413728440067635213372L, 0 },
	TEST_c_c(clog, 0x13836d58a13448d750b4b9p-85L, 0x195ca7bc3ab4f9161edbe6p-85L, 2.8398125044729578740243199963484494962411e-50L, 0.9149964976334130461795060758257083099706L, 0 },
	TEST_c_c(clog, 0x1df515eb171a808b9e400266p-95L, 0x7c71eb0cd4688dfe98581c77p-95L, -3.5048022044913950094635368750889659723004e-57L, 1.3345633256521815205858155673950177421079L, 0 },
	TEST_c_c(clog, 0xe33f66c9542ca25cc43c867p-95L, 0x7f35a68ebd3704a43c465864p-95L, 4.1101771307217268747345114262406964584250e-56L, 1.4596065864518742494094402406719567059585L, 0 },
	TEST_c_c(clog, 0x6771f22c64ed551b857c128b4cp-105L, 0x1f570e7a13cc3cf2f44fd793ea1p-105L, -1.4281333889622737316199756373421183559948e-62L, 1.3673546561165378090903506783353927980633L, 0 },
	TEST_c_c(clog, 0x15d8ab6ed05ca514086ac3a1e84p-105L, 0x1761e480aa094c0b10b34b09ce9p-105L, 1.0027319539522347477331743836657426754857e-62L, 0.8193464073721167323313606647411269414759L, 0 },
	TEST_c_c(clog, 0x187190c1a334497bdbde5a95f48p-106L, 0x3b25f08062d0a095c4cfbbc338dp-106L, -1.7471844652198029695350765775994001163767e-63L, 1.1789110097072986038243729592318526094314L, 0 },
	TEST_c_c(clog, 0x6241ef0da53f539f02fad67dabp-106L, 0x3fb46641182f7efd9caa769dac0p-106L, 4.3299788920664682288477984749202524623248e-63L, 1.4746938237585656250866370987773473745867L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(clog, 0x3e1d0a105ac4ebeacd9c6952d34cp-112L, 0xf859b3d1b06d005dcbb5516d5479p-112L, -1.1683999374665377365054966073875064467108e-66L, 1.3257197596350832748781065387304444940172L, 0 },
	TEST_c_c(clog, 0x47017a2e36807acb1e5214b209dep-112L, 0xf5f4a550c9d75e3bb1839d865f0dp-112L, 1.5077923002544367932999503838191154621839e-65L, 1.2897445708311412721399861948957141824914L, 0 },
	TEST_c_c(clog, 0x148f818cb7a9258fca942ade2a0cap-113L, 0x18854a34780b8333ec53310ad7001p-113L, -7.1865869169568789348552370692485515571497e-67L, 0.8730167479365994646287897223471819363668L, 0 },
	TEST_c_c(clog, 0xfd95243681c055c2632286921092p-113L, 0x1bccabcd29ca2152860ec29e34ef7p-113L, 6.6255694866654064502633121109394710807528e-66L, 1.0526409614996288387567810726095850312049L, 0 },
	TEST_c_c(clog, 0xdb85c467ee2aadd5f425fe0f4b8dp-114L, 0x3e83162a0f95f1dcbf97dddf410eap-114L, 4.6017338806965821566734340588575402712716e-67L, 1.3547418904611758959096647942223384691728L, 0 },
	TEST_c_c(clog, 0x1415bcaf2105940d49a636e98ae59p-115L, 0x7e6a150adfcd1b0921d44b31f40f4p-115L, 2.5993421227864195179698176012564317527271e-67L, 1.4132318089683022770487383611430906982461L, 0 },
#endif
}


static void test_clog10(void)
{
	struct test *testp = lookup("clog10" },

	TEST_c_c(clog10, ZERO_M, 0, INF_M, M_PIl, DIVIDE_BY_ZERO_EXCEPTION },
	TEST_c_c(clog10, ZERO_M, ZERO_M, INF_M, -M_PIl, DIVIDE_BY_ZERO_EXCEPTION },

	TEST_c_c(clog10, 0, 0, INF_M, 0.0, DIVIDE_BY_ZERO_EXCEPTION },
	TEST_c_c(clog10, 0, ZERO_M, INF_M, ZERO_M, DIVIDE_BY_ZERO_EXCEPTION },

	TEST_c_c(clog10, INF_M, INF_P, INF_P, M_PI_34_LOG10El, 0 },

	TEST_c_c(clog10, INF_P, INF_P, INF_P, M_PI4_LOG10El, 0 },
	TEST_c_c(clog10, INF_P, INF_M, INF_P, -M_PI4_LOG10El, 0 },

	TEST_c_c(clog10, 0, INF_P, INF_P, M_PI2_LOG10El, 0 },
	TEST_c_c(clog10, 3, INF_P, INF_P, M_PI2_LOG10El, 0 },
	TEST_c_c(clog10, ZERO_M, INF_P, INF_P, M_PI2_LOG10El, 0 },
	TEST_c_c(clog10, -3, INF_P, INF_P, M_PI2_LOG10El, 0 },
	TEST_c_c(clog10, 0, INF_M, INF_P, -M_PI2_LOG10El, 0 },
	TEST_c_c(clog10, 3, INF_M, INF_P, -M_PI2_LOG10El, 0 },
	TEST_c_c(clog10, ZERO_M, INF_M, INF_P, -M_PI2_LOG10El, 0 },
	TEST_c_c(clog10, -3, INF_M, INF_P, -M_PI2_LOG10El, 0 },

	TEST_c_c(clog10, INF_M, 0, INF_P, M_PI_LOG10El, 0 },
	TEST_c_c(clog10, INF_M, 1, INF_P, M_PI_LOG10El, 0 },
	TEST_c_c(clog10, INF_M, ZERO_M, INF_P, -M_PI_LOG10El, 0 },
	TEST_c_c(clog10, INF_M, -1, INF_P, -M_PI_LOG10El, 0 },

	TEST_c_c(clog10, INF_P, 0, INF_P, 0.0, 0 },
	TEST_c_c(clog10, INF_P, 1, INF_P, 0.0, 0 },
	TEST_c_c(clog10, INF_P, ZERO_M, INF_P, ZERO_M, 0 },
	TEST_c_c(clog10, INF_P, -1, INF_P, ZERO_M, 0 },

	TEST_c_c(clog10, INF_P, QNAN_P, INF_P, QNAN_P, 0 },
	TEST_c_c(clog10, INF_M, QNAN_P, INF_P, QNAN_P, 0 },

	TEST_c_c(clog10, QNAN_P, INF_P, INF_P, QNAN_P, 0 },
	TEST_c_c(clog10, QNAN_P, INF_M, INF_P, QNAN_P, 0 },

	TEST_c_c(clog10, 0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog10, 3, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog10, ZERO_M, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog10, -3, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(clog10, QNAN_P, 0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog10, QNAN_P, 5, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog10, QNAN_P, ZERO_M, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(clog10, QNAN_P, -5, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(clog10, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(clog10, 0.75L, 1.25L, 0.163679467193165171449476605077428975L, 0.447486970040493067069984724340855636L, 0 },
	TEST_c_c(clog10, -2, -3, 0.556971676153418384603252578971164214L, -0.937554462986374708541507952140189646L, 0 },

	TEST_c_c(clog10, 0x1.fffffep+127L, 0x1.fffffep+127L, 38.68235441693561449174780668781319348761L, M_PI4_LOG10El, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, 0x1.fffffep+127L, 1.0L, 38.53183941910362389414093724045094697423L, 1.276276851248440096917018665609900318458e-39L, UNDERFLOW_EXCEPTION_FLOAT },
	TEST_c_c(clog10, 0x1p-149L, 0x1p-149L, -44.70295435610120748924022586658721447508L, M_PI4_LOG10El, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, 0x1p-147L, 0x1p-147L, -44.10089436477324509881274807713822842154L, M_PI4_LOG10El, 0|XFAIL_NATIVE_LINUXF },

#ifndef TEST_FLOAT
	TEST_c_c(clog10, 0x1.fffffffffffffp+1023L, 0x1.fffffffffffffp+1023L, 308.4052305577487344482591243175787477115L, M_PI4_LOG10El, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1.fffffffffffffp+1023L, 0x1p+1023L, 308.3031705664207720674749211936626341569L, 0.2013595981366865903254995612594728746470L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1p-1074L, 0x1p-1074L, -323.1557003452838130619487034867432642357L, M_PI4_LOG10El, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1p-1073L, 0x1p-1073L, -322.8546703496198318667349645920187712089L, M_PI4_LOG10El, 0|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(clog10, 0x1.fp+16383L, 0x1.fp+16383L, 4932.212175672014259683102930239951947672L, M_PI4_LOG10El, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog10, 0x1.fp+16383L, 0x1p+16383L, 4932.112944269463028900262609694408579449L, 0.2069271710841128115912940666587802677383L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog10, 0x1p-16440L, 0x1p-16441L, -4948.884673709346821106688037612752099609L, 0.2013595981366865710389502301937289472543L, 0|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(clog10, 0x1p-149L, 0x1.fp+127L, 38.51805116050395969095658815123105801479L, 0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1p-149L, 0x1.fp+127L, 38.51805116050395969095658815123105801479L, 0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, 0x1p-149L, -0x1.fp+127L, 38.51805116050395969095658815123105801479L, -0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1p-149L, -0x1.fp+127L, 38.51805116050395969095658815123105801479L, -0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1.fp+127L, 0x1p-149L, 38.51805116050395969095658815123105801479L, 1.364376353841841347485783625431355770210L, 0 },
	TEST_c_c(clog10, -0x1.fp+127L, -0x1p-149L, 38.51805116050395969095658815123105801479L, -1.364376353841841347485783625431355770210L, 0 },
#ifdef TEST_FLOAT
	TEST_c_c(clog10, 0x1.fp+127L, 0x1p-149L, 38.51805116050395969095658815123105801479L, ZERO_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(clog10, 0x1.fp+127L, -0x1p-149L, 38.51805116050395969095658815123105801479L, ZERO_M, UNDERFLOW_EXCEPTION },
#endif

#ifndef TEST_FLOAT
	TEST_c_c(clog10, 0x1p-1074L, 0x1.fp+1023L, 308.2409272754311106024666378243768099991L, 0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1p-1074L, 0x1.fp+1023L, 308.2409272754311106024666378243768099991L, 0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, 0x1p-1074L, -0x1.fp+1023L, 308.2409272754311106024666378243768099991L, -0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1p-1074L, -0x1.fp+1023L, 308.2409272754311106024666378243768099991L, -0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1.fp+1023L, 0x1p-1074L, 308.2409272754311106024666378243768099991L, 1.364376353841841347485783625431355770210L, 0 },
	TEST_c_c(clog10, -0x1.fp+1023L, -0x1p-1074L, 308.2409272754311106024666378243768099991L, -1.364376353841841347485783625431355770210L, 0 },
#endif
#if defined TEST_DOUBLE || (defined TEST_LDOUBLE && LDBL_MAX_EXP == 1024)
	TEST_c_c(clog10, 0x1.fp+1023L, 0x1p-1074L, 308.2409272754311106024666378243768099991L, ZERO_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(clog10, 0x1.fp+1023L, -0x1p-1074L, 308.2409272754311106024666378243768099991L, ZERO_M, UNDERFLOW_EXCEPTION },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(clog10, 0x1p-16445L, 0x1.fp+16383L, 4932.061660674182269085496060792589701158L, 0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1p-16445L, 0x1.fp+16383L, 4932.061660674182269085496060792589701158L, 0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, 0x1p-16445L, -0x1.fp+16383L, 4932.061660674182269085496060792589701158L, -0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1p-16445L, -0x1.fp+16383L, 4932.061660674182269085496060792589701158L, -0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1.fp+16383L, 0x1p-16445L, 4932.061660674182269085496060792589701158L, 1.364376353841841347485783625431355770210L, 0 },
	TEST_c_c(clog10, -0x1.fp+16383L, -0x1p-16445L, 4932.061660674182269085496060792589701158L, -1.364376353841841347485783625431355770210L, 0 },
	TEST_c_c(clog10, 0x1.fp+16383L, 0x1p-16445L, 4932.061660674182269085496060792589701158L, ZERO_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(clog10, 0x1.fp+16383L, -0x1p-16445L, 4932.061660674182269085496060792589701158L, ZERO_M, UNDERFLOW_EXCEPTION },
# if LDBL_MANT_DIG >= 113
	TEST_c_c(clog10, 0x1p-16494L, 0x1.fp+16383L, 4932.061660674182269085496060792589701158L, 0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1p-16494L, 0x1.fp+16383L, 4932.061660674182269085496060792589701158L, 0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, 0x1p-16494L, -0x1.fp+16383L, 4932.061660674182269085496060792589701158L, -0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1p-16494L, -0x1.fp+16383L, 4932.061660674182269085496060792589701158L, -0.6821881769209206737428918127156778851051L, 0 },
	TEST_c_c(clog10, -0x1.fp+16383L, 0x1p-16494L, 4932.061660674182269085496060792589701158L, 1.364376353841841347485783625431355770210L, 0 },
	TEST_c_c(clog10, -0x1.fp+16383L, -0x1p-16494L, 4932.061660674182269085496060792589701158L, -1.364376353841841347485783625431355770210L, 0 },
	TEST_c_c(clog10, 0x1.fp+16383L, 0x1p-16494L, 4932.061660674182269085496060792589701158L, ZERO_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(clog10, 0x1.fp+16383L, -0x1p-16494L, 4932.061660674182269085496060792589701158L, ZERO_M, UNDERFLOW_EXCEPTION },
# endif
#endif

	TEST_c_c(clog10, 1.0L, 0x1.234566p-10L, 2.680828048441605163181684680300513080769e-7L, 4.825491868832381486767558728169977751564e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, -1.0L, 0x1.234566p-20L, 2.556638434669064077889576526006849923281e-13L, 1.364375882602207106407956770293808181427L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1.234566p-30L, 1.0L, 2.438200411482400072282924063740535840474e-19L, 6.821881764607257184291586401763604544928e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, -0x1.234566p-40L, -1.0L, 2.325249110681915353442924915876654139373e-25L, -6.821881769213700828789403802671540158935e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1.234566p-50L, 1.0L, 2.217530356103816369479108963807448194409e-31L, 6.821881769209202348667823902864283966959e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1.234566p-60L, 1.0L, 2.114801746467415208319767917450504756866e-37L, 6.821881769209206733143018621078368211515e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1p-61L, 1.0L, 4.084085680564517578238994467153626207224e-38L, 6.821881769209206735545466044044889962925e-1L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1p-62L, 1.0L, 1.021021420141129394559748616788406551878e-38L, 6.821881769209206736487192085600834406988e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1p-63L, 1.0L, 2.552553550352823486399371541971016379740e-39L, 6.821881769209206736958055106378806629019e-1L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUX },
#ifndef TEST_FLOAT
	TEST_c_c(clog10, 0x1p-509L, 1.0L, 7.730698388614835910296270976605350994446e-308L, 6.821881769209206737428918127156778851051e-1L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1p-510L, 1.0L, 1.932674597153708977574067744151337748612e-308L, 6.821881769209206737428918127156778851051e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1p-511L, 1.0L, 4.831686492884272443935169360378344371529e-309L, 6.821881769209206737428918127156778851051e-1L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(clog10, 0x1p-8189L, 1.0L, 1.168114274114528946314738738025008370069e-4931L, 6.821881769209206737428918127156778851051e-1L, UNDERFLOW_EXCEPTION_OK|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog10, 0x1p-8190L, 1.0L, 2.920285685286322365786846845062520925172e-4932L, 6.821881769209206737428918127156778851051e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog10, 0x1p-8191L, 1.0L, 7.300714213215805914467117112656302312931e-4933L, 6.821881769209206737428918127156778851051e-1L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(clog10, 0x1.000566p0L, 0x1.234p-10L, 3.604093470239754109961125085078190708674e-5L, 4.824745078422174667425851670822596859720e-4L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, 0x1.000566p0L, 0x1.234p-100L, 3.577293486783822178310971763308187385546e-5L, 3.897399639875661463735636919790792140598e-31L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(clog10, -0x1.0000000123456p0L, 0x1.2345678p-30L, 1.150487028947346337782682105935961875822e-10L, 1.364376353381646356131680448946397884147L, 0 },
	TEST_c_c(clog10, -0x1.0000000123456p0L, 0x1.2345678p-1000L, 1.150487026509145544402795327729455391948e-10L, 1.364376353841841347485783625431355770210L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(clog10, 0x1.00000000000000123456789abcp0L, 0x1.23456789p-60L, 4.285899851347756188767674032946882584784e-19L, 4.285899850759344225805480528847018395861e-19L, 0 },
	TEST_c_c(clog10, 0x1.00000000000000123456789abcp0L, 0x1.23456789p-1000L, 4.285899851347756186652871946325962330640e-19L, 4.611541215247321502041995872887317363241e-302L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
#endif

	TEST_c_c(clog10, 0x0.ffffffp0L, 0x0.ffffffp-100L, -2.588596909321764128428416045209904492216e-8L, 3.425979381266895667295625489912064603415e-31L, 0 },
#ifndef TEST_FLOAT
	TEST_c_c(clog10, 0x0.fffffffffffff8p0L, 0x0.fffffffffffff8p-1000L, -4.821637332766435821255375046554377090472e-17L, 4.053112396770095089737411317782466262176e-302L, UNDERFLOW_EXCEPTION_LDOUBLE_IBM },
#endif
#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(clog10, 0x0.ffffffffffffffffp0L, 0x0.ffffffffffffffffp-15000L, -2.354315103889861110220423157644627849164e-20L, 1.541165759405643564697852372112893034397e-4516L, 0 },
#endif

	TEST_c_c(clog10, 0x1a6p-10L, 0x3a5p-10L, -6.2126412844802358329771948751248003038444e-07L, 0.4977135139537443711784513409096950995985L, 0 },
	TEST_c_c(clog10, 0xf2p-10L, 0x3e3p-10L, 2.6921240173351112953324592659528481616879e-06L, 0.5785726025799636431142862788413361783862L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, 0x4d4ep-15L, 0x6605p-15L, -7.0781945783414996953799915941870192015212e-09L, 0.4005747524909781155537088181659175147564L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, 0x2818p-15L, 0x798fp-15L, 6.6737261053986614395049481326819059203910e-09L, 0.5438241985991753781478398141908629586460L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, 0x9b57bp-20L, 0xcb7b4p-20L, -1.7182001068739620267773842120965071561416e-11L, 0.3990121149225253562859800593935899629087L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, 0x2731p-20L, 0xfffd0p-20L, 1.9156943718715958194239364991329064049438e-11L, 0.6780326907904082601285090019969008967595L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, 0x2ede88p-23L, 0x771c3fp-23L, -1.9440841725722970687903291200493082253766e-13L, 0.5193774116724956222518530053006822210323L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, 0x11682p-23L, 0x7ffed1p-23L, 5.0916490233953865181284669870035717560498e-13L, 0.6784968969384861816694467029319146542069L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, 0xa1f2c1p-24L, 0xc643aep-24L, -4.5516256421319921959681423447271490869664e-14L, 0.3847315790697197749315054516562206543710L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(clog10, 0x659feap-24L, 0xeaf6f9p-24L, 1.6200701438094619117335617123525612051457e-14L, 0.5049027913635038013499728086604870749732L, 0|XFAIL_NATIVE_LINUXF },
#ifndef TEST_FLOAT
	/* disabled because real result is close to zero, but not exactly determinable */
#ifndef TEST_DOUBLE
	TEST_c_c(clog10, 0x4447d7175p-35L, 0x6c445e00ap-35L, -6.4375803621988389731799033530075237868110e-21L, 0.4378257977686804492768642780897650927167L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x2dd46725bp-35L, 0x7783a1284p-35L, 1.9312741086596516918394613098872836703188e-20L, 0.5231613813514771042838490538484014771862L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x55cb6d0c83af5p-55L, 0x7fe33c0c7c4e90p-55L, -2.2583360179249556400630343805573865814771e-32L, 0.6639894257763289307423302343317622430835L, 0|XFAIL_NATIVE_LINUX },
#endif
	TEST_c_c(clog10, 0x164c74eea876p-45L, 0x16f393482f77p-45L, -1.3155760824064879362415202279780039150764e-26L, 0.3473590599762514228227328130640352044313L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0xfe961079616p-45L, 0x1bc37e09e6d1p-45L, 2.3329549194675052736016290082882121135546e-26L, 0.4561756099441139182878993697611751382976L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0xa4722f19346cp-51L, 0x7f9631c5e7f07p-51L, -2.6979587627476803379953050733225113494503e-30L, 0.6472785229986997177606324374555347813105L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x10673dd0f2481p-51L, 0x7ef1d17cefbd2p-51L, 1.3918041236396763648388478552321724382899e-29L, 0.6263795733790237053262025311642907438291L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x8ecbf810c4ae6p-52L, 0xd479468b09a37p-52L, -4.2289432987513243393180377141513840878196e-30L, 0.4252020027092323591068799049905597805296L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x5b06b680ea2ccp-52L, 0xef452b965da9fp-52L, 3.6079845358966994996207055940336690133424e-30L, 0.5243112258263349992771652393178033846555L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x659b70ab7971bp-53L, 0x1f5d111e08abecp-53L, -1.0893543813872082317104059174982092534059e-30L, 0.5954257879188711495921161433751775633232L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x15cfbd1990d1ffp-53L, 0x176a3973e09a9ap-53L, 4.4163015461643576961232672330852798804976e-31L, 0.3564851427422832755956993418877523303529L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(clog10, 0x1367a310575591p-54L, 0x3cfcc0a0541f60p-54L, 2.2081507730821788480616336165447731164865e-32L, 0.5484039935757001196548030312819898864760L, 0|XFAIL_NATIVE_LINUX },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	TEST_c_c(clog10, 0x298c62cb546588a7p-63L, 0x7911b1dfcc4ecdaep-63L, -5.1816837072162316773907242302011632570857e-37L, 0.5386167838952956925896424154370364458140L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog10, 0x4d9c37e2b5cb4533p-63L, 0x65c98be2385a042ep-63L, 2.7822833698845776001753149807484078521508e-37L, 0.3992725998539071066769046272515417679815L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog10, 0x602fd5037c4792efp-64L, 0xed3e2086dcca80b8p-64L, -1.0146400362652473358437501879334790111898e-37L, 0.5149047982335273098246594109614460842099L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog10, 0x6b10b4f3520217b6p-64L, 0xe8893cbb449253a1p-64L, 1.0529283395205396881397407610630442563938e-37L, 0.4947949395762683446121140513971996916447L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(clog10, 0x81b7efa81fc35ad1p-65L, 0x1ef4b835f1c79d812p-65L, -4.3074341162203896332989394770760901408798e-39L, 0.5709443672155660428417571212549720987784L, 0|XFAIL_NATIVE_LINUXL },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 106
	TEST_c_c(clog10, 0x3f96469050f650869c2p-75L, 0x6f16b2c9c8b05988335p-75L, -4.5643214291682663316715446865040356750881e-46L, 0.4564083863660793840592614609053162690362L, 0 },
	TEST_c_c(clog10, 0x3157fc1d73233e580c8p-75L, 0x761b52ccd435d7c7f5fp-75L, 5.8575458340992751256451490143468457830297e-44L, 0.5103174273246635294300470585396890237265L, 0 },
	TEST_c_c(clog10, 0x155f8afc4c48685bf63610p-85L, 0x17d0cf2652cdbeb1294e19p-85L, -2.0748709499710785084693619097712106753591e-50L, 0.3645447681189598740620098186365764884771L, 0 },
	TEST_c_c(clog10, 0x13836d58a13448d750b4b9p-85L, 0x195ca7bc3ab4f9161edbe6p-85L, 1.2333149003324592532859843519619084433953e-50L, 0.3973779298829931059309198145608711073016L, 0 },
	TEST_c_c(clog10, 0x1df515eb171a808b9e400266p-95L, 0x7c71eb0cd4688dfe98581c77p-95L, -1.5221162575729652613635150540947625639689e-57L, 0.5795934880811949230121092882659698986043L, 0 },
	TEST_c_c(clog10, 0xe33f66c9542ca25cc43c867p-95L, 0x7f35a68ebd3704a43c465864p-95L, 1.7850272475173865337808494725293124613817e-56L, 0.6338990862456906754888183278564382516852L, 0 },
	TEST_c_c(clog10, 0x6771f22c64ed551b857c128b4cp-105L, 0x1f570e7a13cc3cf2f44fd793ea1p-105L, -6.2023045024810589256360494043570293518879e-63L, 0.5938345819561308555003145899438513900776L, 0 },
	TEST_c_c(clog10, 0x15d8ab6ed05ca514086ac3a1e84p-105L, 0x1761e480aa094c0b10b34b09ce9p-105L, 4.3548095442952115860848857519953610343042e-63L, 0.3558376234889641500775150477035448866763L, 0 },
	TEST_c_c(clog10, 0x187190c1a334497bdbde5a95f48p-106L, 0x3b25f08062d0a095c4cfbbc338dp-106L, -7.5879257211204444302994221436282805900756e-64L, 0.5119945461708707332160859198685423099187L, 0 },
	TEST_c_c(clog10, 0x6241ef0da53f539f02fad67dabp-106L, 0x3fb46641182f7efd9caa769dac0p-106L, 1.8804859395820231849002915747252695375405e-63L, 0.6404513901551516189871978418046651877394L, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 113
	TEST_c_c(clog10, 0x3e1d0a105ac4ebeacd9c6952d34cp-112L, 0xf859b3d1b06d005dcbb5516d5479p-112L, -5.0742964549782184008668435276046798273476e-67L, 0.5757527761596220360985719127090110408283L, 0 },
	TEST_c_c(clog10, 0x47017a2e36807acb1e5214b209dep-112L, 0xf5f4a550c9d75e3bb1839d865f0dp-112L, 6.5482587585671294601662599808612773010057e-66L, 0.5601289501766423782280643144987875760229L, 0 },
	TEST_c_c(clog10, 0x148f818cb7a9258fca942ade2a0cap-113L, 0x18854a34780b8333ec53310ad7001p-113L, -3.1210950417524756037077807411854181477733e-67L, 0.3791463562379872585396164879981280044658L, 0 },
	TEST_c_c(clog10, 0xfd95243681c055c2632286921092p-113L, 0x1bccabcd29ca2152860ec29e34ef7p-113L, 2.8774482675253468630312378575186855052697e-66L, 0.4571561610046221605554903008571429975493L, 0 },
	TEST_c_c(clog10, 0xdb85c467ee2aadd5f425fe0f4b8dp-114L, 0x3e83162a0f95f1dcbf97dddf410eap-114L, 1.9985076315737626043096596036300177494613e-67L, 0.5883569274304683249184005177865521205198L, 0 },
	TEST_c_c(clog10, 0x1415bcaf2105940d49a636e98ae59p-115L, 0x7e6a150adfcd1b0921d44b31f40f4p-115L, 1.1288799405048268615023706955013387413519e-67L, 0.6137587762850841972073301550420510507903L, 0 },
#endif
}


static void test_conj(void)
{
	struct test *testp = lookup("conj" },

	TEST_c_c(conj, 0.0, 0.0, 0.0, ZERO_M, NO_INEXACT_EXCEPTION },
	TEST_c_c(conj, 0.0, ZERO_M, 0.0, 0.0, NO_INEXACT_EXCEPTION },
	TEST_c_c(conj, QNAN_P, QNAN_P, QNAN_P, QNAN_P, NO_INEXACT_EXCEPTION },
	TEST_c_c(conj, INF_P, INF_M, INF_P, INF_P, NO_INEXACT_EXCEPTION },
	TEST_c_c(conj, INF_P, INF_P, INF_P, INF_M, NO_INEXACT_EXCEPTION },
	TEST_c_c(conj, 1.0, 2.0, 1.0, -2.0, NO_INEXACT_EXCEPTION },
	TEST_c_c(conj, 3.0, -4.0, 3.0, 4.0, NO_INEXACT_EXCEPTION },
}


static void test_cproj(void)
{
	struct test *testp = lookup("cproj" },

	TEST_c_c(cproj, 0.0, 0.0, 0.0, 0.0, NO_INEXACT_EXCEPTION },
	TEST_c_c(cproj, ZERO_M, ZERO_M, ZERO_M, ZERO_M, NO_INEXACT_EXCEPTION },
	TEST_c_c(cproj, 0.0, ZERO_M, 0.0, ZERO_M, NO_INEXACT_EXCEPTION },
	TEST_c_c(cproj, ZERO_M, 0.0, ZERO_M, 0.0, NO_INEXACT_EXCEPTION },

	TEST_c_c(cproj, QNAN_P, QNAN_P, QNAN_P, QNAN_P, NO_INEXACT_EXCEPTION },

	TEST_c_c(cproj, INF_P, INF_P, INF_P, 0.0, NO_INEXACT_EXCEPTION },
	TEST_c_c(cproj, INF_P, INF_M, INF_P, ZERO_M, NO_INEXACT_EXCEPTION },
	TEST_c_c(cproj, INF_M, INF_P, INF_P, 0.0, NO_INEXACT_EXCEPTION },
	TEST_c_c(cproj, INF_M, INF_M, INF_P, ZERO_M, NO_INEXACT_EXCEPTION },

	TEST_c_c(cproj, 1.0, 0.0, 1.0, 0.0, NO_INEXACT_EXCEPTION },
	TEST_c_c(cproj, 2.0, 3.0, 2.0, 3.0, NO_INEXACT_EXCEPTION|XFAIL_NATIVE_LINUX },
}


static void test_csin(void)
{
	struct test *testp = lookup("csin" },

	TEST_c_c(csin, 1.0e00, 2.0e00, 3.165778547525405883e00, 1.959601044654846191e00, 0 },
	TEST_c_c(csin, 0.0e00, 2.0e00, 0.0e00, 3.626860409975051879e00, 0 },
	TEST_c_c(csin, 1.0e00, 0.0e00, 8.414709866046905517e-01, 0.0e00, 0 },
	TEST_c_c(csin, -2.0e00, 2.0e00, -3.420954853296279907e00, -1.509306490421295166e00, 0 },
	TEST_c_c(csin, -1.0e00, -2.0e00, -3.165778547525405883e00, -1.959601044654846191e00, 0 },
	TEST_c_c(csin, 1.0e00, -2.0e00, 3.165778547525405883e00, -1.959601044654846191e00, 0 },

	TEST_c_c(csin, 0.0, 0.0, 0.0, 0.0, 0 },
	TEST_c_c(csin, ZERO_M, 0.0, ZERO_M, 0.0, 0 },
	TEST_c_c(csin, 0.0, ZERO_M, 0, ZERO_M, 0 },
	TEST_c_c(csin, ZERO_M, ZERO_M, ZERO_M, ZERO_M, 0 },

	TEST_c_c(csin, 0.0, INF_P, 0.0, INF_P, 0 },
	TEST_c_c(csin, ZERO_M, INF_P, ZERO_M, INF_P, 0 },
	TEST_c_c(csin, 0.0, INF_M, 0.0, INF_M, 0 },
	TEST_c_c(csin, ZERO_M, INF_M, ZERO_M, INF_M, 0 },

	TEST_c_c(csin, INF_P, 0.0, QNAN_P, 0.0, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csin, INF_M, 0.0, QNAN_P, 0.0, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csin, INF_P, ZERO_M, QNAN_P, 0.0, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csin, INF_M, ZERO_M, QNAN_P, 0.0, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },

	TEST_c_c(csin, INF_P, INF_P, QNAN_P, INF_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csin, INF_M, INF_P, QNAN_P, INF_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csin, INF_P, INF_M, QNAN_P, INF_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csin, INF_M, INF_M, QNAN_P, INF_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },

	TEST_c_c(csin, INF_P, 6.75, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(csin, INF_P, -6.75, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(csin, INF_M, 6.75, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(csin, INF_M, -6.75,  QNAN_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(csin, 4.625, INF_P, INF_M, INF_M, 0 },
	TEST_c_c(csin, 4.625, INF_M, INF_M, INF_P, 0 },
	TEST_c_c(csin, -4.625, INF_P, INF_P, INF_M, 0 },
	TEST_c_c(csin, -4.625, INF_M, INF_P, INF_P, 0 },

	TEST_c_c(csin, QNAN_P, 0.0, QNAN_P, 0.0, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csin, QNAN_P, ZERO_M, QNAN_P, 0.0, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(csin, QNAN_P, INF_P, QNAN_P, INF_P, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csin, QNAN_P, INF_M, QNAN_P, INF_P, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(csin, QNAN_P, 9.0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csin, QNAN_P, -9.0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(csin, 0.0, QNAN_P, 0.0, QNAN_P, 0 },
	TEST_c_c(csin, ZERO_M, QNAN_P, ZERO_M, QNAN_P, 0 },

	TEST_c_c(csin, 10.0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csin, QNAN_P, -10.0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(csin, INF_P, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csin, INF_M, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(csin, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(csin, 0.75L, 1.25L, 1.28722291002649188575873510790565441L, 1.17210635989270256101081285116138863L, 0 },
	TEST_c_c(csin, -2, -3, -9.15449914691142957346729954460983256L, 4.16890695996656435075481305885375484L, 0 },

	TEST_c_c(csin, 0.75, 89.5, 2.522786001038096774676288412995370563339e38L, 2.708024460708609732016532185663087200560e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(csin, 0.75, -89.5, 2.522786001038096774676288412995370563339e38L, -2.708024460708609732016532185663087200560e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(csin, -0.75, 89.5, -2.522786001038096774676288412995370563339e38L, 2.708024460708609732016532185663087200560e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(csin, -0.75, -89.5, -2.522786001038096774676288412995370563339e38L, -2.708024460708609732016532185663087200560e38L, 0|XFAIL_NATIVE_LINUXF },

#ifndef TEST_FLOAT
	TEST_c_c(csin, 0.75, 710.5, 1.255317763348154410745082950806112487736e308L, 1.347490911916428129246890157395342279438e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(csin, 0.75, -710.5, 1.255317763348154410745082950806112487736e308L, -1.347490911916428129246890157395342279438e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(csin, -0.75, 710.5, -1.255317763348154410745082950806112487736e308L, 1.347490911916428129246890157395342279438e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(csin, -0.75, -710.5, -1.255317763348154410745082950806112487736e308L, -1.347490911916428129246890157395342279438e308L, 0|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(csin, 0.75, 11357.25, 8.385498349388321535962327491346664141020e4931L, 9.001213196851067077465606717616495588201e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(csin, 0.75, -11357.25, 8.385498349388321535962327491346664141020e4931L, -9.001213196851067077465606717616495588201e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(csin, -0.75, 11357.25, -8.385498349388321535962327491346664141020e4931L, 9.001213196851067077465606717616495588201e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(csin, -0.75, -11357.25, -8.385498349388321535962327491346664141020e4931L, -9.001213196851067077465606717616495588201e4931L, 0|XFAIL_NATIVE_LINUXL },
#endif

#ifdef TEST_FLOAT
	TEST_c_c(csin, 0x1p-149, 180, 1.043535896672617552965983803453927655332e33L, INF_P, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXF },
#endif

#if defined TEST_DOUBLE || (defined TEST_LDOUBLE && LDBL_MAX_EXP == 1024)
	TEST_c_c(csin, 0x1p-1074, 1440, 5.981479269486130556466515778180916082415e301L, INF_P, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(csin, 0x1p-16434L, 22730, 1.217853148905605987081057582351152052687e4924L, INF_P, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(csin, SUBNORM_P, MIN_P, SUBNORM_P, MIN_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(csin, MIN_P, SUBNORM_P, MIN_P, SUBNORM_P, UNDERFLOW_EXCEPTION },
}


static void test_csinh(void)
{
	struct test *testp = lookup("csinh" },

	TEST_c_c(csinh, 1.0e00, 2.0e00, -4.890562593936920166e-01, 1.403119236230850219e00, 0 },
	TEST_c_c(csinh, 0.0e00, 2.0e00, -0.0e00, 9.09297426825681709417e-01, 0 },
	TEST_c_c(csinh, 1.0e00, 0.0e00, 1.175201192498207092e00, 0.0e00, 0 },
	TEST_c_c(csinh, -2.0e00, 2.0e00, 1.509306475520133972e00, 3.420954853296279907e00, 0 },
	TEST_c_c(csinh, -1.0e00, -2.0e00, 4.890562593936920166e-01, -1.403119236230850219e00, 0 },
	TEST_c_c(csinh, 1.0e00, -2.0e00, -4.890562593936920166e-01, -1.403119236230850219e00, 0 },

	TEST_c_c(csinh, 0.0, 0.0, 0.0, 0.0, 0 },
	TEST_c_c(csinh, ZERO_M, 0.0, ZERO_M, 0.0, 0 },
	TEST_c_c(csinh, 0.0, ZERO_M, 0.0, ZERO_M, 0 },
	TEST_c_c(csinh, ZERO_M, ZERO_M, ZERO_M, ZERO_M, 0 },

	TEST_c_c(csinh, 0.0, INF_P, 0.0, QNAN_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csinh, ZERO_M, INF_P, 0.0, QNAN_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csinh, 0.0, INF_M, 0.0, QNAN_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csinh, ZERO_M, INF_M, 0.0, QNAN_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },

	TEST_c_c(csinh, INF_P, 0.0, INF_P, 0.0, 0 },
	TEST_c_c(csinh, INF_M, 0.0, INF_M, 0.0, 0 },
	TEST_c_c(csinh, INF_P, ZERO_M, INF_P, ZERO_M, 0 },
	TEST_c_c(csinh, INF_M, ZERO_M, INF_M, ZERO_M, 0 },

	TEST_c_c(csinh, INF_P, INF_P, INF_P, QNAN_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csinh, INF_M, INF_P, INF_P, QNAN_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csinh, INF_P, INF_M, INF_P, QNAN_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csinh, INF_M, INF_M, INF_P, QNAN_P, INVALID_EXCEPTION|IGNORE_ZERO_INF_SIGN },

	TEST_c_c(csinh, INF_P, 4.625, INF_M, INF_M, 0 },
	TEST_c_c(csinh, INF_M, 4.625, INF_P, INF_M, 0 },
	TEST_c_c(csinh, INF_P, -4.625, INF_M, INF_P, 0 },
	TEST_c_c(csinh, INF_M, -4.625, INF_P, INF_P, 0 },

	TEST_c_c(csinh, 6.75, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(csinh, -6.75, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(csinh, 6.75, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(csinh, -6.75, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(csinh, 0.0, QNAN_P, 0.0, QNAN_P, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csinh, ZERO_M, QNAN_P, 0.0, QNAN_P, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(csinh, INF_P, QNAN_P, INF_P, QNAN_P, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(csinh, INF_M, QNAN_P, INF_P, QNAN_P, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(csinh, 9.0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csinh, -9.0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(csinh, QNAN_P, 0.0, QNAN_P, 0.0, 0 },
	TEST_c_c(csinh, QNAN_P, ZERO_M, QNAN_P, ZERO_M, 0 },

	TEST_c_c(csinh, QNAN_P, 10.0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csinh, QNAN_P, -10.0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(csinh, QNAN_P, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csinh, QNAN_P, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(csinh, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(csinh, 0.75L, 1.25L, 0.259294854551162779153349830618433028L, 1.22863452409509552219214606515777594L, 0 },
	TEST_c_c(csinh, -2, -3, 3.59056458998577995201256544779481679L, -0.530921086248519805267040090660676560L, 0 },

	TEST_c_c(csinh, 89.5, 0.75, 2.708024460708609732016532185663087200560e38L, 2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(csinh, -89.5, 0.75, -2.708024460708609732016532185663087200560e38L, 2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(csinh, 89.5, -0.75, 2.708024460708609732016532185663087200560e38L, -2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(csinh, -89.5, -0.75, -2.708024460708609732016532185663087200560e38L, -2.522786001038096774676288412995370563339e38L, 0|XFAIL_NATIVE_LINUXF },

#ifndef TEST_FLOAT
	TEST_c_c(csinh, 710.5, 0.75, 1.347490911916428129246890157395342279438e308L, 1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(csinh, -710.5, 0.75, -1.347490911916428129246890157395342279438e308L, 1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(csinh, 710.5, -0.75, 1.347490911916428129246890157395342279438e308L, -1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(csinh, -710.5, -0.75, -1.347490911916428129246890157395342279438e308L, -1.255317763348154410745082950806112487736e308L, 0|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(csinh, 11357.25, 0.75, 9.001213196851067077465606717616495588201e4931L, 8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(csinh, -11357.25, 0.75, -9.001213196851067077465606717616495588201e4931L, 8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(csinh, 11357.25, -0.75, 9.001213196851067077465606717616495588201e4931L, -8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(csinh, -11357.25, -0.75, -9.001213196851067077465606717616495588201e4931L, -8.385498349388321535962327491346664141020e4931L, 0|XFAIL_NATIVE_LINUXL },
#endif

#ifdef TEST_FLOAT
	TEST_c_c(csinh, 180, 0x1p-149, INF_P, 1.043535896672617552965983803453927655332e33L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXF },
#endif

#if defined TEST_DOUBLE || (defined TEST_LDOUBLE && LDBL_MAX_EXP == 1024)
	TEST_c_c(csinh, 1440, 0x1p-1074, INF_P, 5.981479269486130556466515778180916082415e301L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(csinh, 22730, 0x1p-16434L, INF_P, 1.217853148905605987081057582351152052687e4924L, OVERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(csinh, SUBNORM_P, MIN_P, SUBNORM_P, MIN_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(csinh, MIN_P, SUBNORM_P, MIN_P, SUBNORM_P, UNDERFLOW_EXCEPTION },
}


static void test_csqrt(void)
{
	struct test *testp = lookup("csqrt" },

	TEST_c_c(csqrt, 0.0, 0.0, 0.000000000000000000e00, 0.000000000000000000e00, 0 },
	TEST_c_c(csqrt, 1.0, 2.0, 1.272019654512405395e00, 7.861513718962669372e-01, 0 },
	TEST_c_c(csqrt, 0.0, 2.0, 1.000000000000000000e00, 1.000000000000000000e00, 0 },
	TEST_c_c(csqrt, 1.0, 0.0, 1.000000000000000000e00, 0.000000000000000000e00, 0 },
	TEST_c_c(csqrt, -2.0, 2.0, 6.435942575335502624e-01, 1.553773969411849975e00, 0 },
	TEST_c_c(csqrt, -1.0, 2.0, 7.861513718962669372e-01, 1.272019654512405395e00, 0 },
	TEST_c_c(csqrt, 1.0, 2.0, 1.272019654512405395e00, 7.861513718962669372e-01, 0 },

	TEST_c_c(csqrt, 0, 0, 0.0, 0.0, 0 },
	TEST_c_c(csqrt, 0, ZERO_M, 0, ZERO_M, 0 },
	TEST_c_c(csqrt, ZERO_M, 0, 0.0, 0.0, 0 },
	TEST_c_c(csqrt, ZERO_M, ZERO_M, 0.0, ZERO_M, 0 },

	TEST_c_c(csqrt, INF_M, 0, 0.0, INF_P, 0 },
	TEST_c_c(csqrt, INF_M, 6, 0.0, INF_P, 0 },
	TEST_c_c(csqrt, INF_M, ZERO_M, 0.0, INF_M, 0 },
	TEST_c_c(csqrt, INF_M, -6, 0.0, INF_M, 0 },

	TEST_c_c(csqrt, INF_P, 0, INF_P, 0.0, 0 },
	TEST_c_c(csqrt, INF_P, 6, INF_P, 0.0, 0 },
	TEST_c_c(csqrt, INF_P, ZERO_M, INF_P, ZERO_M, 0 },
	TEST_c_c(csqrt, INF_P, -6, INF_P, ZERO_M, 0 },

	TEST_c_c(csqrt, 0, INF_P, INF_P, INF_P, 0 },
	TEST_c_c(csqrt, 4, INF_P, INF_P, INF_P, 0 },
	TEST_c_c(csqrt, INF_P, INF_P, INF_P, INF_P, 0 },
	TEST_c_c(csqrt, ZERO_M, INF_P, INF_P, INF_P, 0 },
	TEST_c_c(csqrt, -4, INF_P, INF_P, INF_P, 0 },
	TEST_c_c(csqrt, INF_M, INF_P, INF_P, INF_P, 0 },
	TEST_c_c(csqrt, 0, INF_M, INF_P, INF_M, 0 },
	TEST_c_c(csqrt, 4, INF_M, INF_P, INF_M, 0 },
	TEST_c_c(csqrt, INF_P, INF_M, INF_P, INF_M, 0 },
	TEST_c_c(csqrt, ZERO_M, INF_M, INF_P, INF_M, 0 },
	TEST_c_c(csqrt, -4, INF_M, INF_P, INF_M, 0 },
	TEST_c_c(csqrt, INF_M, INF_M, INF_P, INF_M, 0 },

	TEST_c_c(csqrt, INF_M, QNAN_P, QNAN_P, INF_P, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(csqrt, INF_P, QNAN_P, INF_P, QNAN_P, 0 },

	TEST_c_c(csqrt, 0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csqrt, 1, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csqrt, ZERO_M, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csqrt, -1, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(csqrt, QNAN_P, 0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csqrt, QNAN_P, 8, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csqrt, QNAN_P, ZERO_M, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(csqrt, QNAN_P, -8, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(csqrt, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(csqrt, 16.0, -30.0, 5.0, -3.0, 0 },
	TEST_c_c(csqrt, -1, 0, 0.0, 1.0, 0 },
	TEST_c_c(csqrt, 0, 2, 1.0, 1.0, 0 },
	TEST_c_c(csqrt, 119, 120, 12.0, 5.0, 0 },
	TEST_c_c(csqrt, 0.75L, 1.25L, 1.05065169626078392338656675760808326L, 0.594868882070379067881984030639932657L, 0 },
	TEST_c_c(csqrt, -2, -3, 0.89597747612983812471573375529004348L, -1.6741492280355400404480393008490519L, 0 },
	TEST_c_c(csqrt, -2, 3, 0.89597747612983812471573375529004348L, 1.6741492280355400404480393008490519L, 0 },
	/* Principal square root should be returned (i.e., non-negative real
	   part).  */
	TEST_c_c(csqrt, 0, -1, M_SQRT1_2l, -M_SQRT1_2l, 0 },

	TEST_c_c(csqrt, 0x1.fffffep+127L, 0x1.fffffep+127L, 2.026714405498316804978751017492482558075e+19L, 8.394925938143272988211878516208015586281e+18L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(csqrt, 0x1.fffffep+127L, 1.0L, 1.844674352395372953599975585936590505260e+19L, 2.710505511993121390769065968615872097053e-20L, 0 },
	TEST_c_c(csqrt, 0x1p-149L, 0x1p-149L, 4.112805464342778798097003462770175200803e-23L, 1.703579802732953750368659735601389709551e-23L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(csqrt, 0x1p-147L, 0x1p-147L, 8.225610928685557596194006925540350401606e-23L, 3.407159605465907500737319471202779419102e-23L, 0|XFAIL_NATIVE_LINUXF },

	TEST_c_c(csqrt, ZERO_P, 0x1p-149L, 2.646977960169688559588507814623881131411e-23L, 2.646977960169688559588507814623881131411e-23L, 0|XFAIL_NATIVE_LINUXF },
	TEST_c_c(csqrt, 0x1p-50L, 0x1p-149L, 2.980232238769531250000000000000000000000e-8L, 2.350988701644575015937473074444491355637e-38L, 0|XFAIL_NATIVE_LINUXF },
#ifdef TEST_FLOAT
	TEST_c_c(csqrt, 0x1p+127L, 0x1p-149L, 1.304381782533278221234957180625250836888e19L, ZERO_P, UNDERFLOW_EXCEPTION },
#endif
	TEST_c_c(csqrt, 0x1p-149L, 0x1p+127L, 9.223372036854775808000000000000000000000e18L, 9.223372036854775808000000000000000000000e18L, 0 },
	TEST_c_c(csqrt, 0x1.000002p-126L, 0x1.000002p-126L, 1.191195773697904627170323731331667740087e-19L, 4.934094449071842328766868579214125217132e-20L, UNDERFLOW_EXCEPTION_OK },
	TEST_c_c(csqrt, -0x1.000002p-126L, -0x1.000002p-126L, 4.934094449071842328766868579214125217132e-20L, -1.191195773697904627170323731331667740087e-19L, UNDERFLOW_EXCEPTION_OK },

#ifndef TEST_FLOAT
	TEST_c_c(csqrt, 0x1.fffffffffffffp+1023L, 0x1.fffffffffffffp+1023L, 1.473094556905565378990473658199034571917e+154L, 6.101757441282702188537080005372547713595e+153L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(csqrt, 0x1.fffffffffffffp+1023L, 0x1p+1023L, 1.379778091031440685006200821918878702861e+154L, 3.257214233483129514781233066898042490248e+153L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(csqrt, 0x1p-1074L, 0x1p-1074L, 2.442109726130830256743814843868934877597e-162L, 1.011554969366634726113090867589031782487e-162L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(csqrt, 0x1p-1073L, 0x1p-1073L, 3.453664695497464982856905711457966660085e-162L, 1.430554756764195530630723976279903095110e-162L, 0|XFAIL_NATIVE_LINUX },

	TEST_c_c(csqrt, ZERO_P, 0x1p-1074L, 1.571727784702628688909515672805082228285e-162L, 1.571727784702628688909515672805082228285e-162L, 0|XFAIL_NATIVE_LINUX },
	TEST_c_c(csqrt, 0x1p-500L, 0x1p-1074L, 5.527147875260444560247265192192255725514e-76L, 4.469444793151709302716387622440056066334e-249L, 0|XFAIL_NATIVE_LINUX },
#if defined TEST_DOUBLE || (defined TEST_LDOUBLE && LDBL_MAX_EXP == 1024)
	TEST_c_c(csqrt, 0x1p+1023L, 0x1p-1074L, 9.480751908109176726832526455652159260085e153L, ZERO_P, UNDERFLOW_EXCEPTION },
#endif
	TEST_c_c(csqrt, 0x1p-1074L, 0x1p+1023L, 6.703903964971298549787012499102923063740e153L, 6.703903964971298549787012499102923063740e153L, 0 },
	TEST_c_c(csqrt, 0x1.0000000000001p-1022L, 0x1.0000000000001p-1022L, 1.638872094839911521020410942677082920935e-154L, 6.788430486774966350907249113759995429568e-155L, 0 },
	TEST_c_c(csqrt, -0x1.0000000000001p-1022L, -0x1.0000000000001p-1022L, 6.788430486774966350907249113759995429568e-155L, -1.638872094839911521020410942677082920935e-154L, 0 },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(csqrt, 0x1.fp+16383L, 0x1.fp+16383L, 1.179514222452201722651836720466795901016e+2466L, 4.885707879516577666702435054303191575148e+2465L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(csqrt, 0x1.fp+16383L, 0x1p+16383L, 1.106698967236475180613254276996359485630e+2466L, 2.687568007603946993388538156299100955642e+2465L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(csqrt, 0x1p-16440L, 0x1p-16441L, 3.514690655930285351254618340783294558136e-2475L,  8.297059146828716918029689466551384219370e-2476L, 0|XFAIL_NATIVE_LINUXL },

	TEST_c_c(csqrt, ZERO_P, 0x1p-16445L, 4.269191686890197837775136325621239761720e-2476L, 4.269191686890197837775136325621239761720e-2476L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(csqrt, 0x1p-5000L, 0x1p-16445L, 2.660791472672778409283210520357607795518e-753L, 6.849840675828785164910701384823702064234e-4199L, 0|XFAIL_NATIVE_LINUXL },
	TEST_c_c(csqrt, 0x1p+16383L, 0x1p-16445L, 7.712754032630730034273323365543179095045e2465L, ZERO_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(csqrt, 0x1p-16445L, 0x1p+16383L, 5.453740678097079647314921223668914312241e2465L, 5.453740678097079647314921223668914312241e2465L, 0 },
	TEST_c_c(csqrt, 0x1.0000000000000002p-16382L, 0x1.0000000000000002p-16382L, 2.014551439675644900131815801350165472778e-2466L, 8.344545284118961664300307045791497724440e-2467L, 0 },
	TEST_c_c(csqrt, -0x1.0000000000000002p-16382L, -0x1.0000000000000002p-16382L, 8.344545284118961664300307045791497724440e-2467L, -2.014551439675644900131815801350165472778e-2466L, 0 },

# if LDBL_MANT_DIG >= 113
	TEST_c_c(csqrt, ZERO_P, 0x1p-16494L, 1.799329752913293143453817328207572571442e-2483L, 1.799329752913293143453817328207572571442e-2483L, 0 },
	TEST_c_c(csqrt, 0x1p-5000L, 0x1p-16494L, 2.660791472672778409283210520357607795518e-753L, 1.216776133331049643422030716668249905907e-4213L, 0 },
	TEST_c_c(csqrt, 0x1p+16383L, 0x1p-16494L, 7.712754032630730034273323365543179095045e2465L, ZERO_P, UNDERFLOW_EXCEPTION },
	TEST_c_c(csqrt, 0x1p-16494L, 0x1p+16383L, 5.453740678097079647314921223668914312241e2465L, 5.453740678097079647314921223668914312241e2465L, 0 },
	TEST_c_c(csqrt, 0x1.0000000000000000000000000001p-16382L, 0x1.0000000000000000000000000001p-16382L, 2.014551439675644900022606748976158925145e-2466L, 8.344545284118961663847948339519226074126e-2467L, 0 },
	TEST_c_c(csqrt, -0x1.0000000000000000000000000001p-16382L, -0x1.0000000000000000000000000001p-16382L, 8.344545284118961663847948339519226074126e-2467L, -2.014551439675644900022606748976158925145e-2466L, 0 },
# endif
#endif
}


static void test_ctan(void)
{
	struct test *testp = lookup("ctan" },

	TEST_c_c(ctan, 1.0e00, 2.0e00, 3.381283208727836608e-02, 1.014793619513511657e00, 0 },
	TEST_c_c(ctan, 0.0e00, 2.0e00, 0.0e00, 9.640275761485099792e-01, 0 },
	TEST_c_c(ctan, 1.0e00, 0.0e00, 1.557407721877098083e00, 0.0e00, 0 },
	TEST_c_c(ctan, -2.0e00, 2.0e00, 2.839295566082000732e-02, 1.023835599422454834e00, 0 },
	TEST_c_c(ctan, -1.0e00, -2.0e00, -3.381283208727836608e-02, -1.014793619513511657e00, 0 },
	TEST_c_c(ctan, 1.0e00, -2.0e00, 3.381283208727836608e-02, -1.014793619513511657e00, 0 },

	TEST_c_c(ctan, 0, 0, 0.0, 0.0, 0 },
	TEST_c_c(ctan, 0, ZERO_M, 0.0, ZERO_M, 0 },
	TEST_c_c(ctan, ZERO_M, 0, ZERO_M, 0.0, 0 },
	TEST_c_c(ctan, ZERO_M, ZERO_M, ZERO_M, ZERO_M, 0 },

	TEST_c_c(ctan, 0, INF_P, 0.0, 1.0, 0 },
	TEST_c_c(ctan, 1, INF_P, 0.0, 1.0, 0 },
	TEST_c_c(ctan, ZERO_M, INF_P, ZERO_M, 1.0, 0 },
	TEST_c_c(ctan, -1, INF_P, ZERO_M, 1.0, 0 },

	TEST_c_c(ctan, 0, INF_M, 0.0, -1.0, 0 },
	TEST_c_c(ctan, 1, INF_M, 0.0, -1.0, 0 },
	TEST_c_c(ctan, ZERO_M, INF_M, ZERO_M, -1.0, 0 },
	TEST_c_c(ctan, -1, INF_M, ZERO_M, -1.0, 0 },

	TEST_c_c(ctan, INF_P, 0, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctan, INF_P, 2, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctan, INF_M, 0, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctan, INF_M, 2, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctan, INF_P, ZERO_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctan, INF_P, -2, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctan, INF_M, ZERO_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctan, INF_M, -2, QNAN_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(ctan, QNAN_P, INF_P, 0.0, 1.0, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(ctan, QNAN_P, INF_M, 0.0, -1.0, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(ctan, 0, QNAN_P, 0.0, QNAN_P, 0 },
	TEST_c_c(ctan, ZERO_M, QNAN_P, ZERO_M, QNAN_P, 0 },

	TEST_c_c(ctan, 0.5, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ctan, -4.5, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(ctan, QNAN_P, 0, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ctan, QNAN_P, 5, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ctan, QNAN_P, ZERO_M, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ctan, QNAN_P, -0.25, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(ctan, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(ctan, 0.75L, 1.25L, 0.160807785916206426725166058173438663L, 0.975363285031235646193581759755216379L, 0 },
	TEST_c_c(ctan, -2, -3, 0.376402564150424829275122113032269084e-2L, -1.00323862735360980144635859782192726L, 0 },

	TEST_c_c(ctan, 1, 45, 1.490158918874345552942703234806348520895e-39L, 1.000000000000000000000000000000000000001L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },
	TEST_c_c(ctan, 1, 47, 2.729321264492904590777293425576722354636e-41L, 1.0, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },

#ifndef TEST_FLOAT
	TEST_c_c(ctan, 1, 355, 8.140551093483276762350406321792653551513e-309L, 1.0, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(ctan, 1, 365, 1.677892637497921890115075995898773550884e-317L, 1.0, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(ctan, 1, 5680, 4.725214596136812019616700920476949798307e-4934L, 1.0, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(ctan, 1, 5690, 9.739393181626937151720816611272607059057e-4943L, 1.0, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(ctan, 0x3.243f6cp-1, 0, -2.287733242885645987394874673945769518150e7L, 0.0, 0|XFAIL_NATIVE_LINUX },

	TEST_c_c(ctan, 0x1p127, 1, 0.2446359391192790896381501310437708987204L, 0.9101334047676183761532873794426475906201L, 0|XFAIL_NATIVE_LINUXL|XFAIL_NATIVE_LINUXF|XFAIL_FPU },

#ifndef TEST_FLOAT
	TEST_c_c(ctan, 0x1p1023, 1, -0.2254627924997545057926782581695274244229L, 0.8786063118883068695462540226219865087189L, 0|XFAIL_NATIVE_LINUX|XFAIL_FPU },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(ctan, 0x1p16383L, 1, 0.1608598776370396607204448234354670036772L, 0.8133818522051542536316746743877629761488L, 0|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif

	TEST_c_c(ctan, 50000, 50000, ZERO_P, 1.0, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(ctan, 50000, -50000, ZERO_P, -1.0, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(ctan, -50000, 50000, ZERO_M, 1.0, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(ctan, -50000, -50000, ZERO_M, -1.0, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },

	TEST_c_c(ctan, 0x1.921fb6p+0, 0x1p-149, -2.287733242885645987394874673945769518150e7L, 7.334008549954377778731880988481078535821e-31L, 0|ROUND_TONEAREST|XFAIL_NATIVE_LINUX },

#ifndef TEST_FLOAT
	TEST_c_c(ctan, 0x1.921fb54442d18p+0, 0x1p-1074, 1.633123935319536975596773704152891653086e16L, 1.317719414943508315995636961402669067843e-291L, 0|ROUND_TONEAREST|XFAIL_NATIVE_LINUX|XFAIL_FPU },
#endif

#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(ctan, 0x1.921fb54442d1846ap+0L, 0x1p-16445L, -3.986797629811710706723242948653362815645e19L, 5.793882568875674066286163141055208625180e-4912L, 0|ROUND_TONEAREST|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif

	TEST_c_c(ctan, 0x1.921fb6p+0, 0x1p-149, -2.287733242885645987394874673945769518150e7L, 7.334008549954377778731880988481078535821e-31L, 0|ROUND_TOWARDZERO|XFAIL_NATIVE_LINUX },

#ifndef TEST_FLOAT
	TEST_c_c(ctan, 0x1.921fb54442d18p+0, 0x1p-1074, 1.633123935319536975596773704152891653086e16L, 1.317719414943508315995636961402669067843e-291L, 0|ROUND_TOWARDZERO|XFAIL_NATIVE_LINUX|XFAIL_FPU },
#endif

#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(ctan, 0x1.921fb54442d1846ap+0L, 0x1p-16445L, -3.986797629811710706723242948653362815645e19L, 5.793882568875674066286163141055208625180e-4912L, 0|ROUND_TOWARDZERO|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif

	TEST_c_c(ctan, 0x1.921fb6p+0, 0x1p-149, -2.287733242885645987394874673945769518150e7L, 7.334008549954377778731880988481078535821e-31L, 0|ROUND_DOWNWARD|XFAIL_NATIVE_LINUX },

#ifndef TEST_FLOAT
	TEST_c_c(ctan, 0x1.921fb54442d18p+0, 0x1p-1074, 1.633123935319536975596773704152891653086e16L, 1.317719414943508315995636961402669067843e-291L, 0|ROUND_DOWNWARD|XFAIL_NATIVE_LINUX|XFAIL_FPU },
#endif

#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(ctan, 0x1.921fb54442d1846ap+0L, 0x1p-16445L, -3.986797629811710706723242948653362815645e19L, 5.793882568875674066286163141055208625180e-4912L, 0|ROUND_DOWNWARD|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif

	TEST_c_c(ctan, 0x1.921fb6p+0, 0x1p-149, -2.287733242885645987394874673945769518150e7L, 7.334008549954377778731880988481078535821e-31L, 0|ROUND_UPWARD|XFAIL_NATIVE_LINUX },

#ifndef TEST_FLOAT
	TEST_c_c(ctan, 0x1.921fb54442d18p+0, 0x1p-1074, 1.633123935319536975596773704152891653086e16L, 1.317719414943508315995636961402669067843e-291L, 0|ROUND_UPWARD|XFAIL_NATIVE_LINUX|XFAIL_FPU },
#endif

#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(ctan, 0x1.921fb54442d1846ap+0L, 0x1p-16445L, -3.986797629811710706723242948653362815645e19L, 5.793882568875674066286163141055208625180e-4912L, 0|ROUND_UPWARD|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif
}


static void test_ctanh(void)
{
	struct test *testp = lookup("ctanh" },

	TEST_c_c(ctanh, 1.0e00, 2.0e00, 1.166736245155334472e00, -2.434581927955150604e-01, 0 },
	TEST_c_c(ctanh, 0.0e00, 2.0e00, 0.0e00, -2.185039848089218139e00, 0 },
	TEST_c_c(ctanh, 1.0e00, 0.0e00, 7.615941539406776428e-01, 0.0e00, 0 },
	TEST_c_c(ctanh, -2.0e00, 2.0e00, -1.023835599422454834e00, -2.839295566082000732e-02, 0 },
	TEST_c_c(ctanh, -1.0e00, -2.0e00, -1.166736245155334472e00, 2.434581927955150604e-01, 0 },
	TEST_c_c(ctanh, 1.0e00, -2.0e00, 1.166736245155334472e00, 2.434581927955150604e-01, 0 },

	TEST_c_c(ctanh, 0, 0, 0.0, 0.0, 0 },
	TEST_c_c(ctanh, 0, ZERO_M, 0.0, ZERO_M, 0 },
	TEST_c_c(ctanh, ZERO_M, 0, ZERO_M, 0.0, 0 },
	TEST_c_c(ctanh, ZERO_M, ZERO_M, ZERO_M, ZERO_M, 0 },

	TEST_c_c(ctanh, INF_P, 0, 1.0, 0.0, 0 },
	TEST_c_c(ctanh, INF_P, 1, 1.0, 0.0, 0 },
	TEST_c_c(ctanh, INF_P, ZERO_M, 1.0, ZERO_M, 0 },
	TEST_c_c(ctanh, INF_P, -1, 1.0, ZERO_M, 0 },
	TEST_c_c(ctanh, INF_M, 0, -1.0, 0.0, 0 },
	TEST_c_c(ctanh, INF_M, 1, -1.0, 0.0, 0 },
	TEST_c_c(ctanh, INF_M, ZERO_M, -1.0, ZERO_M, 0 },
	TEST_c_c(ctanh, INF_M, -1, -1.0, ZERO_M, 0 },

	TEST_c_c(ctanh, 0, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctanh, 2, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctanh, 0, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctanh, 2, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctanh, ZERO_M, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctanh, -2, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctanh, ZERO_M, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },
	TEST_c_c(ctanh, -2, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION },

	TEST_c_c(ctanh, INF_P, QNAN_P, 1.0, 0.0, IGNORE_ZERO_INF_SIGN },
	TEST_c_c(ctanh, INF_M, QNAN_P, -1.0, 0.0, IGNORE_ZERO_INF_SIGN },

	TEST_c_c(ctanh, QNAN_P, 0, QNAN_P, 0.0, 0 },
	TEST_c_c(ctanh, QNAN_P, ZERO_M, QNAN_P, ZERO_M, 0 },

	TEST_c_c(ctanh, QNAN_P, 0.5, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ctanh, QNAN_P, -4.5, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(ctanh, 0, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ctanh, 5, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ctanh, ZERO_M, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	TEST_c_c(ctanh, -0.25, QNAN_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },

	TEST_c_c(ctanh, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_c_c(ctanh, 0, M_PI_4l, 0.0, 1.0, 0 },

	TEST_c_c(ctanh, 0.75L, 1.25L, 1.37260757053378320258048606571226857L, 0.385795952609750664177596760720790220L, 0 },
	TEST_c_c(ctanh, -2, -3, -0.965385879022133124278480269394560686L, 0.988437503832249372031403430350121098e-2L, 0 },

	TEST_c_c(ctanh, 45, 1, 1.000000000000000000000000000000000000001L, 1.490158918874345552942703234806348520895e-39L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },
	TEST_c_c(ctanh, 47, 1, 1.0, 2.729321264492904590777293425576722354636e-41L, UNDERFLOW_EXCEPTION_FLOAT|XFAIL_NATIVE_LINUXF },

#ifndef TEST_FLOAT
	TEST_c_c(ctanh, 355, 1, 1.0, 8.140551093483276762350406321792653551513e-309L, UNDERFLOW_EXCEPTION_DOUBLE },
	TEST_c_c(ctanh, 365, 1, 1.0, 1.677892637497921890115075995898773550884e-317L, UNDERFLOW_EXCEPTION_DOUBLE|XFAIL_NATIVE_LINUX },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(ctanh, 5680, 1, 1.0, 4.725214596136812019616700920476949798307e-4934L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
	TEST_c_c(ctanh, 5690, 1, 1.0, 9.739393181626937151720816611272607059057e-4943L, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUXL },
#endif

	TEST_c_c(ctanh, 0, 0x3.243f6cp-1, 0.0, -2.287733242885645987394874673945769518150e7L, 0|XFAIL_NATIVE_LINUX },

	TEST_c_c(ctanh, 1, 0x1p127, 0.9101334047676183761532873794426475906201L, 0.2446359391192790896381501310437708987204L, 0|XFAIL_NATIVE_LINUXL|XFAIL_NATIVE_LINUXF|XFAIL_FPU },

#ifndef TEST_FLOAT
	TEST_c_c(ctanh, 1, 0x1p1023, 0.8786063118883068695462540226219865087189L, -0.2254627924997545057926782581695274244229L, 0|XFAIL_NATIVE_LINUX|XFAIL_FPU },
#endif

#if defined TEST_LDOUBLE && LDBL_MAX_EXP >= 16384
	TEST_c_c(ctanh, 1, 0x1p16383L, 0.8133818522051542536316746743877629761488L, 0.1608598776370396607204448234354670036772L, 0|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif

	TEST_c_c(ctanh, 50000, 50000, 1.0, ZERO_P, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(ctanh, 50000, -50000, 1.0, ZERO_M, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(ctanh, -50000, 50000, -1.0, ZERO_P, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },
	TEST_c_c(ctanh, -50000, -50000, -1.0, ZERO_M, UNDERFLOW_EXCEPTION|XFAIL_NATIVE_LINUX },

	TEST_c_c(ctanh, 0x1p-149, 0x1.921fb6p+0, 7.334008549954377778731880988481078535821e-31L, -2.287733242885645987394874673945769518150e7L, 0|ROUND_TONEAREST|XFAIL_NATIVE_LINUX },

#ifndef TEST_FLOAT
	TEST_c_c(ctanh, 0x1p-1074, 0x1.921fb54442d18p+0, 1.317719414943508315995636961402669067843e-291L, 1.633123935319536975596773704152891653086e16L, 0|ROUND_TONEAREST|XFAIL_NATIVE_LINUX|XFAIL_FPU },
#endif

#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(ctanh, 0x1p-16445L, 0x1.921fb54442d1846ap+0L, 5.793882568875674066286163141055208625180e-4912L, -3.986797629811710706723242948653362815645e19L, 0|ROUND_TONEAREST|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif

	TEST_c_c(ctanh, 0x1p-149, 0x1.921fb6p+0, 7.334008549954377778731880988481078535821e-31L, -2.287733242885645987394874673945769518150e7L, 0|ROUND_TOWARDZERO|XFAIL_NATIVE_LINUX },

#ifndef TEST_FLOAT
	TEST_c_c(ctanh, 0x1p-1074, 0x1.921fb54442d18p+0, 1.317719414943508315995636961402669067843e-291L, 1.633123935319536975596773704152891653086e16L, 0|ROUND_TOWARDZERO|XFAIL_NATIVE_LINUX|XFAIL_FPU },
#endif

#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(ctanh, 0x1p-16445L, 0x1.921fb54442d1846ap+0L, 5.793882568875674066286163141055208625180e-4912L, -3.986797629811710706723242948653362815645e19L, 0|ROUND_TOWARDZERO|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif

	TEST_c_c(ctanh, 0x1p-149, 0x1.921fb6p+0, 7.334008549954377778731880988481078535821e-31L, -2.287733242885645987394874673945769518150e7L, 0|ROUND_DOWNWARD|XFAIL_NATIVE_LINUX },

#ifndef TEST_FLOAT
	TEST_c_c(ctanh, 0x1p-1074, 0x1.921fb54442d18p+0, 1.317719414943508315995636961402669067843e-291L, 1.633123935319536975596773704152891653086e16L, 0|ROUND_DOWNWARD|XFAIL_NATIVE_LINUX|XFAIL_FPU },
#endif

#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(ctanh, 0x1p-16445L, 0x1.921fb54442d1846ap+0L, 5.793882568875674066286163141055208625180e-4912L, -3.986797629811710706723242948653362815645e19L, 0|ROUND_DOWNWARD|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif

	TEST_c_c(ctanh, 0x1p-149, 0x1.921fb6p+0, 7.334008549954377778731880988481078535821e-31L, -2.287733242885645987394874673945769518150e7L, 0|ROUND_UPWARD|XFAIL_NATIVE_LINUX },

#ifndef TEST_FLOAT
	TEST_c_c(ctanh, 0x1p-1074, 0x1.921fb54442d18p+0, 1.317719414943508315995636961402669067843e-291L, 1.633123935319536975596773704152891653086e16L, 0|ROUND_UPWARD|XFAIL_NATIVE_LINUX|XFAIL_FPU },
#endif

#if defined TEST_LDOUBLE && LDBL_MIN_EXP <= -16381
	TEST_c_c(ctanh, 0x1p-16445L, 0x1.921fb54442d1846ap+0L, 5.793882568875674066286163141055208625180e-4912L, -3.986797629811710706723242948653362815645e19L, 0|ROUND_UPWARD|XFAIL_NATIVE_LINUXL|XFAIL_FPU },
#endif
}


#if defined(NATIVE_WIN32) || defined(NATIVE_LINUX)
#if defined(TEST_FLOAT)
#include "../src/crcpf.c"
#elif defined(TEST_LDOUBLE)
#include "../src/crcpl.c"
#else
#include "../src/crcp.c"
#endif
#include "../src/p_pmlerrs.c"
#include "../src/p_matherr.c"
#include "../src/s_signgam.c"
#include "../src/s_issignaling.c"
#include "../src/s_issignalingf.c"
#include "../src/s_issignalingl.c"
#endif


static void test_crcp(void)
{
	struct test *testp = lookup("crcp" },

	TEST_c_c(crcp, 1.0e00, 2.0e00, 1.999999992549419403e-01, -3.999999985098838806e-01, 0 },
	TEST_c_c(crcp, 0.0e00, 2.0e00, 0.0e00, -5.0e-01, 0 },
	TEST_c_c(crcp, 1.0e00, 0.0e00, 1.0e00, -0.0e00, 0 },
	TEST_c_c(crcp, -2.0e00, 2.0e00, -2.5e-01, -2.5e-01, 0 },
	TEST_c_c(crcp, -1.0e00, -2.0e00, -1.999999992549419403e-01, 3.999999985098838806e-01, 0 },
	TEST_c_c(crcp, 1.0e00, -2.0e00, 1.999999992549419403e-01, 3.999999985098838806e-01, 0 },
}


static void test_cpow(void)
{
	struct test *testp = lookup("cpow" },

	TEST_cc_c (cpow, 1, 0, 0, 0, 1.0, 0.0, 0 },
	TEST_cc_c (cpow, 2, 0, 10, 0, 1024.0, 0.0, 0 },
#if 0
	/* Disabled until we fix bug 14473.  */
	TEST_cc_c (cpow, M_El, 0, 0, 2 * M_PIl, 1.0, 0.0, 0 },
#endif
	TEST_cc_c (cpow, 2, 3, 4, 0, -119.0, -120.0, 0 },

	TEST_cc_c (cpow, QNAN_P, QNAN_P, QNAN_P, QNAN_P, QNAN_P, QNAN_P, 0 },

	TEST_cc_c (cpow, 0.75L, 1.25L, 0.75L, 1.25L, 0.117506293914473555420279832210420483L, 0.346552747708338676483025352060418001L, 0 },
	TEST_cc_c (cpow, 0.75L, 1.25L, 1.0L, 1.0L, 0.0846958290317209430433805274189191353L, 0.513285749182902449043287190519090481L, 0 },
	TEST_cc_c (cpow, 0.75L, 1.25L, 1.0L, 0.0L, 0.75L, 1.25L, 0 },
	TEST_cc_c (cpow, 0.75L, 1.25L, 0.0L, 1.0L, 0.331825439177608832276067945276730566L, 0.131338600281188544930936345230903032L, 0 },
}
